import bb.cascades 1.0
import bb.multimedia 1.0

//navControl provides safe access to dynamically create pages from a given page.qml file.
//you can also open a sheet and and pages will then be stacked there 
NavigationPane {
    id: navControl
    property bool __isPushing: false
    
    //used to dynmically create pages
    attachedObjects: [
        ComponentDefinition {
            id: pageDefinition;
            source: ""
        },

        //For displaying the a page inside a Sheet
        Sheet {
            id: theSheet;
            property bool isOpen: false
            content: NavigationPane {
                id: theSheetNavControl;
                onPushTransitionEnded: {
                    console.debug("SafePushNavPane::Sheet::onPushTransitionEnded");
                    navControl.__isPushing = false;
                    appContext.searchScreen.enableTypeToSearch(false);
                }
                onPopTransitionEnded: {
                    console.debug("SafePushNavPane::Sheet::onPopTransitionEnded");
                    page.destroy();
                    appContext.searchScreen.enableTypeToSearch(true);
                }
            }
        }
    ]
        
    // signal for when the search page gets popped. Remove this once the search has been converted to a Sheet.
    signal searchScreenPopped()
    
    // signal for when the playon list screen gets popped
    signal playonListScreenPopped()

    onCreationCompleted: {
        // Used to correctly set the now playing overlay style
        Application.fullscreen.connect(decideOverlayStyle);
        Application.invisible.connect(applicationMinimized);
        Application.thumbnail.connect(applicationMinimized);
    }
    
    onTopChanged: {
        console.log("SafePushNavPane::onTopChanged");
        if(tabbedPane.appLoaded){
	        decideOverlayStyle();
        	appContext.addSearchScreenToPage(page);
	    }
    }
    
    onPushTransitionEnded: {
        console.log("SafePushNavPane::onPushTransitionEnded");
        navControl.__isPushing = false;
    }
    onPopTransitionEnded: {
        if (page.objectName == "PlayOnListObjectName") {
            playonListScreenPopped();
        }
        pageCleanup(page);
    }
    
    function decideOverlayStyle() {
        console.log("SafePushNavPane::decideOverlayStyle");
        // If the current screen is the player screen, we want to change the controller overlay style
        if (null != navControl.top && "NewPlayerScreenObjectName" == navControl.top.objectName) {
            localPlaybackHandler.overlayStyle = OverlayStyle.Plain;
        } else {
            localPlaybackHandler.overlayStyle = OverlayStyle.Fancy;
        }
    }
    
    function applicationMinimized() {
        localPlaybackHandler.overlayStyle = OverlayStyle.Fancy;
    }

    //Push a new dynamically created "page". "page" is the qml filename eg file.qml
    function safePushOnePage(pageFile) {
        console.debug("SafePushNavPane::safePushOnePage", pageFile);
        if (navControl.__isPushing) {
            console.debug("SafePushNavPane::safePushOnePage currently pushing. New page not created");
            return null;
        } else {            
            navControl.__isPushing = true;
            pageDefinition.source = pageFile;
            var pageDef = pageDefinition.createObject();
            if (!pageDef) {
                console.debug("SafePushNavPane::safePushOnePage failed to create new page");
                console.debug("SafePushNavPane::safePushOnePage " + pageDef.errorMessage());
                navControl.__isPushing = false;
                return null;
            }
            navControl.push(pageDef);
            return pageDef;
        }
    }
    
    //A sheet is created if it does not exist. If the Sheet is already open, it just does a push
    //Push a new dynamically created "page" on Sheet. "page" is the qml filename eg file.qml
    function safePushOnePageOnSheet(pageFile) {
        //check to see if a push is in progress
        if (navControl.__isPushing) {
            console.debug("SafePushNavPane::safePushOnePageOnSheet currently pushing. New page not created: ", pageFile);
            return null;
        }
        
        //Is a sheet already open
        if (theSheet.isOpen) {
            //push on the current sheet
            console.debug("SafePushNavPane::safePushOnePageOnSheet on current sheet", pageFile);
            pageDefinition.source = pageFile;
            var pageDef = pageDefinition.createObject();
            if (!pageDef) {
                console.log("SafePushNavPane::safePushOnePageOnSheet failed to create new page");
                navControl.__isPushing = false;
                return null;
            }
            theSheetNavControl.push(pageDef);
            return pageDef;
        } // else 
        //open a sheet with it's own nav panel
        console.debug("SafePushNavPane::safePushOnePageOnSheet open sheet", pageFile);
        
        //make sure the stack is empty
        while (theSheetNavControl.top) {
            theSheetNavControl.pop();
        }

        //create the new page
        pageDefinition.source = pageFile;
        var pageDef = pageDefinition.createObject();
        if (!pageDef) {
            console.debug("SafePushNavPane::safePushOnePageOnSheet failed to create page on new sheet", pageFile);
            console.debug("SafePushNavPane::safePushOnePageOnSheet " + pageDef.errorMessage());
            navControl.__isPushing = false;
            return null;
        }
        
        theSheetNavControl.push(pageDef);
        theSheet.open();
        theSheet.peekEnabled = false; //because all of the sheets that use this do not want peek we will disable it
        theSheet.isOpen = true;
        return pageDef;
    }
    
    //push a 'Page' on the stack
    //if a sheet is open, it's put there
    function safePush(page) {
        if (theSheet.isOpen) {
            console.debug("SafePushNavPane::safePush on sheet", page.source);
            theSheetNavControl.push(page);
        } else {
            console.debug("SafePushNavPane::safePush", page.source);
            navControl.push(page);
        }
    }
    
    //pop a 'Page' off the stack
    function safePop() {
        console.debug("SafePushNavPane::safePop");
        if (theSheet.isOpen) {
            theSheetNavControl.pop(page);
        } else {
            navControl.pop(pageDef);
        }
    }
    
    function safeCloseSheet() {
        console.debug("SafePushNavPane::closeSheet");
        theSheet.close();
        navControl.__isPushing = false;
        
        //make sure the stack is empty
        while (theSheetNavControl.top) {
            theSheetNavControl.pop();
        }

        theSheet.isOpen=false;
       
    }
    
    
    function popAndDeleteAll() {
        while (navControl.count() > 1) {
           var page = navControl.top;
            navControl.remove(page);
            pageCleanup(page);
        }
    }
    
    // Pops the artistDetails screen if it is at the top so that we still get the transition effect
    // Removes otherwise so that we don't see an extra pop transition.
    function popOrRemoveArtistDetails() {
        if (navControl.top.objectName == "artistDetails") {
            navControl.pop();
        } else {
            for( var i = navControl.count() - 1 ; i >= 0 ; --i ) {
    	        if (navControl.at(i).objectName == "artistDetails") {
    	            var page = navControl.at(i);
    	            navControl.remove(page);
    	            page.destroy();
    	        }
    	    }
        }
    }
    
    function popLocalPlayerScreen() {
        popScreenByName("NewPlayerScreenObjectName");
    }
    
    function popRadioPlayerScreen() {
        popScreenByName("RadioPlayerScreenObjectName");        
    }
    
    // Closes the named screen (if opened), including any child pages
    function popScreenByName(pageName) {
        console.debug("SafePushNavPane::popScreenByName: " + pageName);
        
        // Scan the nav pane stack to see if the named screen is open
        for( var i = navControl.count() - 1 ; i >= 0 ; --i ) {
            // Is it the named screen?
            if( navControl.at(i).objectName == pageName ) {
                // First, pop any screens that are on top of the named screen
                while( navControl.top.objectName != pageName ) {
                    navControl.pop();
                }
                // Top item is the named screen, pop it
                navControl.pop();
                
                // We are done, break out of the for loop
                break;
            }
        }
    }
    
    //Does some checks and then destroys the page
    function pageCleanup (page)
    {
		if (theSheet.isOpen) { //oops, we just popped the wrong page
			console.log("SafePushNavPane::pageCleanup - Wrong page popped");
		} else {
			console.debug("SafePushNavPane::pageCleanup");		
				if ("NewPlayerScreenObjectName" == page.objectName)
				{
					appContext.onPlayerScreenPopped();
                } else if ("RadioPlayerScreenObjectName" == page.objectName) {
                    RadioPlaybackContext.onRadioPlayerScreenPopped();
				}
				console.debug("SafePushNavPane::pageCleanup - destroying page");
				page.destroy();
		}
    }

    /** Check the navigation stack for dirty pages. */
    function isDirty() {
        for (var i = navControl.count() - 1; i >= 0; --i) {
            var page = navControl.at(i);
            if (page != null && page.isDirty) {
                return true;
            }
        }
        return false;
    }

    
//////////////////////////////////////////////////////////////////////////////////////
//
// Private functions
//
//////////////////////////////////////////////////////////////////////////////////////

}
