import bb.cascades 1.0
import bb.cascades.multimedia.internal 1.0

TabbedPane {
    //this is now set on the root object context via c++
    //id: tabbedPane
    
    showTabsOnActionBar: true
    
    property variant currentNavPane: smartScreenNavControl

    property bool appLoaded: false

    //signal fired when the tab changes, used in smart
    signal tabChanged()
    
    //signal fired when smart has finished loading 
    signal primaryPageLoaded()
    
    //signal fired when application load has completed 
    signal applicationLoadComplete();

    attachedObjects: [
        Tab {
            id: radioScreen
            objectName: "radioScreen"
            title: qsTr("Radio")+Retranslate.onLanguageChanged
            imageSource: "asset:///images/radio/fmradio_ic_radio.png"
            SafePushNavPane {
                id: radioScreenNavControl
            }
        }
    ]

    function setActiveTabByName(name){
        console.log("setActiveTabByName = " + name);
        if(name == "recent"){
            this.activeTab = smartScreen;
        }else if(name == "library"){
            this.activeTab = libraryScreen;
        }else if(name =="playlists"){
            this.activeTab = playlistsScreen;
        }
    }

    function loadPrimaryPage(){
        var component = Qt.createComponent("./core/Smart.qml");
        if (!component.errorString()) {
            var newPage = component.createObject(smartScreenNavControl);    
            smartScreenNavControl.remove(smartPage);
            smartScreenNavControl.insert(0, newPage);
        } else { 
            console.log("critical error loading smart:"  + component.errorString());
        }
    }

    function loadAllPages(){
        if(appContext != null){          
            var radioNav = RadioPlaybackContext.fmRadioSupported ? radioScreenNavControl : null;
            appLoaded = appContext.loadApp(libraryScreenNavControl,
                    playlistsNavControl, radioNav);
            applicationLoadComplete();
        }
    }
    
    function addFMTab(){
        add(radioScreen);
    }

    onActiveTabChanged: {
        currentNavPane = activeTab.content;
        tabChanged();
        //If the objectName changes we'll need to update the if else checks
        if (activeTab.objectName == "smartScreen") {
            commonAnalytics.instance.logActiveTabChanged("Smart");
        } else if (activeTab.objectName == "libraryScreen") {
            commonAnalytics.instance.logActiveTabChanged("Library");
        } else if (activeTab.objectName == "playlistsScreen") {
            commonAnalytics.instance.logActiveTabChanged("Playlists");
        } else if (activeTab.objectName == "radioScreen") {
            commonAnalytics.instance.logActiveTabChanged("Radio");
        } else {
            //We want to log tab changes in Analytics we should make sure
            //we're updating things when there are more tabs
            console.warn("Encountered an unknown Tab Name");
            commonAnalytics.instance.logActiveTabChanged(activeTab.objectName);
        }
    }

    Tab {
        id: smartScreen
        objectName: "smartScreen"
        //: navigation tab label
        title: qsTr("Recent")+Retranslate.onLanguageChanged
        imageSource: "asset:///images/tabs/ic_smart.png"
        SafePushNavPane {
            id: smartScreenNavControl
            SmartStart {
                id: smartPage
            }
        }
    }
    
    Tab {
        id: libraryScreen
        objectName: "libraryScreen"
        //: navigation tab label
        title: qsTr("Library") + Retranslate.onLanguageChanged
        imageSource: "asset:///images/tabs/ic_library.png"
        SafePushNavPane {
            id: libraryScreenNavControl
        }
    }
    
    Tab {
        id: playlistsScreen
        objectName: "playlistsScreen"
        //: navigation tab label
        title: qsTr("Playlists") + Retranslate.onLanguageChanged //USED BY STABILITY TESTING
        imageSource: "asset:///images/tabs/ic_playlists.png"
        SafePushNavPane {
            id: playlistsNavControl
        }
    }
}
