import bb.cascades 1.0
import bb.cascades.multimedia 1.0
import bb.cascades.multimedia.internal 1.0
import "ContentUris.js" as Uris

/*
 * A container that displays a list of albums.
 * When an album is selected it opens a list of the albums tracks
 *
 * @slot addAll called to add all of the tracks currently being viewed
 *
 * @author Scott Milliquet
 */
Container {
    id: addAlbumsContainer
    objectName: "addAlbumsContainer"
    
	property alias itemsCountModified: list.itemsCountChanged
    horizontalAlignment: HorizontalAlignment.Fill
    verticalAlignment: VerticalAlignment.Fill
    
    attachedObjects: [
	    //artists
        HeaderTypeDecorator {
	        id: albumDataModel
	        headerType: "list"
	        itemType: "album"
	        model: PrioritizedDataModelDecorator {
	            model: ModelFilterDecorator {
	                id: searchModel
	                filterFields: ["album_artist","album"]
	                queryFactory:   SearchFilterQueryFactory{
	                        	        id: searchFilter
	                        	        dataQuery: Uris.albums(["album_id","thumb_file","album_artist"])
                                        headerQuery: Uris.albumGroups(["album_artist"]);
	                        	        suppressData: false
	                        	    }
                    model: AsyncHeaderDataModel {
                        objectName: "AddAlbums"
	                    id: albumData
	                }
	            }
		    }
	    }
    ]
    
    onCreationCompleted: {
        addMusicViewPort.filterTextChanging.connect(addAlbumsContainer.handleTextChanging);
        searchModel.applyFilter();
    }
        
    function handleTextChanging(text) {
        searchModel.setCriteria(text);
        searchModel.applyFilter();
    }
    
    layout: DockLayout {
        id: dockLayout
    }
    Container {
        
        horizontalAlignment: HorizontalAlignment.Fill
        verticalAlignment: VerticalAlignment.Fill
                 	    
        layout: StackLayout {
            orientation: LayoutOrientation.TopToBottom
        }
        
     	AddMusicListView {
            id: list
			property bool itemsCountChanged: false;
            dataModel: albumDataModel
            
            layout: StackListLayout {
		        headerMode: ListHeaderMode.Sticky
		    }
		    
            listItemComponents: [
                ListItemComponent {
                    type: "album"
                    AddToPlaylistListItemVisual {
                        id: addAlbumsListItem
                        title: { ListItemData["album"] != null ? ListItemData["album"] : (qsTr("Unknown") + Retranslate.onLanguageChanged) }
                        thumb_file: { ListItemData["thumb_file"] != null ? ListItemData["thumb_file"] : "asset:///images/common/album_placeholder.png" }
                        subTitle: ListItemData["album_artist"]
                        onItemSelected: {
                        	ListItem.view.openAlbum(ListItemData);
                        }
                        onAddTrackToPlaylist: {
                            ListItem.view.addAlbum( ListItemData );
                        }
                    }
                },
                ListItemComponent {
                    type: "list_header"
                    Header {
                        id: addAlbumsHeader
                        title: ListItemData["header"]
                    }
                }
            ]
            
	        /*
	         * called to append the selected album to the end of the playlist
	         */
            function addAlbum(data) {
                addMusicViewPort.playlistDataModel.appendAlbum( data["album_id"] );
				list.itemsCountChanged = true;
            }
            
           /*
            * display the contents of the album
            */
            function openAlbum(data) {
                var page = tabbedPane.currentNavPane.safePushOnePageOnSheet("core/AddTracks.qml");
                commonAnalytics.instance.logPagePush("AddTracks", "AddAlbumsContainer");
                page.titleStr = (data["album"] != null && data["album"] != "") ? data["album"] : (qsTr("Unknown") + Retranslate.onLanguageChanged);
                var album_id = data["album_id"];
                if ( album_id == null ) {
                    album_id = "0";
                }
                page.mode = 1; //album
                page.id = album_id;
                page.contentUri = Uris.albumMembers(["title","fullpath","fid"],album_id);
                page.playlistDataModel = addMusicViewPort.playlistDataModel;
            }
            
            //Ensure an item can be selected consecutively
            onTriggered: {
	            clearSelection();
	            list.select(indexPath, false);
            }
        }
    }
}
