import bb.cascades 1.0
import bb.cascades.multimedia 1.0
import bb.cascades.multimedia.internal 1.0
import "ContentUris.js" as Uris

/*
 * A container that displays a list of artists.
 * When an artist is selected it opens a list of the tracks by that artist
 *
 * @slot addAll called to add all of the tracks currently being viewed
 *
 * @author Scott Milliquet
 */
Container {
    id: addArtistsContainer
    objectName: "addArtistsContainer"
    
    property alias itemsCountModified: list.itemsCountChanged
    horizontalAlignment: HorizontalAlignment.Fill
    verticalAlignment: VerticalAlignment.Fill
    
    attachedObjects: [
	    //artists
        HeaderTypeDecorator {
	        id: artistDataModel
	        headerType: "list"
	        itemType: "artist"
	        model: PrioritizedDataModelDecorator {
	            model: ModelFilterDecorator {
	                id: searchModel
	                filterFields: ["album_artist"]
	                queryFactory:   SearchFilterQueryFactory{
	                        	        id: searchFilter
	                        	        dataQuery: Uris.artists(["album_artist_id","thumb_file","count"])
	                        	        headerQuery: Uris.artistGroups([])
	                        	        suppressData: false
	                        	    }
                    model: AsyncHeaderDataModel {
                        objectName: "AddArtists"
	                    id: artistData
	                }
	            }
		    }
	    }
    ]
    
    onCreationCompleted: {
        addMusicViewPort.filterTextChanging.connect(addArtistsContainer.handleTextChanging);
        searchModel.applyFilter();
    }
        
    function handleTextChanging(text) {
        searchModel.setCriteria(text);
        searchModel.applyFilter();
    }
    
    layout: DockLayout {
        id: dockLayout
    }
    Container {
        
        horizontalAlignment: HorizontalAlignment.Fill
        verticalAlignment: VerticalAlignment.Fill
                 	    
        layout: StackLayout {
            orientation: LayoutOrientation.TopToBottom
        }
        
     	AddMusicListView {
            id: list
            objectName: "list"
			property bool itemsCountChanged: false;
            dataModel: artistDataModel
            
            layout: StackListLayout {
                headerMode: ListHeaderMode.Sticky
            }
            
            listItemComponents: [
                ListItemComponent {
                    type: "artist"
                    AddToPlaylistListItemVisual {
                        id: addArtistListItem
                        title: { ListItemData["album_artist"] != null ? ListItemData["album_artist"] : qsTr("Unknown")  + Retranslate.onLanguageChanged } 
                        thumb_file: { ListItemData["thumb_file"] != null ? ListItemData["thumb_file"] : "asset:///images/common/album_placeholder.png" }
                        subTitle: ListItem.view.getSubtitle(ListItemData);
                        onItemSelected: {
                            ListItem.view.openArtist(ListItemData);
                        }
                        onAddTrackToPlaylist: {
                            ListItem.view.addArtist( ListItemData );
                        }
                    }
                },
                ListItemComponent {
                    type: "list_header"
                    Header {
                        id: addArtistHeader
                        objectName: "addArtistHeader"
                        title: { ListItemData["header"] != null ? ListItemData["header"] : qsTr("Unknown")  + Retranslate.onLanguageChanged } 
                    }
                }
            ]
            
            function getSubtitle(data) {
                var c = data["count"];
                if ( c == "" || c == null ){
                    return qsTr( "%n Song(s)", "", 0 ) + Retranslate.onLanguageChanged;
                }
                return qsTr( "%n Song(s)", "", parseInt(c)) + Retranslate.onLanguageChanged;
            }
            
            /*
             * called to append the selected artist to the end of the playlist
             */
            function addArtist(data) {
                console.debug("AddArtistsContainer:addArtist");
                if ( data["album_artist_id"] != null ) {
                    addMusicViewPort.playlistDataModel.appendArtist( data["album_artist_id"] );
					list.itemsCountChanged = true;
                } else {
                    console.debug("AddArtistsContainer:Album Artist Id is null.  not able to add.");
                }
            }
            
        	/*
            * Display the tracks of the artist
            */
            function openArtist(data) {
                console.debug("AddArtistsContainer:openArtist");
                //__addMusicPageVariant from AddMusicPage.qml
                var page = tabbedPane.currentNavPane.safePushOnePageOnSheet("core/AddTracks.qml");
                commonAnalytics.instance.logPagePush("AddTracks", "AddArtistsContainer");
                page.titleStr = (data["album_artist"] != null && data["album_artist"] != "") ? data["album_artist"] : (qsTr("Unknown") + Retranslate.onLanguageChanged);
                var album_artist = data["album_artist_id"];
                if ( album_artist == null ) {
                    album_artist = "0";
                }
                page.mode = 0; //artist
                page.id = album_artist;
                page.contentUri = Uris.artistMembers(["album","fullpath","fid"],album_artist); 
				page.playlistDataModel = addMusicViewPort.playlistDataModel; 
            }
            
            //Ensure an item can be selected consecutively
            onTriggered: {
	            clearSelection();
	            list.select(indexPath, false);
            }
        }
    }
}
