import bb.cascades 1.0
import bb.cascades.multimedia 1.0
import bb.cascades.multimedia.internal 1.0
import bb.cascades.music 1.0
import bb.system 1.0
import "layoutconstants.js" as LayoutConstants

/*
* This page is the starting point for the add to playlist screens.
* This page contains a title and the Add Music Segmented Container.
* @property playlistDataModel  editable playlist datamodel
* @property useNewPlaylist flag to indicate to use a new empty editable playlist datamodel
* @author Scott Milliquet
*/
MediaPage {
    id: addMusicPage
    objectName: "addMusicPage"
    isDirty: true
    
    property variant playlistDataModel;
    property bool useNewPlaylist: false;
    property bool waitingForPlid: false; 
	property int errSavingPlaylist:PlaylistEditErrorType.PLERROR_OK;
    property Page editPage

    signal toggleFilter()

    titleBar: TitleBar {
        //visibility : Visible
        title : qsTr("Add Music") + Retranslate.onLanguageChanged;
        kind: TitleBarKind.Default;
        acceptAction: ActionItem  {
            id: doneEditPlaylistAction
            title: qsTr("Done") + Retranslate.onLanguageChanged
            onTriggered: {
	            console.debug("AddMusic::acceptAction: Done");
                donePressed();
            }
	    }
    }

    attachedObjects: [
        // Create the filter action item as an attached object so that
        // we can add it dynamically depending on which device we are on
        ActionItem {
            id: filterAction
            ActionBar.placement: ActionBarPlacement.OnBar
            title: qsTr("Filter") + Retranslate.onLanguageChanged
            imageSource: "asset:///images/common/ic_search.png"
            
            onTriggered: {
                addMusicPage.toggleFilter();
            }
        }
    ]
    
    onCreationCompleted: {
        // add action item on the bar
        qmlUtils.addActionItemToPage(filterAction, addMusicPage, 2);
    }
    
    function donePressed() {
        console.debug("AddMusic::donePressed");
        if ( useNewPlaylist && playlistDataModel.isModified() ) {
            popAndOpenPlaylistEdit();
        } else {
            tabbedPane.currentNavPane.safeCloseSheet();
        }
    }

    //We're done with AddMusic, (a save has been done). Close the sheet
    //and open the playlist
    function popAndOpenPlaylistEdit() {
        console.debug("AddMusic::popAndOpenPlaylistEdit");
        //pop previous page
        tabbedPane.currentNavPane.safeCloseSheet();

        tabbedPane.currentNavPane.pushTransitionEnded.connect(onPushTransitionEnded);
        editPage = tabbedPane.currentNavPane.safePushOnePage("core/PlaylistEdit.qml");
        commonAnalytics.instance.logPagePush("PlaylistEdit", "AddMusic");
        editPage.setTitle(playlistDataModel.getName());
        editPage.openPlaylistDetailOnSaved = true;
    }
    
    function onPushTransitionEnded() {
        console.log("AddMusic: set the model after edit page push transition ended");
        currentNavPane.pushTransitionEnded.disconnect(onPushTransitionEnded);
        editPage.playlistDataModel = playlistDataModel;
    }
    
    onUseNewPlaylistChanged: {
   		if ( useNewPlaylist ) {
   			//use the empty playlist datamodel
   			playlistDataModel = musicUtils.createEmptyM3uPlaylistDataModel();
   			//setup the playlistDataModel with a new empty m3u - this function generates a new file name as well
   			playlistDataModel.setNewFileName();
   		}
    }
    
    onPlaylistDataModelChanged: {
        if ( playlistDataModel != null && addMusicContainer != null ) {
            addMusicContainer.playlistDataModel = playlistDataModel;
        }
    }
    
    content: Container {
        layout: StackLayout {
            orientation: LayoutOrientation.TopToBottom
        }
        Container {
            
            horizontalAlignment: HorizontalAlignment.Fill
            
            layout: DockLayout {
            }
            layoutProperties: StackLayoutProperties {                
                spaceQuota: 100
            }
            BackgroundWrapper {}
            Container {
                
                horizontalAlignment: HorizontalAlignment.Fill
                verticalAlignment: VerticalAlignment.Fill
                
                AddMusicSegmentedContainer{
                    id: addMusicContainer
                    objectName: "addMusicContainer"
                    filterActionItem: filterAction
                    
                    onCreationCompleted: {
                        addMusicPage.toggleFilter.connect(addMusicContainer.toggleFilter);
                        localeHandler.localeChanged.connect(addMusicContainer.setActionItemText);
                    }
                }
            }
        }
    }
}