import bb.cascades 1.0
import bb.cascades.multimedia 1.0

/**
 * A list item visual for lists related to adding new tracks, albums, etc. to
 * a playlist
 *
 * @param	title	 		The top row of the track info
 * @param	subTitle		The bottom row of the track info
 * @param 	track			The track number/index
 * @param 	thumb_file		The uri for the image to display
 * @param   addedCount      Number of times an item is on the playlist
 * @signal  addTrackToPlaylist Emitted when "add" button is pressed
 * @signal  removeTrackFromPlaylist Emitted when "remove" button is pressed
 * @signal  itemSelected Emitted when the item is pressed but not add or remove.
 */
Container {
    id: addToPlaylistListItem 

    property string title: ""
    property string subTitle: ""
    property string track: "" 
    property string thumb_file: ""
    
    //"private" do not document exposure
    property bool isTrackAddedAnimationRunning: false

    // the animation duration for all animations in this screen
    property int __animationDuration: 250

    signal addTrackToPlaylist()
    signal removeTrackFromPlaylist()
    signal itemSelected()

    function onAddTrackAnimCompleted() {
        addToPlaylistListItem.isTrackAddedAnimationRunning = false;
    }

    layout: DockLayout {}

	// background and divider
    minHeight: 120
    Divider {
        horizontalAlignment: HorizontalAlignment.Fill
        verticalAlignment: VerticalAlignment.Bottom
        visible: !(addToPlaylistListItem.ListItem.indexInSection + 1 == addToPlaylistListItem.ListItem.sectionSize)
    }

    // track number, title, sub-title, add and remove track buttons
    Container {
        id: mainContainer

        //store whether touch event was handled. 
        property bool touchHandled: false

        horizontalAlignment: HorizontalAlignment.Fill
        verticalAlignment: VerticalAlignment.Fill		

		rightPadding:20
		
	    layout: StackLayout {
	        orientation: LayoutOrientation.LeftToRight
		}

        onTouch: {
            if (event.isUp() && !touchHandled) {
                addToPlaylistListItem.itemSelected();
            }
            touchHandled = false; //reset touch handled
        }

        // track number, track title, sub-title
		Container {
		    id: trackInfoContainer
		    
		    horizontalAlignment: HorizontalAlignment.Left
            verticalAlignment: VerticalAlignment.Fill
		    
		    layout: StackLayout {
		        orientation: LayoutOrientation.LeftToRight
			}
  
            layoutProperties: StackLayoutProperties {                
                spaceQuota:100
            }
	
 			ImageView {
 			    
 			    horizontalAlignment: HorizontalAlignment.Left
                verticalAlignment: VerticalAlignment.Center

                id: albumArtHolder
                preferredHeight: 120
                preferredWidth: 120
                imageSource: ((addToPlaylistListItem.thumb_file == null || addToPlaylistListItem.thumb_file == "") ? "asset:///images/common/album_placeholder.png" : addToPlaylistListItem.thumb_file)
            }
            TrackInfoLabel{
		       id: trackInfo
		       
		       horizontalAlignment: HorizontalAlignment.Left
		       verticalAlignment: VerticalAlignment.Center
		       
			   layoutProperties: StackLayoutProperties {			       
			       spaceQuota: 80
			   }
			   
			   title: ((addToPlaylistListItem.title == null || addToPlaylistListItem.title == "") ? qsTr("Unknown") + Retranslate.onLanguageChanged : addToPlaylistListItem.title)
			   subTitle: ((addToPlaylistListItem.subTitle == null || addToPlaylistListItem.subTitle == "") ? qsTr("Unknown") + Retranslate.onLanguageChanged : addToPlaylistListItem.subTitle)
		       titleStyle: addToPlaylistListItem.ListItem.view.titleStyle
		       subtitleStyle: addToPlaylistListItem.ListItem.view.subtitleStyle
		   }   
	    }
	    
        // add track icon
        ImageButton {
            verticalAlignment: VerticalAlignment.Center
            defaultImageSource: "asset:///images/context/ic_add_stand_alone_dark_theme_inactive.png"  
            pressedImageSource: "asset:///images/context/add_pressed.png"
            //not needed disabledImageSource
            onClicked: {
                if(!addToPlaylistListItem.isTrackAddedAnimationRunning) { 
                    addToPlaylistListItem.isTrackAddedAnimationRunning = true;
                    addTrackAnim.play();
                    addToPlaylistListItem.addTrackToPlaylist();
                }
            }
            leftMargin: 15
            rightMargin: 15
        }

        animations: [
	        SequentialAnimation{
	            id: addTrackAnim
	            
	    	    ParallelAnimation {
	    	        ScaleTransition{
	    	            target: trackInfoContainer
	    	            toY: 0.5
	    	            toX: 0.5
	    	            duration: __animationDuration
	    	        }
	                FadeTransition {
	                    target: trackInfoContainer
	                    toOpacity: 0.0
	                    duration: __animationDuration
	                    easingCurve: StockCurve.CubicInOut
	                }
	    	    }
	    	    
	    	    ParallelAnimation {
		    	    TranslateTransition {
	                    target: trackInfoContainer
	                    toY: 300
	                    duration: 100
	                    easingCurve: StockCurve.CubicInOut
	                }
	                ScaleTransition{
	    	            target: trackInfoContainer
	    	            toY: 1
	    	            toX: 1
	    	            duration: 100
	    	        }
	    	    }
	    	    
	    	    ParallelAnimation {
	    	        id: addTrack
	                TranslateTransition {
	                    target: trackInfoContainer
	                    toY: 0
	                    duration: __animationDuration
	                    easingCurve: StockCurve.CubicInOut
	                }
	                FadeTransition {
	                    target: trackInfoContainer
	                    toOpacity: 1.0
	                    duration: __animationDuration
	                    easingCurve: StockCurve.CubicInOut
	                }
	    	    }
	    	    
	    	    onEnded: {
	    	        onAddTrackAnimCompleted();
	    	    }
	    	}
    	]
	}
}
