import bb.cascades 1.0
import bb.cascades.multimedia 1.0
import "layoutconstants.js" as LayoutConstants

/**
 * A list item visual for lists related to adding new tracks, albums, etc. to
 * a playlist
 *
 * @param	title	 		The top row of the track info
 * @param	subTitle		The bottom row of the track info
 * @param   addedCount      Number of times an item is on the playlist
 * @signal  addTrackToPlaylist Emitted when "add" button is pressed
 * @signal  removeTrackFromPlaylist Emitted when "remove" button is pressed
 * @signal  itemSelected Emitted when the item is pressed but not add or remove.
 */
Container {
    id: addToPlaylistListItem 

    property string title: ""
    property string subTitle: ""
    property int addedCount: 0
    
    //"private" do not document exposure
    property bool isTrackAddedAnimationRunning: false
    
    // the animation duration for all animations in this screen
    property int __animationDuration: 250

    signal addTrackToPlaylist()
    signal removeTrackFromPlaylist()
    signal itemSelected()
    
    
    function onAddTrackAnimCompleted() {
        addToPlaylistListItem.isTrackAddedAnimationRunning = false;
    }

    layout: DockLayout {}

    onAddedCountChanged: {
		if ( addedCount == 0) {
			addedCountLabel.text = "";
		} else {
		    addedCountLabel.text = addedCount;
		}
	}
    
	// background and divider
    minHeight: LayoutConstants.LIST_ITEM_VISUAL_HEIGHT
    Divider {
        horizontalAlignment: HorizontalAlignment.Fill
        verticalAlignment: VerticalAlignment.Bottom
        visible: !(addToPlaylistListItem.ListItem.indexInSection + 1 == addToPlaylistListItem.ListItem.sectionSize)
    }
    
    // track number, title, sub-title, add and remove track buttons
    Container {
        id: mainContainer
        
        //store whether touch event was handled. 
        property bool touchHandled: false	
        
        verticalAlignment: VerticalAlignment.Center
        
        leftPadding: 10
        
	    layout: StackLayout {
	        orientation: LayoutOrientation.LeftToRight
		}

        onTouch: {
            if (event.isUp() && !touchHandled) {
                addToPlaylistListItem.itemSelected();
            }
            touchHandled = false; //reset touch handled
        }
        TrackInfoLabel {
            id: trackInfo
            horizontalAlignment: HorizontalAlignment.Left
            verticalAlignment: VerticalAlignment.Center
            
		       //background: Color.DarkGray
            layoutProperties: StackLayoutProperties {
                spaceQuota: 90 //what is taking the other 10?
            }
            title: ((addToPlaylistListItem.title == null || addToPlaylistListItem.title == "") ? qsTr ("Unknown") + Retranslate.onLanguageChanged : addToPlaylistListItem.title)
            subTitle: ((addToPlaylistListItem.subTitle == null || addToPlaylistListItem.subTitle == "") ? qsTr ("Unknown") + Retranslate.onLanguageChanged : addToPlaylistListItem.subTitle)
            titleStyle: addToPlaylistListItem.ListItem.view.titleStyle
            subtitleStyle: addToPlaylistListItem.ListItem.view.subtitleStyle
        }	    
    
		// add and remove track icons
		Container {
		    
		    rightPadding: 20
		    
		    horizontalAlignment: HorizontalAlignment.Right
	        verticalAlignment: VerticalAlignment.Center
		    
		    layout: StackLayout {
		        orientation: LayoutOrientation.RightToLeft
			}   

			// add track icon
			ImageButton {
			    verticalAlignment: VerticalAlignment.Center
			    defaultImageSource: "asset:///images/context/ic_add_stand_alone_dark_theme_inactive.png"  
			    pressedImageSource: "asset:///images/context/add_pressed.png"
			    //not needed disabledImageSource
			    onClicked: {
			        if(!addToPlaylistListItem.isTrackAddedAnimationRunning) { 
			            addToPlaylistListItem.isTrackAddedAnimationRunning = true;
			            addTrackAnim.play();
			            addToPlaylistListItem.addTrackToPlaylist();
			        }
			    }
			    leftMargin: 15
			    rightMargin: 15
			}
			// remove track icon
			ImageButton {
			    verticalAlignment: VerticalAlignment.Center
			    defaultImageSource: "asset:///images/context/ic_subtract_stand_alone_dark_theme_inactive.png" 
			    pressedImageSource: "asset:///images/context/remove_pressed.png"
			    onClicked: {
			        addToPlaylistListItem.removeTrackFromPlaylist();
			    }
                opacity: (addToPlaylistListItem.addedCount > 0) //enabled is not working properly use opacity for now
			    leftMargin: 15
			    rightMargin: 15
			}
		    //Added Count
		    Label {
		    	id: addedCountLabel
		    	textStyle.color: ColorUtils.get(0xff606060)
		    	
		    	verticalAlignment: VerticalAlignment.Center
		    	
		    	layoutProperties: StackLayoutProperties {		            
		        }
		    }
		}

        animations: [
	        SequentialAnimation{
	            id: addTrackAnim
	    	    ParallelAnimation {
	    	        ScaleTransition{
	    	            target: trackInfo
	    	            toY: 0.5
	    	            toX: 0.5
	    	            duration: __animationDuration
	    	        }
	                FadeTransition {
	                    target: trackInfo
	                    toOpacity: 0.0
	                    duration: __animationDuration
	                    easingCurve: StockCurve.CubicInOut
	                }
	    	    }
	    	    
	    	    ParallelAnimation {
		    	    TranslateTransition {
	                    target: trackInfo
	                    toY: 300
	                    duration: 100
	                    easingCurve: StockCurve.CubicInOut
	                }
	                ScaleTransition{
	    	            target: trackInfo
	    	            toY: 1
	    	            toX: 1
	    	            duration: 100
	    	        }
	    	    }
	    	    
	    	    ParallelAnimation {
	    	        id: addTrack
	                TranslateTransition {
	                    target: trackInfo
	                    toY: 0
	                    duration: __animationDuration
	                    easingCurve: StockCurve.CubicInOut
	                }
	                FadeTransition {
	                    target: trackInfo
	                    toOpacity: 1.0
	                    duration: __animationDuration
	                    easingCurve: StockCurve.CubicInOut
	                }
	    	    }
	    	    
	    	    onEnded: {
	    	        onAddTrackAnimCompleted();
	    	    }
	    	}
    	]
	}
}
