import bb.cascades 1.0
import bb.cascades.multimedia 1.0
import bb.cascades.multimedia.internal 1.0
import bb.cascades.music 1.0

/*
 * A page that displays a list tracks as specified by the URL that is passed in
 *
 * @property titleStr The title at the top of the page
 * @property contentUri  The uri of the tracks to display
 * @property mode Value to control how the visual is displayed for artist = 0 album = 1
 * @property playlistDataModel  editable playlist datamodel
 * @property id The album, album_arist or artist id.
 */
MediaPage {
    property alias titleStr: screenTitle.title
    property string contentUri;
    property int mode: 0;  //artist = 0; album = 1
    property variant playlistDataModel;
    property string id;

    isDirty: list.inEditMode

    actions: [
        ActionItem {
            title: qsTr ("Add All")  + Retranslate.onLanguageChanged
            ActionBar.placement: ActionBarPlacement.OnBar
            imageSource: "asset:///images/playlists/ic_add_to_playlist.png"
            onTriggered: {
                if ( mode == 0 ) { //artist
                	playlistDataModel.appendArtist( id );
                } else if ( mode == 1 ) { //album
                	playlistDataModel.appendAlbum( id );
                }
            }
        }
    ]
    attachedObjects: [
        //all tracks
        HeaderTypeDecorator {
            id: trackDataModel
            headerType: "list"
            itemType: (mode == 0 ? "trackTwoRow" : "track") //artist = 0; album = 1
            
            model: AsyncHeaderDataModel {
                objectName: "AddTracksForURI"
                id: trackData      
                query: MediaDataQueryDecorator {
                    query: ContentUriHeaderDataQuery { 
                        id: dataQuery
                        onError: appContext.showDataQueryErrorDialog(code, message)
                    }
                }
            }
        }
    ]

    onContentUriChanged: {
        dataQuery.uri = contentUri;
        dataQuery.headerUri = "";        
        trackData.load();
    }
    
    onPlaylistDataModelChanged: {
   	    playlistDataModel.newItemsCount.connect( onNewItemsCountChanged );
   	}
   	
   	function onNewItemsCountChanged() {
        list.itemsCountChanged = !list.itemsCountChanged;
    }
    
    content: Container {
        layout: StackLayout {
            orientation: LayoutOrientation.TopToBottom
        }
        ScreenTitle {
            id: screenTitle
            
            horizontalAlignment: HorizontalAlignment.Fill
            
            layoutProperties: StackLayoutProperties {                
            }
        }
	    Container {
	        
	        horizontalAlignment: HorizontalAlignment.Fill
	        
	        layoutProperties: StackLayoutProperties {
	            spaceQuota: 100	            
	        }
	        layout: DockLayout {}
	        BackgroundWrapper {}
	        AddMusicListView {
	            id: list
	            dataModel: trackDataModel
	            property bool itemsCountChanged: false;            
	            layout: StackListLayout {
	                headerMode: ListHeaderMode.Sticky
	            }

	            listItemComponents: [
	                ListItemComponent {
	                    type: "track"
	                    AddTrackToPlaylistSingleRow {
	                        id: addTracksListItem
	                        title: ListItemData["title"]
	                        track: ListItemData["track"]
	                        addedCount: ListItem.view.getAddedCount(ListItemData)
	                        onAddTrackToPlaylist: {
								ListItem.view.addToPlaylist(ListItemData);
								addedCount = ListItem.view.getAddedCount(ListItemData);
	                        }
	                        onRemoveTrackFromPlaylist: {
                            	ListItem.view.removeItem(ListItemData);
                            	addedCount = ListItem.view.getAddedCount(ListItemData);
                            }

                            property bool countChanged: ListItem.view.itemsCountChanged
                            //force the count to update for each item 
                            onCountChangedChanged: {
                                addedCount = ListItem.view.getAddedCount(ListItemData);
                            }
                            
	                        ListItem.onInitializedChanged: {
	                            if (initialized) {
                        	        addedCount = ListItem.view.getAddedCount(ListItemData);
                                }
                            }	
                            
                            // This signal seems to only trigger for some items and not others.	
                            // to catch the missing items we use the ListItem.onInitializedChanged.
                            // Could this be a Cascades issue?				
							ListItem.onDataChanged: {
								addedCount = ListItem.view.getAddedCount(ListItemData);
							}
	                    }
	                },
	                ListItemComponent {
	                    type: "trackTwoRow"
	                    AddTrackToPlaylistListItemVisual {
	                        title: ListItemData["title"]
	                        subTitle: ListItemData["album"]
	                        addedCount: ListItem.view.getAddedCount(ListItemData)
	                        onAddTrackToPlaylist: {
								ListItem.view.addToPlaylist(ListItemData);
								addedCount = ListItem.view.getAddedCount(ListItemData);
	                        }
	                        onRemoveTrackFromPlaylist: {
                            	ListItem.view.removeItem(ListItemData);
                            	addedCount = ListItem.view.getAddedCount(ListItemData);
                            }

                            property bool countChanged: ListItem.view.itemsCountChanged
                            //force the count to update for each item 
                            onCountChangedChanged: {
                                addedCount = ListItem.view.getAddedCount(ListItemData);
                            }
                            
	                        ListItem.onInitializedChanged: {
	                            if (initialized) {
	                        	    addedCount = ListItem.view.getAddedCount(ListItemData);
	                        	}
                            }
							
							ListItem.onDataChanged: {
								addedCount = ListItem.view.getAddedCount(ListItemData);
							}
	                    }
	                },
	                ListItemComponent {
	                    type: "list_header"
	                    Header {
	                        id: addTracksHeader
	                        title: qsTr("Songs") + Retranslate.onLanguageChanged
	                    }
	                }
	            ]
	            
	            function addToPlaylist(data) {
	            	playlistDataModel.appendItem(data["fid"]);
	            }

	            function removeItem(data) {
	                playlistDataModel.removeItemByFileId(data["fid"]);
	            }

	            function getAddedCount(data) {
	                var val = playlistDataModel.getCount(data["fid"]);
	                if ( val == null ) {
	            	   return 0; 
	            	} else {
	            	    return val;
	            	}
	            }

	            onTriggered: {
	                clearSelection();
	                list.select(indexPath, false);
                }
            }
	    }               
	}
}
