import bb.cascades 1.0
import bb.cascades.multimedia 1.0
import bb.cascades.multimedia.internal 1.0
import "ContentUris.js" as Uris

/*
 * A container that displays a list of all the tracks on the device.
 */
Container {
    id: addTracksContainer

    property alias itemsCountModified: list.itemsCountChanged
    
    horizontalAlignment: HorizontalAlignment.Fill
    verticalAlignment: VerticalAlignment.Fill

    attachedObjects: [
        //all tracks
        HeaderTypeDecorator {
            id: trackDataModel
            headerType: "list"
            itemType: "track"
            model: PrioritizedDataModelDecorator {
	            model: ModelFilterDecorator {
	                id: searchModel
    	            filterFields: ["title","artist"]
    	            queryFactory:   SearchFilterQueryFactory{
    	                                id: searchFilter
                                        dataQuery: Uris.allSongsByGroups(["artist","fid"])
                                        headerQuery: Uris.allSongsGroups(["artist"])
    	                                suppressData: false
    	                            }
    	            model: AsyncHeaderDataModel {
    	                objectName: "AddTracks"
    	                id: trackData
    	            }
    	        }
	        }
        }
    ]

    onCreationCompleted: {
        addMusicViewPort.filterTextChanging.connect(addTracksContainer.handleTextChanging);
        searchModel.applyFilter();
    }
    
    function handleTextChanging(text) {
        searchModel.setCriteria(text);
        searchModel.applyFilter();
    }

    layout: DockLayout {
        id: dockLayout
    }
    
    Container {
        
        horizontalAlignment: HorizontalAlignment.Fill
        verticalAlignment: VerticalAlignment.Fill
                 	    
        layout: StackLayout {
            orientation: LayoutOrientation.TopToBottom
        }
     	
     	AddMusicListView {
            id: list
            dataModel: trackDataModel
            property bool itemsCountChanged: false;
            layout: StackListLayout {
                headerMode: ListHeaderMode.Sticky
            }

            listItemComponents: [
                ListItemComponent {
                    type: "track"
                    AddTrackToPlaylistListItemVisual {
                        id: addTracksToPlaylistListItem
                        title: { ListItemData["title"] == null ? qsTr("Unknown") + Retranslate.onLanguageChanged : ListItemData["title"] }
                        subTitle: { ListItemData["artist"] == null ? qsTr("Unknown") + Retranslate.onLanguageChanged : ListItemData["artist"] }
						addedCount: ListItem.view.getAddedCount(ListItemData)
                        onAddTrackToPlaylist: {
							ListItem.view.appendItem(ListItemData);
							addedCount = ListItem.view.getAddedCount(ListItemData);
                        }

                        onRemoveTrackFromPlaylist: {
                        	ListItem.view.removeItem(ListItemData);
                        	addedCount = ListItem.view.getAddedCount(ListItemData);
                        }

                        property bool countChanged: ListItem.view.itemsCountChanged
                        //force the count to update for each item 
                        onCountChangedChanged: {
                            addedCount = ListItem.view.getAddedCount(ListItemData);
                        }
                        
                        ListItem.onInitializedChanged: {
                            if (initialized) {
                    	       addedCount = ListItem.view.getAddedCount(ListItemData);
                    	    }
                        }
						
						ListItem.onDataChanged: {
							addedCount = ListItem.view.getAddedCount(ListItemData);
						}
                    }
                },
                ListItemComponent {
                    type: "list_header"
                    Header {
                        id: addTracksToPlaylistHeader
                        title: ListItemData["header"]
                    }
                }
            ]

            function removeItem( data ) {
            	addMusicViewPort.playlistDataModel.removeItemByFileId(data["fid"]);
            }

            function appendItem( data ) {
            	addMusicViewPort.playlistDataModel.appendItem(data["fid"]);
            }

            function getAddedCount(data) {
                var val = addMusicViewPort.playlistDataModel.getCount(data["fid"]);
                if ( val == null ) {
            	   return 0; 
            	} else {
            	    return val;
            	}
            }

            onTriggered: {
	            clearSelection();
	            list.select(indexPath, false);
            }
        }
    }
}
