import bb.cascades 1.0
import "layoutconstants.js" as LayoutConstants

/**
 * This control is a header visual for the albums details list.
 * It wraps around a HeaderWithCover control
 *
 * @param   album          The text to use for the album name
 * @param   year           The text to use for the year of the album
 * @param   album_id       The album id retrived from mmlibrary
 * @param   thumb_file     The uri for the image of the album
 * @param   topPadVisible  boolean to specify if you want padding on the top of the header.  Default is true
 *
 * @signal shuffleAlbum emitted when the shuffle image is pressed
 */
Container {
    id: albumHeaderWithCover
    
    property TabbedPane tabbedPane: ListItem.view.listTabbedPane
    property alias album: headerWithCover.topText
    property alias year: headerWithCover.middleText
    property alias album_id: headerWithCover.list_id
    property alias thumb_file: headerWithCover.thumb_file
    property bool multiSelectionMode: false
    property bool allowAddToPlaylistAction: true
    property bool allowPlayNextAction: true    
    property bool allowPlayLastAction: true
    property bool allowPropertiesAction: false
    property bool allowDeleteAction: true
    
    signal shuffleAlbum (string album_id)

    property bool isDeleted: false
    scaleY: (isDeleted ? 0 : 1)
    topPadding: LayoutConstants.ALBUM_HEADER_SPACING
    
    layout: StackLayout{}
       
    HeaderWithCover {
        id: headerWithCover
            
        onShuffleList: {
            albumHeaderWithCover.shuffleAlbum( listId );
        }
    }
     
    ListItem.onSelectionChanged: {
    	headerWithCover.headerBackground = (selected && !multiSelectionMode) ? ColorUtils.get(0xff00a8df) : ColorUtils.get(0xffc4c4c4);
    }
   
    ListItem.onActivationChanged: {
        headerWithCover.borderImage.visible = (active && !multiSelectionMode) ? true : false;
    }
    
    contextActions: [
        ActionSet {
            id: myActions
        }
    ]
    
    contextMenuHandler: MusicContextMenuHandler {
        actionSet: myActions
    }
}
