import bb.cascades 1.0
import bb.cascades.multimedia 1.0
import bb.cascades.multimedia.internal 1.0

/**
 * This control is the main albums page.
 * It contains a ListView that displays all the albums on your device in alphabetical order with sticky headers
  */
Container {
    id: albumsPage
    property variant listView: libraryListView
    property alias listModel: libraryListView.dataModel
    signal scrolladexActiveChanged(bool active);
    
    horizontalAlignment: HorizontalAlignment.Fill
    verticalAlignment: VerticalAlignment.Fill
    
    layout: DockLayout {
        id: dockLayout
    }

    attachedObjects: [
        ListViewConnection {
            id: albumsListViewConnection
            listView: libraryListView
            key: "header"
            thumbKey: "thumb_file"
            defaultThumb: "asset:///images/common/album_placeholder.png"
        }
    ]

    Container {     	    
        layout: DockLayout {
        }
        
        horizontalAlignment: HorizontalAlignment.Fill
        verticalAlignment: VerticalAlignment.Fill
        
     	LibraryListView {
	        id: libraryListView
	        
	        contentType: MediaLibraryContentType.AlbumType
	        bottomPadding: 6
	        
	        scrollIndicatorMode: ScrollIndicatorMode.None
	        
	        listItemComponents: [
	            ListItemComponent {
	                type: ""
	                DeletableThumbsListItem {
	                    id: albumsListItem
	                    labelText: (!ListItemData["album"] ? "" : ListItemData["album"])
	                    bottomLabelText: (!ListItemData["album"] ? (!ListItemData["album_artist"] ? "" : ListItemData["album_artist"]) : "")
	                    bottomLabelVisible:  bottomLabelText != ""
	                    thumbUri: (!ListItemData["thumb_file"] ? "" : ListItemData["thumb_file"])
	                    contentType: MediaLibraryContentType.AlbumType
	                    itemSelected: (!ListItemData["isSelected"] ? false : ListItemData["isSelected"])
	                    multiSelectionMode: ListItem.view.isMultiSelect
	                }
	            },
	            ListItemComponent {
	                type: "header"
	                Header {
	                    id: albumsListHeader
	                    title: ListItemData["header"] + Retranslate.onLocaleOrLanguageChanged
	                    attachedObjects: [
	                        LayoutUpdateHandler {
	                            id: layoutHandler
	                            onLayoutFrameChanged: {
	                                albumsListHeader.ListItem.view.setScrolladexTopPadding(layoutHandler.layoutFrame.height);
	                            }
	                        }
	                    ]
	                }
	            }
	        ]
	    	
	    	function pushDetailsScreen(indexPath) {
                var selectedItem = libraryListView.dataModel.data(indexPath);
    	        var page = tabbedPane.currentNavPane.safePushOnePage("core/ArtistDetails.qml");
                commonAnalytics.instance.logPagePush("ArtistDetails", "Albums");
    	        page.openAtAlbumId = selectedItem.album_id;
    	        page.albumArtistIdStr = selectedItem.album_artist_id;
    	        //: album artist is not known for display on top of album art
    	        page.titleStr = selectedItem.album_artist;
    	    }
	    	
            function setScrolladexTopPadding(height) {
                albumsScrolladex.topPadding = height;
            }  
        }
        Scrolladex {
            id: albumsScrolladex
            verticalAlignment: VerticalAlignment.Fill
            listViewConnection: albumsListViewConnection
            toolTipMode: ToolTipMode.TextWithThumb
            fadeListWhileActive: true
            animateListWhileActive: false
            onActiveChanged: {
                scrolladexActiveChanged(albumsScrolladex.active);
            }
        }
        MusicListViewHelper {
	        listView: libraryListView
        }
    }
}
