import bb.cascades 1.0
import bb.cascades.multimedia 1.0
import bb.cascades.music 1.0
import bb.system 1.0
import "layoutconstants.js" as LayoutConstants
import "ContentUris.js" as Uris

/**
 * A control to display the list of albums for a specific artist
 * The list uses AlbumHeaderWithCover for the list header
 * and ListItemVisual for the list items
 *
 * @param   albumArtistId  The album artist id used to retrieve albums for a specific artist
 *
 */
Container {
    id: artistList
    objectName: "artistList"

    property string albumArtistId: "-1"
    property string openAtAlbumId
    property variant scrollToIndexPath
    property alias listView: selectableMusicListView
    property alias selectableDataModel: selectableMusicListView.dataModel
    property string songToPlayOnOpenFid: "-1"
    property bool isEmpty: true;
    // Flag that is set by InvokeUtils.js when the app is invoked and we want to scroll to playing song
    property bool scrollToCurrentSongAfterInvoke: false
    property Search searchScreen
    
    rightPadding: LayoutConstants.TRACKS_LIST_VIEW_PADDING
    leftPadding: LayoutConstants.TRACKS_LIST_VIEW_PADDING
    
    layout: DockLayout {
    }
    attachedObjects: [
        AsyncHeaderDataModel {
            id: dataModel
            objectName: "ArtistAlbumList"
            query: MediaDataQueryDecorator {
                query: ContentUriArtistAlbumListDataQuery {
                    id: dataQuery
                    onError: appContext.showDataQueryErrorDialog(code, message)
                }
            }
        },
        SelectableDataModelDecorator {
            id: selectableModel
            model: PrioritizedDataModelDecorator {
                model: CurrentSongDecorator {
                    id: curSongDecorator
                    model: dataModel
                    playbackControllerRef: appContext.playbackController
                    dynamicPlaylistManager: appContext.dynamicPlaylistManagerRef
                }
            }
        }
    ]

    animations: [
        LoadListTransition {
            target: selectableMusicListView
            dataModel: dataModel
        }
    ]
    
    function handleItemsChanged () {
        var wasEmpty = artistList.isEmpty;
        artistList.isEmpty = !dataModel.childCount(null);

        if(!isEmpty && openAtAlbumId != "0"){
        	var children = dataModel.childCount(null);
        	var idStr = albumArtistId;
			if (idStr == "" || idStr == null) {
				idStr = "0";
			}
    		var albumIndex = musicUtils.getHeaderIndexForAlbumId(openAtAlbumId, Uris.artistAlbums(["album","album_id"], idStr), dataModel);
        	scrollToIndexPath = [albumIndex];
        }
        
        if ( isEmpty && !wasEmpty ) {
             //todo: add this to appContext
            appContext.popOrRemoveArtistDetails();
        } else if (scrollToCurrentSongAfterInvoke) {
            selectableMusicListView.scrollToCurrentSong(appContext.playbackController.currentSong);
            scrollToCurrentSongAfterInvoke = false;
        } else if ( scrollToIndexPath != null ) {
            selectableMusicListView.scrollToAsyncModelItem( scrollToIndexPath );
            console.log("ArtistAlbumlist.qml - handleItemsChanged");
        }
    }
    
    onAlbumArtistIdChanged: {
        console.debug("ArtistAlbumList: onAlbumArtistIdChanged " + albumArtistId);
		var idStr = albumArtistId;
		if (idStr == "" || idStr == null) {
			idStr = "0";
		}
        dataQuery.headerUri = Uris.artistAlbums(["count","album_id","thumb_file"], idStr);
        dataQuery.uri = Uris.artistMembersByTrack(["title","album","album_id","album_artist","album_artist_id","artist","artist_id","fullpath","duration","fid","filename", "date_modified"],idStr);

        dataModel.load();
        if (openAtAlbumId == "" || openAtAlbumId == null) {
            openAtAlbumId = "0";
        }
        if(songToPlayOnOpenFid != "-1"){
            scrollToIndexPath = selectableMusicListView.playAndFindIndexPath(songToPlayOnOpenFid, openAtAlbumId);
        } 
    }
    
    SelectableMusicListView{
        id: selectableMusicListView
        dataModel: selectableModel
        layout: StackListLayout {}
        bottomPadding: LayoutConstants.TRACKS_LIST_VIEW_PADDING

        property string idStr: artistList.albumArtistId 
        property int songIndexPath
        property variant songDecorator: curSongDecorator

        contextMenuHandler: ContextMenuHandler {
            id: contextMenuHandler
            onVisualStateChanged: {
                selectableMusicListView.enableTypeToSearch();
            }
        }

        onDataModelChanged: {
            artistList.isEmpty = !dataModel.childCount(null);
            dataModel.itemsChanged.connect( artistList.handleItemsChanged );
        }

        property bool isMultiSelect: false;
        
        //a function to scroll to the currently playing track
        function scrollToCurrentSong(){
            appContext.playbackController.currentIndexChanged.disconnect(scrollToCurrentSong);
            
            var curSong = appContext.playbackController.currentSong;

            var currFid = curSong.fid;
            var currAlbumId = curSong.album_id;
            if (songToPlayOnOpenFid != -1) {
                currFid = songToPlayOnOpenFid;
                currAlbumId = openAtAlbumId;
            }

            var indexPath = getIndexPath(currFid, currAlbumId);
            songToPlayOnOpenFid = -1;

            selectableMusicListView.scrollToAsyncModelItem(indexPath);
        }
        
        function playAndFindIndexPath(fid, albumId){
            var songIndexPath = getIndexPath(fid, albumId);
            startPlayback(albumId, songIndexPath, null);
            return songIndexPath;                        
        }
        
        function getIndexPath(fid, albumId){
	        var idStr = albumArtistId;
			if (idStr == "" || idStr == null) {
				idStr = "0";
			}
            var albumIndex = musicUtils.getHeaderIndexForAlbumId(albumId, Uris.artistAlbums(["album","album_id"], idStr),dataModel);
            var songIndex = musicUtils.getIndexByKey(fid, Uris.albumMembers(["fid","key_id"], albumId), "key_id=", null, albumIndex);
            return [albumIndex,songIndex];
        }
        
        function startPlayback(albumId, indexPath, selectedItem){
            console.debug("ArtistAlbumList about to call startPlaybackByIndex on DPM: ");
            dynamicPlaylistManager.startPlaybackByIndex(
                Uris.albumMembers(Uris.playerColumns(),albumId),
                appContext.playbackController.shuffleMode == PlaybackController.ShuffleOn, //shuffle works now
                indexPath[indexPath.length - 1],
                "0",
                PlaybackController.PlaybackPlaying,
                selectedItem);        
        }
        
        function enableTypeToSearch() {
            searchScreen.enableTypeToSearch(ActionMenuVisualState.Hidden == contextMenuHandler.visualState);
        }
        
        function dialogCallback(result) {
            deleteDialog.finished.disconnect(selectableMusicListView.dialogCallback);
            if (deleteDialog.result == SystemUiResult.ConfirmButtonSelection) {
                var itemType = dataModel.itemType(selectedIndexPath);
                var selectedItem = dataModel.data(selectedIndexPath);
                
    			if (itemType == "header") {
    			    deleteManager.deleteContentByIds(MediaLibraryContentType.AlbumType, selectedItem.album_id);
    			} else  {
     			    deleteManager.deleteContentByIds(MediaLibraryContentType.AudioType, selectedItem.fid);
    			} 
            }
        }
        
        // callback from when an item is long pressed for delete
        function deleteItems() {
            var itemType = dataModel.itemType(selectedIndexPath);
            deleteDialog = appContext.getDeleteSystemDialog((itemType == "header") ? MediaLibraryContentType.AlbumType:MediaLibraryContentType.AudioType, 1);
            deleteDialog.finished.connect(selectableMusicListView.dialogCallback);
            deleteDialog.show();
        }

        // override function to pop up the select playlist screen to add a track to a playlist
        function addToPlaylist() {
            var itemType = dataModel.itemType(selectedIndexPath);
            var selectedItem = dataModel.data(selectedIndexPath);
            
            //open add to playlist screen
            var page = currentNavPane.safePushOnePage ("core/SelectPlaylist.qml");
            commonAnalytics.instance.logPagePush("SelectPlaylist", "ArtistAlbumList");
			if (itemType == "header") {
			    page.contentType = MediaLibraryContentType.AlbumType;
			    page.idsToAdd = [selectedItem.album_id];
			} else  {
			    page.contentType = MediaLibraryContentType.AudioType;
			    page.idsToAdd = [selectedItem.fid];
			} 
        }
        
        // override function to add the selected track or album to the Now Playing list
        function addToNowPlaying(last) {
            var itemType = dataModel.itemType(selectedIndexPath);
            var selectedItem = dataModel.data(selectedIndexPath);
           
			if (itemType == "header") {
                appContext.addToNowPlaying(MediaLibraryContentType.AlbumType, [selectedItem.album_id], null, last);
			} else  {
                appContext.addToNowPlaying(MediaLibraryContentType.AudioType, [selectedItem.fid], null, last);
			} 
        }
        
        listItemComponents: [
            ListItemComponent {
                type: ""
                SimpleListItemVisual {
                    title: ListItemData["title"]
                    track: ListItemData["track"]
                    duration: ListItemData["duration"]
                    controller: ListItem.view.songDecorator
                    multiSelectionMode: ListItem.view.isMultiSelect
                    itemSelected: ListItemData["isSelected"]
                }
            },
            ListItemComponent {
                type: "fullItem"
                ListItemVisual {
                    title: ListItemData["title"]
                    track: ListItemData["track"]
                    duration: ListItemData["duration"]
                    fullpath: ListItemData["fullpath"]
                    itemSelected: ListItemData["isSelected"]
                    currSong: ListItemData["currSong"]
                    multiSelectionMode: ListItem.view.isMultiSelect
                    controller: ListItem.view.songDecorator
                }
            },
            ListItemComponent {
                type: "header"
                AlbumHeaderWithCover {
                    id: artistDetailsHeader
                    album: ListItemData["album"]
                    year: ListItemData["year"]
                    album_id: ListItemData["album_id"]
                    thumb_file: ListItemData["thumb_file"]
                    multiSelectionMode: ListItem.view.isMultiSelect
                    property string idString: ListItem.view.idStr 
                    onShuffleAlbum:{
                         appContext.playbackController.currentIndexChanged.connect(ListItem.view.scrollToCurrentSong);
                         dynamicPlaylistManager.startPlaybackByIndex(
                             Uris.albumMembers(Uris.playerColumns(),album_id),
                             true, //shuffle on
                             -1, "0", PlaybackController.PlaybackPlaying);
                         commonAnalytics.instance.logShuffleAll("ArtistAlbumListHeader");
                    }
                }
    		}
    	]
    	
    	onTriggered:{
            if (!dataModel.hasChildren (indexPath)) {
                var selectedItem = selectableMusicListView.dataModel.data (indexPath);
                if (selectedItem.fullpath == appContext.playbackController.currentSong.fullpath && appContext.playbackController.currentSong.fullpath != "" && appContext.playbackController.currentSong.fullpath != null) {
                        //selected an already playing track, play/pause toggle
                    if (appContext.playbackController.playbackState == PlaybackController.PlaybackPlaying) {
                        appContext.playbackController.pause()
                    } else {
                        appContext.playbackController.play()
                    }
                } else {
                   startPlayback(selectedItem.album_id, indexPath, selectedItem);
                }
                clearSelection();
                selectableMusicListView.select (indexPath, false);
            }
        }
            
    }
}
