import bb.cascades 1.0
import bb.cascades.multimedia 1.0
import bb.cascades.multimedia.internal 1.0
import bb.cascades.music 1.0
import "ContentUris.js" as Uris

/**
 * This is a page that displays a list of all albums for a specific artist
 * It contains a title and an AlbumArtistList control
 *
 * @param   albumArtistIdStr  The album artist id used to retrieve albums for a specific artist
 * @param   titleStr          The text to use as the title for this page
 * @param   openAtAlbumId     The album id of the album that should be scrolled onto the screen
 *
 */
Page {
    id: artistDetails
    objectName: "artistDetails"
    property alias albumArtistIdStr: artistAlbumList.albumArtistId
    property string titleStr
    property alias openAtAlbumId: artistAlbumList.openAtAlbumId
    property alias songToPlayOnOpenFid: artistAlbumList.songToPlayOnOpenFid
    property alias scrollToCurrentSongAfterInvoke: artistAlbumList.scrollToCurrentSongAfterInvoke
        
    actions: [       
        NowPlayingActionItem {
            page: "ArtistDetails"
        },
	    ActionItem {
            id: playAll
            title: qsTr("Play All") + Retranslate.onLanguageChanged
            imageSource: "asset:///images/playing/ic_play.png"
            
            onTriggered: {
                // Using content ids api instead of content uri api to get the correct order of songs.
                dynamicPlaylistManager.startPlaybackByContentIds(
                    MediaLibraryContentType.AlbumArtistType, // content type
                    [albumArtistIdStr], // content id
                    null,
                	false, //shuffle off 
                	0,
                    "0",
                    PlaybackController.PlaybackPlaying);
            }
        },
        
	    ActionItem {
            title: qsTr("Shuffle All") + Retranslate.onLanguageChanged
            imageSource: "asset:///images/playing/ic_shuffle.png"
            ActionBar.placement: ActionBarPlacement.OnBar
            onTriggered: {
                // Using content ids api instead of content uri api to get the correct order of songs.
                dynamicPlaylistManager.startPlaybackByContentIds(
                    MediaLibraryContentType.AlbumArtistType, // content type
                    [albumArtistIdStr], // content id
                    null,
                	true, //shuffle on 
                	-1,
                    "0",
                    PlaybackController.PlaybackPlaying);
                commonAnalytics.instance.logShuffleAll("ArtistDetails");
            }
        },
	    MultiSelectActionItem {
	        onTriggered: {
	            artistAlbumList.listView.selectableMultiHandler.active = true;
	        }
	    },
        ActionItem {
            //: menu item for search
            title: qsTr("Search") + Retranslate.onLanguageChanged
	        imageSource: "asset:///images/common/ic_search.png"
            onTriggered: {
                artistDetailsSearchScreen.display();
            }
        }
	]

    onCreationCompleted:{
	    tabbedPane.currentNavPane.backButtonsVisible = true;
	    artistDetailsSearchScreen.rootPage = artistDetails;
	    artistDetailsSearchScreen.hide();
    }

    onActionMenuVisualStateChanged: {
        artistDetailsSearchScreen.enableTypeToSearch(ActionMenuVisualState.Hidden == actionMenuVisualState);
    }
    
    content: Container {
        id: mainContainer
        layout: DockLayout {
        }

        Container {
            layout: StackLayout {
                orientation: LayoutOrientation.TopToBottom
            }
            ScreenTitle {
                id: screenTitle
                title: (titleStr == null || titleStr == "") ? (qsTr("Unknown") + Retranslate.onLanguageChanged) : titleStr
                showNowPlaying: true
                horizontalAlignment: HorizontalAlignment.Fill
            }
            Container {
                
                horizontalAlignment: HorizontalAlignment.Fill
                
                layoutProperties: StackLayoutProperties {
                    spaceQuota: 100               
                }
                layout: DockLayout {}
                BackgroundWrapper {}
                ArtistAlbumList {
                    id: artistAlbumList
                    searchScreen: artistDetailsSearchScreen
                }
            }
        }

        Search { // ArtistDetails need its own instance of Search screen
            // because it may be launched from the main Search screen
            // (see also tabbedPane.searchScreen).
            id: artistDetailsSearchScreen
        }
    }
}
