import bb.cascades 1.0
import bb.cascades.multimedia 1.0
import bb.cascades.multimedia.internal 1.0

/**
 * This control is the main artist page.
 * It contains a ListView that displays all the artists on your device in alphabetical order with sticky headers
 */
Container {
    id: artistsPage
    property variant listView: libraryListView
    property alias listModel: libraryListView.dataModel
    signal scrolladexActiveChanged(bool active);
    
    horizontalAlignment: HorizontalAlignment.Fill
    verticalAlignment: VerticalAlignment.Fill

    layout: DockLayout {
        id: dockLayout
    }

    attachedObjects: [
        ListViewConnection {
            id: artistsListViewConnection
            listView: libraryListView
            key: "header"
            thumbKey: "thumb_file"
            defaultThumb: "asset:///images/common/album_placeholder.png"
        }
    ]
    
    Container {
        layout: DockLayout {
        }

        horizontalAlignment: HorizontalAlignment.Fill
        verticalAlignment: VerticalAlignment.Fill

     	LibraryListView {
	        id: libraryListView
	        
	        contentType: MediaLibraryContentType.AlbumArtistType
	        bottomPadding: 6
	        
	        scrollIndicatorMode: ScrollIndicatorMode.None

	    	listItemComponents: [
	    		ListItemComponent {
                           type: ""     
                           DeletableThumbsListItem {
                               id: artistsListItem
                               labelText: (!ListItemData["album_artist"] ? "" : ListItemData["album_artist"])
                               thumbUri: (!ListItemData["thumb_file"] ? "" : ListItemData["thumb_file"])
                               contentType: MediaLibraryContentType.AlbumArtistType
                               itemSelected: (!ListItemData["isSelected"] ? false : ListItemData["isSelected"])
                               multiSelectionMode: ListItem.view.isMultiSelect
                           }
	    		},
	    		ListItemComponent {
	    			type: "header"	    			
					Header {
					    id: artistsListHeader
					    title: ListItemData["header"] + Retranslate.onLanguageChanged
					    attachedObjects: [
					        LayoutUpdateHandler {
					            id: layoutHandler
					            onLayoutFrameChanged: {
					                artistsListHeader.ListItem.view.setScrolladexTopPadding(layoutHandler.layoutFrame.height);
					            }
					        }
					    ]
		            }
	    		}  
	    	]
	    	
	    	function pushDetailsScreen(indexPath) {
	    	    var selectedItem = libraryListView.dataModel.data(indexPath);
	    	    var page = tabbedPane.currentNavPane.safePushOnePage("core/ArtistDetails.qml");
                commonAnalytics.instance.logPagePush("ArtistDetails", "Artists");
	    	    
	    	    page.albumArtistIdStr = selectedItem.album_artist_id;
	    	    //: album artist is not known for display on top of album art
	    	    page.titleStr = selectedItem.album_artist;
	    	}
	    	
            function setScrolladexTopPadding(height) {
                artistsScrolladex.topPadding = height;
            }
	    }
     	Scrolladex {
     	    id: artistsScrolladex
     	    verticalAlignment: VerticalAlignment.Fill
     	    listViewConnection: artistsListViewConnection
            toolTipMode: ToolTipMode.TextWithThumb
            fadeListWhileActive: true
            animateListWhileActive: false
            onActiveChanged: {
                scrolladexActiveChanged(active);
            }
        }
     	MusicListViewHelper {
     	    listView: libraryListView
        }
    }
}
