import bb.cascades 1.0
import bb.cascades.multimedia 1.0
import bb.cascades.multimedia.internal 1.0
import bb.cascades.music 1.0
import "ContentUris.js" as Uris

Page {
    id: byYearPage
    property string allTracksQuery: Uris.allSongsPlayback()
    property variant searchScreenContainer: mainContainer

    actions: [
        NowPlayingActionItem {
            page: "ByYear"
        },
        ActionItem {
            title: qsTr("Play All") + Retranslate.onLanguageChanged
            imageSource: "asset:///images/playing/ic_play.png"
            onTriggered: {
                dynamicPlaylistManager.startPlaybackByIndex(
                	allTracksQuery,
                	false, //shuffle off 
                	0);
            }
        },
        ActionItem {
            title: qsTr("Shuffle All") + Retranslate.onLanguageChanged
            imageSource: "asset:///images/playing/ic_shuffle.png"
            ActionBar.placement: ActionBarPlacement.OnBar

            onTriggered: {
                dynamicPlaylistManager.startPlaybackByIndex(
                	allTracksQuery,
                	true, //shuffle on 
                	-1);
                commonAnalytics.instance.logShuffleAll("ByYear");
            }
        },
        ActionItem {
            //: menu item for search
            title: qsTr("Search") + Retranslate.onLanguageChanged
            imageSource: "asset:///images/common/ic_search.png"
            onTriggered: {
                appContext.searchScreen.display();
            }
        }
    ]

    onActionMenuVisualStateChanged: {
        appContext.searchScreen.enableTypeToSearch(ActionMenuVisualState.Hidden == actionMenuVisualState);
    }

    content: Container {
        id: mainContainer
        layout: DockLayout {
        }

        Container {
            layout: StackLayout {
                orientation: LayoutOrientation.TopToBottom
            }
            ScreenTitle {
                id: screenTitle
                showNowPlaying: true
                
                title: qsTr("By Year") + Retranslate.onLanguageChanged
                
                horizontalAlignment: HorizontalAlignment.Fill
                
                layoutProperties: StackLayoutProperties {               
                }
            }
            attachedObjects: [
                ModelCombiner {
                    id: byYearDataModel                
                    //by decades
                    HeaderTypeDecorator {
                        headerType: "by_decades"
                        itemType: "decade"
                        model: AsyncHeaderDataModel {
                            objectName: "Decades"
                            id: decadeData
                            query: ForcedUpdateQueryDecorator {
                                query: MediaDataQueryDecorator {
                                    query: ContentUriHeaderDataQuery {
                                        //This content URI gets all the tracks and groups them by decade.  The TOPN function is used to grab the first four thumb files for 
                                        //each group to be used with the FourThumbsListItem
                                        headerUri: ""
                                        uri: Uris.decades()
                                        onError: appContext.showDataQueryErrorDialog(code, message)
                                    }
                                }
                            }
                        }
                    }
                    //by year
                    HeaderTypeDecorator {
                        headerType: "by_year"
                        itemType: "year"
                        model: AsyncHeaderDataModel {
                            objectName: "Years"
                            id: yearData
                            query: ForcedUpdateQueryDecorator {
                                query: MediaDataQueryDecorator {
                                    query: ContentUriHeaderDataQuery {
                                        //This content URI gets all the tracks and groups them by decade.  The TOPN function is used to grab the first four thumb files for 
                                        //each group to be used with the FourThumbsListItem
                                        headerUri: ""
                                        uri: Uris.years()
                                        onError: appContext.showDataQueryErrorDialog(code, message)
                                    }
                                }
                            }
                        }
                    }                
                }
            ]
    		
            onCreationCompleted: {
                decadeData.loaded.connect(onDecadeDataLoaded);
    	        decadeData.load();
    	        yearData.load();       
    	    }
            function onDecadeDataLoaded() {
                byYearList.scrollToPosition(ScrollPosition.Beginning, ScrollAnimation.None);
            }
    
            Container {
                
                horizontalAlignment: HorizontalAlignment.Fill
                
                layoutProperties: StackLayoutProperties {
                    spaceQuota: 100                
                }
                layout: DockLayout {
                }
                BackgroundWrapper {
                }
                MusicListView {
                    id: byYearList
                    objectName: "byYearList"
                    dataModel: byYearDataModel
                    
                    verticalAlignment: VerticalAlignment.Fill
                    horizontalAlignment: HorizontalAlignment.Fill
                    layout: GridListLayout {
                        headerMode: ListHeaderMode.Sticky
                        columnCount: 3
                    }
                    
                    listItemComponents: [
                        ListItemComponent {
                            type: "decade"
                            content: [
                                SmartPlaylistListItem {                                
                                    headerURI: Uris.filter(Uris.allSongsHeader(), ListItemData["header_criterion"]);	                            
    	                            contentURI: Uris.filter(Uris.allSongs(["artist","fid","duration","fullpath","decade"]), ListItemData["header_criterion"]);
    
                                    playlist_name: ListItemData["header"];
    						        count: ListItemData["count"];
    						        thumb1Uri: ListItemData["four_square_1"];
    						        thumb2Uri: ListItemData["four_square_2"];
    						        thumb3Uri: ListItemData["four_square_3"];
    						        thumb4Uri: ListItemData["four_square_4"];
    						        defaultImage: "asset:///images/common/album_placeholder.png"
                                }
                            ]
                        },
                        ListItemComponent {
                            type: "year"
                            content: [
                                SmartPlaylistListItem {                                
                                    headerURI: Uris.filter(Uris.allSongsHeader(), ListItemData["header_criterion"]);                                    
                                    contentURI: Uris.filter(Uris.allSongs(["artist","fid","duration","fullpath","year"]), ListItemData["header_criterion"]); 
                                        
                                    playlist_name: ListItemData["header"];
    						        count: ListItemData["count"];
    						        thumb1Uri: ListItemData["four_square_1"];
    						        thumb2Uri: ListItemData["four_square_2"];
    						        thumb3Uri: ListItemData["four_square_3"];
    						        thumb4Uri: ListItemData["four_square_4"];
    						        defaultImage: "asset:///images/common/album_placeholder.png" 
                                }
                            ]
                        },
                        ListItemComponent {
                            type: "by_decades_header"
                            content: [
                                Header {
                                    id: smartPlaylistDecadeHeader
                                    objectName: "smartPlaylistDecadeHeader"
                                    title: qsTr ("Decades") + Retranslate.onLanguageChanged
                                }
                            ]
                        },
                        ListItemComponent {
                            type: "by_year_header"
                            content: [
                                Header {
                                    id: smartPlaylistYearHeader
                                    objectName: "smartPlaylistYearHeader"
                                    title: qsTr ("Years") + Retranslate.onLanguageChanged
                                }
                            ]
                        }
                    ]
                    function processSelectionChanged(data, contentURI, headerURI, playlist_name, indexPath) {
                    	var page = tabbedPane.currentNavPane.safePushOnePage("core/SmartPlaylistDetails.qml");
                    	commonAnalytics.instance.logPagePush("SmartPlaylistDetails", "ByYear");
                        page.headerURI = headerURI;
                        page.contentURI = contentURI;
                        page.titleStr = (playlist_name != null) ? playlist_name : (qsTr("Unknown") + Retranslate.onLanguageChanged);
                        page.headerDataModel = byYearList.dataModel;
                        page.index = indexPath;
                    }
    
                    onTriggered:{
                        var currentItemType = byYearList.dataModel.itemType(indexPath);
                        var pageData = byYearList.dataModel.data(indexPath);
                        var headerURI, contentURI;
                        
                        clearSelection();
                        byYearList.select(indexPath, false);
                        
                        if(currentItemType == "decade"){
                            headerURI = Uris.filter(Uris.allSongsHeader(), pageData["header_criterion"]);
                            contentURI = Uris.filter(Uris.allSongs([ "album", "album_artist", "artist", "fid", "duration", "fullpath", "filename", "date_modified", "decade" ]), pageData["header_criterion"]);
                            byYearList.processSelectionChanged(pageData, contentURI, headerURI, pageData["header"], indexPath);              
                        }else if(currentItemType == "year"){
                            headerURI = Uris.filter(Uris.allSongsHeader(), pageData["header_criterion"]);
                            contentURI = Uris.filter(Uris.allSongs([ "album", "album_artist", "artist", "fid", "duration", "fullpath", "filename", "date_modified", "year" ]), pageData["header_criterion"]);
                            byYearList.processSelectionChanged(pageData, contentURI, headerURI, pageData["header"], indexPath);
                        }                    
                    }                
                }
                MusicListViewHelper {
                    listView: byYearList 
                }
            }
        }
    }
}
