import bb.cascades 1.0
import bb.cascades.multimedia 1.0
import bb.cascades.multimedia.internal 1.0
import bb.system 1.0
import "layoutconstants.js" as LayoutConstants

/**
 * This control is a wrapper for a FourThumbListItem
 *
 * @param   labelText           The text to use for the top row of the label
 * @param   bottomLabelText     The text to use for the bottom row of the label.  bottomLabelVisible must be set to true for this to appear
 * @param   bottomLabelVisible  This must be set to true for the bottom label to be displayed
 * @param   thumb1Uri           The uri for the top let image
 * @param   thumb2Uri           The uri for the top right image
 * @param   thumb3Uri           The uri for the bottom left image
 * @param   thumb4Uri           The uri for the bottom right image 
 * @param   contentTopMargin    The top margin
 * @param   contentBottomMargin The bottom margin
 * @param   contentRightMargin  The right margin
 * @param   contentLeftMargin   The left margin
 * @param   defaultImage        The uri for the image to use as a default
 */

FourThumbsListItem {
    id: thumbListItem 
    property TabbedPane tabbedPane: ListItem.view.listTabbedPane
    
     property alias thumb1Uri: thumbListItem.thumb1Uri
     property alias thumb2Uri: thumbListItem.thumb2Uri
     property alias thumb3Uri: thumbListItem.thumb3Uri 
     property alias thumb4Uri: thumbListItem.thumb4Uri
     
    defaultImage: "asset:///images/common/album_placeholder.png"
    pressImageSource: "asset:///images/common/core_listitem_pressed.png.amd"
    selectImageSource: "asset:///images/common/core_griditem_pressed.png.amd"
    defaultPadding: 0
    leftPadding: LayoutConstants.LIBRARY_LIST_VIEW_SPACING
    rightPadding: ListItem.indexInSection % ListItem.view.layout.columnCount == (ListItem.view.layout.columnCount - 1) ? LayoutConstants.LIBRARY_LIST_VIEW_SPACING : 0

    property SystemDialog deleteDialog
    
    property bool multiSelectionMode: false
    property bool active: ListItem.active
    
    // properties to specify which action items to set
    property bool allowAddToPlaylistAction: true
    property bool allowPlayNextAction: true
    property bool allowPlayLastAction: true
    property bool allowMultiSelectAction: true
    property bool allowDeleteAction: true
    property bool allowPlayOnAction: false
    property bool allowPropertiesAction: false

    // property that defines the content type for deleting
    property variant contentType: MediaLibraryContentType.AudioType
    
    preferredHeight: 256
    preferredWidth: 256
    
	opacity: (!multiSelectionMode || ListItem.view.dataModel.selectionList.length == 0 || itemSelected) ? 1.0: 0.7

    contextActions: [
        ActionSet {
            id: myActions
        }
    ]

    contextMenuHandler: MusicContextMenuHandler {
        actionSet: myActions

        onVisualStateChanged: {
           appContext.searchScreen.enableTypeToSearch(ActionMenuVisualState.Hidden == visualState);
        }
    }
    
    onCreationCompleted:{
        ListItem.activationChanged.connect(thumbListItem.onActivationChanged);
        itemSelectedChanged.connect(thumbListItem.onItemSelectedChanged);
    }
}
