import bb.cascades 1.0
import bb.cascades.multimedia 1.0
import bb.cascades.music 1.0
import bb.system 1.0
import "layoutconstants.js" as LayoutConstants

import "PlUtils.js" as PlUtils

/**
 * This component is used as the visual for each individual song in the lists
 * It contains a track number, name, duration and an image for "favourites"
 *
 * @param   title                The text to be used for the title of the song
 * @param   subTitle             The text to be used for the sub-title of the song
*/
Container {
    id: editListItem
    
    //following properties are set from the external qml list that uses this ListItemVisual    
    property string title
    property string artist
    property string playlist_entry_text
    property string fid
    property bool active: false
	
	// These are properties set by DragDropHandler
	property DragDropHandler handler
	property variant indexPath
	property variant itemData
    property bool selected: false 
	               
    attachedObjects:[
         ComponentDefinition{
       	            id: borderImage
       	           	ImageView {
       	           	    id: overlay
      	                imageSource: "asset:///images/common/core_item_border.amd"
       	           	}
         }
    ]
               	    
    layout: DockLayout {}
    
	minHeight: 120 // hardcoded until a better method is found
     			        
    horizontalAlignment: HorizontalAlignment.Fill
    verticalAlignment: VerticalAlignment.Fill

    onItemDataChanged: {
        if (itemData) {
            labelTitle.title = itemData["fid"] != "" ? itemData["title"] : PlUtils.getFileName(itemData["playlist_entry_text"]);
            labelTitle.subTitle = itemData["fid"] != "" ? itemData["artist"] : PlUtils.fileMissingInfo(itemData["playlist_entry_text"])
        }
    }
    
    // When the item is selected, make the background solid black, remove the divider and show the 
    // selection image with a drop shadow.  To show the drop shadow, the outside container and 
    // selection image needs to be greater than the inside contents.
    onSelectedChanged: {
        contentArea.background = Color.Black;
        divider.visible = false;
        selectionBorder.visible = true;
        contentArea.minHeight = 114; 
        selectionBorder.minHeight = 140; 
        editListItem.minHeight = 140; 
    }
 
 	
 	Divider {
 	    id: divider
 	}
 
    // delete icon, song title, drag image
    Container {
        id:contentArea 
        
        horizontalAlignment: HorizontalAlignment.Fill
        verticalAlignment: VerticalAlignment.Center
         
        leftPadding: 10
        
	    layout: StackLayout {
	        orientation: LayoutOrientation.LeftToRight
		}
		
		// track number, track title
		Container {   
		    
		    horizontalAlignment: HorizontalAlignment.Left
            verticalAlignment: VerticalAlignment.Fill
		    
		    layout: StackLayout {
		        orientation: LayoutOrientation.LeftToRight
			}
            layoutProperties: StackLayoutProperties {                
                spaceQuota:100
            }
            
            // Delete Button
            Container{
                
                horizontalAlignment: HorizontalAlignment.Left
		        verticalAlignment: VerticalAlignment.Fill
                
 			    layoutProperties: StackLayoutProperties { 			        
 			        spaceQuota:10
 			    }
 			    layout:DockLayout{
 			    }
                ImageView{
                    id: removeButton
                    
                    horizontalAlignment: HorizontalAlignment.Right
 			        verticalAlignment: VerticalAlignment.Center                    
                    
                    scalingMethod: ScalingMethod.AspectFit             
                    imageSource: "asset:///images/context/ic_subtract_stand_alone_dark_theme_inactive.png"
 //USED BY STABILITY TESTING
                }
		        //add the onTouch on the Container to increase the touch area
		        onTouch: {
                    if(event.isUp()) {
                        editListItem.handler.dataModel.removeItemByIndexPath(indexPath);
                    }
                }
		    }
		    
			// track title
		    TrackInfoLabel {
			    id: labelTitle
	            verticalAlignment: VerticalAlignment.Center
                labelTitle.textStyle.fontSize: LayoutConstants.EDIT_PL_TITLE_FONT_SIZE
                labelTitle.textStyle.fontSizeValue: LayoutConstants.EDIT_PL_TITLE_FONT_SIZE_VALUE
                labelTitle.textStyle.color: ColorUtils.getPrimaryTextColor()
                labelSubTitle.textStyle.fontSize: LayoutConstants.EDIT_PL_SUBTITLE_FONT_SIZE
                labelSubTitle.textStyle.fontSizeValue: LayoutConstants.EDIT_PL_SUBTITLE_FONT_SIZE_VALUE
                labelSubTitle.textStyle.color: ColorUtils.getSecondaryTextColor()
                leftPadding: 30
	            layoutProperties: StackLayoutProperties {	                
	                spaceQuota: 90
	            }
	        }
	    }
	   
		// icon drag image
		Container {
		    rightPadding:30
		    
		    horizontalAlignment: HorizontalAlignment.Right
	        verticalAlignment: VerticalAlignment.Center
		    
		    layout: StackLayout {
		        orientation: LayoutOrientation.RightToLeft
			}   
            ImageView{
                verticalAlignment: VerticalAlignment.Center

                id: dragButton
                imageSource: "asset:///images/playlists/core_contextualmenu_handle.png"
            }
		}	
	
	}

	
    // selection image with drop shadow
    Container {
        id: selectionBorder
        visible: false
        horizontalAlignment: HorizontalAlignment.Fill
        verticalAlignment: VerticalAlignment.Fill
        layout: DockLayout {}

        ImageView {
            horizontalAlignment: HorizontalAlignment.Fill
            verticalAlignment: VerticalAlignment.Fill
            scalingMethod: ScalingMethod.Fill
            imageSource: "asset:///images/playlists/ic_drag_pressed.amd"
        }
    }

}
