import bb.cascades 1.0
import "ContentUris.js" as Uris

/**
*  This is a page that displays a list of all songs for a specific genre
 * It contains a title and a GenreList control
 *
 * @param   genreIdStr  The genre id used to retrieve songs for a secific genre
 * @param   titleStr: The text to use as the title for this page
 *
 */

Page {
    id: genreDetails
    property alias genreIdStr: genreList.genreId
    property string titleStr
    property string genreTracksQuery: Uris.genreMembers(["album_artist","album","artist","fullpath","duration","fid"],genreIdStr)

    actions: [
        NowPlayingActionItem {
            page: "GenreDetails"
        },
        ActionItem {
            title: qsTr("Play All") + Retranslate.onLanguageChanged
            imageSource: "asset:///images/playing/ic_play.png"
            onTriggered: {
                dynamicPlaylistManager.startPlaybackByIndex(
                	genreTracksQuery,
                	false, //shuffle off 
                	0);
            }
        },
        ActionItem {
	        title: qsTr("Shuffle All") + Retranslate.onLanguageChanged
	        imageSource: "asset:///images/playing/ic_shuffle.png"
	        ActionBar.placement: ActionBarPlacement.OnBar
	        onTriggered: {
                dynamicPlaylistManager.startPlaybackByIndex(
	            	genreTracksQuery,
	            	true, //shuffle on 
	            	-1);
                commonAnalytics.instance.logShuffleAll("GenreDetails");
	        }
        },
        MultiSelectActionItem {
	        onTriggered: {
	            genreList.listView.selectableMultiHandler.active = true;
	        }
	    },
	    ActionItem {
	        //: menu item for search
	        title: qsTr("Search") + Retranslate.onLanguageChanged
	    	imageSource: "asset:///images/common/ic_search.png"
	        onTriggered: {
                genreDetailsSearchScreen.display();
            }
	    }
	]
	
    onCreationCompleted: {
        genreDetailsSearchScreen.rootPage = genreDetails;
        genreDetailsSearchScreen.hide();
    }

    onActionMenuVisualStateChanged: {
        genreDetailsSearchScreen.enableTypeToSearch(ActionMenuVisualState.Hidden == actionMenuVisualState);
    }
    content: Container {
        id: mainContainer
        layout: DockLayout {
        }

        Container {
            layout: StackLayout {
                orientation: LayoutOrientation.TopToBottom
            }
            ScreenTitle {
                id: screenTitle
                title: (titleStr == null || titleStr == "") ? qsTr("Unknown") + Retranslate.onLanguageChanged : titleStr
                showNowPlaying: true
                
                horizontalAlignment: HorizontalAlignment.Fill
            }
            Container {
                
                horizontalAlignment: HorizontalAlignment.Fill
                
                layoutProperties: StackLayoutProperties {
                    spaceQuota: 100               
                }
                layout: DockLayout {}
                BackgroundWrapper {}
                GenreList {
                    id: genreList
                }
            }
        }
        
        Search { // PlaylistDetails need its own instance of Search screen
        	// because it may be launched from the main Search screen
        	// (see also tabbedPane.searchScreen).
        	id: genreDetailsSearchScreen
        }
    }
}
