import bb.cascades 1.0
import bb.cascades.multimedia 1.0
import bb.cascades.multimedia.internal 1.0
import bb.cascades.music 1.0
import "../radio/components"
import "ContentUris.js" as Uris
import "layoutconstants.js" as LayoutConstants

Container {
    id: mainContainer
    background: Color.Black
    
    property string curSongTitle: ""
    property string curSongArtist: ""
    property string curSongArtFile: ""
    
    property bool noContent: false;
    property bool notEnoughContent: false;  
    property bool noRecentlyPlayedContent: false;
    property bool notEnoughRecentlyPlayedContent: false;
    property string default_thumb_uri: "asset:///images/common/hdcover_album_placeholder.png"
    
    onCreationCompleted: {
        appContext.playbackController.currentMediaItemChanged.connect(updateCurrentSong);
        updateCurrentSong();
    }
    
    layout: DockLayout {}
    
    // HDCover visual when there is no track playing
    // Recently Played Songs
	Container {
	    id: hdCoverRecentlyPlayed
        visible: !playerStub.playbackActive && !RadioPlaybackContext.radioPlaybackActive && !noRecentlyPlayedContent
	    layout: StackLayout {
	        orientation: LayoutOrientation.TopToBottom
	    }
 	    background: Color.Black
 	    
 	    // Fill the entire active frame space given
	    horizontalAlignment: HorizontalAlignment.Fill
	    verticalAlignment: VerticalAlignment.Fill
 	    
 	    property variant recentlyPlayedItem1
 	    property variant recentlyPlayedItem2
 	    property variant recentlyPlayedItem3
 	    property variant recentlyPlayedItem4
 	    
	    attachedObjects: [          
	       AsyncDataModel{
	           id: hdCoverDataModel
	           objectName: "HDRecentPlayed"
	           query: MediaDataQueryDecorator{
		            query: ContentUriHeaderDataQuery{
		                uri: Uris.recentlyPlayedAlbums(["thumb_file","date_modified","artist","album"], LayoutConstants.HDCOVER_GRID_ALBUM_COUNT)                             
		            }
	           }
       	      onItemsChanged:{
				  var count = hdCoverDataModel.childCount(null);
				  if (count == 0 || count == null){
				      mainContainer.noRecentlyPlayedContent = true;
				  }
				  else {
					mainContainer.noRecentlyPlayedContent = false;
					
					if(count < LayoutConstants.HDCOVER_GRID_ALBUM_COUNT){
					    mainContainer.notEnoughRecentlyPlayedContent = true;
					}
					
					var i;
					for (i=0; i<count; i++) {
					    hdCoverRecentlyPlayed["recentlyPlayedItem"+(i+1)] = hdCoverDataModel.data([i]);
					}
					for (;i<LayoutConstants.HDCOVER_GRID_ALBUM_COUNT; i++) {
					    hdCoverRecentlyPlayed["recentlyPlayedItem"+(i+1)] = null;
					}
				}
       	      }
	      }
	    ]
	        
	    onCreationCompleted: {
	        hdCoverDataModel.load();
	    }
	    
	    HDCoverHeader {
            // use up whatever height is left
            layoutProperties: StackLayoutProperties {
    	        spaceQuota: 1
    	    }
    	    
    	    // fill the entire width
    	    horizontalAlignment: HorizontalAlignment.Fill
	        
	        text: qsTr("Recently Played")  + Retranslate.onLanguageChanged
        }
        
        Container{
            layout: DockLayout{
            }
	    
	        HDCoverThumbnailGrid{
	            id: hdCoverRecentlyPlayedGrid
	            
	            visible: !notEnoughRecentlyPlayedContent
		        preferredWidth: LayoutConstants.HDCOVER_ALBUM_COVER_WIDTH
		        preferredHeight: LayoutConstants.HDCOVER_ALBUM_COVER_HEIGHT
		        minHeight: LayoutConstants.HDCOVER_ALBUM_COVER_HEIGHT
		        maxHeight: LayoutConstants.HDCOVER_ALBUM_COVER_HEIGHT
		        minWidth: LayoutConstants.HDCOVER_ALBUM_COVER_WIDTH
		        maxWidth: LayoutConstants.HDCOVER_ALBUM_COVER_WIDTH
		        
		        topMargin: 0
	            thumbUri1: (!hdCoverRecentlyPlayed || !hdCoverRecentlyPlayed.recentlyPlayedItem1) ? "" : hdCoverRecentlyPlayed.recentlyPlayedItem1.thumb_file
	            thumbUri2: (!hdCoverRecentlyPlayed || !hdCoverRecentlyPlayed.recentlyPlayedItem2) ? "" : hdCoverRecentlyPlayed.recentlyPlayedItem2.thumb_file
	            thumbUri3: (!hdCoverRecentlyPlayed || !hdCoverRecentlyPlayed.recentlyPlayedItem3) ? "" : hdCoverRecentlyPlayed.recentlyPlayedItem3.thumb_file
	            thumbUri4: (!hdCoverRecentlyPlayed || !hdCoverRecentlyPlayed.recentlyPlayedItem4) ? "" : hdCoverRecentlyPlayed.recentlyPlayedItem4.thumb_file
	        }
		    
		    ImageView{
		        id: recentlyPlayedAlbumArt
		        
		        visible: !hdCoverRecentlyPlayedGrid.visible
		        imageSource: ( hdCoverRecentlyPlayed.recentlyPlayedItem1 != null && hdCoverRecentlyPlayed.recentlyPlayedItem1.thumb_file != null && hdCoverRecentlyPlayed.recentlyPlayedItem1.thumb_file != "") ?
		                        hdCoverRecentlyPlayed.recentlyPlayedItem1.thumb_file : default_thumb_uri
		        scalingMethod: ScalingMethod.AspectFill
		        preferredWidth: LayoutConstants.HDCOVER_ALBUM_COVER_WIDTH
		        preferredHeight: LayoutConstants.HDCOVER_ALBUM_COVER_HEIGHT
		        minHeight: LayoutConstants.HDCOVER_ALBUM_COVER_HEIGHT
		        maxHeight: LayoutConstants.HDCOVER_ALBUM_COVER_HEIGHT
		        minWidth: LayoutConstants.HDCOVER_ALBUM_COVER_WIDTH
		        maxWidth: LayoutConstants.HDCOVER_ALBUM_COVER_WIDTH
		        
		        topMargin: 0
		    }
		    
		    // Gradient overlay image
	    	ImageView {
	    	    visible: LayoutConstants.HDCOVER_SHOW_RECENTLY_PLAYED_OVERLAY
	            imageSource: "asset:///images/common/recentlyPlayed_active_frame_overlay_n_series.png"      
	            verticalAlignment: VerticalAlignment.Bottom
	        }
	    }
	}
	
	// HD Cover when no track is playing and we have NO recently played songs
	// Recently Added Albums
	Container {
	    id: hdCoverAlbums
        visible: !playerStub.playbackActive && !RadioPlaybackContext.radioPlaybackActive && !hdCoverRecentlyPlayed.visible && !noContent
	    layout: StackLayout {
	        orientation: LayoutOrientation.TopToBottom
	    }
 	    background: Color.Black
 	    
 	    // Fill the entire active frame space given
	    horizontalAlignment: HorizontalAlignment.Fill
	    verticalAlignment: VerticalAlignment.Fill
 	      
 	    property variant albumItem1
 	    property variant albumItem2
 	    property variant albumItem3
 	    property variant albumItem4
 	    
	    attachedObjects: [          
	       AsyncDataModel{
	           id: hdCoverAlbumsDataModel
	           objectName: "HDRecentAdded"
	           query: MediaDataQueryDecorator{
		            query: ContentUriHeaderDataQuery{
		                uri: Uris.recentlyAddedAlbums(["album","thumb_file","date_modified","album_artist","album_artist_id","album_id"],LayoutConstants.HDCOVER_GRID_ALBUM_COUNT)		            
	                }
	           }
       	      onItemsChanged:{
				  var count = hdCoverAlbumsDataModel.childCount(null);
				  if (count == 0 || count == null){
				      mainContainer.noContent = true;
				  }
				  else {
					mainContainer.noContent = false;
					
					if(count < LayoutConstants.HDCOVER_GRID_ALBUM_COUNT){
					    mainContainer.notEnoughContent = true;
					}
					
                    var i;
                    for (i=0; i<count; i++) {
                        hdCoverAlbums["albumItem"+(i+1)] = hdCoverAlbumsDataModel.data([i]);
                    }
                    for (;i<LayoutConstants.HDCOVER_GRID_ALBUM_COUNT; i++) {
                        hdCoverAlbums["albumItem"+(i+1)] = null;
                    }
				}
       	      }
	      }
	    ]
	        
	    onCreationCompleted: {
	        hdCoverAlbumsDataModel.load();
	    }
	    
	    HDCoverHeader {
            // use up whatever height is left
            layoutProperties: StackLayoutProperties {
    	        spaceQuota: 1
    	    }
    	    
    	    // fill the entire width
    	    horizontalAlignment: HorizontalAlignment.Fill
	        
	        text: qsTr("Library")  + Retranslate.onLanguageChanged
        }
        
        Container{
            layout: DockLayout{
            }
        
	        HDCoverThumbnailGrid{
	            id: hdCoverAlbumsGrid
	            
	            visible: !notEnoughContent
		        preferredWidth: LayoutConstants.HDCOVER_ALBUM_COVER_WIDTH
		        preferredHeight: LayoutConstants.HDCOVER_ALBUM_COVER_HEIGHT
		        minHeight: LayoutConstants.HDCOVER_ALBUM_COVER_HEIGHT
		        maxHeight: LayoutConstants.HDCOVER_ALBUM_COVER_HEIGHT
		        minWidth: LayoutConstants.HDCOVER_ALBUM_COVER_WIDTH
		        maxWidth: LayoutConstants.HDCOVER_ALBUM_COVER_WIDTH
		        
		        topMargin: 0
	            thumbUri1: (!hdCoverAlbums || !hdCoverAlbums.albumItem1) ? "" : hdCoverAlbums.albumItem1.thumb_file
	            thumbUri2: (!hdCoverAlbums || !hdCoverAlbums.albumItem2) ? "" : hdCoverAlbums.albumItem2.thumb_file
	            thumbUri3: (!hdCoverAlbums || !hdCoverAlbums.albumItem3) ? "" : hdCoverAlbums.albumItem3.thumb_file
	            thumbUri4: (!hdCoverAlbums || !hdCoverAlbums.albumItem4) ? "" : hdCoverAlbums.albumItem4.thumb_file
	        }
	        
	        ImageView{
		        id: recentlyAddedAlbumArt
		        
		        visible: !hdCoverAlbumsGrid.visible
		        imageSource: (  hdCoverAlbums.albumItem1 != null && hdCoverAlbums.albumItem1.thumb_file != null && hdCoverAlbums.albumItem1.thumb_file != "") ?
		                        hdCoverAlbums.albumItem1.thumb_file : default_thumb_uri
		        scalingMethod: ScalingMethod.AspectFill
		        preferredWidth: LayoutConstants.HDCOVER_ALBUM_COVER_WIDTH
		        preferredHeight: LayoutConstants.HDCOVER_ALBUM_COVER_HEIGHT
		        minHeight: LayoutConstants.HDCOVER_ALBUM_COVER_HEIGHT
		        maxHeight: LayoutConstants.HDCOVER_ALBUM_COVER_HEIGHT
		        minWidth: LayoutConstants.HDCOVER_ALBUM_COVER_WIDTH
		        maxWidth: LayoutConstants.HDCOVER_ALBUM_COVER_WIDTH
		        
		        topMargin: 0
		    }
		    
		    // Gradient overlay image - for N-series only
	    	ImageView {
	    	    visible: LayoutConstants.HDCOVER_SHOW_RECENTLY_PLAYED_OVERLAY
	            imageSource: "asset:///images/common/recentlyPlayed_active_frame_overlay_n_series.png"      
	            verticalAlignment: VerticalAlignment.Bottom
	        }
	    }
	}        

	HDCoverNowPlaying{
		id: hdCoverNowPlaying
		visible: playerStub.playbackActive
		background: Color.Black
		
		artistNameText: mainContainer.curSongArtist != "" ? mainContainer.curSongArtist : qsTr("Unknown") + Retranslate.onLanguageChanged
		songTitleText: mainContainer.curSongTitle != "" ? mainContainer.curSongTitle: qsTr("Unknown")  + Retranslate.onLanguageChanged
		albumArtSource: (mainContainer.curSongArtFile != "" )? mainContainer.curSongArtFile: default_thumb_uri 
	}
	
	HDCoverFM{
	    id: hdCoverFM
        visible: RadioPlaybackContext.radioPlaybackActive
        background: Color.Black
        stationFrequencyText: RadioPlaybackContext.currentRadioFrequency
	}
	
	// Gradient overlay image
	ImageView {
	    visible: LayoutConstants.HDCOVER_SHOW_NOW_PLAYING_OVERLAY
        imageSource: "asset:///images/common/active_frame_overlay.png"      
        verticalAlignment: VerticalAlignment.Bottom
    }
	
	function updateCurrentSong () {
        if (appContext.playbackController != null && appContext.playbackController.currentSong != null) {
            var currentSong = appContext.playbackController.currentSong;

            mainContainer.curSongTitle = currentSong.title != null ? currentSong.title : "";
            mainContainer.curSongArtist = currentSong.artist != null ? currentSong.artist : "";
            mainContainer.curSongArtFile = currentSong.art_file != null ? currentSong.art_file : "";
        }
    }
}
