import bb.cascades 1.0
import bb.cascades.multimedia 1.0
import bb.cascades.multimedia.internal 1.0
import "layoutconstants.js" as LayoutConstants

/**
 * A reusable component that allows an image to be displayed as well as three rows of text
 *
 * @param	topText	 	The top row of text
 * @param	middleText	The middle row of text
 * @param 	thumbUri	The uri for the image to display	
 */

Container {
    id: imageThreeRowTextHeader
    preferredHeight: LayoutConstants.ALBUM_HEADER_HEIGHT
    background: ColorUtils.get(0xffc4c4c4)
    horizontalAlignment: HorizontalAlignment.Fill
    
    property string thumbUri: "."  //need a non empty string to ensure onThumbUriChanged event is generated if we do not have album art.
    property string thumb1Uri
    property string thumb2Uri
    property string thumb3Uri
    property string thumb4Uri
    property string topText
    property string middleText
    property alias shuffleButtonVisible: shuffleButton.visible
    property bool fourSquareVisible
    property string defaultImage: "asset:///images/common/album_placeholder.png"
    
    signal shuffleClicked
    
    function getThumbUri(uri) {
        return (uri == "" || uri == null) ? defaultImage : uri;
    }
    
    // binding for ControlDelegate is flaky, set it directly
    onThumbUriChanged: {
        if (!fourSquareVisible && controlDelegate.control) {
            controlDelegate.control.imageSource = getThumbUri(thumbUri);
        }
    }

    attachedObjects: [
        ComponentDefinition{
            id: singleImageComponent
    	    ImageView {
    	        id: albumArtHolder
                imageSource: getThumbUri(thumbUri)
    	        verticalAlignment: VerticalAlignment.Fill
                horizontalAlignment: HorizontalAlignment.Fill
    	    }
        },
        ComponentDefinition{
            id: fourImageComponent
    	    FourThumbsListItem{
                id: fourThumb
                thumb1Uri: imageThreeRowTextHeader.thumb1Uri
                thumb2Uri: imageThreeRowTextHeader.thumb2Uri
                thumb3Uri: imageThreeRowTextHeader.thumb3Uri
                thumb4Uri: imageThreeRowTextHeader.thumb4Uri
                topLabelVisible: false
    	    	bottomLabelVisible: false
    	    	overlayOpacity: 0.0
    	    	defaultImage: imageThreeRowTextHeader.defaultImage
    	    }
        }
    ]

    layout: StackLayout {
        orientation: LayoutOrientation.LeftToRight
    }
    Container {
        preferredHeight: LayoutConstants.ALBUM_HEADER_HEIGHT
        preferredWidth: LayoutConstants.ALBUM_HEADER_HEIGHT
        layout: DockLayout { }
	    ControlDelegate {
	        id: controlDelegate
	        sourceComponent: imageThreeRowTextHeader.fourSquareVisible ? fourImageComponent : singleImageComponent
	    }
    }
    //container to keep the two rows of text
    Container {
        leftPadding: 20
        rightPadding: 10
        verticalAlignment: VerticalAlignment.Fill
        layout: DockLayout { }
        layoutProperties: StackLayoutProperties {
            spaceQuota: 100
        }
        Container {  //This container is required if we want the Label to resize properly
            layout: StackLayout {
                orientation: LayoutOrientation.TopToBottom
            }
            horizontalAlignment: HorizontalAlignment.Left
            verticalAlignment: VerticalAlignment.Fill
            topPadding: LayoutConstants.ALBUM_HEADER_TEXT_TOP_PADDING
            rightPadding: LayoutConstants.ALBUM_HEADER_TEXT_RIGHT_PADDING
            Label {
                id: topTextLabel
                layoutProperties: StackLayoutProperties {
                    spaceQuota: 50
                }
                bottomMargin: 0
                implicitLayoutAnimationsEnabled: false
                //:Default text to specify unknown album name - genre and playlist will always override this with duration
                text: (topText == null || topText == "") ? (qsTr("Unknown") + Retranslate.onLanguageChanged) : topText
                
                textStyle.base: SystemDefaults.TextStyles.BigText
                
                textStyle.color: ColorUtils.get(0xff262626)
                textStyle.fontWeight: FontWeight.W400
                textStyle.fontSize: LayoutConstants.ALBUM_HEADER_TOP_FONT_SIZE
                textStyle.fontSizeValue: LayoutConstants.ALBUM_HEADER_TOP_FONT_SIZE_VALUE
                verticalAlignment: VerticalAlignment.Center
                textFormat: TextFormat.Plain
	        }
            Label {
                id: middleTextLabel
                layoutProperties: StackLayoutProperties {
                    spaceQuota: 50
                }
                text: middleText
                topMargin: 0
                implicitLayoutAnimationsEnabled: false
                textStyle.base: SystemDefaults.TextStyles.TitleText
                textStyle.color: ColorUtils.get(0xff444444)
                textStyle.fontWeight: FontWeight.W400
                textStyle.fontSize: LayoutConstants.ALBUM_HEADER_LOWER_FONT_SIZE
                textStyle.fontSizeValue: LayoutConstants.ALBUM_HEADER_LOWER_FONT_SIZE_VALUE
                verticalAlignment: VerticalAlignment.Center
                textFormat: TextFormat.Plain
                visible: (middleText == "" || middleText == null) ? false : true
            }	        
        }
        ImageButton {
            id: shuffleButton
            defaultImageSource: "asset:///images/playing/ic_shuffle_dk.png";
            pressedImageSource: "asset:///images/playing/ic_shuffle_active.png";
            disabledImageSource: "asset:///images/playing/ic_shuffle.png";
            verticalAlignment: LayoutConstants.ALBUM_HEADER_SHUFFLE_ALIGNMENT
            horizontalAlignment: HorizontalAlignment.Right
            onClicked: {
                shuffleClicked();
            }
        }
    }
}