/**
 * InvokeUtils.js
 * 
 * Research In Motion Limited proprietary and confidential.
 * Copyright (C) Research In Motion Limited, 2012.
 */

Qt.include("ContentUris.js")

function handleInvoke(request,dataModel,dynamicPlaylistManager,playbackActive) {
	
    var params = request.uri.queryParams;
	
    //If we are already set up for playback and receive a 'resumeOrShuffle', resume
    if (params && params.play && params.play.toLowerCase() == 'resumeorshuffle' && playbackActive) {    	
    	console.log ("Received resumeorshuffle and playback was active, calling play()");
    	dynamicPlaylistManager.playbackController.play();    	
    	return;
    }	
		
    var invokeHandler = new InvokeHandler(dataModel, dynamicPlaylistManager);

    var action = request.uri.authority;
    
    // Play music by full path
    if (!request.uri.authority && request.uri.path) {
    	var contentUri = filter(invokeLookup(["album_id", "fullpath"]), "fullpath=:path");
        var result = invokeHandler._loadData(contentUri, {':path': request.uri.path});
        if (result) {
            // Invoke into artist details and start playback of the file.
            action = "albums-from-fullpath";
            params.id = result["album_id"];
            params.play = result["fid"];
            params.playId = params.play;
        } else {
            invokeHandler.onError(qsTr("Playback failed"), qsTr("Cannot find file \"%1\"").arg(request.uri.path.replace(/\/accounts\/1000\/(removable|shared)/g, "")));
        }
    } else {
        // Special arrangements for play=shuffle.
        if (params && params.play) {
            params.shuffle = params.play.toLowerCase()=='shuffle' || params.play.toLowerCase() == 'resumeorshuffle';
            params.playId = params.shuffle ? -1 : params.play;
        }

        if (params && (params.id || params.playId)) {
            // Backwards compatibility for album_id and genre_id.
            var compatibilityMap = {
                albums: {
                    play: [ "fid", "album_id" ],
                    show: [ "db_album_id", "album_id" ]
                },
                genres: {
                    play: [ "fid", "genre_id" ],
                    show: [ "db_genre_id", "genre_id" ]
                }
            };

            if (compatibilityMap.hasOwnProperty(action)) {
                var cm = compatibilityMap[action];
                if (params.playId > 0) {
                    params.id = invokeHandler._mapColumn(cm.play[0], cm.play[1], params.playId);
                } else {
                    params.id = invokeHandler._mapColumn(cm.show[0], cm.show[1], params.id);
                }
            }
        }
    }
    
    var allowSwitchView = invokeHandler.startPlayback(action, params);
    if (allowSwitchView)	//only switch view if requested music exists on device
    {
    	invokeHandler.switchView(action, params, tabbedPane.activePane);
    }
}

function InvokeHandler(synchronousDataModel, dynamicPlaylistManager) {
    this.dataModel = synchronousDataModel;
    this.dynamicPlaylistManager = dynamicPlaylistManager;
}

InvokeHandler.prototype = {

    /**
     * Start music playback for contentURI, starting at the index found from the given
     * SQL where clause.
     * 
     * @param contentUri - content uri to start playback for
     * @param songData - any relevant data about the song to play back. Can contain field's such as fid, album_id, etc.
     * @param shuffle - true if we are to launch in shuffle mode.
     */
    playMusicById: function(contentUri, songData, shuffle) {
        // Find the starting index.
        // playbackController.setPlaylistUsingFid() results in ambiguous columns errors.
        var index = -1;
        if (!shuffle) {
            var fid = songData["fid"];
            if (fid < 0) { // negative fid means play the first item
                index = 0;
            } else {
                var i = musicUtils.getIndexByUrl(contentUri, "fid=" + fid);
                if (i > 0) {
                    index = i-1;
                } else {
                    return false;
                }
            }
        }
        console.debug("Starting playback: URI=\"" + contentUri + "\"");
        console.debug("Starting playback: shuffle=" + shuffle + "; index=" + index);

        // TODO(emaccaull): Remove this try/catch once ListItemVisual no longer references
        // the playbackController or QTBUG-29676 is fixed.
        //
        // A known Qt issue (https://bugreports.qt-project.org/browse/QTBUG-29676) is
        // causing the JS engine to bail after playback starts.
        try {
            this.dynamicPlaylistManager.startPlaybackByIndex(contentUri, shuffle, index, "0", PlaybackController.PlaybackPlaying, songData);
        } catch (error) {
            var message = "playback error: ";
            for (var prop in error) {
                if (error.hasOwnProperty(prop)) {
                    message += prop + "=" + error[prop] + "; ";
                }
            }
            console.log("Invoke:", message);
        }
        return true;
    },

    /**
     * Start audio playback for the given action and parameters.  The action and 
     * parameters are expected to have been parsed from a music:// URI.
     * 
     * @see http://foundry51.qnx.com/sf/wiki/do/viewPage/projects.internal_deckard/wiki/ApplicationArguments
     */
    startPlayback: function(action, params) {
        console.debug("Invoked: action = " + action);
        for (var p in params) {
            console.debug("    key:" + p + "; value:" + params[p]);
        }
        
        // used to dummy in the information for the current song for the playlist page we are
        // pushing
        var songData;
        if (action=="songs") {
            console.debug("Invoked: play all songs");
            songData = {"fid" : params.playId};
            return this.checkContentAndPlay(allSongsPlayback(), songData, params.shuffle);
        }
        if (!params.id || !params.play) {
            console.debug("Invoked: ID or playback file not specified. Nothing to play.");
            return true;
        }
        
        var playbackUri = "";
        switch (action) {
        case "artists":
            playbackUri = artistAlbumMembersPlayback(params.id);
            // gets the albumId from the fid.
            var albumId = this._mapColumn("fid", "album_id", params.playId);
            songData = {"album_id":albumId, "fid":params.playId, "artist_id": params.id};
            break;
        case "albums-from-fullpath":
            playbackUri = albumMembers(playerColumns(),params.id);
            songData = {"album_id":params.id, "fid":params.playId};
            break;
        case "albums":
            playbackUri = albumMembers(playerColumns(),params.id);
            songData = {"album_id":params.id, "fid":params.playId};
            break;
        case "playlists":
            playbackUri = playlistPlayback(params.id);
            songData = {"fid" : params.playId, "plid" : params.id};
            break;
        case "genres":
            playbackUri = genreMembers(playerColumns(),params.id);
            songData = {"fid" : params.playId};
            break;
        }
        
        if (playbackUri) {
        	return this.checkContentAndPlay(playbackUri, songData, params.shuffle);
        } else {
            console.log("Invoked: can't play anything for action " + action);
        }
        return true;
    },

    // True if an object is undefined, null or has no properties.
    isEmpty: function(object) {
        return object == undefined || Object.keys(object).length == 0;
    },

    /*
     * Synchronously load data.
     */
    _loadData: function(contentUri, map) {
    	// Assuming the model is behind a DataModelDecorator
    	var query = dataQueryFactory.createContentUriDataQuery(this.dataModel);
        query.uri = contentUri;
        if (map) {
        	query.bindValues = map;
        }
        this.dataModel.query = query;       
        this.dataModel.load();
        var res = this.dataModel.data([0]);

        // Return null if res has no properties
        return this.isEmpty(res) ? null : res;
    },

    _mapColumn: function(from, to, fromValue) {
        var uri = filter(invokeLookup([from, to]), from + "=:id");
        var result = this._loadData(uri, {":id": fromValue});
        return result && result[to];
    },

    _openAllTracksPlaylist: function() {
    	tabbedPane.activePane.popAndDeleteAll();
        var page = tabbedPane.activePane.safePushOnePage("core/SmartPlaylistDetails.qml");
        page.headerURI = allSongsHeader();
        page.contentURI = allSongs(["album","album_artist","artist","fid","duration","fullpath"]);
        page.titleStr = qsTr("All Songs");
        var fourSquare = this._loadData(fourSquare());
        var defaultImage = "asset:///images/common/album_placeholder.png";

        for (var i = 1; i <= 4; i++) {
        	page["thumb" + i + "Uri"] = fourSquare["four_square_" + i] || defaultImage;
        }
    },

    pushPage: function(contentUri, navPane, pageName, resultMap) {
        var results = this._loadData(contentUri);
        if (results) {
            navPane.popAndDeleteAll();
            var page = navPane.safePushOnePage(pageName);
            if (page) {
                for (var prop in resultMap) {
                    var value = results[ resultMap[prop] ];
                    if (!value && typeof page[prop] == 'string') {
                        page[prop] = qsTr("Unknown");
                    } else {
                        page[prop] = results[ resultMap[prop] ];
                    }
                }
            }
            return page;
        }
        return null;
    },

    /**
     * Switch to a view for the action and parameters.  The action and params are expected
     * to have been parsed from a music:// URI.
     *
     * @param navPane The current SafePushNavPane that pages should be pushed to.
     * 
     * @see http://foundry51.qnx.com/sf/wiki/do/viewPage/projects.internal_deckard/wiki/ApplicationArguments
     */
    switchView: function(action, params, navPane) {
    	console.log("Invoked: BG navPane=" + navPane);
    	if (navPane.isDirty()) {
            console.log("Invoked: Nav pane is dirty; not pushing new page.");
            return;
        }
        if (action) {
            if (this.isEmpty(params) && appContext.isLibraryAction(action)) {
                // TODO(emaccaull): remove global references!
            	navPane.popAndDeleteAll();
                console.debug("Invoked: switch to library screen");
                appContext.invokeLibrary(action);
            } else if (action == "songs") {
                // Push the all songs screen.
                this._openAllTracksPlaylist();
            } else if (params && params.id) {
                // Push the appropriate page onto the current navigation stack.
                switch (action) {
                case "artists":
                    var uri = artistAlbums(["album_artist_id","album_artist","artist_id"],params.id);
                    var page = this.pushPage(uri, navPane, "core/ArtistDetails.qml",
                            {'albumArtistIdStr': 'album_artist_id', 'titleStr': 'album_artist'});
                    page.scrollToCurrentSongAfterInvoke = true;
                    break;
                case "albums-from-fullpath":  // Fall through
                case "albums":
                    var uri = album(["album_id","album_artist_id","album_artist"],params.id);
                    var page = this.pushPage(uri, navPane, "core/ArtistDetails.qml",
                            {'openAtAlbumId': 'album_id', 'albumArtistIdStr': 'album_artist_id', 'titleStr': 'album_artist'});
                    page.scrollToCurrentSongAfterInvoke = true;
                    break;
                case "playlists":
                    var uri = playlist(["playlist_name","plid"],params.id);
                    this.pushPage(uri, navPane, "core/PlaylistDetails.qml",
                            {'playlistIdStr': 'plid', 'titleStr': 'playlist_name'});
                    commonAnalytics.instance.logPagePush("PlaylistDetails", "InvokeUtils");
                    break;
                case "genres":
                    var uri = genre(["genre","genre_id"], params.id);
                    this.pushPage(uri, navPane, "core/GenreDetails.qml",
                            {'genreIdStr': 'genre_id', 'titleStr': 'genre'});
                    commonAnalytics.instance.logPagePush("GenreDetails", "InvokeUtils");
                    break;
                }
            }
        }
    },
    
    /**
     * Checks if content exists, and plays music by id.
     * 
     * @param uri - content uri to play
     * @param songData - relevant data to the view, ie: fid, album_id, etc..
     * @param shuffle - true if shuffle, false if not.
     */
    checkContentAndPlay: function(uri, songData, shuffle)
    {
    	//check if requested data exists
    	var res = this._loadData(uri + "&type=count");
        if (res && res["COUNT(*)"] > 0)
        {
            if (this.playMusicById(uri, songData, shuffle)) {
                return true;
            } else {
                this.onError(qsTr("Playback failed"), qsTr("Unable to find media."));
                console.debug("Invoked: FID not available");
                return false;
            }
        }
        else
        {
        	this.onError(qsTr("Playback failed"), qsTr("Unable to find media."));
        	console.debug("Invoked: no music available");
        	return false;
        }
    },
    
    onError: function(title, body)
    {
    	var dialog = systemDialogUtils.getSystemDialog();
        dialog.title = title;
        dialog.body = body;
        dialog.confirmButton.label = qsTr("OK");
        dialog.cancelButton.label = undefined;
        dialog.show();
    }

}; // InvokeHandler.prototype