import bb.cascades 1.0
import bb.cascades.multimedia 1.0
import bb.cascades.multimedia.internal 1.0
import "ContentUris.js" as Uris

Page {
    property alias libraryContentQMLPath:libraryPage.contentQMLPath
    /**
     * property used to check if there is any content in this page
     */
    property bool noContentViewVisible: false

    property variant searchScreenContainer: libraryPage

    id: libPage

    actions: [
        NowPlayingActionItem {
            page: "Library"
        },
        ActionItem {
            enabled: !noContentViewVisible
            title: qsTr("Play All") + Retranslate.onLanguageChanged
            imageSource: "asset:///images/playing/ic_play.png"
            onTriggered: {
                dynamicPlaylistManager.startPlaybackByIndex(
                	Uris.allSongsPlayback(),
                	false, //shuffle off 
                	0);
            }
        },
        
        ActionItem {
            enabled: !noContentViewVisible
            title: qsTr("Shuffle All") + Retranslate.onLanguageChanged
            imageSource: "asset:///images/playing/ic_shuffle.png"
            onTriggered: {
                dynamicPlaylistManager.startPlaybackByIndex(
                	Uris.allSongsPlayback(),
                	true, //shuffle on 
                	-1);
                commonAnalytics.instance.logShuffleAll("Library");
            }
        },
        ActionItem {
	        //: menu item for search
	        title: qsTr("Search") + Retranslate.onLanguageChanged
	        imageSource: "asset:///images/common/ic_search.png"
	        onTriggered: {
                appContext.searchScreen.display();
            }
        },
        MultiSelectActionItem {
            enabled: !noContentViewVisible
            onTriggered: {
                if (!libraryPage.at(0).libraryListView.multiHandler.active) {
                    libraryPage.at(0).libraryListView.multiHandler.active = true;
                } else {
                    resetMultiSelectMode ();
                }
            }
        }
    ]
    
    onCreationCompleted: {
        appContext.addAppWorldActionItem(libPage);
    }

    onActionMenuVisualStateChanged: {
        appContext.searchScreen.enableTypeToSearch(ActionMenuVisualState.Hidden == actionMenuVisualState);
    }

    content: SwitchableContainer {
        id: libraryPage
        horizontalAlignment: HorizontalAlignment.Fill
        verticalAlignment: VerticalAlignment.Fill

        layout: DockLayout {
        }
        contentQMLPath: "EmptyContainer.qml"
        onContentQMLPathChanged: {
            if (libraryPage.at(0).noContentView) {
                libraryPage.at(0).noContentView.visibleChanged.connect(onNoContentViewVisibleChanged);
            }
        }
        function onNoContentViewVisibleChanged(visible) {
            noContentViewVisible = visible;
        }
    }

    function resetMultiSelectMode() {
        libraryPage.at(0).selectableLibraryDataModel.clearSelections();
        libraryPage.at(0).libraryListView.clearSelection();
    }

    function handleInvokeAction(actionName) {
        libraryPage.at(0).selectSegment(actionName);
    }

    // Check if the invocation action, actionName, should update this library page.
    function isLibraryInvokeAction(actionName) {
        return libraryPage.at(0).getSegmentIndex(actionName) >= 0;
    }
}
