import bb.cascades 1.0
import bb.cascades.multimedia 1.0
import bb.cascades.multimedia.internal 1.0
import "ContentUris.js" as Uris
import "layoutconstants.js" as LayoutConstants
/**
 * This control contains buttons at the top and a content view
 * that loads different QML documents depending on what button
 * is tapped. This will eventually be replaced by a core control.
 */
Container {
    property variant libraryListView
    property variant segmentDataModel: artistsData
    property variant noContentView: noContentView
    property variant artistsPage
    property variant albumsPage
    property variant genresPage
     
    attachedObjects: [
        AsyncHeaderDataModel {
            objectName: "Artists"
            id: artistsData
            query: LibraryHeaderQueryDecorator {
                query: ContentUriHeaderDataQuery {
                    uri: Uris.artists(["album_artist_id","thumb_file"])
                    headerUri: Uris.artistGroups([])
                    indexEnabled: true
                    onError:  appContext.showDataQueryErrorDialog(code, message)
                }
            }
        },
        SelectableDataModelDecorator {
            id: artistsSelectableModel
            model: MediaDataModelDecorator {
                model: PrioritizedDataModelDecorator {
                    model: artistsData
                }
            }
        },
        AsyncHeaderDataModel {
            objectName: "Albums"
            id: albumsData
            query: LibraryHeaderQueryDecorator {
                query: ContentUriHeaderDataQuery {
                    uri: Uris.albums(["album_id","album_artist_id","thumb_file","album_artist"])
                    headerUri: Uris.albumGroups([])
                    indexEnabled: true
                    onError:  appContext.showDataQueryErrorDialog(code, message)
                }
            }
        },
        SelectableDataModelDecorator {
            id: albumsSelectableModel
            model: MediaDataModelDecorator {
                model: PrioritizedDataModelDecorator {
                    model: albumsData
                }
            }
        },
        AsyncHeaderDataModel {
            objectName: "Genres"
            id: genresData
            query: MediaDataQueryDecorator {
                query: ContentUriHeaderDataQuery {
                    uri: Uris.genres(["genre_id","thumb_file"])
                    headerUri: "" //if header uri is empty, the model will provide one dummy header for all the data
                    indexEnabled: true
                    onError:  appContext.showDataQueryErrorDialog(code, message)
                }
            } 
        },
        SelectableDataModelDecorator {
            id: genresSelectableModel
            model: PrioritizedDataModelDecorator {
                model: genresData
            }
        }
    ]

    layout: StackLayout {
        orientation: LayoutOrientation.TopToBottom
    }
    verticalAlignment: VerticalAlignment.Center
    horizontalAlignment: HorizontalAlignment.Fill

    ScreenTitle {
        largeText: false
        showNowPlaying: true
        title: qsTr("Music Library") + Retranslate.onLanguageChanged
        horizontalAlignment: HorizontalAlignment.Fill
        verticalAlignment: VerticalAlignment.Top
    }

    Container {

        layout: DockLayout {}
        layoutProperties: StackLayoutProperties {
            spaceQuota: 100
        }
        BackgroundWrapper {
            useAltPaint: (libraryListView ? libraryListView.isMultiSelect : false)
        }

        NoContentView {
            id: noContentView
            image: "asset:///images/common/music_no_content_icon.png"
            title: qsTr("No Music") + Retranslate.onLanguageChanged
            subTitle: (MediaApplication.corporate ? qsTr("You are currently in the work space. Please switch to the personal space to see any personal files.\n\nFiles stored on the SD card are not accessible from the work space. Please switch to the personal space to see the files stored on the SD card.") : qsTr("You can use BlackBerry Link to sync music between your device and computer. \n\nTo download BlackBerry Link on your computer, visit\nwww.blackberry.com/BlackBerryLink")) + Retranslate.onLanguageChanged
            pageDataModel: segmentDataModel
            verticalAlignment: VerticalAlignment.Center
        }

        Container{
            layout: StackLayout {
                orientation: LayoutOrientation.TopToBottom
            }
            Container {
                topPadding: LayoutConstants.CONTROL_PADDING
                bottomPadding: LayoutConstants.CONTROL_PADDING
	            SegmentedControl {
		            id: segmentedControl
		            objectName: "segmentedControl"
		            property variant optionNames: ["Artists", "Albums", "Genres"]
		            horizontalAlignment: HorizontalAlignment.Center
		            visible: !noContentView.visible
		            
		            Option {
		                id: option1
		                objectName: "option1"
		                text: qsTr("Artists") + Retranslate.onLanguageChanged
		                value: "asset:///core/Artists.qml"
		                selected: true
		            }
		            Option {
		                id: option2
		                objectName: "option2"
		                text: qsTr("Albums") + Retranslate.onLanguageChanged
		                value: "asset:///core/Albums.qml"
		            }
		            Option {
		                id: option3
		                objectName: "option3"
		                text: qsTr("Genres") + Retranslate.onLanguageChanged
		                value: "asset:///core/Genres.qml"
		            }
	                onSelectedIndexChanged: {
	                    loadSegmentModel(segmentedControl.selectedValue);
	                }
	            }
            }
    	    Container {
    	        id: switchableContainer
    	        preferredWidth: PrimaryDisplay.pixelSize.width
    	        maxWidth: PrimaryDisplay.pixelSize.width
    	        minWidth: PrimaryDisplay.pixelSize.width
    	        visible: !noContentView.visible
    	        
    	        verticalAlignment: VerticalAlignment.Fill
    	        
    	        layout: DockLayout {}
    	        
    	        onCreationCompleted: {
                    artistsData.load();
                    loadSegmentModel("asset:///core/Artists.qml");
                    
                    albumsData.load();
                    genresData.load();
    		    }
    	    }
        }
    }
    function addPage(page) {
        if (switchableContainer.count() == 0) {
            switchableContainer.add(page);
        } else {
            switchableContainer.replace(0, page);
        }
    }

    function loadSegmentModel(switchableContainerQMLPath) {
        if (switchableContainer.at(0) != null) {
            switchableContainer.at(0).listModel = null; // so that all data model's slots are disconnected
        }
        switch(switchableContainerQMLPath){
            case "asset:///core/Artists.qml":
                if (artistsPage == null) {
                    var component = Qt.createComponent(switchableContainerQMLPath);
                    artistsPage = component.createObject(switchableContainer);
                    artistsPage.scrolladexActiveChanged.connect(activeChangedSlot);
                }
                addPage(artistsPage);
                switchableContainer.at(0).listModel = artistsSelectableModel;
                segmentDataModel.data([0,0]); //request data at indexpath [0,0] before switching datamodel to prevent cache misses
                segmentDataModel = artistsData;
                break;
            case "asset:///core/Albums.qml":
                if (albumsPage == null) {
                    var component = Qt.createComponent(switchableContainerQMLPath);
                    albumsPage = component.createObject(switchableContainer);
                    albumsPage.scrolladexActiveChanged.connect(activeChangedSlot);
                }
                addPage(albumsPage);
                switchableContainer.at(0).listModel = albumsSelectableModel;
                segmentDataModel.data([0,0]); //request data at indexpath [0,0] before switching datamodel to prevent cache misses
                segmentDataModel = albumsData;
                break;
            case "asset:///core/Genres.qml":
                if (genresPage == null) {
                    var component = Qt.createComponent(switchableContainerQMLPath);
                    genresPage = component.createObject(switchableContainer);
                }
                addPage(genresPage);
                switchableContainer.at(0).listModel = genresSelectableModel;
                segmentDataModel.data([0,0]); //request data at indexpath [0,0] before switching datamodel to prevent cache misses
                segmentDataModel = genresData;
                break;
        }
        if (libraryListView) {
            libraryListView.multiselectActiveChanged.disconnect(activeChangedSlot);
        }
        libraryListView = switchableContainer.at(0).listView;
        libraryListView.multiselectActiveChanged.connect(activeChangedSlot);
    }

    // Put the segment given by segmentName into focus.
    function selectSegment(segmentName) {
        var index = getSegmentIndex(segmentName);
        if (index >= 0) {
            segmentedControl.setSelectedIndex(index);
        }
    }

    // Return the index in the SegmentedControl of the option with the given text.
    // Returns -1 if optionName is invalid.
    function getSegmentIndex(optionName) {
        // Transform the given optionName into the same case as the list of optionNames.
        var name = optionName.charAt(0).toUpperCase() + optionName.slice(1).toLowerCase();
        return segmentedControl.optionNames.indexOf(name);
    }
	
    function activeChangedSlot(active) {
        segmentedControl.enabled = !active;
    }	
}