import bb.cascades 1.0
import bb.cascades.multimedia 1.0
import bb.system 1.0

/**
 * This component is used as the visual for each individual song in the lists
 * It contains a track number, name, duration
 * 
 * @param   title                The text to be used for the title of the song
 * @param   track                The text for the track number
 * @param   duration             The integer for the duration of the song
 */

Container {
    id: listItem

    property TabbedPane tabbedPane: ListItem.view.listTabbedPane

    //following properties are set from the external qml list that uses this ListItemVisual
    property string title: ""
    property string track: ""
    property int duration: 0
    property bool multiSelectionMode: false
    property bool itemSelected: false

    // properties to specify which action items to set
    property bool allowSearchAppWorld: !MediaApplication.corporate // BB World - Work has no music to purchase
    property bool allowAddToPlaylistAction: true
    property bool allowPlayNextAction: true
    property bool allowPlayLastAction: true
    property bool allowMultiSelectAction: true
    property bool allowDeleteAction: true
    property bool allowPlayOnAction: appContext.showPlayOnAction
    property bool allowPropertiesAction: true
    property bool allowInvokeAction: true
    
    //internal property used by qml files that are wrapped in ListItemParent
    property string durationText
    
    onDurationChanged: {
        reevaluateDurationText();
    }
    
    function reevaluateDurationText(){
        if (listItem.duration == -1) {
            durationText = ""; //File is missing
        } else {
            durationText = localeUtils.getColonSeparatedDuration(listItem.duration);
        }
    }
    
    onCreationCompleted:{
        localeHandler.localeChanged.connect(reevaluateDurationText);
    }
    
    opacity: (itemSelected || !multiSelectionMode) ? 1.0 : 0.7
    layout: DockLayout {
    }
    background: itemSelected ? ColorUtils.get(0xff00a8df) : ColorUtils.get(0xffe4e4e4)

    contextActions: [
        ActionSet {
            id: myActions
        }
    ]
    
    onAllowPlayOnActionChanged: {
        updatePlayOnAction();
    }
    
    function updatePlayOnAction() {
        if (listItem.allowPlayOnAction && listItem.ListItem.view.playOnActionItem != null) {
            myActions.add(listItem.ListItem.view.playOnActionItem);
        } else {
            myActions.remove(listItem.ListItem.view.playOnActionItem);
        }
    }

    contextMenuHandler: MusicContextMenuHandler {
        actionSet: myActions
            checkForValidFid: true
    }
 	
    //Image to create divider between lists of songs
    ImageView {
		id: dividerImage
        visible: ! (listItem.ListItem.indexInSection + 1 == listItem.ListItem.sectionSize)
        horizontalAlignment: HorizontalAlignment.Fill
        verticalAlignment: VerticalAlignment.Bottom
        imageSource: "asset:///images/playing/tracks_divider.amd"
    }

	//Image that wraps the item when it is activated.
    ImageView {
        id: borderImage
        horizontalAlignment: HorizontalAlignment.Fill
        verticalAlignment: VerticalAlignment.Fill
        visible: false
        imageSource: "asset:///images/common/core_item_border.amd"
    }

    ListItem.onActivationChanged: {
        if (active) {
            borderImage.visible = true;
        } else {
            borderImage.visible = false;
        }
    }

    //Update divider image when index changes (if no longer last item, need to add divider)
    ListItem.onIndexInSectionChanged: {
        dividerImage.visible = !(listItem.ListItem.indexInSection + 1 == listItem.ListItem.sectionSize)
    }
}
