import bb.cascades 1.0
import bb.cascades.multimedia 1.0
import bb.system 1.0
import "layoutconstants.js" as LayoutConstants

/**
 * This component is used as the visual for each individual song in the lists
 * It contains a track number, name, duration
 *
 * @param   title                The text to be used for the title of the song
 * @param   subTitle             The text to be used for the sub-title of the song
 * @param   track                The text for the track number
 * @param   fullPath             The full path of the song
 * @param   duration             The integer for the duration of the song
 * @param   showSubTitle         Boolean value to make the second row of text visible (default is false)
 */

ListItemParent {
    id: listItem
    
    //following properties are set from the external qml list that uses this ListItemVisual    
    property string subTitle: ""
    property string fullpath: ""
    property bool showSubTitle: false
    property bool isCurrentlyPlaying: false
    
    //property to determine whether or not to show the remote playback spinner
    property bool minimizedIndicatorVisible: false

    //keeps a reference to the current song decorator
    property variant controller

    signal removeTrack();
    signal reorderTrack();
 
    //keeps track of whether this list item is a currently playing track
    property bool currSong

    layout: DockLayout {}
    background: itemSelected ? ColorUtils.get(0xff00a8df) : LayoutConstants.LIST_ITEM_VISUAL_PLAYING_BG
    minHeight: LayoutConstants.LIST_ITEM_VISUAL_HEIGHT

    onControllerChanged: {
        controller.trackChanged.connect(handleTrackChanged);
        controller.playbackStateUpdate.connect(handlePlaybackStateChanged);
        controller.updateRemoteIndicator.connect(updateIndicator);
    }

    function handlePlaybackStateChanged(currentlyPlaying){
        isCurrentlyPlaying = currentlyPlaying;
    }

    function handleTrackChanged() {
        ListItem.view.dataModel.itemUpdated(ListItem.indexPath);
    }

    function updateIndicator (remoteActivity) {
        if (remoteActivity && listItem.currSong) {
    		//Hide the Play/Pause icon and show the activity indicator
    		minimizedIndicatorVisible = true;
		} else {
		    minimizedIndicatorVisible = false;
		}
    }
	ListItem.onInitializedChanged: {
	    if ( initialized && controller) { //ensure controller is not null
            isCurrentlyPlaying = controller.currentlyPlaying;
        }
	}

    Container {
        leftPadding: 10
        rightPadding: 20
        horizontalAlignment: HorizontalAlignment.Fill
        verticalAlignment: VerticalAlignment.Fill

        layout: StackLayout {
            orientation: LayoutOrientation.LeftToRight
        }

        ActivityIndicator {
            id: minimizedIndicator
            visible: minimizedIndicatorVisible
            horizontalAlignment: HorizontalAlignment.Center
            verticalAlignment: VerticalAlignment.Center

    	
            onVisibleChanged: {
                if (visible) {
                    minimizedIndicator.start();
                } else {
                    minimizedIndicator.stop();
                }
            }
            layoutProperties: StackLayoutProperties {
                spaceQuota: 12
            }
            rightMargin: visible ? 10 : 0
        }

        // track number
        PlayPauseTrackButton {
            id: playPauseButton
            visible: !minimizedIndicatorVisible
            isPlaying: isCurrentlyPlaying
            trackNumber: track
            currentlyPlaying: listItem.currSong
            currentlySelected: listItem.itemSelected
            textAlignment: HorizontalAlignment.Center
            horizontalAlignment: HorizontalAlignment.Center
            verticalAlignment: VerticalAlignment.Center
            textStyleBase: listItem.ListItem.view.trackNumberNormalStyle
            layoutProperties: StackLayoutProperties {
                spaceQuota: 12
            }
            rightMargin: visible ? 10 : 0
        }

        // track title
        TrackInfoLabel {
            id: labelTitleFull
            leftPadding: 0
            titleStyle: itemSelected
                ? listItem.ListItem.view.titleHighlightStyle
                : listItem.ListItem.view.titlePlayingStyle
            subtitleStyle: itemSelected
                ? listItem.ListItem.view.subtitleHighlightStyle
                : listItem.ListItem.view.subtitlePlayingStyle
            verticalAlignment: VerticalAlignment.Center
            title: listItem.title
            subTitle: listItem.subTitle
            layoutProperties: StackLayoutProperties {
                spaceQuota: 88
            }
        }
        Label {
            verticalAlignment: VerticalAlignment.Center
            horizontalAlignment: HorizontalAlignment.Right
            id: labelDurationFull
            text: listItem.durationText
            textStyle.base: itemSelected
                ? listItem.ListItem.view.durationHighlightStyle
                : listItem.ListItem.view.durationPlayingStyle
            leftMargin: 30
        }
    }

}
