import bb.cascades 1.0
import "layoutconstants.js" as LayoutConstants


/**
 * This control allows titles to be specified for the labels at the top of a list
 *
 * @param	text1	 	The leftmost text
 * @param	text2		The rightmost text
 * @param 	textColor1	Optional: The color of the leftmost text. Default is white.
 * @param 	textColor2	Optional: The color of the rightmost text. Default is white.
 * @param 	background	Optional: The background that can be set.  Default is #2a524c
 * @param   leftMarg    The margin left of the leftmost text
 * @param   rightMarg   The margin right of the rightmost text
 */

Container {
    id: listTextLabel
    objectName: "listTextLabel"
    property alias text1: label1.text
    property alias text2: label2.text
    property alias textStyle1:label1.textStyle
    property alias textStyle2:label2.textStyle
    property variant textColor1: ColorUtils.getPrimaryTextColor()
    property variant textColor2: ColorUtils.getPrimaryTextColor()
 
   
    leftPadding: 20
    rightPadding: 20
    bottomPadding: 5

    layout: StackLayout {
        orientation: LayoutOrientation.LeftToRight
    }
    background: LayoutConstants.TRACKS_LIST_TITLE_COLOR
    Label {
        id: label1
        
        textStyle{
            base: listHeaderTsd.style
            color: textColor1
        } 
 
        verticalAlignment: VerticalAlignment.Center
        textFormat: TextFormat.Plain

        layoutProperties: StackLayoutProperties {            
            spaceQuota: 100
        }
    }
    Label {
        id: label2
        
        textStyle{
            base: listHeaderTsd.style
            color: textColor2
        } 
 
        rightMargin: 20
        
        verticalAlignment: VerticalAlignment.Center
        horizontalAlignment: HorizontalAlignment.Right
        textFormat: TextFormat.Plain
    }
    
    attachedObjects: [
        TextStyleDefinition {
            id: listHeaderTsd
            base: SystemDefaults.TextStyles.BodyText
            fontSize: LayoutConstants.TRACKS_LIST_HEADER_FONT_SIZE
            fontSizeValue: LayoutConstants.TRACKS_LIST_HEADER_FONT_SIZE_VALUE
            fontWeight: FontWeight.W500
        }
    ]
}
