import bb.cascades 1.0
import bb.cascades.multimedia 1.0
import bb.cascades.multimedia.internal 1.0
import bb.cascades.music 1.0
import bb.multimedia 1.0
import bb.system 1.0
import "InvokeUtils.js" as InvokeUtils
import "layoutconstants.js" as LayoutConstants
import "ContentUris.js" as Uris

Container {
    // Globals that are accessible to all components in the tree
    property variant playbackController
    property variant dynamicPlaylistManagerRef
    
    property SystemDialog rendererUnavailableDialog: null
    // Keeps track if the player screen has been closed at some point
    // during the app lifetime
    property bool playerScreenHasBeenClosed:false
    property int windowState: -1 //-1 for uninitialized, 0 for fullscreen, 1 for thumbnailed
    property bool noContentViewVisible: false //used to determine if we should show the HD cover, set from smart.qml

	property bool showPlayOnAction: securityStatus.dlnaPlayOnEnabled
	
    //internal reference to library page for invoke (ugly)
    property variant __libraryPage
    
    property variant searchScreen

    //signal fired when the playback controller changes    
    signal newPlaybackController(variant playbackController)
    signal deactivateMultiSelect()

    //signal fired when the player screen is closed
    signal playerScreenClosed()
    
    // signal fired when a playback error occurs.
    signal playBackErrorOccured(int errorCode);

    attachedObjects: [
        ComponentDefinition{
            id: hdCoverContainerComponent
            source: "HDCover.qml"
        },
        ComponentDefinition {
            id: searchScreenDef
            source: "Search.qml"
        },
        SecurityStatus {
        	id: securityStatus 
        }
    ]
        
    onPlaybackControllerChanged: {
        dynamicPlaylistManager.playbackController = playbackController;
        newPlaybackController(playbackController);
    }

    function loadQmlObject(sourcePath, parentNav) {
        var component = Qt.createComponent(sourcePath);
        if (!component.errorString()) {
            var qmlObject = component.createObject(parentNav);
            parentNav.insert(0, qmlObject);
            return qmlObject;
        } else {
            console.error("Error loading '" + sourcePath + "':", component.errorString());
            return null;
        }
    }

    function loadApp(libNav, playlistNav, radioNav) {
        //link the playback controller componetns together
        __switchPlaybackController(null, localPlaybackController);

        //load the library page
        var libPage = loadQmlObject("./Library.qml", libNav);
        if (libPage) {
            libPage.libraryContentQMLPath = "asset:///core/LibrarySegmentedContainer.qml";
            __libraryPage = libPage;
        }

        //load the playlist page
        var playlistsPage = loadQmlObject("./Playlists.qml", playlistNav);
        if (playlistsPage) {
            playlistsPage.playlistsContentQMLPath = "asset:///core/PlaylistsContainer.qml";
        }

        // Radio is optional - make sure its nav pane has been provided.
        if (radioNav) {
            var radioPage = loadQmlObject("../radio/pages/RadioPage.qml", radioNav);
            if (radioPage) {
            	radioPage.radioContentQMLPath = "asset:///radio/components/RadioSegmentedControl.qml";
            }
        }

        //listen to tab changed events for search
		tabbedPane.tabChanged.connect(__handleTabChanged);

        //setup search on initial tab 
        __handleTabChanged();
        
        postApplicationSetup();
        onFullscreen();
        return true;   
    }

    function isLibraryAction(action){
        return __libraryPage.isLibraryInvokeAction(action);
    }

    function invokeLibrary(action) {
        tabbedPane.setActiveTabByName("library");
       __libraryPage.handleInvokeAction(action);
    }
		
    function onPlayerScreenPopped(){
    	playerScreenHasBeenClosed = true;
    	playerScreenClosed();
    }    
    
    // We receieve this event when we enter fullscreen
    function onFullscreen() {
        windowState = 0;
        Application.resetCover();
    }
    
    function onThumbnail() {
        // we will only engage the HDCover if we thumbnailed and AFTER intial launch of the app
        // since we get a thumbnailed event at first launch of the application
        if (windowState == 0) {
            windowState = 1;
            updateHDCoverInfo();
        }
    }
    
    function onInvisible() {
        localPlaybackController.setPlaybackAppFullscreen(false);
    }

    function showDeleteManagerError(msg) {
        systemDialogUtils.showSystemToast(qsTr("Unable to delete file: ") + msg);
    }
     
     function updateHDCoverInfo(){
        if ( windowState == 1) {
            //todo scott fix this
            //if (!smartPage.noContentViewVisible) {
            if (Application.cover == null) {
                if (!hdCoverContainerComponent.hasErrors()) {
                    MediaApplication.updateHDCover(hdCoverContainerComponent.createObject());
                } else {
                }
            }
            //} else {
           //     Application.resetCover();
            //}
         }
    }
    
    // Show dialog for data query related errors, e.g. database and query errors.    
    function showDataQueryErrorDialog(code, msg) {
        if ((MediaLibraryErrorType.DATABASE_ERROR == code) || (MediaLibraryErrorType.QUERY_ERROR == code)) {
            errorHandler.fatalError(code, msg);
        } else {
            errorHandler.showError(code, msg);
        }
    }

    function onSdCardSyncStarted() {
        systemDialogUtils.showMediaSyncSystemToast();
    }

    //Launch the Now Playing screen if playback has been started for the first time
    function pushPlayerScreenIfNotPlaying(playbackActive) {
        // if playback is not active, then launch the now playing screen
        if (playerStub.errorCode == MediaErrorEx.ResourceNotFound) {
            console.log("Error: ResourceNotFound PlayerScreen not pushed");
            playerStub.errorCode = MediaError.None;
        } else if (!playbackActive) {
            tabbedPane.currentNavPane.safePushOnePage("nowplaying/NewPlayerScreen.qml");
            commonAnalytics.instance.logPagePush("NewPlayerScreen", "MusicContext");
        }

    }

    //to be called from c++ after scene is created
    function postApplicationSetup() {
        console.log("PostCreationSetup called");
        
        windowState = 0;
        
        playbackController = localPlaybackController;
        //This just needs to be called once
        localPlaybackController.sleepBehaviour = SleepBehaviour.MinimizeUpdates;
        
        dynamicPlaylistManagerRef = dynamicPlaylistManager;
        dynamicPlaylistManager.dynamicPlaybackUri = Uris.dynamicPlaylistPlayback();
        dynamicPlaylistManager.playbackReady.connect(onDynamicPlaylistPlaybackReady);

        localPlaybackController.error.connect(onShowError);
        localPlaybackController.dataQueryError.connect(showDataQueryErrorDialog);
        remotePlaybackController.error.connect(onShowError);
        remotePlaybackController.dmsError.connect(onShowError);
        remotePlaybackController.rendererUnavailable.connect(onRendererUnavailable);
        remotePlaybackController.dataQueryError.connect(showDataQueryErrorDialog);
        remotePlaybackController.playOnDeviceChanged.connect(playerStub.onPlayOnDeviceChanged);

        localPlaybackController.currentMediaItemChanged.connect(updateHDCoverInfo);
        remotePlaybackController.currentMediaItemChanged.connect(updateHDCoverInfo);

        playerStub.playOnDeviceChangedSecondSignal.connect(__onPlayOnDeviceChanged);

        invokeNotifier.invoked.connect(__handleInvoke);

        deleteManager.error.connect(showDeleteManagerError);

        if (MediaApplication.isSynchronizingSdCard()) {
            systemDialogUtils.showMediaSyncSystemToast();
        }
        
        commonAnalytics.instance.logApplicationStarted(
            {"volume" : playbackController.volume,
             "audioOutputDevice" : playbackController.activeAudioDeviceName()});
    }

    function onShowError(code, msg) {
        console.debug("onShowError()");
        if (code == MediaManagerStatus.WIFI_OFF) {
            // We already handle Wifi disconnection elsewhere.
            return;
        }

        //Music app behaves differently from video while video displays error message with an OK and then shows an error container,
        //Music app shows a system toast which fades aways after few seconds while making choosing the next track to be played
        if( playbackController != remotePlaybackController &&
            ( code == MediaError.UnsupportedMediaType || 
              code == MediaError.DrmProtected || 
              code == MediaError.DrmNoRights ||
              code == MediaError.DrmExpiredLicense ) ) {
            systemDialogUtils.execSystemToast(msg);
            playbackController.handleUnsupportedMediaError();
        } else {
            errorHandler.showError(code, msg);
        }

        if (code != MediaError.ResourceUnavailable ) {
            playBackErrorOccured(code); // signals the playerScreen to hide playon controls

            if (playbackController == remotePlaybackController) {
                switchToLocalPlayback();
            }
        }
    }

    function onRendererUnavailableConfirm(value){
        console.debug("+onRendererUnavailableConfirm");
        rendererUnavailableEject();
        console.debug("-onRendererUnavailableConfirm");
    }

    function rendererUnavailableEject(){
       console.debug( "+rendererUnavailableEject()" );
       playBackErrorOccured( MediaErrorEx.ConnectionNotAvailable ); // signals the playerScreen to hide playon controls

        if (playbackController == remotePlaybackController) 
        {
            switchToLocalPlayback();
        }
        console.debug( "-rendererUnavailableEject()" );    
    }

    function showRendererUnavailableDialog(msg){        
        console.debug( "+showRendererUnavailableDialog()" );
        if (rendererUnavailableDialog == null)             {
            rendererUnavailableDialog = systemDialogUtils.getSystemDialog();
            rendererUnavailableDialog.title = qsTr("Error") + Retranslate.onLanguageChanged; 
            rendererUnavailableDialog.cancelButton.label = undefined;
            //Wait for the user to acknowledge before we switch back to local playback.
    		rendererUnavailableDialog.finished.connect(onRendererUnavailableConfirm);
        }
        
		rendererUnavailableDialog.body = msg;
		rendererUnavailableDialog.show();
		console.debug( "-showRendererUnavailableDialog()" );
    }
    
    function onRendererUnavailable( code, msg, rendererName )
    {
        console.debug( "+onRendererUnavailable()" );
        // check for sticky mode
        if ((playbackController == remotePlaybackController)
                && (playbackController.playbackState != PlaybackController.PlaybackPlaying)
                && (playbackController.playbackState != PlaybackController.PlaybackPaused))
        {
            // show toast for sticky mode
            systemDialogUtils.showSystemToast(qsTr("%1 is not available").arg(rendererName));
            rendererUnavailableEject();
        }
        else
        {
            // show an error dialog if we are not in sticky mode
            showRendererUnavailableDialog(msg);    		            
        }
        console.debug( "-onRendererUnavailable()" ); 
    }     
    
    /**
    * Switch from the remote DLNA playback to the local device based playback.
    */
    function switchToLocalPlayback()
    {
        console.debug("+switchToLocalPlayback()");

        commonAnalytics.instance.logPlayOnStopped(remotePlaybackController.position);

        remotePlaybackController.playOnDevice = false;
        
        console.debug("-switchToLocalPlayback()");
    }
        
    function switchToRemotePlayback()
    {
        if (!MediaApplication.corporate) {                    
            var page = tabbedPane.currentNavPane.safePushOnePage ("core/PlayOnListPageWrapper.qml")
            commonAnalytics.instance.logPagePush("PlayOnListPageWrapper", "MusicContext");
            page.navControl = tabbedPane.currentNavPane
            page.loadData(ExternalRenderer.DLNA)
            page.rendererSelected.connect(__rendererSelected);
        
        } else {
            systemDialogUtils.showSystemToast(qsTr("Play on functionality has been disabled for corporate mode."));
        }
    }
    
    function __handleTabChanged(){
        if (tabbedPane.currentNavPane.top.searchScreenContainer != null) {
            addSearchScreenToPage(tabbedPane.currentNavPane.top);
        }
	}
    
    //Add "Search component" to a page.
    //A page must define a variable, called searchScreenContainer, that points to a container
    //where the Search screen will be added.
    function addSearchScreenToPage(page) {
        if (null != page && page.searchScreenContainer != null) {
            if (searchScreen == null) {
                searchScreen = searchScreenDef.createObject();
            }
            if (page.searchScreenContainer.indexOf(searchScreen) < 0) { // only added if it hasn't been added
                page.searchScreenContainer.add(searchScreen);
                searchScreen.rootPage = page;
                searchScreen.hide();
            }
        }
    }    

    function __handleInvoke(request){
        deactivateMultiSelect();
        InvokeUtils.handleInvoke(request,invokeDataModel,dynamicPlaylistManager,playerStub.playbackActive);	    
    }
    
    function __rendererSelected(renderer) {
        console.log("rendererSelected", "->");
        
        // Stop radio playback before starting DLNA
        if(RadioPlaybackContext.fmRadioSupported) {
            RadioPlaybackContext.stopPlayback();            
        }
        
        //Going to do DLNA but we need to figure out our current position so we can seek to it via DLNA
     	console.log("rendererSelected", "Playing remotely to dlna: " + renderer["name"]);
        
        commonAnalytics.instance.logPlayOnStarted(renderer["type"],  renderer["name"], renderer["id"], 
                                                  renderer["devicetypeinfo"], renderer["macaddress"], 
                                                  renderer["version"], renderer["manufacturer"], localPlaybackController.position);
     	
      	remotePlaybackController.playOnDeviceName = renderer["name"];  
    	remotePlaybackController.playOnDeviceId = renderer["id"];
    	
    	console.log("rendererSelected", "About to set dlnaOutput uuid to: " + renderer["id"]);
        dlnaPlaybackHandler.uuid = renderer["id"];

        console.log("rendererSelected()", "playOnDevice flag is: " + remotePlaybackController.playOnDevice);
        remotePlaybackController.playOnDevice = true;  // remotePlaybackController.playOnDevice doesn't actually become true if an error happens. onPlayOnDeviceChanged() is only called on success.

        console.log("----------------------------------------------------------------------------------------------------------------");
        console.log("rendererSelected", "rendererSelected <-");
    }   

    function __onPlayOnDeviceChanged(enabled) {    	
        console.log("MusicContext.qml: +__onPlayOnDeviceChanged()");
        if (enabled) {
            __switchPlaybackController(localPlaybackController, remotePlaybackController);
	    } else {
            __switchPlaybackController(remotePlaybackController, localPlaybackController);
        }
        console.log("MusicContext.qml: -__onPlayOnDeviceChanged()");
	}

    function __switchPlaybackController(oldPlaybackController, newPlaybackController)
    {
        console.log("+switchPlaybackController()");
        
        if (playbackController != newPlaybackController) {
	        timeSliderController.playbackControllerRef = newPlaybackController;
	        timeSliderController.seekInterval = 100;  // Should this be larger for remote playback?
	       
            // Make sure that the playlist is copied over whenever the playback controller changes,
            // so the playlist is never empty.
            playbackController = newPlaybackController;            
            
            if(oldPlaybackController != null){
	            __copyPlayList(oldPlaybackController, newPlaybackController);
	            
	            if (oldPlaybackController == localPlaybackController) {
	                console.log("__switchPlaybackController: The old playback state is: " + oldPlaybackController.playbackState);
	                console.log("__switchPlaybackController: Pausing the old controller!");
	                oldPlaybackController.pause();
	            }
	            oldPlaybackController.reset();
	        }
        }

        console.log("-switchPlaybackController()");
    }
       
    function __copyPlayList(oldPlaybackController, newPlaybackController)
    {
        console.log("+__copyPlayList");
        console.log("__copyPlayList: The current contentURI is: " + oldPlaybackController.currentContentUri);
        
        var contentUri 			= oldPlaybackController.currentContentUri;
        var currentIndex 		= oldPlaybackController.currentIndex;
        var currentPosition 	= oldPlaybackController.position;
        var currentShuffleMode	= oldPlaybackController.shuffleMode;
        var startState;

        if (oldPlaybackController == remotePlaybackController) {
            if (oldPlaybackController.playbackState == PlaybackController.PlaybackCompleted) {
                startState = PlaybackController.PlaybackPrepared;
            } else {
                startState = PlaybackController.PlaybackPaused;
            }
        } else {
            startState = PlaybackController.PlaybackPlaying;
        }
         

        if (contentUri.length && currentIndex.length) {
            console.log("qml to call setPlaylistUsingController() with start state " + startState);
            newPlaybackController.setPlaylistUsingController( oldPlaybackController, startState );
        }
        
        console.log("-__copyPlayList");
    }
    // Add the specified type to the NowPlaying dynamic playlist.
    function addToNowPlaying(type, id, config, last)
    {
        console.log("DPL: addToNowPlaying() called type: " + type + " id: " + id + " config: " + config + " last: " + last);
        if (last) {
            dynamicPlaylistManager.appendItem(type, id, config);
        } else {
            dynamicPlaylistManager.insertItemNext(type, id, config);
        }
    }

    function onDynamicPlaylistPlaybackReady() {
        pushPlayerScreenIfNotPlaying(playerStub.playbackActive);
    }

    function getPlatformActionItems(filePath, data, metadata) {
        return invokeHelper.getInvokeActionItemForFile("", "", filePath, mimeAssociation.getMimeTypeFromFilePath(filePath), data, metadata);
    }

    function addAppWorldActionItem(page) {
        // BB World - Work has no music to purchase, don't display the menu item
        if (!MediaApplication.corporate) {
	        invokeHelper.addGetMoreActionIfAvailable(
	                     page, 
	                     qsTr("BlackBerry World") + Retranslate.onLanguageChanged, 
	                     true, 
	                     "asset:///images/common/ic_app_world.png");
         }
    }

    // launch BlackBerry World - kept name for consistancy
    function launchAppWorld(artist, album) {
        console.debug("launchAppWorld(" + artist + ", " + album + ")");
        //The API to BBW will probably change to this
        //	searchFor = "exact=true&artist=";
        //	searchFor += artist;
        //	searchFor += "&album=";
        //	searchFor += album;
        var searchFor = artist;
        if (album != null) {
            searchFor += " " + album;
        }
        invokeHelper.invokeTargetApp("bb.action.SEARCH.EXTENDED", "sys.appworld.search", "", "application/vnd.bb.search.criteria", searchFor);
    }

    function launchProperties(fid) {
        var propsPage = tabbedPane.currentNavPane.safePushOnePage("core/PropertiesWrapper.qml");
        propsPage.propFid = fid;
        propsPage.forceFileNameReadOnly = true;
        commonAnalytics.instance.logPagePush("PropertiesWrapper", "MusicContext");
    }

    function getDeleteSystemDialog(type, count) {
		var deleteDialog = systemDialogUtils.getSystemDialog();
		
		deleteDialog.title = qsTr("Delete", "", count) + Retranslate.onLanguageChanged; //USED BY STABILITY TESTING)
		deleteDialog.confirmButton.label = qsTr("Delete") + Retranslate.onLanguageChanged;
		deleteDialog.cancelButton.label = qsTr("Cancel") + Retranslate.onLanguageChanged;
		
		switch(type) {
			case MediaLibraryContentType.AlbumType:
				// Singular form is "Are you sure that you want to permanently delete this album?"
				// Plural form is "Are you sure that you want to permanently delete these %n albums?"
				deleteDialog.body = qsTr("Are you sure that you want to permanently delete these %n albums?", "", count) + Retranslate.onLanguageChanged;
				break;
				
			case MediaLibraryContentType.AlbumArtistType:
				deleteDialog.body = qsTr("Are you sure that you want to permanently delete these %n artists?", "", count) + Retranslate.onLanguageChanged;
				break;
				
			case MediaLibraryContentType.GenreType:
				deleteDialog.body = qsTr("Are you sure that you want to permanently delete these %n genres?", "", count) + Retranslate.onLanguageChanged;
				break;
				
			case MediaLibraryContentType.PlaylistType:
				deleteDialog.body = qsTr("Are you sure that you want to permanently delete these %n playlists?", "", count) + Retranslate.onLanguageChanged;
				break;
				
			case MediaLibraryContentType.AudioType:
				deleteDialog.body = qsTr("Are you sure that you want to permanently delete these %n songs?", "", count) + Retranslate.onLanguageChanged;
				break;
				
			default:
				deleteDialog.body = qsTr("Are you sure that you want to permanently delete these %n items?", "", count) + Retranslate.onLanguageChanged;
				break;
		}
        return deleteDialog;
    }
}
