import bb.cascades 1.0
import bb.cascades.multimedia 1.0

ContextMenuHandler {
    
    property ActionSet actionSet
    property bool checkForValidFid: false
    
    onPopulating: {
        /**
         * When the context menu is showing, we want to add the specific action items
         * for our type.  The actions themselves are created in the parent ListView
         * so that we can reuse them and not create additional objects.
         * Note: We don't remove the actions from our set becausse calling remove on an
         * action also destroys it so it can't be used again. 
         */
        
		// Abort the showing of the context menu if the user long-pressed a deleted song
		if (checkForValidFid && (parent.ListItem.data.fid == null || parent.ListItem.data.fid == "")) {
            event.abort();
            return;
        }
        
        // add the PlayNextAction item to our action set if it wasn't already added
        if ( parent.allowPlayNextAction
             && parent.ListItem.view.playNextActionItem != null
             && actionSet.indexOf(parent.ListItem.view.playNextActionItem) < 0 
             && playerStub.playbackActive ) {
            
            actionSet.add( parent.ListItem.view.playNextActionItem );
        }
        
        // add the Add To NowPlaying action item to our action set if it wasn't already added
        if ( parent.allowPlayLastAction
             && parent.ListItem.view.playLastActionItem != null
             && actionSet.indexOf(parent.ListItem.view.playLastActionItem) < 0 
             && playerStub.playbackActive) {
            // find location of "Play Next" and insert this item below it.
            var index = actionSet.indexOf(parent.ListItem.view.playNextActionItem);
            if (index >= 0) {
                actionSet.insert( index + 1,  parent.ListItem.view.playLastActionItem );
            } else {
                actionSet.add( parent.ListItem.view.playLastActionItem );
            }
        }
        
        // add the Add To Playlist action item to our action set if it wasn't already added
        if ( parent.allowAddToPlaylistAction
             && parent.ListItem.view.addToPlaylistActionItem != null
             && actionSet.indexOf(parent.ListItem.view.addToPlaylistActionItem) < 0 ) {
            
            actionSet.add( parent.ListItem.view.addToPlaylistActionItem );
        }
        
        // add the Play On action item to our action set if it wasn't already added
        if ( parent.allowPlayOnAction
             && parent.ListItem.view.playOnActionItem != null
             && actionSet.indexOf(parent.ListItem.view.playOnActionItem) < 0 ) {
            
            /** The index where the action will be placed.
             * If index < 0 the action is inserted as the first item. (this why use -1 in here)
             * If index > the number of items in the ActionSet it is added as the last item.
             * can not guarantee to insert the action item to the first place if set index to 0, 
             * and add this call to the very beginning, can not guarantee to insert the action item to the first place either.
             */  
            actionSet.insert(-1, parent.ListItem.view.playOnActionItem );
            
            // set playon enabled if necessary
            parent.ListItem.view.playOnActionItem.enabled = !appContext.playbackController.playOnDevice;
        }

        // add the Properties action item to our action set if it wasn't already added
        if (parent.allowPropertiesAction 
            && parent.ListItem.view.propertiesActionItem != null 
            && actionSet.indexOf(parent.ListItem.view.propertiesActionItem) < 0) {

            actionSet.add(parent.ListItem.view.propertiesActionItem);
            
            actionSet.title = parent.ListItem.data.filename;
            //: Date modified
            actionSet.subtitle = qsTr("Modified: %1").arg(localeUtils.formatDateMedium(parent.ListItem.data.date_modified));
        }

        // add the Select More action item to our action set if it wasn't already added
        if ( parent.allowMultiSelectAction
             && parent.ListItem.view.multiSelectActionItem != null
             && actionSet.indexOf(parent.ListItem.view.multiSelectActionItem) < 0 ) {
            
            actionSet.add( parent.ListItem.view.multiSelectActionItem );
        }
        
        // add the Delete Action action item to our action set if it wasn't already added
        if ( parent.allowDeleteAction
             && parent.ListItem.view.deleteActionItem != null
             && actionSet.indexOf(parent.ListItem.view.deleteActionItem) < 0 ) {

            actionSet.add( parent.ListItem.view.deleteActionItem );
        }

        // add the context sensative search for AppWorld action item to our action set if it wasn't already added
        // Note: it need to appear here so the position in the menu is the same as the global on
        // which is created below. 
        if (parent.allowSearchAppWorld 
            && parent.ListItem.view.searchAppWorld != null 
            && actionSet.indexOf(parent.ListItem.view.searchAppWorld) < 0
            && invokeHelper.isMusicAppWorldAvailable()) {

            actionSet.add(parent.ListItem.view.searchAppWorld);
        }

        if ( parent.allowInvokeAction) {
	        // If the invoke action item is not null and the current action set tied to the invoke action
	        // set is not null, then remove it to create a new one 
	        if ( parent.ListItem.view.invokeAction != null
	             && parent.ListItem.view.currentInvokeActionSet != null
	             && parent.ListItem.view.currentInvokeActionSet.indexOf(parent.ListItem.view.invokeAction) >= 0) {
	            
	            // this will destroy the action item
	            parent.ListItem.view.currentInvokeActionSet.remove(parent.ListItem.view.invokeAction)
	        }

            var thumb  = invokeHelper.getThumbFileForSingleInvoke( MediaLibraryContentType.AudioType, parent.ListItem.data.key_id );
	        
	        // create the invoke action item specific to this list item
	        var metadata;
            if (thumb) {
                metadata = {"thumb": thumb};
            }    
	        parent.ListItem.view.invokeAction = appContext.getPlatformActionItems(parent.ListItem.data.fullpath, "", metadata);

	        // add the invoke action item to our action set
	        actionSet.add( parent.ListItem.view.invokeAction );
	        
	        // set our action set as the current action set tied to the invoke action
	        parent.ListItem.view.currentInvokeActionSet = actionSet;
   
	    }
	    
	    // If the action set is empty don't show the menu at all
	    if (actionSet.count() < 1) {
            event.abort();
            return;
        }
        
        //For analytics : connect to "onTriggered" signal every time context menu is created/shown
        connectToActions();
    }
    
    onVisualStateChanged : {
    	//For analytics : disconnect from "onTriggered" signal once context menu is dismissed
	    if(ActionMenuVisualState.Hidden == visualState){
	    	disconnectFromActions();
		}
    }
    
    function connectToActions(){
    	if(parent.ListItem.view.playNextActionItem != null){
    		parent.ListItem.view.playNextActionItem.triggered.connect(onPlayNextActionItemTriggered);
    	}
    	
		if(parent.ListItem.view.playLastActionItem != null){
			parent.ListItem.view.playLastActionItem.triggered.connect(onPlayLastActionItemTriggered);
		}
		
		if(parent.ListItem.view.addToPlaylistActionItem != null){
			parent.ListItem.view.addToPlaylistActionItem.triggered.connect(onAddToPlaylistActionItemTriggered);
		}
		
		if(parent.ListItem.view.playOnActionItem != null){
			parent.ListItem.view.playOnActionItem.triggered.connect(onPlayOnActionItemTriggered);
		}
		
		if(parent.ListItem.view.propertiesActionItem != null){
			parent.ListItem.view.propertiesActionItem.triggered.connect(onPropertiesActionItemTriggered);
		}
		
		if(parent.ListItem.view.multiSelectActionItem != null){
			parent.ListItem.view.multiSelectActionItem.triggered.connect(onMultiSelectActionItemTriggered);
		}
		
		if(parent.ListItem.view.deleteActionItem != null){
			parent.ListItem.view.deleteActionItem.triggered.connect(onDeleteActionItemTriggered);
		}
		
		if(parent.ListItem.view.searchAppWorld != null){
			parent.ListItem.view.searchAppWorld.triggered.connect(onSearchAppWorldTriggered);
		}
		
		if(parent.ListItem.view.invokeAction != null){
			parent.ListItem.view.invokeAction.triggered.connect(onInvokeActionItemTriggered);
		}
    }
    
    function disconnectFromActions(){
    	if(parent.ListItem.view.playNextActionItem != null){
    		parent.ListItem.view.playNextActionItem.triggered.disconnect(onPlayNextActionItemTriggered);
    	}
    	
		if(parent.ListItem.view.playLastActionItem != null){
			parent.ListItem.view.playLastActionItem.triggered.disconnect(onPlayLastActionItemTriggered);
		}
		
		if(parent.ListItem.view.addToPlaylistActionItem != null){
			parent.ListItem.view.addToPlaylistActionItem.triggered.disconnect(onAddToPlaylistActionItemTriggered);
		}
		
		if(parent.ListItem.view.playOnActionItem != null){
			parent.ListItem.view.playOnActionItem.triggered.disconnect(onPlayOnActionItemTriggered);
		}
		
		if(parent.ListItem.view.propertiesActionItem != null){
			parent.ListItem.view.propertiesActionItem.triggered.disconnect(onPropertiesActionItemTriggered);
		}
		
		if(parent.ListItem.view.multiSelectActionItem != null){
			parent.ListItem.view.multiSelectActionItem.triggered.disconnect(onMultiSelectActionItemTriggered);
		}
		
		if(parent.ListItem.view.deleteActionItem != null){
			parent.ListItem.view.deleteActionItem.triggered.disconnect(onDeleteActionItemTriggered);
		}
		
		if(parent.ListItem.view.searchAppWorld != null){
			parent.ListItem.view.searchAppWorld.triggered.disconnect(onSearchAppWorldTriggered);
		}
		
		if(parent.ListItem.view.invokeAction != null){
			parent.ListItem.view.invokeAction.triggered.disconnect(onInvokeActionItemTriggered);
		}
    }
    
    function onPlayNextActionItemTriggered(){
    	commonAnalytics.instance.logContextMenuInvoked("PlayNextActionItem");
    }
    
    function onPlayLastActionItemTriggered(){
    	commonAnalytics.instance.logContextMenuInvoked("PlayLastActionItem");
    }
    
    function onAddToPlaylistActionItemTriggered(){
    	commonAnalytics.instance.logContextMenuInvoked("AddToPlaylistActionItem");
    }
    
    function onPlayOnActionItemTriggered(){
    	commonAnalytics.instance.logContextMenuInvoked("PlayOnActionItem");
    }
    
    function onPropertiesActionItemTriggered(){
    	commonAnalytics.instance.logContextMenuInvoked("PropertiesActionItem");
    }
    
    function onMultiSelectActionItemTriggered(){
    	commonAnalytics.instance.logContextMenuInvoked("MultiSelectActionItem");
    }
    
    function onDeleteActionItemTriggered(){
    	commonAnalytics.instance.logContextMenuInvoked("DeleteActionItem");
    }
    
    function onSearchAppWorldTriggered(){
    	commonAnalytics.instance.logContextMenuInvoked("SearchAppWorld");
    }
    
    function onInvokeActionItemTriggered(){
    	commonAnalytics.instance.logContextMenuInvoked("InvokeActionItem");
    }
}