import bb.cascades 1.0
import bb.cascades.multimedia 1.0
import bb.cascades.multimedia.internal 1.0
import bb.cascades.music 1.0
import bb.system 1.0
import "layoutconstants.js" as LayoutConstants

/**
 * A wrapper on ListView to expose top level components to ListItemComponent scope and
 * to add some functionality needed by the screens in the music app
 */

ListView {
    id: musicListView

    property variant listTabbedPane: tabbedPane
    property alias trackNumberNormalStyle: trackNumberNormal.style
    property alias titleNormalStyle: titleNormal.style
    property alias titleHighlightStyle: titleHighlight.style
    property alias titlePlayingStyle: titlePlaying.style
    property alias subtitleNormalStyle: subtitleNormal.style
    property alias subtitleHighlightStyle: subtitleHighlight.style
    property alias subtitlePlayingStyle: subtitlePlaying.style
    property alias durationNormalStyle: durationNormal.style
    property alias durationHighlightStyle: durationHighlight.style
    property alias durationPlayingStyle: durationPlaying.style
    property variant selectedIndexPath     
    property variant activeFid: 0
    property variant activeKeyId: 0
    property variant activePlid: 0
    property variant activeFullpath
    property variant activeArtist
    property variant activeAlbum
    property variant contentType: MediaLibraryContentType.AudioType
    property SystemDialog deleteDialog
    
    // A reference used to disconnect signals once DataModel is reset
    property variant prevDataModel
    
    // Internal variable used to perform custom scrolling
    property variant scrollToIndexPath
    property bool scrolling: false
    
    // Common invoke action item
    property InvokeActionItem invokeAction
    // Action set currently tied to the invoke action item
    property ActionSet currentInvokeActionSet
    
    scrollRole: ScrollRole.Main

    property DeleteActionItem deleteActionItem: DeleteActionItem {
        objectName: "MusicListViewDelete"
        onTriggered: {
            deleteItems();
        }
    }
    property ActionItem addToPlaylistActionItem: ActionItem {
        objectName: "MusicListViewAddToPlaylist"
        title: qsTr("Add to Playlist") + Retranslate.onLanguageChanged
        imageSource: "asset:///images/playlists/ic_add_to_playlist.png"
        onTriggered: {
            addToPlaylist();
        }
    }
    
    property ActionItem playNextActionItem: ActionItem {
        objectName: "MusicListViewPlayNext"
        title: qsTr("Play Next") + Retranslate.onLanguageChanged
        imageSource: "asset:///images/common/ic_add_to_play_next.png"
        onTriggered: {
            addToNowPlaying(false);
        }
    }
    
    property ActionItem playLastActionItem: ActionItem {
        objectName: "MusicListViewPlayLast"
        title: qsTr("Play Last") + Retranslate.onLanguageChanged
        imageSource: "asset:///images/common/ic_add_to_play_last.png"
        onTriggered: {
            addToNowPlaying(true);
        }
    }
    
    property ActionItem playOnActionItem: ActionItem {
        objectName: "MusicListViewPlayOn"
        title: qsTr ("Play on") + Retranslate.onLanguageChanged
        imageSource: "asset:///images/playing/ic_play_on.png"
        onTriggered: {
            playOnSelected();
        }
    }
    property ActionItem searchAppWorld: ActionItem {
        objectName: "MusicListViewAppWorld"
        title: qsTr("BlackBerry World") + Retranslate.onLanguageChanged
        imageSource: "asset:///images/common/ic_app_world.png"
        onTriggered: {
            searchAppWorldSelected(activeArtist, activeAlbum);
        }
    }

    property ActionItem propertiesActionItem: ActionItem {
        objectName: "MusicListViewProperties"
        title: qsTr("Properties") + Retranslate.onLanguageChanged
        imageSource: "asset:///images/playing/ic_properties.png"
        onTriggered: {
            appContext.launchProperties(activeFid);
        }
    }

    onActivationChanged: {
        if (active) {
            selectedIndexPath = indexPath;
            var activeData = dataModel.data(indexPath);
            var activeItemType = dataModel.itemType(indexPath);
            activeFid = activeData["fid"];
            activeKeyId = activeData["key_id"];
            activePlid = activeData["plid"];
            activeFullpath = activeData["fullpath"];
            activeArtist = activeData["album_artist"];
            if (activeArtist == null) {
                activeArtist = activeData["artist"];
            }
            activeAlbum = activeData["album"];
            switch (activeItemType) {
                case "playlist":
                    contentType = MediaLibraryContentType.PlaylistType;
                    break;
                case "track":
                    contentType = MediaLibraryContentType.AudioType;
                    break;
                case "album":
                    contentType = MediaLibraryContentType.AlbumType;
                    break;
                case "artist":
                    contentType = MediaLibraryContentType.AlbumArtistType;
            }
        }
    }
    
    onDataModelChanged: {
        if (prevDataModel != null) {
            prevDataModel.itemUpdated.disconnect(onItemUpdated);
            prevDataModel.itemsChanged.disconnect(onItemsChanged);
        }
        if (dataModel != null) {
            dataModel.itemUpdated.connect(onItemUpdated);
            dataModel.itemsChanged.connect(onItemsChanged);
        }
        prevDataModel = dataModel;
    }
    
    // callback from when an item is long pressed for delete
    function deleteItems() {
        console.debug("[Delete Item] Begin MusicListView::DeleteActionItem::onTriggered with contentType: " + contentType);
        deleteDialog = appContext.getDeleteSystemDialog(contentType, 1);
        console.debug("[Delete Item] Connecting deleteDialog::finished to MusicListView::dialogCallback");
        deleteDialog.finished.connect(musicListView.dialogCallback);
        console.debug("[Delete Item] About to show deleteDialog");
        deleteDialog.show();
        console.debug("[Delete Item] Leaving MusicListView::DeleteActionItem::onTriggered");
    }
    
    function dialogCallback(result) {
        console.debug("[Delete Item] Begin MusicListView::dialogCallback with result: " + result + ", activeKeyId: " + activeKeyId + ", contentType: " + contentType);
        if (deleteDialog.result == SystemUiResult.ConfirmButtonSelection) {
            console.debug("[Delete Item] Recognized confirm pressed");
            if (activeKeyId) {
                console.debug("[Delete Item] About to call deleteManager::deleteContentByIds");
                deleteManager.deleteContentByIds(contentType, activeKeyId);
            }
        }
        console.debug("[Delete Item] Leaving MusicListView::dialogCallback");
    }
    
    // function to pop up the select playlist screen to add a track to a playlist
    function addToPlaylist() {
        if (activeFid != null || activeFid != "") { //push Select Playlist
            var page = currentNavPane.safePushOnePage("core/SelectPlaylist.qml");
            commonAnalytics.instance.logPagePush("SelectPlaylist", "MusicLisView");
            page.contentType = MediaLibraryContentType.AudioType;
            page.idsToAdd = [activeFid];
        }
    }
    
    // function to add the select item to the Now Playing list
    function addToNowPlaying(last) {
        if (activeKeyId != null && activeKeyId != "") {
            var config = {};
            if (activePlid != null && activePlid != "") {
                config.plid = activePlid;
            }
            appContext.addToNowPlaying(contentType, [activeKeyId], config, last);
        }
    }

    // callback from when an item is long pressed for Play On
    function playOnSelected() {
        if(selectedIndexPath != -1) {
            if (!dataModel.hasChildren(selectedIndexPath)) {
                var selectedItem = dataModel.data(selectedIndexPath);
                if (!appContext.playbackController.playOnDevice) { // called from overflow menu
                    playerStub.playOnDeviceChangedSecondSignal.connect(onPlayOnDeviceChanged);
                    appContext.playBackErrorOccured.connect(disconnectPlayOnSignals);
		            appContext.switchToRemotePlayback();
		        } else {
		       		triggered(selectedIndexPath); // simulate a select on the item.
		        }
            }
        }
    }
  
   function onPlayOnDeviceChanged(enabled) {
        console.log("+MusicListView::onPlayOnDeviceChanged");
        disconnectPlayOnSignals();
        if (enabled) {
		    console.log("onPlayOnDeviceChanged: Triggered: " + selectedIndexPath);
            triggered(selectedIndexPath);
        }
        console.log("-MusicListView::onPlayOnDeviceChanged");
        }

    function searchAppWorldSelected(artist,album) {
        
        console.log("+MusicListView::searchAppWorldSelected(" + artist + ", " + album + ")");
        if (artist != null || artist != "") { //only launch if the artist is valid
            appContext.launchAppWorld(artist, album);
        }
        console.log("-MusicListView::searchAppWorldSelected");
    }

    function disconnectPlayOnSignals() {
        playerStub.playOnDeviceChangedSecondSignal.disconnect(onPlayOnDeviceChanged);
        appContext.playBackErrorOccured.disconnect(disconnectPlayOnSignals);        
    }
    
    /**
     * The method check if the index path points to a valid item in the DataModel.
     *
     * @param indexPath The index path.
     */
    function validItemIndexPath(indexPath) {
        if (dataModel != null) {
            var len = indexPath.length;
            if (len <= 0) {
                return false;
            }
            var path = indexPath.slice(0, len-1);
            if (dataModel.hasChildren(path) && (dataModel.childCount(path) > indexPath[len-1])) {
                return true;
            }
        }
        return false;
    }
    
    /**
     * A scrollToItem implementation for AsyncDataModel. AsyncDataModel has a cache window on the
     * data set. Calling scrollToItem to an out of range index on the ListView of the AsyncDataModel
     * could impose an unpleasant user experience that blank screen is displayed before the AsyncDataModel
     * move the cache window to the target index range.
     *
     * To avoid the issue, scrollToAsyncModelItem determines if the target index is out-of-range. If so,
     * it tries to move the cache window to the target range and then performs the scrollToItem after that.
     * Please also note that, ScrollAnimation is set to None to avoid blank screen.
     *
     * @param indexPath The index path
     */
    function scrollToAsyncModelItem(indexPath) {
        if (dataModel != null) {
            if (!validItemIndexPath(indexPath)) {
                console.debug("MusicListView::scrollToAsyncModelItem: unable to scroll to invalid indexPath");
                return;
            }
            var item = dataModel.data(indexPath);
            if (item != null) {
                scrolling = false;
                musicListView.scrollToItem(indexPath, ScrollAnimation.None);
            } else {
                scrolling = true;
                scrollToIndexPath = indexPath;
            }
        }
    }
    
    function onItemUpdated() {
        if (scrolling) {
            scrolling = false;
            musicListView.scrollToItem(scrollToIndexPath, ScrollAnimation.None);
        }
    }
    
    function onItemsChanged() {
        if (scrolling) {
            scrolling = false;
        }
    }
    
    attachedObjects: [
        TextStyleDefinition {
            // The track numbers, by default.
            id: trackNumberNormal
            base: SystemDefaults.TextStyles.SubtitleText
            color: ColorUtils.get(0xff444444)
        },
        TextStyleDefinition {
            /*
             * The track names, by default.
             * Should be "Primary R"
             */
            id: titleNormal
            base: FontUtils.PrimaryR
            color: ColorUtils.get(0xff212223)
        },
        TextStyleDefinition {
            /*
             * The track names, when highlighted
             * Should be "Primary R"
             */
            id: titleHighlight
            base: FontUtils.PrimaryR
            color: ColorUtils.getPrimaryTextColor()
        },
        TextStyleDefinition {
            /*
             * The track names, when playing
             * Should be "Primary R"
             */
            id: titlePlaying
            base: FontUtils.PrimaryR
            color: LayoutConstants.LIST_ITEM_VISUAL_PLAYING_TEXT
        },
        TextStyleDefinition {
            /*
             * The text under a track name, by default.
             * e.g. the Artist when looking at a Playlist
             * Should be "P3R"
             */
            id: subtitleNormal
            base: FontUtils.P3R
            color: ColorUtils.get(0xff606060)
        },
        TextStyleDefinition {
            /*
             * The text under a track name, when highlighted
             * e.g. the Artist when looking at a Playlist
             * Should be "P3R"
             */
            id: subtitleHighlight
            base: FontUtils.P3R
            color: ColorUtils.getPrimaryTextColor()
        },
        TextStyleDefinition {
            /*
             * The text under a track name, when playing
             * e.g. the Artist when looking at a Playlist
             * Should be "P3R"
             */
            id: subtitlePlaying
            base: FontUtils.P3R
            color: LayoutConstants.LIST_ITEM_VISUAL_PLAYING_TEXT
        },
        TextStyleDefinition {
            /*
             * The track lengths, by default
             * Should be "Primary R"
             */
            id: durationNormal
            base: FontUtils.PrimaryR
            color: ColorUtils.get(0xff444444)
        },
        TextStyleDefinition {
            id: durationHighlight
            base: FontUtils.PrimaryR
            color: ColorUtils.getPrimaryTextColor()
        },
        TextStyleDefinition {
            id: durationPlaying
            base: FontUtils.PrimaryR
            color: LayoutConstants.LIST_ITEM_VISUAL_PLAYING_TEXT
        }
    ]
}
