/**
 * PlUtils.js
 * 
 * Research In Motion Limited proprietary and confidential.
 * Copyright (C) Research In Motion Limited, 2012.
 */

/**
 * @brief Generate a human readable text for the corresponding error code when
 * saving a playlist. 
 * 
 * @param errorCode The error code in the persistCompleted signal from the PlaylistDataModel.
 * @return A human readable text for the error code.
 */
function getErrorText(errorCode) {
	var errorStr = "";
	switch ( errorCode ) {
		case PlaylistEditErrorType.PLERROR_CONFLICTNAME: //file already exists.
			errorStr = qsTr("File exists. Please type a new name.") + Retranslate.onLanguageChanged;
			break;
		case PlaylistEditErrorType.PLERROR_INVALIDNAME: //filename not set
			errorStr = qsTr("Please enter a valid playlist name.") + Retranslate.onLanguageChanged;
			break;	
		case PlaylistEditErrorType.PLERROR_LOWMEMORY:
			errorStr = qsTr("Device internal memory is low. Please free up memory before saving.") + Retranslate.onLanguageChanged;
			break;
		case PlaylistEditErrorType.PLERROR_SAVE: //unable to overwrite existing playlist
		default:
			errorStr = qsTr("An error occurred saving the playlist.") + Retranslate.onLanguageChanged;
			break;  
	}
	return errorStr;
}

/**
 * @brief Get the file name of the playlist_entry_text.
 * 
 * @param entryTextStr The playlist_entry_text from the database.
 * @return The file name.
 */
function getFileName(entryTextStr){
    var entryTextArr = entryTextStr.split("/");
    return entryTextArr[entryTextArr.length - 1];
}

/**
 * 
 */
function getPlaylistTrackTitle(fid, title, filename) {
	if ( fid != "" ){
		return title;
	} else {
		return getFileName(filename);
	}
}

/**
 * @brief Get the file missing information for the playlist_entry_text.
 * 
 * This method determines the file location of the text entry and generates
 * the corresponding file missing info. 
 * 
 * @param entryTextStr The playlist_entry_text from the database.
 * @return The file missing information.
 */
function fileMissingInfo(entryTextStr){
	var entryTextArr = entryTextStr.split("/");
	if (entryTextArr.indexOf("sdcard") != -1){
		// "/accounts/1000/removable/sdcard/music/54-40"
		return qsTr("SDCard file missing") + Retranslate.onLanguageChanged;
	}
	else {
		// "/accounts/1000/shared/music"
		return qsTr("Local file missing") + Retranslate.onLanguageChanged;
	}
}
