import bb.cascades 1.0
import bb.cascades.multimedia 1.0
import bb.cascades.multimedia.internal 1.0

/**
 * This control switches between a track number and Play/Pause button depending on 
 * @param currentlyPlaying Represents the current playback state
 * @param trackNumber Represents the current track number
 * @signal playPausePressed() emitted when Play/Pause button is pressed
 */
Container {
    id: playPauseTrack
    property bool currentlyPlaying: false
    property alias isPlaying:playPause.isPlaying
    property alias trackNumber:labelTrack.text
    property bool currentlySelected: false
    property alias textAlignment: labelTrack.horizontalAlignment
    property variant textStyleBase

    signal playPausePressed()
    layout: DockLayout {}
    Label {
	    id: labelTrack
	    textStyle.base: playPauseTrack.textStyleBase
        textStyle.color: currentlySelected ? ColorUtils.getPrimaryTextColor() : ColorUtils.get(0xff444444)
    }
    MultiStateImageView {
        id: playPause
        property bool isPlaying: false
        opacity: ( ( !appContext.playbackController.pausable && isPlaying ) ? 0.4 : 1.0 )
        
        visible: false
        upImages: 
            ["asset:///images/playing/ic_playinlist.png",       // playing, not selected
             "asset:///images/playing/ic_playinlist_pause.png", // pause, not selected
             "asset:///images/playing/ic_play.png",             // playing, selected
             "asset:///images/playing/ic_pause.png"]            // pause, selected
        scalingMethod: ScalingMethod.AspectFill
        
        onIsPlayingChanged: {
            updatePlayPauseState();
        }        
    }
    onCurrentlySelectedChanged: {
        updatePlayPauseState();
    }
    onCurrentlyPlayingChanged: {
        updatePlayPauseState();
        if (currentlyPlaying) {
            labelTrack.visible = false;
            playPause.visible = true;
        } else {
            labelTrack.visible = true;
            playPause.visible = false;
        }
    }
    
    function updatePlayPauseState() {
        if (isPlaying) {
            if (currentlySelected) {
                playPause.currentState = 3;
            } else {
                playPause.currentState = 1;
            }
            
        } else {
            if (currentlySelected) {
                playPause.currentState = 2;
            } else {
                playPause.currentState = 0;
            }
        }
    }
}
