import bb.cascades 1.0
import bb.cascades.multimedia 1.0
import bb.cascades.multimedia.internal 1.0
import bb.cascades.music 1.0
import bb.multimedia 1.0
import "../"

Container {
    id: playerStub
    
    //used to determine when playback is playing or paused
    property bool playbackActive: false
    
    //dlna properties which need to be persisted when the player screen is closed
    property variant __currentController

    signal playOnDeviceChangedFirstSignal(bool enabled)
    signal playOnDeviceChangedSecondSignal(bool enabled)
    signal albumPlayed(string albumId)
        
    //recently played properties     
    property bool addedToSDB: false
    property int threshold: 3000 //threshold to add to recently played list

    property int errorCode: MediaError.None
    
    onCreationCompleted: {
        if(tabbedPane.appLoaded){
            appLoadComplete();
        } else {
            tabbedPane.applicationLoadComplete.connect(appLoadComplete);                    
        }
    }
    
    function appLoadComplete(){
        //listen to further change events
        appContext.newPlaybackController.connect(playbackControllerChanged);
        //kick the initial changed
        playbackControllerChanged(appContext.playbackController);
    }

    function __handlePlaybackStateChanged(){
        console.debug("PlayerStub, playback state changed " + appContext.playbackController.playbackState);
        if( appContext.playbackController.playbackState == PlaybackController.PlaybackStopped && appContext.playbackController.totalTracks() < 1 ) {
            console.debug("PlayerStub,  playlist is empty!");
            // If controller is in a stopped state, check the track count, if zero we want to disable access
            // to the Now Playing Screen. And in the future, automatically close it if already opened.
            // We query the controller directly, as the DataModel object we have access to may
            // not be updated.
            playbackActive = false;
            // Close the Now Playing screen if it is open
            tabbedPane.currentNavPane.popLocalPlayerScreen();
        } else if(appContext.playbackController.playbackState == PlaybackController.PlaybackCompleted) {
            console.debug("PlayerStub,  playback complete!");
            playbackActive = false;
            // Close the Now Playing screen if it is open
            tabbedPane.currentNavPane.popLocalPlayerScreen();
            analytics.logEvent("PlaybackCompleted");
        } else if (appContext.playbackController.playbackState != PlaybackController.PlaybackStopped && !playbackActive) {
            // only change the active state (which fires the state changed signal) if we're actually changing state
            console.debug("PlayerStub,  playback active!");
            playbackActive = true;
        } else {
            console.debug("PlayerStub,  playbackStateChanged - no action taken");
        }
        if ( errorCode == MediaErrorEx.ResourceNotFound ) {
            errorCode = MediaError.None;
        }
    }

    function playbackControllerChanged(playbackController){
        if(__currentController != null){
            __currentController.playbackStateChanged.disconnect(__handlePlaybackStateChanged);
            __currentController.currentMediaItemChanged.disconnect(__trackChanged);
            __currentController.error.disconnect(__onError);
            if(!addedToSDB){
                __currentController.positionChanged.disconnect(__trackElapsedTimeChanged);            
            }
        }
        if(playbackController != null){
            __currentController = playbackController
            __currentController.playbackStateChanged.connect(__handlePlaybackStateChanged);
            __currentController.currentMediaItemChanged.connect(__trackChanged);
            __currentController.error.connect(__onError);
            if(!addedToSDB){
                __currentController.positionChanged.connect(__trackElapsedTimeChanged);            
            }            
        }
        errorCode = MediaError.None;
    }
    
    function __trackElapsedTimeChanged( timeElapsed ){    
        if(!addedToSDB && timeElapsed >= threshold){
        	addedToSDB = true; //set flag to ensure accurate play count               
        	var track = __currentController.currentSong;
        	var fid = track.fid;
            if(fid != null){
                console.debug("PlayerStub,  marking recently played, fid=" + fid);
                //console.debug("PlayerStub debug marking recently played, fid=" + track.fid + ", title=" + track.title);
                var currentTime = new Date();
                statsDBM.setDatePlayed(fid, currentTime.getTime()); 
                statsDBM.incrementCount(fid);        
                albumPlayed(_currentController.currentSong.albumId);
            }                     
        }
    }
    
    function __trackChanged(){
        console.debug("PlayerStub,  track changed");
        addedToSDB = false;
        __currentController.positionChanged.connect(__trackElapsedTimeChanged);
        //var track = __currentController.currentSong;
        //console.debug("PlayerStub debug track changed, fid=" + track.fid + ", title=" + track.title);
    }   
    
    function onPlayOnDeviceChanged(enabled) {
        playOnDeviceChangedFirstSignal(enabled);
        playOnDeviceChangedSecondSignal(enabled);
        console.debug("PlayerStub playOnDeviceChanged signals are sent");
    }  

    function __onError( code, msg ) {
        errorCode = code;
        if ( errorCode == MediaError.ResourceUnavailable ) {
            playbackActive = false;
        }
    }
}
