import bb.cascades 1.0
import bb.cascades.multimedia 1.0
import bb.cascades.multimedia.internal 1.0
import bb.cascades.music 1.0
import "ContentUris.js" as Uris

/**
 * This is a page that displays a list of all songs for a specific playlist
 * It contains a title and a PlaylistList control
 * @param   playlistIdStr  The playlist id used to retrieve songs for a secific playlist
 * @param   titleStr       The text to use as the title for this page
 */

MediaPage {
    id: playlistDetailsPage

    property alias playlistIdStr: playlistList.playlistId
    property alias titleStr: screenTitle.title
    property alias thumb1Uri: playlistList.thumb1Uri
    property alias thumb2Uri: playlistList.thumb2Uri
    property alias thumb3Uri: playlistList.thumb3Uri
    property alias thumb4Uri: playlistList.thumb4Uri
    property string playlistTracksQuery: Uris.playlistPlayback(playlistIdStr)
    
    // Internal properties
    property bool popEditPageOnLoaded: false // pop the edit page when PlaylistList is loaded
    property Page editPage: null
    
    paneProperties: NavigationPaneProperties {
        backButton: ActionItem {
            id: playlistDetailsBackButton
            objectName: "playlistDetailsBackButton"
            onTriggered: {
            	playlistList.backButtonPressed();
           }
        }
    }

    property ActionItem editPlaylist: ActionItem {
                    id: playlistDetailsEditActionItem
                    title: qsTr("Edit") + Retranslate.onLanguageChanged //USED BY STABILITY TESTING
                    //ActionBar.placement: ActionBarPlacement.OnBar
                    imageSource: "asset:///images/common/ic_edit.png"
                    onTriggered: {
                        currentNavPane.pushTransitionEnded.connect(onPushTransitionEnded);
                        editPage = tabbedPane.currentNavPane.safePushOnePage("core/PlaylistEdit.qml");
                        commonAnalytics.instance.logPagePush("PlaylistEdit", "PlaylistDetails");
                        editPage.loadDataModel = true;			 
                        editPage.setTitle(playlistDetailsPage.titleStr); 
                        playlistDataModel.plid = playlistIdStr;
                        editPage.playlistSaved.connect(playlistSaved);
                    }
                    function playlistSaved(plid, filename, four_square_list) {
                        titleStr = filename;
                        thumb1Uri = four_square_list[0];
                        thumb2Uri = four_square_list[1];
                        thumb3Uri = four_square_list[2];
                        thumb4Uri = four_square_list[3];
                        popEditPageOnLoaded = true; // pop the edit page after PlaylistList is loaded
                        if (playlistIdStr != plid) {
                            playlistIdStr = plid;
                        } else {
                            playlistList.getDataModel().load();
                        }
	                }
                    function onPushTransitionEnded() {
                        console.log("PlaylistDetails: set the model after push transition ended");
                        currentNavPane.pushTransitionEnded.disconnect(onPushTransitionEnded);
                        editPage.playlistDataModel = playlistDataModel;
                    }
                }

    property ActionItem playAll:ActionItem {
    	                title: qsTr("Play All") + Retranslate.onLanguageChanged
    	                imageSource: "asset:///images/playing/ic_play.png"
    	                onTriggered: {
    	                    
							if( playlistList.getDataModel() && playlistList.getDataModel().childCount([]) > 0 ) {
							    dynamicPlaylistManager.startPlaybackByIndex(
	    	                    	playlistTracksQuery,
	    	                    	false, //shuffle off 
	    	                    	0);
	    	                 }
    	                }
    	            }

    property ActionItem shuffleAll:ActionItem {
	                title: qsTr("Shuffle All") + Retranslate.onLanguageChanged
	                imageSource: "asset:///images/playing/ic_shuffle.png"
	                //ActionBar.placement: ActionBarPlacement.OnBar
	                
	                onTriggered: {
	                    if( playlistList.getDataModel() && playlistList.getDataModel().childCount([]) > 0 ) {
	                        dynamicPlaylistManager.startPlaybackByIndex(
		                    	playlistTracksQuery,
		                    	true, //shuffle on 
		                    	-1);
                            commonAnalytics.instance.logShuffleAll("PlaylistDetails");
		                 }
	                }
	            }
   property ActionItem search: ActionItem {
            //: menu item for search
            title: qsTr("Search") + Retranslate.onLanguageChanged
	        imageSource: "asset:///images/common/ic_search.png"
            onTriggered: {
            playlistDetailsSearchScreen.display();
        }
        } 
    
    property ActionItem goToPlayer: NowPlayingActionItem {
        page: "PlaylistDetails"
    }

                
   property MultiSelectActionItem multiSelectActionItem: MultiSelectActionItem {
        onTriggered: {
            playlistList.listView.selectableMultiHandler.active = true;
        }
    }     
    

    attachedObjects: [
        AsyncHeaderDataModel {
            objectName: "Playlist"
            id: noContentModel
            query: MediaDataQueryDecorator {
                query: ContentUriHeaderDataQuery {
                    id: noContentQuery
                }
            }
        },
        M3uPlaylistDataModel {
        	id: playlistDataModel
        }
    ]

    onCreationCompleted:{
        qmlUtils.addActionItemToPage(goToPlayer, playlistDetailsPage);
        qmlUtils.addActionItemToPage(playAll, playlistDetailsPage);
        qmlUtils.addActionItemToPage(shuffleAll, playlistDetailsPage, 2);
        qmlUtils.addActionItemToPage(editPlaylist, playlistDetailsPage, 2);
        qmlUtils.addActionItemToPage(search, playlistDetailsPage);
        qmlUtils.addActionItemToPage(multiSelectActionItem, playlistDetailsPage);
        tabbedPane.currentNavPane.backButtonsVisible = true;
        playlistList.getDataModel().loaded.connect(onLoaded);
        playlistDetailsSearchScreen.rootPage = playlistDetailsPage;
        playlistDetailsSearchScreen.hide();
    }

    onActionMenuVisualStateChanged: {
        playlistDetailsSearchScreen.enableTypeToSearch(ActionMenuVisualState.Hidden == actionMenuVisualState);
    }

    function getFileName() {
        return screenTitle.cacheText;
    }

    function setTitle( title ) {
        screenTitle.title = title;
    }
    
    function onLoaded() {
        if (popEditPageOnLoaded) {
            popEditPageOnLoaded = false;
            editPage.popPlaylistEditPage();
            editPage = null;
        }
    }

    onPlaylistIdStrChanged: {
        if(playlistIdStr != null && playlistIdStr != ""){
            noContentQuery.headerUri = ""; //if header uri is empty, the model will provide one dummy header for all the data
            noContentQuery.uri = Uris.playlistMembers(["COUNT(*)"], playlistIdStr);
            noContentModel.load();
        }
    }

    content: Container {
        id: mainContainer
        layout: DockLayout {
        }
        
        Container {
            layout: StackLayout {
                orientation: LayoutOrientation.TopToBottom
            }
            ScreenTitle {
                id: screenTitle
                showNowPlaying: true
                horizontalAlignment: HorizontalAlignment.Fill            
                layoutProperties: StackLayoutProperties {            
                }
            }
            Container {
                horizontalAlignment: HorizontalAlignment.Fill
                
                layoutProperties: StackLayoutProperties {
                    spaceQuota: 100                
                }
                layout: DockLayout {}
                BackgroundWrapper {}
                Container {
                    horizontalAlignment: HorizontalAlignment.Fill
                    verticalAlignment: VerticalAlignment.Fill
                    
                    NoContentView {
                        id: noContentView
                        title: qsTr("No Music") + Retranslate.onLanguageChanged
                        subTitle: (MediaApplication.corporate ? qsTr("You are currently in the work space. Please switch to the personal space to see any personal files.\n\nFiles stored on the SD card are not accessible from the work space. Please switch to the personal space to see the files stored on the SD card.") 
               	 		: qsTr("You can use BlackBerry Link to sync music between your device and computer. \n\nTo download BlackBerry Link on your computer, visit\nwww.blackberry.com/BlackBerryLink")) + Retranslate.onLanguageChanged
                        pageDataModel: noContentModel
                        horizontalAlignment: HorizontalAlignment.Fill
                        verticalAlignment: VerticalAlignment.Center
                    }
    	            PlaylistList {
    	                id: playlistList
    	                objectName: "playlistList"
    	                visible: !noContentView.visible
    	                searchScreen: playlistDetailsSearchScreen
                    }
                }
            }
        }

        Search { // PlaylistDetails need its own instance of Search screen
                 // because it may be launched from the main Search screen
                 // (see also tabbedPane.searchScreen).
            id: playlistDetailsSearchScreen
        }
    }
}
