import bb.cascades 1.0
import bb.cascades.multimedia 1.0
import bb.system 1.0
import bb.cascades.music 1.0
import "PlUtils.js" as PlUtils
import "layoutconstants.js" as LayoutConstants
import "ContentUris.js" as Uris

/**
 * A control to display the list of songs for a specific playlist
 * The list uses CountDurationHeaderWithCover for the list header
 * and ListItemVisual for the list items
 *
 * @param   playlistId  The playlist id used to retrieve songs for a specific playlist
 * @param   thumb1Uri   Thumb location used by 4 thumb item   
 * @param   thumb2Uri   Thumb location used by 4 thumb item 
 * @param   thumb3Uri   Thumb location used by 4 thumb item 
 * @param   thumb4Uri   Thumb location used by 4 thumb item 
 */

Container {
    property string playlistId
    property alias thumb1Uri: list.fourSquare1
    property alias thumb2Uri: list.fourSquare2
    property alias thumb3Uri: list.fourSquare3
    property alias thumb4Uri: list.fourSquare4
    property Search searchScreen
    property alias listView: list

    id: playlistList
    
    rightPadding: LayoutConstants.TRACKS_LIST_VIEW_PADDING
    leftPadding: LayoutConstants.TRACKS_LIST_VIEW_PADDING
    
    layout: DockLayout {        
    }
    
    attachedObjects: [
		SelectableDataModelDecorator {
            id: selectableModel
        	model: PrioritizedDataModelDecorator {
	            id: prioritizedModel
	            model: CurrentSongDecorator {
	                id: curSongDecorator
                    checkOid: true
                    playbackControllerRef: appContext.playbackController
                    dynamicPlaylistManager: appContext.dynamicPlaylistManagerRef
	                model: AsyncHeaderDataModel {
	                    objectName: "PlaylistList"
	                    id: dataModel
	                }
	            }
			}
        },
        SystemDialog {
            id: fileMissingDialog
			title: qsTr("Error playing media") + Retranslate.onLanguageChanged
			body: qsTr("The media file is missing.") + Retranslate.onLanguageChanged
            confirmButton.label: qsTr("OK") + Retranslate.onLanguageChanged
            cancelButton.label: undefined
        }
    ]
    
    animations: [
        LoadListTransition {
            target: list
            dataModel: dataModel
        }
    ]
    
    function getDataModel() {
        if ( dataModel == null ) {
        	console.log("dataModel null");
        }
    	return dataModel;
    }

    onPlaylistIdChanged: {
        var queryDecorator = dataQueryFactory.createMediaDataQueryDecorator(dataModel);        
        var query = dataQueryFactory.createContentUriHeaderDataQuery(queryDecorator);
        query.headerUri = Uris.playlist(["playlist_name","duration","count","thumb_file"],playlistId);
        query.uri = Uris.playlistMembers([ "album", "album_artist","title","artist","duration","fid","fullpath","plid","playlist_entry_text","filename","date_modified"],playlistId);       

        queryDecorator.query = query;
        dataModel.query = queryDecorator;
        dataModel.query.error.connect(appContext.showDataQueryErrorDialog);
        
        console.log("onPlaylistIdChanged() setting playlistId to: " + playlistId);

        dataModel.load();
    }

    SelectableMusicListView {
        id:list
        dataModel: selectableModel
        layout: StackListLayout {}
        topPadding: LayoutConstants.TRACKS_LIST_VIEW_PADDING
        bottomPadding: LayoutConstants.TRACKS_LIST_VIEW_PADDING
        
        property bool isMultiSelect: false;
        contentType: MediaLibraryContentType.PlaylistEntryType

        property string fourSquare1
        property string fourSquare2
        property string fourSquare3
        property string fourSquare4
        property string playlistIdStr: playlistList.playlistId
        property int songIndexPath
        property alias editTitleStyle:titleNormalEdit.style
        property alias editSubtitleStyle:subtitleNormalEdit.style
        property variant songDecorator: curSongDecorator

        contextMenuHandler: ContextMenuHandler {
            id: contextMenuHandler
            onVisualStateChanged: {
                list.enableTypeToSearch();
            }
        }

        //a function to scroll to the currently playing track
        function scrollToCurrentSong(){
            appContext.playbackController.currentIndexChanged.disconnect(scrollToCurrentSong);
            songIndexPath = musicUtils.getIndexByKey(appContext.playbackController.currentSong.oid, Uris.playlistMembers(["oid"],playlistIdStr), "oid=", dataModel, 0);
            list.scrollToAsyncModelItem([0,songIndexPath]); //[0,] is added to indicate every song is under the first and only header
        }
        function enableTypeToSearch() {
            searchScreen.enableTypeToSearch(ActionMenuVisualState.Hidden == contextMenuHandler.visualState);
        }

    	listItemComponents: [
            ListItemComponent {
                type: ""
                SimpleTwoRowListItemVisual {
                    title: PlUtils.getPlaylistTrackTitle(ListItemData["fid"], ListItemData["title"], ListItemData["playlist_entry_text"])
                    subTitle: {(ListItemData["fid"] != "" ? ListItemData["artist"] : PlUtils.fileMissingInfo(ListItemData["playlist_entry_text"]))}
                    track: ListItem.indexInSection + 1
                    duration: {(ListItemData["fid"] != "" ? ListItemData["duration"] : -1)}
                    controller: ListItem.view.songDecorator
                    allowDeleteAction: false
                    multiSelectionMode: ListItem.view.isMultiSelect
                    itemSelected: ListItemData["isSelected"]
                }
            },
            ListItemComponent {
                type: "fullItem"
                ListItemVisual {
                    id: playlistListItemVisual
                    title: PlUtils.getPlaylistTrackTitle(ListItemData["fid"], ListItemData["title"], ListItemData["playlist_entry_text"])
                    subTitle: {(ListItemData["fid"] != "" ? ListItemData["artist"] : PlUtils.fileMissingInfo(ListItemData["playlist_entry_text"]))}
                    track: ListItem.indexInSection + 1
                    duration: {(ListItemData["fid"] != "" ? ListItemData["duration"] : -1)}
                    fullpath: ListItemData["fullpath"]
                    currSong: ListItemData["currSong"]
                    controller: ListItem.view.songDecorator
                    showSubTitle: true
                    allowDeleteAction: false
                    itemSelected: ListItemData["isSelected"]
                    multiSelectionMode: ListItem.view.isMultiSelect
                }
            },
            ListItemComponent {
    			type: "header"
				CountDurationHeaderWithCover {
				    id: playlistListItemHeader
                    fourSquareVisible: true
				    duration: ListItemData["duration"]
			        count: ListItemData["count"]
			        thumb1Uri: ListItem.view.fourSquare1
			        thumb2Uri: ListItem.view.fourSquare2
			        thumb3Uri: ListItem.view.fourSquare3
			        thumb4Uri: ListItem.view.fourSquare4
                    property string listId: ListItem.view.playlistIdStr 
			        defaultImage: "asset:///images/common/album_placeholder.png"
                    onShuffleList:{
                        appContext.playbackController.currentIndexChanged.connect(ListItem.view.scrollToCurrentSong);
                        dynamicPlaylistManager.startPlaybackByIndex(
                        Uris.playlistPlayback(ListItem.view.playlistIdStr),                                                
                        true, //shuffle on 
                        -1);
                        commonAnalytics.instance.logShuffleAll("PlaylistListHeader");
                    }  
	            }
    		}
    	]

     	onTriggered: {
	        if ( !dataModel.hasChildren(indexPath) ) {
	            var selectedItem = list.dataModel.data(indexPath);
				if (selectedItem.fid == null || selectedItem.fid == "") {
					fileMissingDialog.show();
				}
				else {
					var state = playerStub.playbackActive;
                    if (state && selectedItem.fullpath == appContext.playbackController.currentSong.fullpath && appContext.playbackController.currentSong.fullpath != "" && appContext.playbackController.currentSong.fullpath != null &&
                        musicUtils.getOidFromCurrentSong(appContext.playbackController.currentSong) == selectedItem.oid) {
						//selected an already playing track, play/pause toggle
						if ( appContext.playbackController.playbackState == PlaybackController.PlaybackPlaying ) {
							appContext.playbackController.pause()
						} else {
							appContext.playbackController.play()
						}
					} else{
						var playlistTracksQuery = Uris.playlistPlayback(playlistId);
						// the playlistPlayback contenturi filters out the invalid fid, we should not pass DataModel to MusicUtils::getIndex.
                        var songIndex = musicUtils.getIndexByKey(selectedItem.key_id, playlistTracksQuery, "key_id=", null, 0);
                        dynamicPlaylistManager.startPlaybackByIndex(
						   playlistTracksQuery, 
						   appContext.playbackController.shuffleMode == PlaybackController.ShuffleOn, //shuffle works now
						   songIndex,
						   "0",
						   PlaybackController.PlaybackPlaying,             
						   selectedItem);
					}
				}
			}
    	}

        attachedObjects: [
            TextStyleDefinition {
                /*
                 * The track names.
                 * Should be "Primary R"
                 */
                id: titleNormalEdit
                base: FontUtils.PrimaryR
                color: ColorUtils.get(0xffe5e5e5)
            },
            TextStyleDefinition {
                /*
                 * The track name subtitles i.e. the Artist.
                 * Should be "P3R"
                 */
                id: subtitleNormalEdit
                base: FontUtils.P3R
                color: ColorUtils.get(0xffe5e5e5)
            }
    	]
    }

     function backButtonPressed() {
        tabbedPane.currentNavPane.pop();
    }

}
