import bb.cascades 1.0
import bb.cascades.multimedia 1.0
import bb.cascades.multimedia.internal 1.0
import bb.cascades.music 1.0
import bb.system 1.0
import "ContentUris.js" as Uris

/**
 * This control is the main playlist page.
 * It contains a ListView that displays all the playlists on your device.
 */
Page {
    id: rootPage
    property alias playlistsContentQMLPath:playlistsPage.contentQMLPath
    /**
     * property used to check if there is any content in this page
     */
    property bool noContentViewVisible: false

    property variant searchScreenContainer: playlistsPage

    actions: [
        NowPlayingActionItem {
            page: "Playlists"
        },
        ActionItem {
            title: qsTr("Play All") + Retranslate.onLanguageChanged
            imageSource: "asset:///images/playing/ic_play.png"
            enabled: !noContentViewVisible
            onTriggered: {
                dynamicPlaylistManager.startPlaybackByIndex(
                    Uris.allSongsPlayback(),
                    false, //shuffle off 
                    0);
            }
        },
        ActionItem {
            title: qsTr("Shuffle All") + Retranslate.onLanguageChanged
            imageSource: "asset:///images/playing/ic_shuffle.png"
            enabled: !noContentViewVisible
            onTriggered: {
                dynamicPlaylistManager.startPlaybackByIndex(
                    Uris.allSongsPlayback(),
                    true, //shuffle on 
                    -1);
                commonAnalytics.instance.logShuffleAll("Playlists");
            }
        },
        ActionItem {
            //: menu item for search
            title: qsTr("Search") + Retranslate.onLanguageChanged
	        imageSource: "asset:///images/common/ic_search.png"            
            onTriggered: {
                appContext.searchScreen.display();
            }
        },
        MultiSelectActionItem {
            enabled: !noContentViewVisible
            onTriggered: {
                if (!playlistsPage.at(0).playlistsListView.multiHandler.active) {
                    playlistsPage.at(0).playlistsListView.multiHandler.active = true;
                } else {
                    playlistsPage.playlistsListModel.clearSelections();
                    playlistsPage.playlistsListView.clearSelection();
                }
            }
        }     
    ]
    
    onCreationCompleted: {
        appContext.addAppWorldActionItem(rootPage);
    }

    onActionMenuVisualStateChanged: {
        appContext.searchScreen.enableTypeToSearch(ActionMenuVisualState.Hidden == actionMenuVisualState);
    }

    content: SwitchableContainer {
        id: playlistsPage

        horizontalAlignment: HorizontalAlignment.Fill
        verticalAlignment: VerticalAlignment.Fill

        layout: DockLayout {
        }
        onContentQMLPathChanged: {
            if (playlistsPage.at(0).noContentView) {
                playlistsPage.at(0).noContentView.visibleChanged.connect(onNoContentViewVisibleChanged);
            }
        }
        function onNoContentViewVisibleChanged(visible) {
            noContentViewVisible = visible;
        }
    }
}

