import bb.cascades 1.0
import bb.cascades.multimedia 1.0
import bb.cascades.multimedia.internal 1.0
import bb.cascades.music 1.0
import bb.system 1.0
import "ContentUris.js" as Uris

Container {
    property variant playlistsListView: playlistsPage.listView
    property variant playlistsListModel: playlistsPage.listModel
    property variant noContentView: noContentViewTracks
    
    layout: StackLayout {
        orientation: LayoutOrientation.TopToBottom
    }
    
    ScreenTitle {
        largeText: false
        showNowPlaying: true
        title: qsTr("Playlists") + Retranslate.onLanguageChanged
        horizontalAlignment: HorizontalAlignment.Fill
    }
    
    Container {
	    id: playlistsPage
	    property alias listView: playlistContainerList
	    property alias listModel: mySelectablePlaylists
	    property bool hasPlaylists: false
	    property bool hasContent: false
	    layout: DockLayout {}
	    
        layoutProperties: StackLayoutProperties {
            spaceQuota: 100
        }
	    BackgroundWrapper {
            useAltPaint: playlistContainerList.isMultiSelect
        }
	    attachedObjects: [
	        ModelCombiner {
	            id: playlistDataModel
	            
	            //my playlists
	            CreatePlaylistDecorator { //decorator to add "Create Playlist" item
	                id: createPlaylistDecorator
	                model: HeaderTypeDecorator {
                        headerType: "my_playlists"
                        itemType: "default"
                        model: PrioritizedDataModelDecorator {
                            model: AsyncHeaderDataModel {
                                objectName: "Playlists"
                                id: data
                                query: MediaDataQueryDecorator {
                                    query: ContentUriHeaderDataQuery {
                                        headerUri: "" //if header uri is empty, the model will provide one dummy header for all the data
                                        uri: Uris.playlists(["plid","four_square","playlist_path","count","duration","thumb_file","key_id"])
                                        onError: appContext.showDataQueryErrorDialog(code, message)
                                    }
                                }
                                onItemsChanged: {
                                    var playlistCount = data.childCount(null);
                                    playlistsPage.hasPlaylists = (playlistCount != null && playlistCount > 0);
                                }
                            }
						}
	                }
	            }
	            
	            //smart model that contains the smart playlists
        
	            MediaDataModelDecorator {
	                model: SmartPlaylistDataModel {
	                    id: smartPlaylistDataModel
	                }
	            }

	        },
	        AsyncDataModel {
	            objectName: "PlaylistsNoContent"
	            id: noContentDecorator 
                query: ContentUriDataQuery {
                    uri: Uris.allSongsNoContent()
                    onError: appContext.showDataQueryErrorDialog(code, message)
                }
                onItemsChanged: {
                    var songCount = noContentDecorator.childCount(null);
                    playlistsPage.hasContent = (songCount != null && songCount > 0);
                }
            },
	        SelectableDataModelDecorator {
	            id: mySelectablePlaylists
	            model: playlistDataModel
	        }
	    ]
	    
	    onCreationCompleted: {
	        data.load();
            noContentDecorator.load();
        }

        NoContentView {
            horizontalAlignment: HorizontalAlignment.Fill
            verticalAlignment: VerticalAlignment.Center
            visible: !playlistsPage.hasContent && !playlistsPage.hasPlaylists
            id: noContentViewTracks
            image: "asset:///images/common/music_no_content_icon.png"
            title: qsTr("No Music") + Retranslate.onLanguageChanged
            subTitle: (MediaApplication.corporate ? qsTr("You are currently in the work space. Please switch to the personal space to see any personal files.\n\nFiles stored on the SD card are not accessible from the work space. Please switch to the personal space to see the files stored on the SD card.") : qsTr("You can use BlackBerry Link to sync music between your device and computer. \n\nTo download BlackBerry Link on your computer, visit\nwww.blackberry.com/BlackBerryLink")) + Retranslate.onLanguageChanged
	    }
	    
	    LibraryListView {
	        id: playlistContainerList
            dataModel: mySelectablePlaylists
            contentType: MediaLibraryContentType.PlaylistType
    
	        visible: !noContentViewTracks.visible
	        bottomPadding: 6
	        
	        layout: GridListLayout {
	            headerMode: ListHeaderMode.Sticky
	            columnCount: 3
	        }

            contextActions:[] //ensure create new playlist, smart playlists do not have an empty menu

			function getPlaylistName(data) {
				if (data == null || data["playlist_name"] == null) {
					return qsTr("Unknown") + Retranslate.onLanguageChanged;
				}
				else if (data["type"] == "last_played_item") {
                    return qsTr("Last Played") + Retranslate.onLanguageChanged;
                }
                else if (data["type"] == "all_tracks_item") {
                    return qsTr("All Songs") + Retranslate.onLanguageChanged;
                }
                else if (data["type"] == "recently_added_item") {
                    return qsTr("Recently Added") + Retranslate.onLanguageChanged;
                }
                else if (data["type"] == "by_year_item") {
                    return qsTr("By Year") + Retranslate.onLanguageChanged;
                }
                else {
                    return data["playlist_name"]
                }
            }
			
	        listItemComponents: [
	            ListItemComponent {
	                type: "default"
	                content: [
	                    PlaylistListItem {
	                        id: myPlaylistListItem
	                        playlist_name: ListItemData["playlist_name"]
	                        count: ListItemData["count"]
	                        duration: ListItemData["duration"]
	                        thumb1Uri: ListItemData["four_square_1"]
	                        thumb2Uri: ListItemData["four_square_2"]
	                        thumb3Uri: ListItemData["four_square_3"]
	                        thumb4Uri: ListItemData["four_square_4"]
                            contentType: MediaLibraryContentType.PlaylistType
	                        itemSelected: ListItemData["isSelected"]
	                        multiSelectionMode: ListItem.view.isMultiSelect
	                    }
	                ]
	            },
	            ListItemComponent {
	                type: "add_playlist_item"
	                content: [
	                    PlaylistCreateListItem {
	                        id: createPlaylistListItem
	                    }
	                ]
	            },
	            ListItemComponent {
	                type: "last_played_item"
	                content: [
	                    SmartPlaylistListItem {
	                        id: smartLastPlayedListItem
	                        playlist_name: ListItemData["playlist_name"]
	                        count: ListItemData["count"]
	                        thumb1Uri: ListItemData["four_square_1"]
	                        thumb2Uri: ListItemData["four_square_2"]
	                        thumb3Uri: ListItemData["four_square_3"]
	                        thumb4Uri: ListItemData["four_square_4"]
	                        defaultImage: "asset:///images/common/album_placeholder.png" 
	                   }
	                ]
	            },
	            ListItemComponent {
	                type: "all_tracks_item"
	                content: [
	                    SmartPlaylistListItem {
	                        id: smartAllTracksListItem
	                        playlist_name: ListItemData["playlist_name"]
	                        count: ListItemData["count"]
	                        thumb1Uri: ListItemData["four_square_1"]
	                        thumb2Uri: ListItemData["four_square_2"]
	                        thumb3Uri: ListItemData["four_square_3"]
	                        thumb4Uri: ListItemData["four_square_4"]
	                        defaultImage: "asset:///images/common/album_placeholder.png" 
	                   }
	                ]
	            },
	            ListItemComponent {
	                type: "recently_added_item"
	                content: [
	                    SmartPlaylistListItem {
	                        id: smartRecentlyAddedListItem
	                        playlist_name: ListItemData["playlist_name"]
	                        count: ListItemData["count"]
	                        thumb1Uri: ListItemData["four_square_1"]
		                    thumb2Uri: ListItemData["four_square_2"]
		                    thumb3Uri: ListItemData["four_square_3"]
		                    thumb4Uri: ListItemData["four_square_4"]
		                    defaultImage: "asset:///images/common/album_placeholder.png" 
	                   }
	                ]
	            },
	            ListItemComponent {
	                type: "by_year_item"
	                content: [
	                    SmartPlaylistListItem {
	                        id: smartByYearListItem
	                        playlist_name: ListItemData["playlist_name"]
	                        count: ListItemData["count"]
	                        thumb1Uri: ListItemData["four_square_1"]
	                        thumb2Uri: ListItemData["four_square_2"]
	                        thumb3Uri: ListItemData["four_square_3"]
	                        thumb4Uri: ListItemData["four_square_4"]
	                        defaultImage: "asset:///images/common/album_placeholder.png"
	                    }
	                ]
	            },
	            ListItemComponent {
	                type: "my_playlists_header"
	                content: [
	                    Header {
	                        id: myPlaylistsListHeader
	                        title: qsTr("My Playlists") + Retranslate.onLanguageChanged
	                    }
	                ]
	            },
	            ListItemComponent {
	                type: "smart_playlists_header"
	                content: [
	                    Header {
	                        id: smartPlaylistsListHeader
	                        title: qsTr("Collections") + Retranslate.onLanguageChanged
	                    }
	                ]
	            }
	        ]
	        function openUserAddedPlaylist(indexPath) {
				var selectedItem = playlistContainerList.dataModel.data(indexPath);
				var page = tabbedPane.currentNavPane.safePushOnePage("core/PlaylistDetails.qml");
                commonAnalytics.instance.logPagePush("PlaylistDetails", "PlaylistsContainer");
				page.playlistIdStr = selectedItem.plid;
				page.titleStr = (selectedItem.playlist_name != null) ? selectedItem.playlist_name : (qsTr("Unknown") + Retranslate.onLanguageChanged);
				page.thumb1Uri = selectedItem.four_square_1;
				page.thumb2Uri = selectedItem.four_square_2;
				page.thumb3Uri = selectedItem.four_square_3;
				page.thumb4Uri = selectedItem.four_square_4;
	        }
	        // open the Add Playlist page
	        function createNewPlaylist() {
	            console.debug("createNewPlaylist");
	            if (playlistsPage.hasContent) {
                    var page = tabbedPane.currentNavPane.safePushOnePageOnSheet("core/AddMusic.qml");
                    commonAnalytics.instance.logPagePush("AddMusic", "PlaylistsContainer");
                    page.useNewPlaylist = true;
                } else {
                    var dialog = systemDialogUtils.getSystemDialog();
                    dialog.title = qsTr("No Music") + Retranslate.onLanguageChanged;
                    dialog.body = qsTr("A playlist can't be created because there's no music on your device.") + Retranslate.onLanguageChanged;
                    dialog.confirmButton.label = qsTr("OK") + Retranslate.onLanguageChanged;
                    dialog.cancelButton.label = undefined;
                    dialog.show();
                } 
	        }
	        // open the ByYear page that contains decades and year list items
	        function openByYear(data) {
	            var page;
	            if(data.count > 0)
	            {
                    page = tabbedPane.currentNavPane.safePushOnePage ("core/ByYear.qml");
                    commonAnalytics.instance.logPagePush("ByYear", "PlaylistsContainer");
                }         
	        }
	        // open the smart playlist screen with the data specifed from a selected listview item
            function openSmartPlaylist(data, indexPath, type, showSaveAsPlaylist) {     
	            var page;	            
	            if (data.count > 0) {
    	            page = tabbedPane.currentNavPane.safePushOnePage ("core/SmartPlaylistDetails.qml");
                    commonAnalytics.instance.logPagePush("SmartPlaylistDetails", "PlaylistsContainer");
                    page.showSaveAsPlaylist = showSaveAsPlaylist;
                    page.type = type;
    	            page.headerURI = data["header_uri"];
    	            page.contentURI = data["contentURI"];
    	            page.titleStr = playlistContainerList.getPlaylistName(data);
    	            page.headerDataModel = playlistContainerList.dataModel;
    	            page.index = indexPath;
    	        }
	        }
	        function pushDetailsScreen(indexPath) {
	            var currentItemType = playlistContainerList.dataModel.itemType(indexPath);
	            var pageData = playlistContainerList.dataModel.data(indexPath);
	            
	            clearSelection();
	            
	            if(currentItemType == "default"){
	                playlistContainerList.openUserAddedPlaylist(indexPath);                
	            }else if(currentItemType == "add_playlist_item"){
	                playlistContainerList.createNewPlaylist();  
	            }else if(currentItemType == "last_played_item"){
	                 playlistContainerList.openSmartPlaylist(pageData, indexPath, "lastplayed", true);        
	            }else if(currentItemType == "all_tracks_item"){
	                playlistContainerList.openSmartPlaylist(pageData, indexPath, "alltracks", false);
	            }else if(currentItemType == "recently_added_item"){
	                playlistContainerList.openSmartPlaylist(pageData, indexPath, "recentlyadded", false);
	            }else if(currentItemType == "by_year_item"){
	                playlistContainerList.openByYear(pageData, false);
	            }
	            
	        }
	    }
	    MusicListViewHelper {
	        listView: playlistContainerList 
	    }
    }
}
