import bb.cascades 1.0
import bb.cascades.multimedia 1.0
import bb.cascades.multimedia.internal 1.0
import "layoutconstants.js" as LayoutConstants

/**
 * A simple component to display a title at the top of a page.
 * It contains a label
 *
 * @param title    The text for the title
 *
 */
Container {
    id: screenTitle
    property alias title: titleLabel.text
    property bool largeText: true
    property bool showNowPlaying: false
    
    property int padding: 10
    
    background: ColorUtils.get(0xff121212)
    minHeight: LayoutConstants.SCREEN_TITLE_HEIGHT

    layout: DockLayout {}

    onCreationCompleted: {
        if(tabbedPane.appLoaded){
            appLoadComplete();
        } else {
            tabbedPane.applicationLoadComplete.connect(appLoadComplete);                    
        }
    }
    
    function appLoadComplete(){
        if(tabbedPane.appLoaded){
            obfuscatorDelegate.delegateActive = true;
        }
    }
    
    ImageView{
        id: titleBar
        imageSource: "asset:///images/common/core_titlebar_gradient_overlay_d.png"
        verticalAlignment: VerticalAlignment.Fill
        horizontalAlignment: HorizontalAlignment.Fill
    }    
     
    Container {
        leftPadding: padding
        rightPadding: padding
        layout: StackLayout { 
	        orientation: LayoutOrientation.LeftToRight    
	    }
	    verticalAlignment: VerticalAlignment.Fill
	    horizontalAlignment: HorizontalAlignment.Fill
        
		Container {
			leftPadding: padding
			rightPadding: padding	    
		    layout: DockLayout {}
	        verticalAlignment: VerticalAlignment.Center
	        
	        layoutProperties: StackLayoutProperties {
                spaceQuota: 1
            }
		    Label {
  		        // The top title should be "P5R"
  		        id: titleLabel
  		        textStyle.base: FontUtils.P5R
  		        textStyle.color: ColorUtils.getPrimaryTextColor()

  	            verticalAlignment: VerticalAlignment.Center
  	            horizontalAlignment: HorizontalAlignment.Left
  	            textFormat: TextFormat.Plain
  		    }  
		}
		ControlDelegate {
	        id: obfuscatorDelegate
	        delegateActive: false
	        source: "../nowplaying/NowPlayingScreenTitle.qml"
	        verticalAlignment: VerticalAlignment.Center
	        
	        onControlChanged: {
	            if(control != null){
	                control.showNowPlaying = showNowPlaying;
	            }
	        }
	        
	        onError: {
	            console.log("ScreenTitle.obfuscatorDelegate, error loading control: " + errorMessage);
	        }
	        
	    }
	}
}
