import bb.cascades 1.0
import bb.cascades.multimedia 1.0
import bb.cascades.multimedia.internal 1.0
import bb.system 1.0
import "ContentUris.js" as Uris
import "layoutconstants.js" as LayoutConstants

Container {
    id: musicSearchPage
    objectName: "SearchObjectName"
    
    visible: false
    translationY: PrimaryDisplay.pixelSize.height

    property Page topPage: tabbedPane.currentNavPane.top
    property Page rootPage // assign to the Page where this container is located
    
    property bool typeToSearchEnabled: false

    keyListeners: [
        KeyListener {
            onKeyPressed: {
                //if the symbol key is pressed then show the search screen if it isnt visible already 
                if (typeToSearchEnabled && isOffScreen() && event.key == 211) {
                    display();
                }
            }
        }
    ]

    animations: [
        TranslateTransition {
            id: goOffScreen
            target: musicSearchPage
            toY: PrimaryDisplay.pixelSize.height
            duration: 350
            easingCurve: StockCurve.CubicInOut
            onEnded: {
                musicSearchPage.visible = false;
                enableTypeToSearch(LayoutConstants.TYPE_TO_SEARCH_ENABLED);
            }
        },

        TranslateTransition {
            id: goOnScreen
            target: musicSearchPage
            toY: 0
            duration: 350
            easingCurve: StockCurve.CubicInOut
            onStarted: {
                musicSearchPage.visible = true;
            }
        }
    ]

    function isOffScreen() {
        return (musicSearchPage.translationY > 0);
    }

    function focusSearchField(focused) {
        musicSearchScreen.focusSearchField(focused);
    }

    function display() {
        rootPage.actionBarVisibility = ChromeVisibility.Hidden;
        musicSearchScreen.enableVirtualKeyboard(true);
        goOnScreen.play(); // animate to on screen
        musicSearchScreen.displaySearchResult(true);
        focusSearchField(true);
    }

    function hide() {
        focusSearchField(false); // don't focus the text field while doing the animation
        musicSearchScreen.enableVirtualKeyboard(false);
        musicSearchScreen.displaySearchResult(false); // hide the search result while doing the animation
        goOffScreen.play(); // animate to off screen
        rootPage.actionBarVisibility = ChromeVisibility.Default;
    }

    /**
     * Temporarily enable or disable "type to search".
     * For example, when an action menu or context menu is displayed, "type to search"
     * is disabled temporarily 
     *
     * If LayoutConstants.TYPE_TO_SEARCH_ENABLED is false; this function does nothing.
     */
    function enableTypeToSearch(enabled) {
        if (LayoutConstants.TYPE_TO_SEARCH_ENABLED) {
            typeToSearchEnabled = enabled && !tabbedPane.invokeSheetShowing;
            musicSearchScreen.setAlwaysFocused(typeToSearchEnabled);
        }
    }

    onTopPageChanged: {
        if (topPage == rootPage) {
            if (!isOffScreen()) {
                rootPage.actionBarVisibility = ChromeVisibility.Hidden;
            }
        }
    }
    
    layout: DockLayout {}
    BackgroundWrapper { }
	
    SearchScreen {
        id: musicSearchScreen
        
        searchListViewDef: musicSearchListViewDef

        hintText: qsTr("Search music") + Retranslate.onLanguageChanged
        noContentLabel: qsTr("Can't find a match. \nTry different search terms.") + Retranslate.onLanguageChanged

        onSearchTextChanged: {
            if (musicSearchPage.typeToSearchEnabled && musicSearchPage.isOffScreen()) {
                musicSearchPage.display();
                musicSearchScreen.enableVirtualKeyboard(true);
                commonAnalytics.instance.logSearchTextChanged(text);
            }
        }

        attachedObjects: [
            ComponentDefinition {
                id: borderImage
                ImageView {
                    id: overlay
                    imageSource: "asset:///images/common/core_item_border.amd"
                }
            },
            
            SelectableDataModelDecorator {
                id: selectableMusicSearchDataModel
                model:  SearchModelCombiner {
                    id: musicSearchDataModel
                    //artist results
                    HeaderTypeDecorator {
                        headerType: "artist"
                        itemType: "artist"
                        model: ModelFilterDecorator {
                            filterFields: ["album_artist"]
                            queryFactory:   SearchFilterQueryFactory{
                                                dataQuery: Uris.artistSearch()
                                                headerQuery: ""
                                            }
                            model: AsyncHeaderDataModel{
                                objectName: "SearchArtist"
                            }  
                        }
            
                    }         
                    //album results
                    HeaderTypeDecorator {
                        headerType: "album"
                        itemType: "album"
                        model:  ModelFilterDecorator {
                            filterFields: ["album"]
                            queryFactory:   SearchFilterQueryFactory{
                                                dataQuery: Uris.albumSearch()
                                                headerQuery: ""
                                            }
                            model: AsyncHeaderDataModel{
                                objectName: "SearchAlbum"
                            }  
                        }
            
                    }
                    
                    //playlist results
                    HeaderTypeDecorator {
                        headerType: "playlist"
                        itemType: "playlist"
                        model:  ModelFilterDecorator {
                            filterFields: ["playlist_name"]
                            queryFactory:   SearchFilterQueryFactory{
                                                dataQuery: Uris.playlistSearch()
                                                headerQuery: ""
                                            }
                            model: AsyncHeaderDataModel{
                                objectName: "SearchPlaylist"
                            }  
                        } 
                    }        
                    
                    //track results
                    HeaderTypeDecorator {
                        headerType: "track"
                        itemType: "track"
	                    model:  ModelFilterDecorator {
                            filterFields: ["title"]
                            queryFactory:   SearchFilterQueryFactory{
                                                dataQuery: Uris.searchTracks()
                                                headerQuery: ""
                                            }
                            model: AsyncHeaderDataModel{
                                objectName: "SearchTrack"
                            }  
                        } 
            
                    }                    
                    onInitComplete: {
                        musicSearchScreen.modelsChanged();
                    }    
                }
            },
            
            ComponentDefinition {
               id: musicSearchListViewDef
               MusicListView {
                    id: musicListView
                    dataModel: selectableMusicSearchDataModel 
                    layout: GridListLayout   {
                        headerMode: ListHeaderMode.Sticky
                        columnCount: 3
                        horizontalCellSpacing: 0
                    }    
                	property variant highlightImage: borderImage
                	
                	scrollRole: ScrollRole.None
                   	
                    listItemComponents : [
                        //album components
                        ListItemComponent {
                            type: "album"
                            DeletableThumbsListItem { 
                                labelText: (!ListItemData || !ListItemData["album"]) ? "" : ListItemData["album"]
                                thumbUri: (!ListItemData || !ListItemData["album"]) ? "" : ListItemData["thumb_file"]
                                defaultImage: "asset:///images/common/album_placeholder.png"
                                preferredHeight: 256
                                preferredWidth: 256
                                itemSelected: (!ListItemData || !ListItemData["isSelected"]) ? false : ListItemData["isSelected"]
                                allowMultiSelectAction: false
                                contentType: MediaLibraryContentType.AlbumType
                            }
                        },
                        ListItemComponent {
                            type: "album_header"
                            SearchHeaderItem {
                                headerText: qsTr("Albums") + Retranslate.onLanguageChanged
                            }
                        },
                        //artist components
                        ListItemComponent {
                            type: "artist"
                            DeletableThumbsListItem { 
                                labelText: (!ListItemData || !ListItemData["album_artist"]) ? "" : ListItemData["album_artist"]
                                thumbUri: (!ListItemData || !ListItemData["thumb_file"]) ? "" : ListItemData["thumb_file"]
                                defaultImage: "asset:///images/common/album_placeholder.png"
                                preferredHeight: 256
                                preferredWidth: 256
                                itemSelected: (!ListItemData || !ListItemData["isSelected"]) ? false : ListItemData["isSelected"]
                                allowMultiSelectAction: false
                                contentType: MediaLibraryContentType.AlbumArtistType
                            }
                        },
                        ListItemComponent {
                            type: "artist_header"
                            SearchHeaderItem {
                                headerText: qsTr("Artists") + Retranslate.onLanguageChanged
                            }
                        },
                        //playlist components
                        ListItemComponent {
                            type: "playlist"
                            content: [
                                PlaylistListItem {
                                    playlist_name: ListItemData["playlist_name"]
                                    count: ListItemData["count"]
                                    duration: ListItemData["duration"]
    								thumb1Uri: ListItemData["four_square_1"]
    								thumb2Uri: ListItemData["four_square_2"]
    								thumb3Uri: ListItemData["four_square_3"]
    								thumb4Uri: ListItemData["four_square_4"]
                                    itemSelected: ListItemData["isSelected"]
                                    allowMultiSelectAction: false
                                    contentType: MediaLibraryContentType.PlaylistType
                                }
                                
                            ]
                        },
                        ListItemComponent {
                            type: "playlist_header"
                            SearchHeaderItem {
                                headerText: qsTr("Playlists") + Retranslate.onLanguageChanged
                            }
                        },
                        //track components
                        ListItemComponent {
                            type: "track"
                            DeletableThumbsListItem { 
                                id: deleteListItem
                                defaultImage: "asset:///images/common/album_placeholder.png"
                                preferredHeight: 256
                                preferredWidth: 256
                                labelText: ListItemData["title"]
                                thumbUri: ListItemData["thumb_file"]
                                itemSelected: ListItemData["isSelected"]
                                allowMultiSelectAction: false
                                allowPlayOnAction: true
                                allowPropertiesAction: true
                                contentType: MediaLibraryContentType.AudioType
                            }
                        },
                        ListItemComponent {
                            type: "track_header"
                            SearchHeaderItem {
                                headerText: qsTr("Songs") + Retranslate.onLanguageChanged
                            }
                        }                            
                    ]
                    
                    // function to pop up the select playlist screen to add tracks to a playlist
                    function addToPlaylist() {
                        var itemType = dataModel.itemType(selectedIndexPath);
                        var selectedItem = dataModel.data(selectedIndexPath);
                        
                        //open add to playlist screen
                        var page = tabbedPane.currentNavPane.safePushOnePage ("core/SelectPlaylist.qml");
                        commonAnalytics.instance.logPagePush("SelectPlaylist", "Search");
                       
						if (itemType == "album") {
						    page.contentType = MediaLibraryContentType.AlbumType;
						    page.idsToAdd = [selectedItem.album_id];
						} else if (itemType == "artist") {
						    page.contentType = MediaLibraryContentType.AlbumArtistType;
						    page.idsToAdd = [selectedItem.album_artist_id];
						} else if (itemType == "playlist") {
						    page.contentType = MediaLibraryContentType.PlaylistType;
						     page.idsToAdd = [selectedItem.plid];
						} else if (itemType == "track") {
						    page.contentType = MediaLibraryContentType.AudioType;
						    page.idsToAdd = [selectedItem.fid];
						} 
                    }
                    
                    // override function to add the select item to the Now Playing list
                    function addToNowPlaying(last) {
                        var itemType = dataModel.itemType(selectedIndexPath);
                        var selectedItem = dataModel.data(selectedIndexPath);
                                          
						if (itemType == "album") {
                            appContext.addToNowPlaying(MediaLibraryContentType.AlbumType, [selectedItem.album_id], null, last);
						} else if (itemType == "artist") {
                            appContext.addToNowPlaying(MediaLibraryContentType.AlbumArtistType, [selectedItem.album_artist_id], null, last);
						} else if (itemType == "playlist") {
                            appContext.addToNowPlaying(MediaLibraryContentType.PlaylistType, [selectedItem.plid], null, last);
						} else if (itemType == "track") {
                            appContext.addToNowPlaying(MediaLibraryContentType.AudioType, [selectedItem.fid], null, last);
						} 
                    }
                    
                 
                    
                    onTriggered: {
                        console.debug("OnShowing:Search.qml: onTriggered indexPath:" + indexPath);
                        musicListView.requestFocus(); // steal focus from the search text field
                              
                        var itemType = dataModel.itemType(indexPath);
                        var selectedItem = dataModel.data(indexPath);
                        if(itemType == "playlist" && selectedItem.plid != null){
                            var page = tabbedPane.currentNavPane.safePushOnePage("core/PlaylistDetails.qml");
                            commonAnalytics.instance.logPagePush("PlaylistDetails", "Search");
                            page.playlistIdStr = selectedItem.plid;
                            page.titleStr = (selectedItem.playlist_name != null) ? selectedItem.playlist_name : (qsTr("Unknown") + Retranslate.onLanguageChanged);
                            page.thumb1Uri = selectedItem.four_square_1;
                            page.thumb2Uri = selectedItem.four_square_2;
                            page.thumb3Uri = selectedItem.four_square_3;
                            page.thumb4Uri = selectedItem.four_square_4;
                        } else if(selectedItem != null && selectedItem.album_artist_id != null) {
                            //if: ensure we can at least push the artist detail screen
                            var page = tabbedPane.currentNavPane.safePushOnePage("core/ArtistDetails.qml");
                            commonAnalytics.instance.logPagePush("ArtistDetails", "Search");
                            if(itemType == "album"){
                                page.openAtAlbumId = selectedItem.album_id;
                            } else if(itemType == "track"){
                                page.openAtAlbumId = selectedItem.album_id;
                                page.songToPlayOnOpenFid = selectedItem.fid;        
                                page.scrollToCurrentSongAfterInvoke = true;
                            }
                            page.albumArtistIdStr = selectedItem.album_artist_id;
                            //: album artist is not known for display on top of album art
                            page.titleStr = selectedItem.album_artist;
                        }
                    }
                    
                    onSelectionChanged: {
                        selectableMusicSearchDataModel.changeSelection(indexPath, selected);
                    }
                }
            }
        ]

        onSearchScreenCancelled: {
            hide();
        }
    }

    MusicListViewHelper {
        id: musicListViewHelper
        listView: musicSearchScreen.searchListView
    }
}