import bb.cascades 1.0
import bb.cascades.multimedia 1.0
import bb.cascades.multimedia.internal 1.0
import bb.cascades.music 1.0
import bb.system 1.0
import "PlUtils.js" as PlUtils
import "ContentUris.js" as Uris

/*
* This Page provides a list of playlists to select from
*/
Page {
    id: selectPlaylistPage
    property variant idsToAdd
    property variant activePlid
    property variant contentType: MediaLibraryContentType.AudioType
	property int errSavingPlaylist:PlaylistEditErrorType.PLERROR_OK;
	property bool popped: false
	
	// This signal is emitted when a playlist is selected
	signal selectedPlaylist(string plid)
	
    attachedObjects: [
        M3uPlaylistDataModel {
            id: createNewM3uPlaylistDataModel
        },
        SystemPrompt {
           id: createNewDialog
           title: qsTr ("Type a name") + Retranslate.onLanguageChanged
 //USED BY STABILITY TESTING
           inputField.defaultText: createNewM3uPlaylistDataModel.generateNewFileName() + Retranslate.onLanguageChanged
 //USED BY STABILITY TESTING
           inputField.emptyText: qsTr("Type Playlist Name") + Retranslate.onLanguageChanged
           confirmButton.label: qsTr ("Create") + Retranslate.onLanguageChanged
           cancelButton.label: qsTr ("Cancel") + Retranslate.onLanguageChanged
        },

	    SystemDialog {
            id: createNewErrorDialog
			title: qsTr("Error creating playlist") + Retranslate.onLanguageChanged
            confirmButton.label: qsTr("OK") + Retranslate.onLanguageChanged
			cancelButton.label: undefined
        }
   ]
   
    onCreationCompleted: {
        createNewM3uPlaylistDataModel.persistCompleted.connect(onPersistCompleted);
        //connect to the new playlist dialog.
        createNewDialog.finished.connect(onCreateNewDialogAccepted);
		createNewErrorDialog.finished.connect(onCreateNewErrorDialogAccepted);
        tabbedPane.currentNavPane.backButtonsVisible = true;
    }
    
    paneProperties: NavigationPaneProperties {
        backButton: ActionItem {
            id: selectPlaylistBackButton
            onTriggered: {
                if (!popped) {
                    popped = true;
                    tabbedPane.currentNavPane.pop();
                }
            }
        }
    }
    
    function onPersistCompleted(result, plid, filename) {
        if (result !=PlaylistEditErrorType.PLERROR_OK) {
            handleError(result);
        } else if (!popped) {
            //leave screen if there was no error
            popped = true;
            selectedPlaylist(plid);
            systemDialogUtils.showSystemToast(qsTr("Song added to playlist."));
            tabbedPane.currentNavPane.pop();
        }
    }
    
    function createNewPlaylistWithId(filename, idsToAdd) {
        createNewM3uPlaylistDataModel.plid = "";
        createNewM3uPlaylistDataModel.load();
    
        //pass the filename
        createNewM3uPlaylistDataModel.setName(filename);
        
        var config = {};
        if (activePlid != null && activePlid != "") {
            config.plid = activePlid;
            createNewM3uPlaylistDataModel.appendItemsQml(contentType, idsToAdd, config);
        }
        else {       
        	//append the item
        	createNewM3uPlaylistDataModel.appendItemsQml(contentType, idsToAdd);
        }
        //write the data
        createNewM3uPlaylistDataModel.persistData();
    }
    
    function onCreateNewDialogAccepted() {
		if(createNewDialog.result == SystemUiResult.ConfirmButtonSelection){
			var filename = createNewDialog.inputFieldTextEntry();
			createNewPlaylistWithId(filename, idsToAdd);
		}
    }
	
	function onCreateNewErrorDialogAccepted() {
        if (errSavingPlaylist ==PlaylistEditErrorType.PLERROR_LOWMEMORY) {
            // Pop the current "Add to Playlist" page and return to the previous page
            // if creating or saving playlist fails due to low internal memory
			console.log("Leaving the current page because Device internal memory is low.");
            tabbedPane.currentNavPane.pop();
        }
        else {
            createNewDialog.show();
        }
    }
    
	function handleError( error ) {
		errSavingPlaylist = error;
		createNewErrorDialog.body = PlUtils.getErrorText(error);
        createNewErrorDialog.show();
    }
	
    content: Container {
        
        horizontalAlignment: HorizontalAlignment.Fill
        
        layout: StackLayout {
            orientation: LayoutOrientation.TopToBottom
        }
        
        ScreenTitle{
            id: selectPlaylistTitle
            title: qsTr("Add to Playlist") + Retranslate.onLanguageChanged
            
            horizontalAlignment: HorizontalAlignment.Fill
            
            layoutProperties: StackLayoutProperties {                
            }
        }
        
        SelectPlaylistCreateNewItem {
            horizontalAlignment: HorizontalAlignment.Fill
            
            onSelected: {
                createNewDialog.show();
            }
        }
        
        Container {
             attachedObjects: [
                //all playlists
                HeaderTypeDecorator {
                    id: data
                    headerType: "list"
                    itemType: "playlist"
                    model: AsyncHeaderDataModel {
                        objectName: "SelectPlaylist"
                        id: playlistData
                        query: MediaDataQueryDecorator {
	                        query: ContentUriHeaderDataQuery {
	                            headerUri: ""
	                            uri: Uris.playlists(["plid","count","four_square"])
	                            onError:  appContext.showDataQueryErrorDialog(code, message)
	                        }
	                    }
                    }
                }
            ]
            
	        onCreationCompleted: {
	            playlistData.load();   
	        }            
            
            layoutProperties: StackLayoutProperties {
                spaceQuota: 100                
            }
            layout: DockLayout {
            }
            BackgroundWrapper {
            }
            ListView {
                id: list
                property alias titleStyle:titleNormal.style
                property alias subtitleStyle:subtitleNormal.style
                dataModel: data
                scrollRole: ScrollRole.Main
                layout: StackListLayout {
	                headerMode: ListHeaderMode.Sticky
	            }
                listItemComponents: [
                    ListItemComponent {
                        type: "playlist"
                        SelectPlaylistListItem {
                            id: selectPlaylistListItem
                            title: ListItemData["playlist_name"]
                            subTitle: ListItem.view.getTracks( ListItemData["count"] )
                            thumb1Uri: ListItemData["four_square_1"]
                            thumb2Uri: ListItemData["four_square_2"]
                            thumb3Uri: ListItemData["four_square_3"]
                            thumb4Uri: ListItemData["four_square_4"]
							defaultImage: "asset:///images/common/album_placeholder.png" 
                        }
                    },
                    ListItemComponent {
                        type: "list_header"
                        Container {}
                    }
                ]

                function getTracks( count ) {
                    if ( count != null && count != "" ) {
                        return qsTr("%n Song(s)", "", parseInt(count)) + Retranslate.onLanguageChanged;
                    } else {
                        return qsTr("Unknown") + Retranslate.onLanguageChanged;
                    }
                }
                
                function handlePlaylistSelected(data) {
                    console.log("handlePlaylistSelected type: " + selectPlaylistPage.contentType + "ids: " + selectPlaylistPage.idsToAdd);
                    var config = {};
                    if (activePlid != null && activePlid != "") {
                        config.plid = activePlid;
                        musicUtils.appendToPlaylist( data["plid"], selectPlaylistPage.contentType, selectPlaylistPage.idsToAdd, config );  
                    }
                    else {
                    	musicUtils.appendToPlaylist( data["plid"], selectPlaylistPage.contentType, selectPlaylistPage.idsToAdd );
                    }
                    selectedPlaylist(data["plid"]);
                    
                    systemDialogUtils.showSystemToast(qsTr("Song added to playlist."));
                    //close the screen
                    tabbedPane.currentNavPane.pop(); 
                }

                onTriggered: {                	
			        var pageData = list.dataModel.data(indexPath);

			        clearSelection();
			        
			        list.handlePlaylistSelected(pageData);  
                }
                attachedObjects: [
                    TextStyleDefinition {
	                    /*
	                     * The playlist names.
	                     * Should be "Primary R"
	                     */
                        id: titleNormal
                        base: FontUtils.PrimaryR
                        color: ColorUtils.getPrimaryTextColor()
                    },
                    TextStyleDefinition {
	                    /*
	                     * The playlist name subtitles i.e. the number of song(s)).
	                     * Should be "P3R"
	                     */
                        id: subtitleNormal
                        base: FontUtils.P3R
                        color: ColorUtils.getSecondaryTextColor()
                    }
                ]
            }
        }
    }
}
