import bb.cascades 1.0
import bb.cascades.multimedia 1.0
import bb.cascades.multimedia.internal 1.0

/**
 * A list item visual to display a title, subtitle with a 4 thumbs list item on the left.
 *
 * @param	title	 		The top row of the track info
 * @param	subTitle		The bottom row of the track info
 * @param   thumb1Uri           The uri for the top let image
 * @param   thumb2Uri           The uri for the top right image
 * @param   thumb3Uri           The uri for the bottom left image
 * @param   thumb4Uri           The uri for the bottom right image
 * @param   defaultImage        The uri for the image to use as a default
 */
Container {
    id: selectPlaylistListItem 

    property string title: ""
    property alias subTitle: trackInfo.subTitle
    property alias thumb1Uri: fourThumbs.thumb1Uri
    property alias thumb2Uri: fourThumbs.thumb2Uri
    property alias thumb3Uri: fourThumbs.thumb3Uri
    property alias thumb4Uri: fourThumbs.thumb4Uri
    property alias overlayImage: fourThumbs.overlayImage
    property alias defaultImage: fourThumbs.defaultImage
    property bool active: ListItem.active

    signal itemSelected()

    layout: DockLayout {}
    
    attachedObjects:[
        ComponentDefinition{
            id: borderImage
           	ImageView {
           	    id: overlay
                imageSource: "asset:///images/common/core_item_border.amd"
           	}
        }
    ]    

    Container {
        id: mainContainer

        horizontalAlignment: HorizontalAlignment.Fill
        verticalAlignment: VerticalAlignment.Fill		
        layout: StackLayout {
            orientation: LayoutOrientation.LeftToRight	        
        }

        onTouch: {
            if (event.isUp()) {
                selectPlaylistListItem.itemSelected();
            }
        }
		// track number, track title, sub-title
		Container {
		    id: trackInfoContainer
		    
		    horizontalAlignment: HorizontalAlignment.Left
            verticalAlignment: VerticalAlignment.Fill
		    
		    layout: StackLayout {
		        orientation: LayoutOrientation.LeftToRight
			}
			
            layoutProperties: StackLayoutProperties {                
                spaceQuota:100
            }
	
            FourThumbsListItem {
                id: fourThumbs
                                    
                preferredHeight: 123
    			preferredWidth: 123
    			topLabelVisible: false
    			bottomLabelVisible: false
            }
            TrackInfoLabel {
                id: trackInfo
                
                horizontalAlignment: HorizontalAlignment.Left
                verticalAlignment: VerticalAlignment.Center
                
                layoutProperties: StackLayoutProperties {                    
                    spaceQuota: 80
                }
                title: (selectPlaylistListItem.title == null || selectPlaylistListItem.title == "") ? qsTr ("Unknown") + Retranslate.onLanguageChanged : selectPlaylistListItem.title
                titleStyle: selectPlaylistListItem.ListItem.view.titleStyle
                subtitleStyle: selectPlaylistListItem.ListItem.view.subtitleStyle
            }
        }
	}
    Divider {
        horizontalAlignment: HorizontalAlignment.Fill
        verticalAlignment: VerticalAlignment.Bottom
    }

	
	//highlight
    ControlDelegate{
        id: highlight
        sourceComponent: borderImage
        delegateActive: active
        horizontalAlignment: HorizontalAlignment.Fill
        verticalAlignment: VerticalAlignment.Fill
    } 
}
