import bb.cascades 1.0
import bb.cascades.multimedia 1.0
import bb.cascades.multimedia.internal 1.0
import bb.cascades.music 1.0
import bb.system 1.0

MusicListView {
    id: selectableMusicListView
    
    /**
     * Emitted when multiselect state on this list has changed
     */
    signal multiselectActiveChanged(bool active)

    property alias selectableMultiHandler: selectableMusicMultiSelectHandler
    
    attachedObjects: [
	    MultiSelectHandler {
	        id: selectableMusicMultiSelectHandler
	        actions: [
	            ActionItem {
                    title: qsTr("Play Next") + Retranslate.onLanguageChanged
                    imageSource: "asset:///images/common/ic_add_to_play_next.png"
                    enabled: dataModel.selectionList.length > 0
                    onTriggered: {
                        var config = {};
                        if (activePlid != null && activePlid != "") {
                            config.plid = activePlid;
                        }
                        appContext.addToNowPlaying(contentType, dataModel.selectionList, config, false);
                        commonAnalytics.instance.logMultiSelectAction("PlayNext", dataModel.selectionList.length);
                    }
                },
                ActionItem {
                    title: qsTr("Play Last") + Retranslate.onLanguageChanged
                    imageSource: "asset:///images/common/ic_add_to_play_last.png"
                    enabled: dataModel.selectionList.length > 0 && playerStub.playbackActive
                    onTriggered: {
                        var config = {};
                        if (activePlid != null && activePlid != "") {
                            config.plid = activePlid;
                        }
                        appContext.addToNowPlaying(contentType, dataModel.selectionList, config, true);
                        commonAnalytics.instance.logMultiSelectAction("PlayLast", dataModel.selectionList.length);
                    }
                },
                ActionItem {
                    property variant selectionList
                    title: qsTr("Add to Playlist")  + Retranslate.onLanguageChanged
                    imageSource: "asset:///images/playlists/ic_add_to_playlist.png" //USED BY STABILITY TESTING
                    enabled: dataModel.selectionList.length > 0
                    onTriggered: {
                        //open add to playlist screen
                        var page = currentNavPane.safePushOnePage ("core/SelectPlaylist.qml");
                        commonAnalytics.instance.logPagePush("SelectPlaylist", "SelectableMusicListView");
                        page.contentType = contentType;
                        page.idsToAdd = dataModel.selectionList;
                        selectionList = dataModel.selectionList;
                        page.activePlid = activePlid;
                        page.selectedPlaylist.connect(onSelectedPlaylist);
                    }
                    function onSelectedPlaylist(plid) {
                        //analytics.logAddToPlaylistTrackCount(selectionList.length)
                        commonAnalytics.instance.logMultiSelectAction("AddToPlaylist", selectionList.length);
                    }
                },
                InvokeActionItem { //Work around for making menu title blank
                    id: menuTitle
                    enabled: false
                    title: " "
                },
	            InvokeActionItem {
	                id: multiInvokeActionItem
	                enabled: dataModel.selectionList.length > 0
	                query {
	                    mimeType: "filelist/audio"
	                    uri: "list://"
	                    invokeActionId: "" //for platform actions
	                }
	                onTriggered: {
                        var config = {};
                        if (activePlid != null && activePlid != "") {
                            config.plid = activePlid;
                        }
                        multiInvokeActionItem.data = invokeHelper.getDataForMultiInvoke(contentType, dataModel.selectionList, config);
                        commonAnalytics.instance.logInvokeAction(dataModel.selectionList.length);
                        commonAnalytics.instance.logMultiSelectAction("Invoke", dataModel.selectionList.length);
	                }
	            },
	            DeleteActionItem {
	                property SystemDialog deleteDialog
	                property variant selectionList
	                enabled: (dataModel.selectionList.length > 0) && (contentType != MediaLibraryContentType.LastPlayedEntryType)
	                && (contentType != MediaLibraryContentType.PlaylistEntryType)             
	                onTriggered: {
	                    selectionList = dataModel.selectionList;
	                    deleteDialog = appContext.getDeleteSystemDialog(MediaLibraryContentType.AudioType, selectionList.length);
	                    deleteDialog.finished.connect(dialogCallback);
	                    deleteDialog.show();
	                }
	                function dialogCallback (result) {
	                    if (deleteDialog.result == SystemUiResult.ConfirmButtonSelection) {
	                        if (selectionList) {
	                            deleteManager.deleteContentByIds (MediaLibraryContentType.AudioType, selectionList);
                                commonAnalytics.instance.logMultiSelectAction("Delete", selectionList.length);
	                        }
	                    }
                    }
	            }
	        ]
	        onActiveChanged: {
	            if (active) {
	                appContext.deactivateMultiSelect.connect(disableMultiSelect);
	                isMultiSelect = true;
	            } else {
	                appContext.deactivateMultiSelect.disconnect(disableMultiSelect);
	                dataModel.clearSelections ();
	                isMultiSelect = false;
	            }
	            
	            // emit our signal that the state changed
	            selectableMusicListView.multiselectActiveChanged(active);
	        }
            status: qsTr ("%Ln Item(s) Selected", "", dataModel.selectionList.length) + Retranslate.onLocaleOrLanguageChanged
        }
    ]
    onSelectionChanged: {
        var selectedData = dataModel.data(indexPath);
        // This is important to make sure some multiselect actions related to playlist works 
        activePlid = selectedData["plid"];
        if (isMultiSelect) {
	        if (!dataModel.hasChildren(indexPath)) {
	            dataModel.changeSelection(indexPath, !dataModel.isSelected(indexPath));
	        }
	    } else if (!dataModel.hasChildren(indexPath)) {
		    dataModel.changeSelection(indexPath, selected);
		}
    }
   
    property MultiSelectActionItem multiSelectActionItem: MultiSelectActionItem{
        multiSelectHandler: selectableMusicMultiSelectHandler
        onTriggered: {
            multiSelectHandler.active = true;
        }
    }
    
    function disableMultiSelect(){
        selectableMusicMultiSelectHandler.active = false;
    }
}
