import bb.cascades 1.0
import bb.cascades.multimedia 1.0
import bb.cascades.multimedia.internal 1.0
import bb.cascades.music 1.0
import bb.system 1.0

import "ContentUris.js" as Uris
import "layoutconstants.js" as LayoutConstants

/**
 * This control is the main smart list page.
 * It contains a ListView that displays album thumnails for recently played and recently added tracks
 */
Page {
    id: rootPage
    
    property alias noContentViewVisible: noContentView.visible
    property variant searchScreenContainer: mainContainer

   /*********************************************
    * <STARTUP comment="DO NOT MEDDLE HERE!">  
    *********************************************/
    
    onNoContentViewVisibleChanged: {
        if(appContext != null){    
            appContext.noContentViewVisible = noContentView.visible;
        }  
    }
    
    function appLoadComplete(){
        appContext.addAppWorldActionItem(rootPage);
        appContext.noContentViewVisible = noContentView.visible;
        noContentView.handlePageDataModelChanged();
    }
    
    onCreationCompleted: {
        if(tabbedPane.appLoaded){
            appLoadComplete();
        } else {
            tabbedPane.applicationLoadComplete.connect(appLoadComplete);
        }
    }
   /*********************************************
    * </STARTUP>
    *********************************************/
         
    

    actions: [
        NowPlayingActionItem {
            page: "Smart"
        },
        ActionItem {
            enabled: !noContentView.visible
            title: qsTr("Play All") + Retranslate.onLanguageChanged
            imageSource: "asset:///images/playing/ic_play.png"
            onTriggered: {
                dynamicPlaylistManager.startPlaybackByIndex(
                    Uris.allSongs(["album","album_artist","artist","fid","duration"]),
                    false, //shuffle off 
                    0);
            }
        },
        ActionItem {
            enabled: !noContentView.visible
            title: qsTr("Shuffle All") + Retranslate.onLanguageChanged
            imageSource: "asset:///images/playing/ic_shuffle.png"
            onTriggered: {
                dynamicPlaylistManager.startPlaybackByIndex(
                    Uris.allSongs(["album","album_artist","artist","fid","duration"]),
                    true, //shuffle on 
                    -1);
                commonAnalytics.instance.logShuffleAll("Smart");
            }
        },
        ActionItem {
            //: menu item for search
            title: qsTr("Search") + Retranslate.onLanguageChanged
	        imageSource: "asset:///images/common/ic_search.png"
            onTriggered: {
                appContext.searchScreen.display();
            }
        },
        MultiSelectActionItem {
            enabled: !noContentView.visible
            onTriggered: {
                smartListView.multiHandler.active = true;
            }
        }
    ]

    onActionMenuVisualStateChanged: {
        appContext.searchScreen.enableTypeToSearch(ActionMenuVisualState.Hidden == actionMenuVisualState);
    }    

   /*********************************************
    * <STARTUP comment="DO NOT MEDDLE HERE!">  
    *********************************************/    
    attachedObjects: [
        RenderFence {
            raised: true
            onReached: {
                tabbedPane.primaryPageLoaded();
                smartPage.visible = true;
            }
            onFailedToReach: {
                tabbedPane.primaryPageLoaded();
                smartPage.visible = true;
            }
        }
    ]
   /*********************************************
    * </STARTUP>  
    *********************************************/
    content: Container {
        id: mainContainer
    	layout: DockLayout { }
        
        Container {
            layout: StackLayout {
                orientation: LayoutOrientation.TopToBottom
    	    }
    	    ScreenTitle {
    	        largeText: false
    	        showNowPlaying: true
    	        title: qsTr("Recent Music") + Retranslate.onLanguageChanged
                horizontalAlignment: HorizontalAlignment.Fill 
    	    } 
            Container {
    	        layout: DockLayout { }
    	        id: smartPage
    	        visible: false // update to "true" when RenderFence is reached (MLTMA-11492)
    	        
    	        layoutProperties: StackLayoutProperties {
    	            spaceQuota: 100
    	        }
    	        
    	        property int defaultInitialCount: LayoutConstants.SMART_SCREEN_SECTION_SIZE
    	        property int defaultMaxCount: LayoutConstants.SMART_SCREEN_MAX_SIZE
    	        property int defaultIncrement: LayoutConstants.SMART_SCREEN_SECTION_SIZE 
    
    	        attachedObjects: [            
    	            //combined expanding data models
    	            ModelCombiner {
    	                id: smartDataModel
    	
    	                //recently added
    	                ExpandingDecorator {
    	                    initialCount: smartPage.defaultInitialCount
    	                    maxCount: smartPage.defaultMaxCount
    	                    expandIncrement: smartPage.defaultIncrement
    	                    objectName: "RecentlyAdded" //used by analytics
    	                    model: HeaderTypeDecorator {
    	                        headerType: "added"
    	                        itemType: "default"
    	                        model: smartRecentlyAddedDataModel
							}
		       			}//end recently added 
	               		ExpandingDecorator {
    	                    initialCount: smartPage.defaultInitialCount
    	                    maxCount: smartPage.defaultMaxCount
    	                    expandIncrement: smartPage.defaultIncrement
                            objectName: "RecentlyPlayed" //used by analytics
    	                    model: HeaderTypeDecorator {
    	                        headerType: "played"
    	                        itemType: "default"
				                //created in MusicApp::startPrimaryDataLoad
				                model: smartRecentlyPlayedDataModel 
			                }
		                }//recently played
    	            },//end combiner
    	            SelectableDataModelDecorator {
    	                id: selectableSmartDataModel
    	                model: smartDataModel
    	            }
    	        ]
	        
    	        onCreationCompleted: {
    	            smartDataModel.relay.connect(tabbedPane, "tabChanged()", "collapseAll()");
    	        }
	        
    	        function onFirstHeaderLoaded() {
    	            smartListView.scrollToPosition(ScrollPosition.Beginning, ScrollAnimation.None);
    	        }
    	        
    	        function updateRecentlyPlayed(albumId){
    	            if(visible){
    	                //kick the model for the initial data pull
    	                if (albumId != 0) {
    	                    recentlyPlayedModel.load();
    	                }
    	            }    
    	        }

    	        BackgroundWrapper {
                    useAltPaint: smartListView.isMultiSelect
    	        }

    	        NoContentView {
    	            id: noContentView
                    image: "asset:///images/common/music_no_content_icon.png"
                    title: qsTr("No Music") + Retranslate.onLanguageChanged
                    subTitle: (MediaApplication.corporate ? qsTr("You are currently in the work space. Please switch to the personal space to see any personal files.\n\nFiles stored on the SD card are not accessible from the work space. Please switch to the personal space to see the files stored on the SD card.") 
                    : qsTr("You can use BlackBerry Link to sync music between your device and computer. \n\nTo download BlackBerry Link on your computer, visit\nwww.blackberry.com/BlackBerryLink")) + Retranslate.onLanguageChanged
                    horizontalAlignment: HorizontalAlignment.Fill
                    verticalAlignment: VerticalAlignment.Center
                    pageDataModel: smartRecentlyAddedDataModel
                }
    
    	        LibraryListView {
    	            id: smartListView 
    	            visible: !noContentView.visible
    	            contentType: MediaLibraryContentType.AlbumType
	                implicitLayoutAnimationsEnabled: false
    	            
    	            dataModel: selectableSmartDataModel
     	            property variant expandDataModel: smartDataModel
     	            bottomPadding: 6
    
                    layout: GridListLayout {
    	                headerMode: ListHeaderMode.Sticky
    	                columnCount: 3
    	            }
    	            
    	            listItemComponents: [
    	                ListItemComponent {
    	                    type: "default"
                            DeletableThumbsListItem {
                                id: smartScreenListItem
                                //data binding 
                                labelText: ListItemData["album"]
                                bottomLabelText: !ListItemData["album"] ? ListItemData["album_artist"] : ""
                                bottomLabelVisible: !ListItemData["album"]
                                thumbUri: ListItemData["thumb_file"]
                                contentType: MediaLibraryContentType.AlbumType
                                itemSelected: ListItemData["isSelected"]
                                multiSelectionMode: ListItem.view.isMultiSelect
                                property alias isExpandTrigger: expandOverlay.visible
                                clipContentToBounds: false       
                                //enable the overlay if this item is the trigger 
                                ListItem.onInitializedChanged: {
                                    if (initialized) {
                                        evaluateTriggerStatus();
                                    }
                                }
                                ListItem.onIndexInSectionChanged: {
                                    evaluateTriggerStatus();
                                }
                                function evaluateTriggerStatus() {
                                    isExpandTrigger = false;
                                    // Found in live update case, when this function is called upon indexInSectionChanged signal,
                                    // ListItem.sectionSize is not correct when size is less than defaultInitialCount. Need further
                                    // investigation to see if it is a Cascades bug. For now use ModelCombiner.childCount instead here 
                                    // since it is accurate.
                                    var ip = ListItem.indexPath;
                                    var pip = ip.slice(0, ip.length-1); //parent indexPath
                                    var actualSectionSize = ListItem.view.expandDataModel.childCount(pip);
                                    if (ListItem.indexInSection == actualSectionSize - 1) {
                                        var context = ListItem.view.expandDataModel.getContext(ListItem.indexPath);
                                        if (ListItem.view.expandDataModel.relay.invokeSlot("canExpandParent(QVariant)", context.getLocalIndexPath(ListItem.indexPath), context, true)) {
                                            isExpandTrigger = true;
                                            if (ip[0] == 0) {
                                                // we only need to translate the expander down if it is in the Recently Added section.  The Recently Played expander is aligned properly already.
                                                expandOverlay.translationY = LayoutConstants.EXPANDER_ICON_TRANSLATION_Y
                                            }
                                        }
                                    }
                                }
                                function expandList() {
                                    var context =  ListItem.view.expandDataModel.getContext(ListItem.indexPath);
                                    isExpandTrigger = false;
                                    ListItem.view.expandDataModel.relay.invokeSlot("expandParent(QVariant)", context.getLocalIndexPath(ListItem.indexPath), context, false);
    	                            var indexPath = ListItem.indexPath;
                                    if (LayoutConstants.EXPANDER_SCROLL_EXTRA_ROW)
                                    {
                                        //scrolls one row further to ensure 'expand' button fully shows
                                        indexPath[1] = indexPath[1]+1;
                                    }
                                    ListItem.view.scrollToItem(indexPath);
                                }
                                
                                function refreshItems(){
                                    ListItem.view.dataModel.itemUpdated(ListItem.indexPath);
                                }
                                
                                onCreationCompleted:{
                                    ListItem.view.dataModel.selectionChanged.connect(refreshItems)
                                }
                                Container {
                                    clipContentToBounds: false
                                    ImageButton {
                                        objectName: "RecentExpanderObjectName"
                                        id: expandOverlay
                                        visible: false
                                        defaultImageSource: "asset:///images/common/core_expand_inline.png"
                                        pressedImageSource: "asset:///images/common/core_expand_inline_pressed.png"
                                        disabledImageSource: "asset:///images/common/core_expand_inline.png"
                                        translationX: LayoutConstants.EXPANDER_ICON_TRANSLATION_X
                                        onClicked: {
                                            expandList();
                                        }
                                    }
                                    horizontalAlignment: HorizontalAlignment.Right
                                    verticalAlignment: VerticalAlignment.Bottom
                                }
                            
                            }
    	                },
    	                ListItemComponent {
    	                    type: "played_header"
    	                    Header {
    	                        id: smartScreenRecentlyPlayedHeader
    	                        title: qsTr("Recently Played") + Retranslate.onLanguageChanged
    	                    }
    	                },
    	                ListItemComponent {
    	                    type: "added_header"
    	                    Header {
    	                        id: smartScreenRecentlyAddedHeader
    	                        title: qsTr("Recently Added") + Retranslate.onLanguageChanged
    	                    }
    	                }            
    	            ]
    	            function pushDetailsScreen(indexPath) {
                        var data = smartListView.dataModel.data(indexPath);
                        var page = currentNavPane.safePushOnePage("core/ArtistDetails.qml");
                        commonAnalytics.instance.logPagePush("ArtistDetails", "Smart");
    	                page.openAtAlbumId = data.album_id;
    	                page.albumArtistIdStr = data.album_artist_id;
    	                //: album artist is not known for display on top of album art
    	                page.titleStr = data.album_artist;
    	            }
    	        }
    	        MusicListViewHelper {
    	            listView: smartListView 
    	        }
    	    }
    	}
	}
}
