import bb.cascades 1.0
import bb.cascades.multimedia 1.0
import bb.cascades.multimedia.internal 1.0
import bb.system 1.0

/**
 * The list item visual for thumbnails on the Smart.qml page.
 * @param isExpandTrigger     Boolean for the visibility of the expanding list overlay
 */
ThumbsListItem{
    id: smartListItem
    signal expandList();
    signal itemSelected();
    property bool __ignoreUp : false
    property alias isExpandTrigger: expandOverlay.visible
    property bool active: ListItem.active
    
    property bool multiSelectionMode: false
    property bool selected: false
   
   property variant contentType
   property SystemDialog deleteDialog
   
	//default properties
	defaultImage: "asset:///images/common/album_placeholder.png"
	pressImageSource: "asset:///images/common/core_listitem_pressed.png.amd"
	selectImageSource: "asset:///images/common/core_griditem_pressed.png.amd"
    
	defaultContentPadding: 0
	preferredHeight: 256
	preferredWidth: 256
	
	opacity: (selected || !multiSelectionMode) ? 1.0: 0.6
	
    onTouch: {
        if (event.isUp() && enabled) {
            if(__ignoreUp){
                __ignoreUp = false;
            } else {
                if(!multiSelectionMode){
                    itemSelected();
                }
            }
        }
    }
	
	contextActions: [
        ActionSet {
            DeleteActionItem {
                onTriggered: {
                    deleteDialog = appContext.getDeleteSystemDialog(contentType, 1);
                    deleteDialog.finished.connect(dialogCallback);
                    deleteDialog.show();
                }
            }
        }
    ]
    
    function dialogCallback(result) {
        if (deleteDialog.result == SystemUiResult.ConfirmButtonSelection) {
            if (ListItemData["key_id"]) {
                deleteManager.deleteContentByIds(contentType, ListItemData["key_id"]);
            }
       }
    }
	
    Container {
        horizontalAlignment: HorizontalAlignment.Right
        verticalAlignment: VerticalAlignment.Bottom
        
        ImageView {
            id: expandOverlay
            visible: false
            imageSource: "asset:///images/common/core_expand_inline.png"
        }
        onTouch: {
            if (event.isUp () && enabled) {
                __ignoreUp = true;
                expandList ();
                expandOverlay.imageSource = "asset:///images/common/core_expand_inline.png";
            }else if(event.isCancel()){
                expandOverlay.imageSource = "asset:///images/common/core_expand_inline.png"
            }else if(event.isDown()){
                expandOverlay.imageSource = "asset:///images/common/core_expand_inline_pressed.png";
            }                          
        }
    }
    
    onCreationCompleted:{
        ListItem.activationChanged.connect(smartListItem.onActivationChanged);
    }
}