import bb.cascades 1.0
import bb.cascades.multimedia 1.0
import bb.cascades.multimedia.internal 1.0
import bb.cascades.music 1.0
import bb.system 1.0
import "PlUtils.js" as PlUtils
import "ContentUris.js" as Uris

Page {
    id: smartPlaylistDetails

    property alias contentURI: playlistList.contentURI
    property alias headerURI: playlistList.headerURI
    property alias type: playlistList.type
    property alias titleStr: screenTitle.title
    property variant headerDataModel // Dont use alias for custom type or it will result in segfault
    property alias index: playlistList.index
    property bool  showSaveAsPlaylist: false
    property bool  saveAsPlaylistAdded: false
    
    property M3uPlaylistDataModel createNewPlaylistDataModel // used to save the last played playlist
    

    actions: [
        NowPlayingActionItem {
            page: "SmartPlaylistDetails"
        },
	    ActionItem {
            title: qsTr("Play All") + Retranslate.onLanguageChanged
            imageSource: "asset:///images/playing/ic_play.png"
            onTriggered: {
                dynamicPlaylistManager.startPlaybackByIndex(
                    contentURI,
                    false, //shuffle off 
                    0);
            }
        },
	    ActionItem {
	        title: qsTr("Shuffle All") + Retranslate.onLanguageChanged
	        imageSource: "asset:///images/playing/ic_shuffle.png"
	        ActionBar.placement: ActionBarPlacement.OnBar
	        
	        onTriggered: {
                dynamicPlaylistManager.startPlaybackByIndex(
	                contentURI,
	                true, //shuffle on 
	                -1);
                commonAnalytics.instance.logShuffleAll("SmartPlaylistDetails");
	        }
	    },
        ActionItem {
            //: menu item for search
            title: qsTr("Search") + Retranslate.onLanguageChanged
	        imageSource: "asset:///images/common/ic_search.png"
            onTriggered: {
                playlistDetailsSearchScreen.display();
            }
        },
	    MultiSelectActionItem {
	        onTriggered: {
                playlistList.listView.selectableMultiHandler.active = true;
	        }
	    }
    ]
    
    attachedObjects: [
        ActionItem {
            id: saveAsPlaylistAction
            title: qsTr("Save as Playlist") + Retranslate.onLanguageChanged
            imageSource: "asset:///images/common/ic_done.png"
            onTriggered: {
                createNewPlaylistDataModel = musicUtils.createEmptyM3uPlaylistDataModel();
                saveDialog.inputField.defaultText = createNewPlaylistDataModel.generateNewFileName();
                saveDialog.show();
            }
        },
        SystemPrompt {
           id: saveDialog
           title: qsTr ("Type a name") + Retranslate.onLanguageChanged
           inputField.emptyText: qsTr("Type Playlist Name") + Retranslate.onLanguageChanged
           confirmButton.label: qsTr ("Create") + Retranslate.onLanguageChanged
           cancelButton.label: qsTr ("Cancel") + Retranslate.onLanguageChanged
        }
    ]
    
    onCreationCompleted: {
        playlistDetailsSearchScreen.rootPage = smartPlaylistDetails;
        playlistDetailsSearchScreen.hide();
    }
    
    onShowSaveAsPlaylistChanged: {
        
        if (showSaveAsPlaylist && !saveAsPlaylistAdded) {
            // add action item on the bar
            qmlUtils.addActionItemToPage(saveAsPlaylistAction, smartPlaylistDetails, 2);
            
            // Connect to the Confirm button on the Save As dialog to commit
            // the "Last Played" playlist to a user created playlist
            saveDialog.finished.connect(onSaveDialogAccepted);
            
            saveAsPlaylistAdded = true;
        }    
    }
    
    function onSaveDialogAccepted() {
		if(saveDialog.result == SystemUiResult.ConfirmButtonSelection){
			createNewPlaylistDataModel.setName(saveDialog.inputFieldTextEntry());
			createNewPlaylistDataModel.append(Uris.lastPlayedPlayListForSaving());
            createNewPlaylistDataModel.persistCompleted.connect(onPersistCompleted);
            createNewPlaylistDataModel.persistData();
        } else {
            //clean up since user canceled the dialog
            createNewPlaylistDataModel.destroy();
            createNewPlaylistDataModel = null;
        }
    }

    function onPersistCompleted(result, plid, filename) {
        if (result == PlaylistEditErrorType.PLERROR_OK) {
            systemDialogUtils.showSystemToast(qsTr("A new playlist: \"%1\" has been created").arg(createNewPlaylistDataModel.getName()));
        } else {
            systemDialogUtils.showSystemToast(PlUtils.getErrorText(result));
        }
        //clean up the model
        createNewPlaylistDataModel.destroy();
        createNewPlaylistDataModel = null;
    }
    
    onActionMenuVisualStateChanged: {
        playlistDetailsSearchScreen.enableTypeToSearch(ActionMenuVisualState.Hidden == actionMenuVisualState);
    }

    content: Container {
        id: mainContainer
        layout: DockLayout {
        }

        Container {
            layout: StackLayout {
                orientation: LayoutOrientation.TopToBottom
            }
            ScreenTitle {
                id: screenTitle
                showNowPlaying: true
                horizontalAlignment: HorizontalAlignment.Fill
                layoutProperties: StackLayoutProperties {                
                }
            }
            Container {
                horizontalAlignment: HorizontalAlignment.Fill
                layoutProperties: StackLayoutProperties {
                    spaceQuota: 100
                }
                layout: DockLayout {}
                BackgroundWrapper {}
                SmartPlaylistList {
                    id: playlistList
                    headerModel: headerDataModel
                }
            }
        }
        
        Search { // PlaylistDetails need its own instance of Search screen
        	// because it may be launched from the main Search screen
        	// (see also tabbedPane.searchScreen).
        	id: playlistDetailsSearchScreen
        }
    }
}
