import bb.cascades 1.0
import bb.cascades.multimedia 1.0
import bb.cascades.multimedia.internal 1.0
import bb.cascades.music 1.0
import bb.system 1.0
import "layoutconstants.js" as LayoutConstants

Container {
    property string contentURI
    property string headerURI
    property string type
    property bool lastPlayed: type == "lastplayed"
    property variant headerModel
    property variant index
    property alias listView: selectableList
    
    id: playlistList
    
    rightPadding: LayoutConstants.TRACKS_LIST_VIEW_PADDING
    leftPadding: LayoutConstants.TRACKS_LIST_VIEW_PADDING
    
    layout: DockLayout {
    }
    
    attachedObjects: [
        AsyncHeaderDataModel {         
            id: dataModel
            objectName: "SmartPlaylist"
            query: MediaDataQueryDecorator {
                query: ContentUriHeaderDataQuery {
                    id: dataQuery   
                }  
            }
        },
        SelectableDataModelDecorator {
            id: selectableModel
            model: PrioritizedDataModelDecorator {
                model: CurrentSongDecorator {
                    id: curSongDecorator
                    checkLpOid: lastPlayed
                    playbackControllerRef: appContext.playbackController
                    dynamicPlaylistManager: appContext.dynamicPlaylistManagerRef
                    model: dataModel
                }
            }
        },
        ListViewConnection {
            id: smartListViewConnection
            listView: selectableList
            key: "title"
        }
    ]
    
    animations: [
        LoadListTransition {
            target: selectableList
            dataModel: dataModel
        }
    ]
    
    onTypeChanged: {
        if (type == "alltracks") {
        	dataQuery.indexEnabled = true;
        } else if (type == "recentlyadded") {
            dataQuery.indexEnabled = true;
            // set a unique index id since recently added query changes periodically.
            dataQuery.indexId = "smartplaylist_recentlyadded";
        }
    }
    
    onContentURIChanged: {
        dataQuery.uri = contentURI;
        dataQuery.headerUri = headerURI;
        dataModel.load();
    }

    function handleItemsChanged() {
        var data = headerModel.data(index);
        selectableList.fourSquare1 = data["four_square_1"];
        selectableList.fourSquare2 = data["four_square_2"];
        selectableList.fourSquare3 = data["four_square_3"];
        selectableList.fourSquare4 = data["four_square_4"];
    }

    SelectableMusicListView{
        id:selectableList
        dataModel: selectableModel
        topPadding: LayoutConstants.TRACKS_LIST_VIEW_PADDING
        bottomPadding: LayoutConstants.TRACKS_LIST_VIEW_PADDING
        contentType: lastPlayed? MediaLibraryContentType.LastPlayedEntryType : MediaLibraryContentType.AudioType
        bufferedScrollingEnabled: true
        scrollIndicatorMode: ScrollIndicatorMode.Default

        property bool isMultiSelect: false;
        
        property variant outsideContainer: playlistList 
        property string fourSquare1
        property string fourSquare2
        property string fourSquare3
        property string fourSquare4
        property int songIndexPath
        property variant songDecorator: curSongDecorator

        //a function to scroll to the currently playing track
        function scrollToCurrentSong(){
            appContext.playbackController.currentIndexChanged.disconnect(scrollToCurrentSong);
            songIndexPath = musicUtils.getIndexByKey(appContext.playbackController.currentSong.fid, outsideContainer.contentURI, "fid=", dataModel, 0);
            selectableList.scrollToAsyncModelItem([0,songIndexPath]); //[0,] is added to indicate every song is under the first and only header
        }

       onDataModelChanged: {
            dataModel.itemsChanged.connect(playlistList.handleItemsChanged);
            dataModel.itemAdded.connect(playlistList.handleItemsChanged);
            dataModel.itemRemoved.connect(playlistList.handleItemsChanged);
        }

        layout: StackListLayout {}
        
    	listItemComponents: [
            ListItemComponent {
                type: ""
                SimpleTwoRowListItemVisual {
                    title: ListItemData["title"]
                    subTitle: ListItemData["artist"]
                    track: ListItem.indexInSection + 1
                    duration: ListItemData["duration"]
                    controller: ListItem.view.songDecorator
                    multiSelectionMode: ListItem.view.isMultiSelect
                    itemSelected: ListItemData["isSelected"]
                    allowDeleteAction: (ListItem.view.contentType != MediaLibraryContentType.LastPlayedEntryType)
                }
            },
            ListItemComponent {
    			type: "fullItem"
                ListItemVisual{
	                title: ListItemData["title"]
                    track: ListItem.indexInSection + 1
                    duration: ListItemData["duration"]
                    fullpath: ListItemData["fullpath"]
                    subTitle: ListItemData["artist"]
                    showSubTitle: true
                    itemSelected: ListItemData["isSelected"]
                    multiSelectionMode: ListItem.view.isMultiSelect
                    controller: ListItem.view.songDecorator
                    currSong: ListItemData["currSong"]
                    allowDeleteAction: (ListItem.view.contentType != MediaLibraryContentType.LastPlayedEntryType)
                    
                }
    		},
            ListItemComponent {
    			type: "header"
				CountDurationHeaderWithCover {
				    fourSquareVisible: true
				    id: smartPlaylistHeader
				    duration: ListItemData["duration"]
			        count: ListItemData["count"]
			        thumb1Uri: ListItem.view.fourSquare1
                    thumb2Uri: ListItem.view.fourSquare2
                    thumb3Uri: ListItem.view.fourSquare3
                    thumb4Uri: ListItem.view.fourSquare4
                    defaultImage: "asset:///images/common/album_placeholder.png"
                    onShuffleList: {
                        appContext.playbackController.currentIndexChanged.connect(ListItem.view.scrollToCurrentSong);
                        dynamicPlaylistManager.startPlaybackByIndex(
                            ListItem.view.outsideContainer.contentURI,
                            true, //shuffle on
                            -1);
                        commonAnalytics.instance.logShuffleAll("SmartPlaylistListHeader");
                     }
                }
    		}
    	]

    	onTriggered: {
	        if ( !dataModel.hasChildren(indexPath) ) {
	            var selectedItem = selectableList.dataModel.data(indexPath);
                if(selectedItem.fullpath == appContext.playbackController.currentSong.fullpath && 
                    appContext.playbackController.currentSong.fullpath != "" &&  appContext.playbackController.currentSong.fullpath != null &&
                    (!lastPlayed || musicUtils.getLpOidFromCurrentSong(appContext.playbackController.currentSong) == selectedItem.lp_oid)){
                    //selected an already playing track, play/pause toggle
                    if ( appContext.playbackController.playbackState == PlaybackController.PlaybackPlaying ) {
			            appContext.playbackController.pause()
			        } else {
			            appContext.playbackController.play()
		            }
                }
                else{
	                var albumId = selectedItem.album_id;
                    var copyToLastPlayed = true;
                    if (type == "lastplayed") {
                        copyToLastPlayed = false;
                    }
                    dynamicPlaylistManager.startPlaybackByIndex(
                       playlistList.contentURI,
                       appContext.playbackController.shuffleMode == PlaybackController.ShuffleOn, //shuffle works now 
                       indexPath[indexPath.length -1],
                       "0",
                       PlaybackController.PlaybackPlaying,              
                       selectedItem,
                       copyToLastPlayed);
       	        }
       	        clearSelection();
                selectableList.select (indexPath, false);
            }
    	}
    }
    
    // TODO: Disable Scrolladex for now because the performance is really bad on Smart Playlist screens.
    /*Scrolladex {
        id: smartScrolladex
        horizontalAlignment: HorizontalAlignment.Right
        verticalAlignment: VerticalAlignment.Fill
        listViewConnection: smartListViewConnection
        toolTipMode: ToolTipMode.None
    }*/
}
