import bb.cascades 1.2
import bb.cascades.multimedia 1.0
import bb.cascades.multimedia.internal 1.0

import "ContentUriHelper.js" as ContentUriHelper

Page {
  id: albumsPageId
  objectName: "AlbumsPageObjectName"

  property string rootAlbumsContentUri: ""
  property string subAlbumContentUri: ""
  property string subAlbumMembersContentUri: ""

  property string previousAlbumName: ""
  property string albumName: ""
  property string albumId: "0"
  property bool isMultiSource: false

  property int listViewPaddingBottom: 0
  property int gridListItemPadding: 0
  property int albumsColumnCount: 0
  property int itemColumnCount: 0
  property double itemAspectRatio: 1

  property string itemType: ""

  property alias selectedStorage: mainContainer.selectedStorage

  //For folders, key_id and path_norm are the same.
  property string folderKeyId: ""

  // We load the data when albumId changed. Since albumId is not set onCreationCompleted.
  onAlbumIdChanged: {
      // Check if it is the root album
      if (albumId == 1) {
          dataQueryAlbums.uri = rootAlbumsContentUri;
      } else {
          dataQueryAlbums.uri = ContentUriHelper.subAlbumContentDataUri(subAlbumContentUri, albumId);
          dataQueryMembers.uri = ContentUriHelper.subAlbumMembersContentDataUri(subAlbumMembersContentUri, albumId);
      }

      albumsDataModel.load(); //load the sub album query first and then as this model has loaded, load the content model.
  }

  attachedObjects: [
    ModelCombiner {
        // Albums Model 
        id: albumsPageModel
        property bool modelsLoaded: false
        HeaderTypeDecorator {
            headerType: "empty"
            itemType: "folder_item"
            model: FolderDataModelDecorator {
                model: AsyncHeaderDataModel {
                    id: albumsDataModel
                    query: ForcedUpdateQueryDecorator{
                        query: MediaDataQueryDecorator {
                            query: ContentUriHeaderDataQuery {
                                id: dataQueryAlbums
                                // This will lead to that we return one dummy header item.
                                // Needed since a FlowListLayout can not layout the transition from folders
                                // to items correctly. 
                                headerUri: ""  
                            }
                        }
                    }

                    onItemsChanged: {
                      thumbListView.itemCount = getAlbumsCount();
                    }

                    onLoaded: {
                      if (albumsPageId.albumId != 1) {
                        albumsMembersDataModel.load();
                      } else {
                        albumsPageModel.modelsLoaded = true;
                      }
                    }                  
                }
            }
        }
        HeaderTypeDecorator {
            headerType: "empty"
            itemType: albumsPageId.itemType
            model: AsyncHeaderDataModel {
                id: albumsMembersDataModel
                query: MediaDataQueryDecorator {
                    query: ContentUriHeaderDataQuery {
                        id: dataQueryMembers
                        // This will lead to that we return one dummy header item.
                        // Needed since a FlowListLayout can not layout the transition from folders
                        // to items correctly.
                        headerUri: ""
                    }    
                }

                onLoaded: {
                    albumsPageModel.modelsLoaded = true;
                }
            }
        }

        onItemsChanged: {
            albumsPageModel.checkIfDataModelEmpty();
        }

        function checkIfDataModelEmpty() {
            if (albumsPageModel.modelsLoaded) {
                if (albumsPageId.getTotalItemCount() == 0) {
                    popPageFromNavigationPane();
                }
            }
        }
    },
    SelectableDataModelDecorator {
        id: selectableModel
        model: albumsPageModel
    }
  ]

  MovePageContainer {
    id: mainContainer
    showDropDown: albumId != "1" && isMultiSource
    layout: StackLayout {}

    Header {
      //: Title in list header that displays the amount of items/albums in the list. %1 is the amount of albums.
      title: albumsPageId.albumName != "" ? albumsPageId.albumName : (qsTr ("Albums: %1") + Retranslate.onLanguageChanged).arg(thumbListView.itemCount)
    }

    ListView {
      id : thumbListView
      // Related to automatic hiding of an eventual action/tab bar when scrolling down. The default value of Page.actionBarAutoHideBehavior
      // means that an eventual action/tab bar will be hidden on small screens and stay visible on large screens. In order to achieve
      // the hide behaviour on all devices, actionBarAutoHideBehavior must be set to ActionBarAutoHideBehavior.HideOnScroll
      scrollRole: ScrollRole.Main
      dataModel: selectableModel
      
      property int itemCount: 0;
      property int albumsColumnCount: albumsPageId.albumsColumnCount
      property int itemColumnCount: albumsPageId.itemColumnCount
      property int gridListItemPadding: albumsPageId.gridListItemPadding
      property double itemAspectRatio: albumsPageId.itemAspectRatio
      property string itemType: albumsPageId.itemType

      bottomPadding: listViewPaddingBottom

      layout: FlowListLayout {
      }

      attachedObjects: [
        ComponentDefinition {
            id: albumPageDefinition
            source: "AlbumsPage.qml"
        }
      ]

      listItemComponents: [
        ListItemComponent {
            type: "empty_header"
            Container {
                visible: false 
            }
        },
        ListItemComponent {
            type: "folder_item"
            MediaFolderListItem {
                id: folderItemId
                layoutProperties: FlowListLayoutProperties {
                    aspectRatio: folderItemId.ListItem.view.itemAspectRatio
                    fillRatio: 1 / folderItemId.ListItem.view.albumsColumnCount
                }

                  thumbUri1: ListItemData.four_square_1 ? ListItemData.four_square_1 : ""
                  thumbUri2: ListItemData.four_square_2 ? ListItemData.four_square_2 : ""
                  thumbUri3: ListItemData.four_square_3 ? ListItemData.four_square_3 : ""
                  thumbUri4: ListItemData.four_square_4 ? ListItemData.four_square_4 : ""
                  folderName: ListItemData.localizedFolderName ? ListItemData.localizedFolderName + Retranslate.onLanguageChanged : ""
                  
                  imageCount: ListItemData.image_count ? ListItemData.image_count : 0
                  // For contentUri with video as contentType (eg content://media/shared/video/*) we rely on count instead.
                  videoCount: ListItemData.video_count ? ListItemData.video_count : ListItemData.count ? ListItemData.count : 0

                  leftPadding: ListItem.view.gridListItemPadding
                  rightPadding: ListItem.view.getItemPaddingRight(ListItem.indexInSection, ListItem.view.albumsColumnCount,
                              ListItem.view.gridListItemPadding)
                  topMargin: ListItem.view.gridListItemPadding
                  bottomMargin: ListItem.view.gridListItemPadding
            }
        },
        ListItemComponent {
          type: "picture_item"
          MediaListItem {
            id: itemId
            layoutProperties: FlowListLayoutProperties {
                aspectRatio: itemId.ListItem.view.itemAspectRatio
                fillRatio: 1 / itemId.ListItem.view.itemColumnCount
            }

            opacity: 0.7

            leftPadding: ListItem.view.gridListItemPadding
            rightPadding: ListItem.view.getItemPaddingRight(ListItem.indexInSection, ListItem.view.itemColumnCount,
                          ListItem.view.gridListItemPadding)
            topMargin: ListItem.view.gridListItemPadding
            bottomMargin: ListItem.view.gridListItemPadding
          }
        },
        ListItemComponent {
          type: "video_item"
          VideoListItem {
            id: itemId
            layoutProperties: FlowListLayoutProperties {
                aspectRatio: itemId.ListItem.view.itemAspectRatio
                fillRatio: 1 / itemId.ListItem.view.itemColumnCount
            }

            opacity: 0.7

            title: ListItemData.title
            duration: ListItemData.duration
            thumb_uri: ListItemData.thumb_file

            leftPadding: ListItem.view.gridListItemPadding
            rightPadding: ListItem.view.getItemPaddingRight(ListItem.indexInSection, ListItem.view.itemColumnCount,
                          ListItem.view.gridListItemPadding)
            topMargin: ListItem.view.gridListItemPadding
            bottomMargin: ListItem.view.gridListItemPadding
          }
        }
      ] // end ListItemComponents

      verticalAlignment: VerticalAlignment.Fill
      horizontalAlignment: HorizontalAlignment.Fill

      // Signal emitted when a user tap an item. Is not emitted when multiSelecthandler.active is set to true
      // for some reason.
      onTriggered: {
          // Check if triggered item is an image item (not header)
          if (!dataModel.hasChildren(indexPath)) {
              var data = dataModel.data(indexPath);
              //differentiate between a folder and a image/video (a folder doesn't have an fid)
              if(data.fid == null) {
                  var subAlbumPage = pushSubPage();

                  // Here we need to check the folderName if it's the specialized root album.
                  // In that case in order to workaround an mmlibrary issue with recursive albums
                  // we need to have a dummy album contentUri that doesn't return any albums.
                  if(data.folder_name == "s/" || data.folder_name == "os/" || data.folder_name == "/") {
                      subAlbumPage.subAlbumContentUri = ContentUriHelper.dummyContentDataUri();
                  } else {
                      subAlbumPage.subAlbumContentUri = albumsPageId.subAlbumContentUri;
                  }

                  subAlbumPage.subAlbumMembersContentUri = albumsPageId.subAlbumMembersContentUri;
                  subAlbumPage.albumName = data.localizedFolderName;
                  subAlbumPage.previousAlbumName = albumsPageId.albumName
                  subAlbumPage.albumId = data.folder_id;
                  subAlbumPage.isMultiSource = data.multi_source;
                  subAlbumPage.folderKeyId = data.key_id;
              }
          }
      }

      function getItemPaddingRight(indexInSection, columnCount, padding) {
        return indexInSection % columnCount == (columnCount - 1) ? padding : 0
      }

    } // End ListView
  } // End Container

  function getAlbumsCount () {
    var indexPath = new Array ();
    indexPath.push(0);
    var count = 0;
    if (albumsDataModel) {
      count = albumsDataModel.childCount(indexPath);
    }
    return count;
  }

  function getTotalItemCount() {
    var indexPath = new Array ();
    indexPath.push(0);
    var count = 0;
    if (albumsMembersDataModel) {
      count = albumsMembersDataModel.childCount(indexPath);
    }
    count += getAlbumsCount();
    return count;
  }

  function popPageFromNavigationPane() {
      var indexOfThisPage = navigationPane.indexOf (albumsPageId);
      if (indexOfThisPage == -1) {
          console.log("Album page already popped, returning!");
          return;
      }
      
      var pagesToPop = navigationPane.count() - indexOfThisPage;
      if (pagesToPop > 1) {
          console.log("Album page: pop " + pagesToPop + " pages");
      }
      for (var i = 0; i < pagesToPop; ++i) {
          navigationPane.pop();
      }
  }
}
