/**
 * AudioUtils.js
 * 
 * Research In Motion Limited proprietary and confidential.
 * Copyright (C) Research In Motion Limited, 2013.
 */

/**
 * Get the audio device string.
 * 
 * @param type The type of the audio device.
 * @return The name of the audio device type.
 */
function getAudioDeviceName(type) {
	var deviceName = "";
	switch (type) {
		case AudioDevice.Speaker:
			deviceName = qsTr("Speaker") + Retranslate.onLanguageChanged;
			break;
		case AudioDevice.Headset:
		case AudioDevice.Headphone:
		case AudioDevice.TTY:
		case AudioDevice.Toslink:
		case AudioDevice.Lineout:
			deviceName = qsTr("Headset") + Retranslate.onLanguageChanged;
			break;
		case AudioDevice.A2DP:
		case AudioDevice.BTSCO:
			deviceName = qsTr("Bluetooth") + Retranslate.onLanguageChanged;
			break;
		case AudioDevice.Handset:
			deviceName = qsTr("Handset") + Retranslate.onLanguageChanged;
			break;
		case AudioDevice.HAC:
			deviceName = qsTr("Hearing Aid") + Retranslate.onLanguageChanged;
			break;
		case AudioDevice.HDMI:
			deviceName = qsTr("HDMI") + Retranslate.onLanguageChanged;
			break;
		case AudioDevice.USB:
			deviceName = qsTr("USB") + Retranslate.onLanguageChanged;
			break;
		case AudioDevice.WifiDisplay:
			deviceName = qsTr("Miracast") + Retranslate.onLanguageChanged;
			break;
		default:
			deviceName = qsTr("Unknown") + Retranslate.onLanguageChanged;
			break;
	}
	return deviceName;
}

/**
 * Get the icon path of the audio device type.
 * @param type The audio device type.
 * @returns The path of the icon.
 */
function getAudioDeviceIcon(type) {
	var icon = "";
	switch (type) {
		case AudioDevice.Speaker:
			icon = "asset:///imports/bb/cascades/multimedia/internal/images/ic_speaker.png";
			break;
		case AudioDevice.Headset:
			icon = "asset:///imports/bb/cascades/multimedia/internal/images/ic_headset.png";
			break;	
		case AudioDevice.Headphone:
			icon = "asset:///imports/bb/cascades/multimedia/internal/images/ic_headphone.png";
			break;
		case AudioDevice.A2DP:
			icon = "asset:///imports/bb/cascades/multimedia/internal/images/ic_bluetooth.png";
			break;
		case AudioDevice.Handset:
			icon = "asset:///imports/bb/cascades/multimedia/internal/images/ic_bb_generic.png";
			break;
		case AudioDevice.HAC:
			icon = "asset:///imports/bb/cascades/multimedia/internal/images/ic_hac.png";
			break;
		case AudioDevice.BTSCO:
			icon = "asset:///imports/bb/cascades/multimedia/internal/images/ic_bt_headset.png";
			break;
		case AudioDevice.HDMI:
			icon = "asset:///imports/bb/cascades/multimedia/internal/images/ic_hdmi.png";
			break;
		case AudioDevice.Toslink:
			icon = "asset:///imports/bb/cascades/multimedia/internal/images/ic_toslink.png";
			break;
		case AudioDevice.TTY:
			icon = "asset:///imports/bb/cascades/multimedia/internal/images/ic_tty.png";
			break;
		case AudioDevice.Lineout:
			icon = "asset:///imports/bb/cascades/multimedia/internal/images/ic_lineout.png";
			break;
		case AudioDevice.USB:
			icon = "asset:///imports/bb/cascades/multimedia/internal/images/ic_usb.png";
			break;
		case AudioDevice.WifiDisplay:
			icon = "asset:///imports/bb/cascades/multimedia/internal/images/ic_miracast.png";
			break;
		default:
			icon = "";
			break;
	}
	return icon;
}