import bb.cascades 1.0
import bb.cascades.multimedia 1.0

/**
 * This is a handy ListView for ContentUriDataModel.
 * 
 * itemSelected(variant selectedItem) signal is notified when an item is selected.
 * The "selectedItem" contains all the data of the selected item from the DataModel
 *
 * @param   dataModel          The ContentUriDataModel for the ListView
 * @param   listItemManager    The ListItemManager for the ListView
 * @param   headers            boolean value. If true, enable the sticky headers;
 *                             otherwise false                           
 */
Container {
    id: listView
    
    // in order to fill the entire region given
    layout: StackLayout{ orientation: LayoutOrientation.TopToBottom }
    
    property variant listItemManager
    property variant dataModel
    property alias headers: list.stickyHeaders
    
    function propertiesInited() {
        if (listItemManager != null && dataModel != null) {
            list.listItemManager = listItemManager;
            list.dataModel = dataModel;
        }
    }
    
    onListItemManagerChanged: {
        propertiesInited();
    }
    
    onDataModelChanged: {
        propertiesInited();
    }
    
    signal itemSelected(variant selectedItem, variant model, variant indexPath)

    // the list view
    ListView {
        id: list
        property int items: 0

        horizontalAlignment: HorizontalAlignment.Fill

        // a weight of 1 is used to fill the entire region
        layoutProperties: StackLayoutProperties { 
            spaceQuota: 100            
        }

        preferredWidth: listView.preferredWidth

        onSelectionChanged: {
            console.debug("indexpath: "+indexPath+", selected: "+selected);
            if (selected) {
                listView.itemSelected(dataModel.data(indexPath), dataModel, indexPath);
            }
        }
    }
}