/*
 * MediaFolderListItem.qml
 *
 * Copyright Research In Motion Limited, 2012-2013.
 * Research In Motion Limited. All rights reserved.
 */

import bb.cascades 1.0

Container {
    id: mediaFolderItem
    objectName: "mediaFolderItem"

    property int videoCount: 0                                  // the count of the videos present in the folder
    property int imageCount: 0                                  // the count of the images present in the folder
    property string thumbUri1: ""                               // path to thumbnail 1
    property string thumbUri2: ""                               // path to thumbnail 2
    property string thumbUri3: ""                               // path to thumbnail 3
    property string thumbUri4: ""                               // path to thumbnail 4
    property alias folderName: folderNameLabel.text             // name of this folder
    property bool showOverlay: true                             // Determins whether or not to show the overlay
    property alias overlayHeight: overlayContainerId.minHeight  // the height of the overlay
    property int labelPadding: 14                               // the padding for the text labels in the overlay

    layout: DockLayout {}
    implicitLayoutAnimationsEnabled: false

    Container {
        // container that includes thumbnails images 
        id: rootImageContainer
        horizontalAlignment: HorizontalAlignment.Fill
        verticalAlignment: VerticalAlignment.Fill
        implicitLayoutAnimationsEnabled: false

        Container {
            // Container for the 2x2 thumbnail grid
            id: imagesContainer
            
            layout: StackLayout  {
                orientation: LayoutOrientation.LeftToRight
            }

            layoutProperties: StackLayoutProperties {
                spaceQuota: 1.0
            }

            horizontalAlignment: HorizontalAlignment.Fill
            verticalAlignment: VerticalAlignment.Fill
            implicitLayoutAnimationsEnabled: false

            Container {
                // left thumbnails container.
                // When thumbnails count is 1, it shows thumb 1
                // When thumbnails count is 2, it shows thumb 1
                // When thumbnails count is 3, it shows thumb 1
                // When thumbnails count is 4, it shows thumb 1 and 3 
                layout: StackLayout  {
                    orientation: LayoutOrientation.TopToBottom
                }

                layoutProperties: StackLayoutProperties {
                    spaceQuota: 1.0
                }

                horizontalAlignment: HorizontalAlignment.Fill
                verticalAlignment: VerticalAlignment.Fill
                implicitLayoutAnimationsEnabled: false

                ThumbItem {
                    id: thumbItem1
                    thumbUri: thumbUri1

                    layoutProperties: StackLayoutProperties {
                        spaceQuota: 1
                    }
                }
                ThumbItem {
                    id: thumbItem3
                    thumbUri: thumbUri3

                    layoutProperties: StackLayoutProperties {
                        spaceQuota: 1
                    }
                }
            }

            Container {
                // right thumbnails container.
                // When thumbnails count is 1, it's invisible
                // When thumbnails count is 2, it shows thumb 2
                // When thumbnails count is 3, it shows thumb 2
                // When thumbnails count is 4, it shows thumb 2 and 4 
                id: rightContainer

                layout: StackLayout  {
                    orientation: LayoutOrientation.TopToBottom
                }

                layoutProperties: StackLayoutProperties {
                    spaceQuota: 1.0
                }

                horizontalAlignment: HorizontalAlignment.Fill
                verticalAlignment: VerticalAlignment.Fill
                implicitLayoutAnimationsEnabled: false
                ThumbItem {
                    id: thumbItem2
                    thumbUri: thumbUri2

                    layoutProperties: StackLayoutProperties {
                        spaceQuota: 1
                    }
                }
                ThumbItem {
                    id: thumbItem4
                    thumbUri: thumbUri4

                    layoutProperties: StackLayoutProperties {
                        spaceQuota: 1
                    }
                }
            }
        }
        ThumbItem {
            id: bottomThumbItem
            thumbUri: thumbUri3

            layoutProperties: StackLayoutProperties {
                spaceQuota: 1
            }
        }
    }

    // Overlay top level
    Container {
        layout: StackLayout {}
        
        visible: showOverlay
        horizontalAlignment: HorizontalAlignment.Fill
        verticalAlignment: VerticalAlignment.Bottom
        implicitLayoutAnimationsEnabled: false

        // Horizontal rule
        Container {
            horizontalAlignment: HorizontalAlignment.Fill
            verticalAlignment: VerticalAlignment.Top
            implicitLayoutAnimationsEnabled: false
            preferredHeight: 1
            background: Color.White
            opacity: 0.15
        }

        // Actual overlay
        Container {
            id: overlayContainerId
            layout: DockLayout {}
            
            background: Color.create("#A6000000")  // #000000 with 65% opacity, according to PR 231116
            horizontalAlignment: HorizontalAlignment.Fill
            implicitLayoutAnimationsEnabled: false
            minHeight: 80
            
            // Used to position the stack of labels/icons vertically. Must be child of DockLayout
            Container {
                verticalAlignment: VerticalAlignment.Center

                // Wrapper container for text label, to enable padding
                Container {
                    horizontalAlignment: HorizontalAlignment.Left
                    verticalAlignment: VerticalAlignment.Top
                    leftPadding: mediaFolderItem.labelPadding
                    rightPadding: mediaFolderItem.labelPadding
                    
                    Label {
                        // label that displays folder name
                        id: folderNameLabel
                        objectName: "folderNameLabel"
                        horizontalAlignment: HorizontalAlignment.Left
                        textStyle {
                            color: SystemDefaults.defaultText
                            fontSize: FontSize.XSmall
                            fontWeight: FontWeight.W500
                        }
                    }
                }

                // Wrapper container for icon labels going left->right
                Container {
                    layout: StackLayout {
                        orientation: LayoutOrientation.LeftToRight
                    }
                    horizontalAlignment: HorizontalAlignment.Left
                    topPadding: 0
                    leftPadding: mediaFolderItem.labelPadding
                    rightPadding: mediaFolderItem.labelPadding
                    
                    // Image count overlay
                    OverlayTextIcon {
                        visible: mediaFolderItem.imageCount > 0
                        icon: "asset:///imports/bb/cascades/multimedia/internal/images/ic_pictures.png"
                        text: "" + mediaFolderItem.imageCount
                        rightPadding: 10
                    }

                    // Video count overlay
                    OverlayTextIcon {
                        visible: mediaFolderItem.videoCount > 0
                        icon: "asset:///imports/bb/cascades/multimedia/internal/images/ic_videos.png"
                        text: "" + mediaFolderItem.videoCount
                        leftPadding: mediaFolderItem.imageCount > 0 ? 10 : 0
                    }
                }
            }
        }
    }

    onVideoCountChanged: {
        relayoutThumbnails();
    }

    onImageCountChanged: {
        relayoutThumbnails();
    }

    function relayoutThumbnails() {
        var totalMediaCount = imageCount + videoCount;
        bottomThumbItem.visible = false
        rightContainer.visible = true
        if (totalMediaCount > 3) {
            thumbItem1.visible = true
            thumbItem2.visible = true
            thumbItem3.visible = true
            thumbItem4.visible = true
            bottomThumbItem.visible = false
        } else if (totalMediaCount == 2) {
            thumbItem1.visible = true
            thumbItem2.visible = true
            thumbItem3.visible = false
            thumbItem4.visible = false
        } else if (totalMediaCount == 3) {
            bottomThumbItem.visible = true
            thumbItem3.visible = false
            thumbItem4.visible = false
        } else if (totalMediaCount == 1) {  
            rightContainer.visible = false
            thumbItem1.visible = true
            thumbItem2.visible = false
            thumbItem3.visible = false
            thumbItem4.visible = false
        }
    }
}
