import bb.cascades 1.2
import bb.cascades.multimedia 1.0
import bb.cascades.multimedia.internal 1.0

Sheet {
    id: moveToNewAlbumSheet

    signal dismissed()
    signal itemsMoved()

    property alias selectedItemsModel: selectedItemsList.dataModel
    property bool isMultiSource: false
    property int itemColumnCount: 1
    property double itemAspectRatio: 1
    property string itemType: ""
    property int textFieldPadding;

    content: Page {
        titleBar: TitleBar {
            //: Title of the move sheet used for moving pictures and videos to a new album
            title: qsTr("New Album") + Retranslate.onLanguageChanged
            acceptAction: ActionItem {
                id: moveActionId
                //: Move the selected items to the new folder
                title: qsTr("Move") + Retranslate.onLanguageChanged
                onTriggered: {
                    moveSelected();
                }
                enabled: false
            }
            dismissAction: ActionItem {
                //: Cancel the ongoing create new album action and return to the move sheet
                title: qsTr("Cancel") + Retranslate.onLanguageChanged
                onTriggered: {
                    dismissed();
                }
            }
        }

        MovePageContainer {
            id: mainContainer
            layout: StackLayout{}
            showDropDown: isMultiSource
            topPadding: 10
            bottomPadding: 10
            leftPadding: 10
            rightPadding: 10
            verticalAlignment: VerticalAlignment.Fill
            horizontalAlignment: HorizontalAlignment.Fill

            Container {
                topPadding: textFieldPadding
                bottomPadding: 10 + textFieldPadding
                rightPadding: textFieldPadding
                leftPadding: textFieldPadding
       
	            TextField {
	                id: textField
	
	                inputMode: TextFieldInputMode.Text
	                //:Text describing the input text field. Expecting the user to enter a name for the new album.
	                hintText: qsTr("Enter album name") + Retranslate.onLanguageChanged
	                validator: Validator {
	                    id: albumNameValidator
	                    mode: ValidationMode.Immediate
	                    onValidate:{
	                        if(moveToNewAlbumSheet.isValidAlbumName()){
	                            albumNameValidator.state = ValidationState.Valid;
	                        }else{                            
	                            albumNameValidator.state = ValidationState.Invalid;
	                        }
	                    }
	                    onValidChanged: {
	                        moveActionId.enabled = valid;
	                    }
	                }
	                input {
	                    submitKey: SubmitKey.Done
	                    onSubmitted: {
	                        if (albumNameValidator.state == ValidationState.Valid) {
	                            moveSelected();
	                        }
	                    }
	                }   
	            }             
            }
            ListView {
                id: selectedItemsList
                verticalAlignment: VerticalAlignment.Fill
                horizontalAlignment: HorizontalAlignment.Fill

                layout: GridListLayout {
                    columnCount: moveToNewAlbumSheet.itemColumnCount
                    headerMode: ListHeaderMode.None
                    cellAspectRatio: moveToNewAlbumSheet.itemAspectRatio
                }

                listItemComponents: [
                    ListItemComponent {
                        type: "picture_item"
                        MediaListItem {
                        }
                    },
                    ListItemComponent {
                        type: "video_item"
                        VideoListItem {
                            title: ListItemData["video.title"]
                            duration: ListItemData["video.duration"]
                            thumb_uri: ListItemData.thumb_file
                        }
                    }
                ]
            }
        }
    }
    
    function isValidAlbumName(){
        var albumName = textField.text.trim();
        if(albumName.length > 0 && albumName.indexOf(".") == 0){
            albumNameValidator.errorMessage = "'.' can't be used as the first letter. Try again.";
        	return false;
        }
        if(albumName.length <= 0){
            albumNameValidator.errorMessage = "You need to give your album a name.";
            return false;
        }
        
        var pattern = new RegExp("[\\\\/:*\"?<>|]");
        if(pattern.test(albumName)){
            albumNameValidator.errorMessage = "\\\\/:*\"?<>| can't be used in the name. Try again.";
            return false;
        }
        
        return true;
    }

    function moveSelected() {
        console.log("Move to new album named " + textField.text.trim());
        if (moveItems(mainContainer.selectedStorage, textField.text.trim())) {
            itemsMoved();
        } else {
            //NOTE: moveItems can return false and show a toast if mmsync is suspended,
            //      don't show duplicate toasts when implementing TODO.
            //TODO: Show error toast
        }
    }

    onOpened: {
        textField.requestFocus();
    }
}
