import bb.cascades 1.0
import bb.cascades.multimedia.internal 1.0
/**
 * This control allows creation of images with arbitrary number of states. The user of this component must
 * fullfill the following contract:
 *     - provide appropriate number of up images based on desired number of states in normalImages property
 *     - provide corresponding number of down images in the same order in downImages. If an image can't be found in 
 *       downImages property, no "down" image will be displayed
 * @param upImages Array of images corresponding to up state
 * @param downImages (optional) Array of images corresponding to down state
 * @param currentState Current state of the ImageView
 */
Container {
    id: outL
    layout: DockLayout{}
    property variant upImages
    property variant downImages
    property int currentState:0
    property alias scalingMethod: icon.scalingMethod
    
    onCurrentStateChanged: {
        icon.imageSource = upImages[currentState];
    }
	onTouch: {
        if (event.isDown() && enabled) {
            if (downImages != null && downImages[currentState] != null) {
                icon.imageSource = downImages[currentState];
            }
        } else if (event.isUp() && enabled) {
            icon.imageSource = upImages[currentState];
        }
    }
    ImageView {
        id: icon
        imageSource: outL.upImages[0]
        horizontalAlignment: HorizontalAlignment.Center
        verticalAlignment: VerticalAlignment.Center
    }
}