import bb.cascades 1.0
import bb.cascades.multimedia.internal 1.0

Container {      
    property alias additionalText: additionalLabel.text
    property alias deviceTypeText: deviceTypeLabel.text
    property alias activityIndicator: indicator
    
    property bool deviceEnabled: false
    property bool showDisconnectButton: false

    background: MediaApplication.playOnBackgroundColor
    opacity: 0.0
    layout: DockLayout {}
    
    ListItem.onInitializedChanged: {
        opacity = 0.0;
        if (initialized) {
            // If the item is being added to the list play the animation otherwise
            // just hide it
            itemFadeIn.play()
        }
    }

    animations: [
        FadeTransition {
            id: itemFadeIn
            duration: 1000
            easingCurve: StockCurve.CubicOut
            toOpacity: 1.0
        }
    ]
    
    Container {
        topPadding: 10  
        leftPadding: 20 
        
        verticalAlignment: VerticalAlignment.Center
        horizontalAlignment: HorizontalAlignment.Fill
        
        layout: StackLayout {
            orientation:LayoutOrientation.TopToBottom           
        }

        Container {
        	verticalAlignment: VerticalAlignment.Center
        	horizontalAlignment: HorizontalAlignment.Fill
            bottomPadding: 20
            minHeight: 120
            
    	    layout: StackLayout {
    	    	orientation:LayoutOrientation.LeftToRight	    	
    	    }
    	    
    	    // Activity Indicator and Icon - only 1 is shown at any given time
            ActivityIndicator {
                id: indicator
                
                visible: false
                preferredWidth: 80
                preferredHeight: 80
            }
            
            ImageView {
                id: icon
                
                imageSource:ListItemData.image
                verticalAlignment: VerticalAlignment.Center
                preferredHeight: 80
                preferredWidth:80   
                scalingMethod: ScalingMethod.AspectFit 
                layoutProperties: StackLayoutProperties {               
                    spaceQuota: -1
                }               
            }
            
            // Name and type text
            Container {
                verticalAlignment: VerticalAlignment.Center
                horizontalAlignment: HorizontalAlignment.Fill
                leftPadding: 20
                
                layout: StackLayout {
                    orientation: LayoutOrientation.TopToBottom
                }
                layoutProperties: StackLayoutProperties {
                    spaceQuota: 100
                }
                
                Container {
                    verticalAlignment: VerticalAlignment.Center
                    layout: StackLayout {
                        orientation: LayoutOrientation.LeftToRight
                    }
                    
                    Label {
                        /*
                         * Should be "Primary R", which means
                         * FontSize.ScalablePointValue,
                         * FontWeight.W400
                         */
                        text: ListItemData.name
                        textStyle.base: SystemDefaults.TextStyles.TitleText
                        textStyle.color: MediaApplication.textColor
                        textStyle.fontSize: FontSize.ScalablePointValue
                        textStyle.fontWeight: FontWeight.W400
                    }
    
                    Label {
                        /*
                         * Should be "Primary R", which means
                         * FontSize.ScalablePointValue,
                         * FontWeight.W400
                         */
                        id: additionalLabel
                        verticalAlignment: VerticalAlignment.Center
                        layoutProperties: StackLayoutProperties {
                            spaceQuota: 100
                        }
                        
                        textStyle.base: SystemDefaults.TextStyles.TitleText
                        textStyle.color: MediaApplication.textColor
                        textStyle.fontSize: FontSize.ScalablePointValue
                        textStyle.fontWeight: FontWeight.W400
                    }
                }
                
                Container {
                    verticalAlignment: VerticalAlignment.Center
                    Label {
                        /*
                         * Should be "Primary R", which means
                         * FontSize.ScalablePointValue,
                         * FontWeight.W400
                         */
                        id: deviceTypeLabel
                        text: ""
                        visible: text == "" ? false : true
                        textStyle.base: SystemDefaults.TextStyles.TitleText
                        textStyle.color: deviceEnabled ? MediaApplication.activeDeviceTypeTextColor : MediaApplication.deviceTypeTextColor
                        textStyle.fontSize: FontSize.XXSmall
                        textStyle.fontWeight: FontWeight.W400
                    }
                }
            } // Name and Type
        }
    
        // Disconnect Button
        Container {
            horizontalAlignment: HorizontalAlignment.Fill
            rightPadding: 20
            bottomPadding: 20
            
            visible: deviceEnabled && showDisconnectButton
            
            layout: DockLayout {}
            
            Button {
                id: disconnectButton
                text: "Disconnect"
                
                horizontalAlignment: HorizontalAlignment.Fill
                
                onClicked: {
                    playOnListItem.ListItem.view.dataModel.rendererDisconnected.connect(playOnListItem.onDisconnectCompleted);
                    startDisconnect();
                }
            }
        }
    } // Outer Container
    
    Divider {
        verticalAlignment: VerticalAlignment.Bottom
        visible: !deviceEnabled || showDisconnectButton // Don't show divider line when disconect button is shown
    }
    
    function startDisconnect() {
        console.log("startDisconnect"); 
        deviceTypeText = qsTr("Disconnecting") + Retranslate.onLanguageChanged;
        icon.setVisible(false);
        indicator.setVisible(true);
        indicator.start();
        
        ListItem.view.dataModel.rendererDisconnected.connect(onDisconnectCompleted);
        ListItem.view.dataModel.disconnectRenderer(ListItemData.id);
    }
    
    function stopDisconnect() {
        console.log("stopDisconnect");
        indicator.setVisible(false);
        icon.setVisible(true);
        indicator.stop();
    }
    
    function onDisconnectCompleted(id, successfully) {
        if (successfully) {
            console.log("Disconnect Miracast Completed");
        } else {
            deviceEnabled = true; // Restore if still connected
            console.log("Disconnect Miracast Device Failed");
            if (ListItem.view.dataModel.connected.id == id) {
                showToast(qsTr("Can’t disconnect from %1. Try to disconnect again.").arg(ListItemData.name) + Retranslate.onLanguageChanged);
            } 
        }
        
        console.log("Disconnect Miracast Device Finish");
        playOnListItem.stopDisconnect(); // Stop spinner
        playOnListItem.deviceTypeText = ListItemData.devicetypeinfo;
    }
}
