import bb.cascades 1.0
import bb.cascades.multimedia 1.0
import bb.cascades.multimedia.internal 1.0

Page {
    id: playOnListPage
    objectName: "PlayOnListObjectName"
    // The NavigationPane for the PlayOnListPage
    property NavigationPane navControl
    property alias backButtonText: backButton.title
    property alias paint: playOnList.paint
    property bool selectRenderer: false
   
    //Signals
    signal rendererSelected (variant renderer)
    signal rendererSelectionCanceled()
    
    paneProperties: NavigationPaneProperties {
        backButton: ActionItem {
            id: backButton
            objectName: "PlayOnListBack"
            onTriggered: {
                navControl.pop();
	        }
	    }
    }
    
    
    titleBar: TitleBar {
        kind: TitleBarKind.Default
        title: qsTr("Select a device to play on") + Retranslate.onLanguageChanged
    }
    
    function loadData(rendererTypes){
	    playOnList.loadData(rendererTypes);
	}
    
    actions : [
		ActionItem { 
		    id: dlnaSettings
		    ActionBar.placement: ActionBarPlacement.OnBar
		    attachedObjects: [
		InvokeHelper {
		    id: invokeHelper
		}
            ]
		    
            objectName: "PlayOnListMediaSharing"
		    imageSource: "images/play_on_list_dlna_settings_tab.png"	
		    //: navigation tab label in Play On page	    
			title: qsTr("Media Sharing") + Retranslate.onLanguageChanged
			onTriggered: {
			    invokeHelper.invokeTargetApp("bb.action.OPEN",
				"sys.settings.card",
				"settings://sharing",
				"settings/view",
				[]);
			}
		}
    ]

    content: [
        PlayOnList {    
            id:playOnList	                  
        }
    ]
    
    onCreationCompleted: {
        playOnList.rendererSelected.connect(rendererSelection);
        if (typeof(currentNavPane) != 'undefined') {
        	currentNavPane.playonListScreenPopped.connect(disconnectPage);
        }
    }
    
	onNavControlChanged: {
	    //Ensure that our back button is visible
	    navControl.backButtonsVisible = true;
	}
	
	function addDLNASettings() {
		qmlUtils.addActionItemToPage(dlnaSettings, playOnListPage);
	}
	
	function removeDLNASettings() {
	    qmlUtils.removeActionItemFromPage(dlnaSettings, playOnListPage);
	}
    
     function rendererSelection(renderer){
         selectRenderer = true;
         navControl.pop();
         rendererSelected(renderer);
     }

	 function disconnectPage() {
         playOnList.rendererSelected.disconnect(rendererSelection);
         if (typeof(currentNavPane) != 'undefined') {
             currentNavPane.playonListScreenPopped.disconnect(disconnectPage);
         }
         if (!selectRenderer) {
             rendererSelectionCanceled();
         }
	 }

}
