import bb.cascades 1.0
import bb.cascades.multimedia 1.0

Page {
    id: page
    
    property variant propertyDataModel
    property string defaultImage
    property int screenWidth: (OrientationSupport.orientation == UIOrientation.Landscape) ? PrimaryDisplay.pixelSize.height : PrimaryDisplay.pixelSize.width
    property bool forceFileNameReadOnly: false
    property int listItemHeight: 111;

    attachedObjects: [
        CommonAnalytics {
            id: commonAnalytics
        }
    ]

    onCreationCompleted: {
        propertyDataModel.loaded.connect(onDataModelLoaded);
        commonAnalytics.instance.logPropertiesPageLaunched();
        qmlUtils.enableActionBarDocking(page);
    }

    ListView {
        id: listView
        scrollIndicatorMode: ScrollIndicatorMode.None

        property string defaultImagePath: defaultImage
        property int thumbnailWidth: screenWidth
        property variant propertiesPage: page
        property int itemHeight: listItemHeight;

        listItemComponents: [
            ListItemComponent {
                type: "videothumbnailitem"
                PropertyVideoThumbnail {
                    preferredWidth: ListItem.view.thumbnailWidth
                    imageSource: (ListItemData.thumbnail != "") ? ListItemData.thumbnail : ListItem.view.defaultImagePath
                }
            },
            ListItemComponent {
                type: "picturethumbnailitem"
                PropertyPictureThumbnail {
                    preferredWidth: ListItem.view.thumbnailWidth
                    imageSource: (ListItemData.thumbnail != "") ? ListItemData.thumbnail : ListItem.view.defaultImagePath
                }
            },
            ListItemComponent {
                type: "folderthumbnailitem"
                PropertyFolderThumbnail {     
                    preferredWidth: ListItem.view.thumbnailWidth
                }
            },
            ListItemComponent {
                type: "filenameitem"
                PropertyFileName {
                    title: ListItemData.title
                    forceReadOnly: ListItem.view.propertiesPage.forceFileNameReadOnly
                    height: ListItem.view.itemHeight
                }
            },
            ListItemComponent {
                type: "foldernameitem"
                PropertyFolderName {
                    title: ListItemData.title
                    height: ListItem.view.itemHeight
                }
            },
            ListItemComponent {
                type: "item"
                PropertyItem {
                    title: ListItemData.title
                    description: ListItemData.description
                    height: ListItem.view.itemHeight
                }
            },
            ListItemComponent {
                type: "header"
                Header {
                    title: ListItemData
                    // First header is invisible
                    visible: ListItem.indexPath != 0
                }
            },
            ListItemComponent {
                type: "segmentedcontrol"
                PropertySegmentedControl {
                }
            }
        ]
    }

    function onDataModelLoaded() {
        listView.dataModel = propertyDataModel;
    }
}