import bb.cascades 1.0
import bb.cascades.multimedia 1.0

Container {
    id: propertyFolderNameContainer
    property alias title : titleLabel.text
    property string folderName: ListItemData.folderName
    
    property int height;
    
    minHeight: height
    
    layout: DockLayout {
    }

	Container {
	    leftPadding: 20
	    rightPadding: 20
        bottomPadding: ListItemData.isWritable ? 20 : 10
	
	    horizontalAlignment: HorizontalAlignment.Fill
        verticalAlignment:  VerticalAlignment.Center
	
	    layout: StackLayout {
	        orientation: LayoutOrientation.TopToBottom
	    }
	
	    Label {
	        id: titleLabel
	        horizontalAlignment: HorizontalAlignment.Fill
	        textStyle {
	            color: Color.Gray
	            fontSize: FontSize.Small
	            fontWeight: FontWeight.W400
	        }
            topMargin: 0
            bottomMargin: 0
	    }
	
		// Visibility of the Label (fileNameLabel)
		// and the TextArea (folderNameTextField) are mutually exclusive.
	    Label {
	        id: fileNameLabel
	        text: folderName
	        visible: !ListItemData.isWritable
	        multiline: true
	        horizontalAlignment: HorizontalAlignment.Fill
	        textStyle {
	            fontSize: FontSize.Large
	            fontWeight: FontWeight.W400
	        }
            topMargin: 0
            bottomMargin: 0
	    }
	
	    TextField {
	        id: folderNameTextField
	        text: folderName
	        hintText: qsTr("Enter album name");
	        enabled: ListItemData.isWritable
	        visible: ListItemData.isWritable
	        inputMode: TextAreaInputMode.Text
	        content.flags: TextContentFlag.EmoticonsOff | TextContentFlag.ActiveTextOff;
	        horizontalAlignment: HorizontalAlignment.Fill
            topMargin: 10
	
	        input {
	        	submitKey: SubmitKey.Done;
	        	
	            onSubmitted: {
	                loseFocus();
	            }
	     
	            flags: TextInputFlag.SpellCheckOff | TextInputFlag.PredictionOff | TextInputFlag.AutoCorrectionOff | TextInputFlag.AutoCapitalizationOff | TextInputFlag.AutoPeriodOff | TextInputFlag.WordSubstitutionOff
	        }
	
	        textStyle {
	            fontSize: FontSize.Large
	            fontWeight: FontWeight.W400
	        }
	
			property string originalFolderName: ""
	        onFocusedChanged: {
	            if (!focused) {
	                var newFolderName = folderNameTextField.text.trim();
	                if (newFolderName.length < 1) {
	                    newFolderName = folderName;
	                } 
	
	                if (newFolderName != folderName) {
	                    var folderRenamed = propertyFolderNameContainer.ListItem.view.dataModel.renameFolder(newFolderName);
	                    if (folderRenamed) {
	                        commonAnalytics.instance.logFolderNameChanged(folderName, newFolderName);
	                        propertyFolderNameContainer.ListItem.view.propertiesPage.addAction(discardChangesAction);
	
	                        // Save the original name the first time we do a rename
	                        if (originalFolderName == "") {
	                          originalFolderName = folderName;
	                        }
	
	                        folderName = newFolderName;
	                    } else {
	                        dialogUtils.showSystemToast(qsTr("This album can't be renamed."));
	                    }
	                }
	                folderNameTextField.text = folderName;
	            }
	        }
	        attachedObjects: [
	            ActionItem {
	                id: discardChangesAction
	                title: qsTr("Discard Changes") + Retranslate.onLanguageChanged
	                onTriggered: {
	                    if (!propertyFolderNameContainer.ListItem.view.dataModel.renameFolder(folderNameTextField.originalFolderName)) {
	                        dialogUtils.showSystemToast(qsTr("Can't revert to old album name."));
	                    } else {
	                    	folderName = folderNameTextField.originalFolderName;
	                        folderNameTextField.text = folderNameTextField.originalFolderName;
	                    }
	
	                    propertyFolderNameContainer.ListItem.view.propertiesPage.removeAction(discardChangesAction);
	                }
	                ActionBar.placement: ActionBarPlacement.OnBar
	                imageSource: "images/ic_discard_changes.png"
	            },
	            SystemDialogUtils{
	                id: dialogUtils
	            },
	            CommonAnalytics {
	                id: commonAnalytics
	            }
	        ]
	    }
    }

    Divider {
        verticalAlignment: VerticalAlignment.Bottom
    }
}
