import bb.cascades 1.0
import "extensionLayoutConstants.js" as ExtensionLayoutConstants

Container {
    id: searchControl
    
    property alias backgroundColor : searchControl.background
    property int horizontalPadding: ExtensionLayoutConstants.CONTROL_PADDING
    property alias buttonText: cancelButton.text
    property string searchText
    property alias hint: searchTextField.hintText
    property alias cancelButtonVisible: cancelButton.visible
    property bool finalSearch: false
    signal searchTextChanged (string text)
    signal searchCancelled ()
    function getSearchText () {
        return searchControl.searchText;
    }

	bottomPadding: ExtensionLayoutConstants.CONTROL_PADDING
	topPadding: ExtensionLayoutConstants.CONTROL_PADDING
    leftPadding: horizontalPadding
    rightPadding: horizontalPadding

    layout: StackLayout {
        id: controlStackLayout
        orientation: LayoutOrientation.LeftToRight
    }

    TextField {
        id: searchTextField
        onTextChanging: {
            if (text.localeCompare (searchControl.searchText) != 0) {
	                    //HACK: store text value as I cannot access it after the fact
                searchControl.searchText = text; 
	                    //only show the clear button when we have text
                searchTextField.clearButtonVisible = text.length > 0;

                //fire external signal
                if (searchScreen.__state != "searching") {
                    searchControl.finalSearch = false;
                    searchTextChanged(text);
                }else{
                    searchControl.finalSearch=true;
                }
            }
        }
        layoutProperties: StackLayoutProperties {            
            spaceQuota: 70
        }
        // We want the spacing between the text field and cancel button to be 10px
        rightMargin: 10
        verticalAlignment: VerticalAlignment.Center
        input {
            submitKey: SubmitKey.Search
        }
        hintText: qsTr("Search") + Retranslate.onLanguageChanged
        input.flags: TextInputFlag.VirtualKeyboardOff
        inputMode: TextFieldInputMode.Default
        inputRoute.primaryKeyTarget: true 
    }
    
    /*
     * This function will toggle the VirtualKeyboardOff flag for the text field.
     */
    function enableVirtualKeyboard(enabled){
        if ( enabled ) {
            searchTextField.input.flags = (searchTextField.input.flags & ~TextInputFlag.VirtualKeyboardOff) | TextInputFlag.VirtualKeyboard;
        } else {
            searchTextField.input.flags = (searchTextField.input.flags & ~TextInputFlag.VirtualKeyboard) | TextInputFlag.VirtualKeyboardOff;
        }
    }
    
    /*
     * For requesting focus to work, the entire QML tree must be ready,
     * so we call this from the outermost elements in the apps when they
     * load this screen.
     *
     * @param focused set to true to set focus to the search text field;
     *                set to false to lose focus from the search text field 
     */
    function focus(focused) {
        if (focused) {
            searchTextField.requestFocus();
        } else {
            searchTextField.loseFocus();
        }
    }
    
    /*
     * Sets the search text field to be always focused or not.
     * To support "Type to Search" feature, the search text field has to always be focused;
     * otherwise the text field won't be able to capture key presses.
     *
     * @param alwaysFocused if true, the search text field will always get focus
     *                      unless another text control requests the focus.
     */
    function setAlwaysFocused(alwaysFocused) {
        if (alwaysFocused) {
            searchTextField.focusRetentionPolicyFlags = FocusRetentionPolicy.LoseToFocusable;
        } else {
            searchTextField.focusRetentionPolicyFlags = FocusRetentionPolicy.Default;
        }
    }

    /**
     * Clears the search text.
     */
    function clearText() {
        searchTextField.resetText();
    }
    
    Button {
        id: cancelButton
        onCreationCompleted: {
            clicked.connect (searchCancelled);
            clicked.connect (searchControl.clearText);
        }
        layoutProperties: StackLayoutProperties {
            spaceQuota: 30
        }
        // We want the spacing between the text field and cancel button to be 10px
        // and this is taken care of by the rightMargin of the TextField above
        leftMargin: 0
    }
}
