import bb.cascades 1.0

/**
 * This control allows the content of the container to be switched to
 * another QML document.
 * 
 * A point to note is that since we are dynamically created object for the content of
 * of the container at runtime, we must destroy them manually as well. They will NOT
 * be automatically cleaned up and can cause memory leaks.
 *
 * @param contentQMLPath The path to the content QML document.
 * 
 * @author schoudhry
 */

Container {
    id: _switchableContainer
        
    property string contentQMLPath
    property variant currentQMLObject

    onContentQMLPathChanged: {
        _switchableContainer.removeAll();
        if (currentQMLObject) {
            // if we have already created a dynamic object, destroy it
            currentQMLObject.destroy();
        }        
        var component = Qt.createComponent(contentQMLPath);
        if (component.errorString()) {
            console.debug("SwitchableContainer: "+component.errorString());
        } else {
            currentQMLObject = component.createObject (_switchableContainer);    
            _switchableContainer.add (currentQMLObject);
        }
    }
}