import bb.cascades 1.0

/**
 * This control is an item that contains an image and a label
 * which can be set.
 *
 * @param   labelText   The text to use for the top row of the label
 * @param   bottomLabelText The text to use for the bottom row of the label.  bottomLabelVisible must be set to true for this to appear
 * @param   bottomLabelVisible This must be set to true for the bottom label to be displayed
 * @param   thumbUri    The uri for the image
 * @param   contentTopPadding   The top Padding
 * @param   contentBottomPadding    The bottom Padding
 * @param   contentRightPadding The right Padding
 * @param   contentLeftPadding  The left Padding
 * @param   defaultImage    The uri for the image to use as a default
  * @param   shadowImage   The uri for an image that is overlayed over the album art.  This is typically used to add a shadow border effect
 */
Container {
    id: thumbListItem 

    property alias labelText: caption.topText
    property alias labelVisible: caption.topVisible
    property alias bottomLabelText: caption.bottomText
    property alias bottomLabelVisible: caption.bottomVisible
    property string thumbUri: defaultImage
    property int   defaultContentPadding: 10 
    property alias contentTopPadding:   thumbListItem.topPadding  //todo  I think this should be able to be set on the Container and we can remove one level 
    property alias contentBottomPadding:    thumbListItem.bottomPadding
    property alias contentRightPadding: thumbListItem.rightPadding
    property alias contentLeftPadding:  thumbListItem.leftPadding
    property string defaultImage
    property alias overlayImage: overlay.imageSource
    property alias overlayOpacity: overlay.opacity
    property alias pressImageSource: pressImage.imageSource
    property alias selectImageSource: selectImage.imageSource
    property bool itemSelected: false
    property string totalDuration;
    
    onThumbUriChanged: {
        thumbImage.imageSource = (thumbUri == "" || thumbUri == null) ? defaultImage : thumbUri;
    }
    
    topPadding: defaultContentPadding
    bottomPadding: defaultContentPadding
    rightPadding: defaultContentPadding
    leftPadding: defaultContentPadding
    
    horizontalAlignment: HorizontalAlignment.Fill
    verticalAlignment: VerticalAlignment.Fill
    
    layout: DockLayout {
        id: contLayout
    }

    // item image
    ImageView {
        id: thumbImage
        horizontalAlignment: HorizontalAlignment.Fill
        verticalAlignment: VerticalAlignment.Fill
        imageSource: defaultImage
        scalingMethod: ScalingMethod.ScalingMethodAspect
    }
    
    ThumbsListItemCaption {
        id: caption
    }
    
	//shadow for the image
	ImageView {
		id: overlay
        opacity: 0.0	
        horizontalAlignment: HorizontalAlignment.Fill
        verticalAlignment: VerticalAlignment.Fill
	}
	
	ImageView {
        id: pressImage
        horizontalAlignment: HorizontalAlignment.Fill
        verticalAlignment: VerticalAlignment.Fill
        scalingMethod: ScalingMethod.Fill
        opacity: 0
        attachedObjects: ImplicitAnimationController {
            enabled: false
            propertyName: "opacity"
        }
        animations: [
            FadeTransition {
                id: showPressedBorderAnimation
                toOpacity: 1
                duration: 50
            },
            FadeTransition {
                id: hidePressedBorderAnimation
                delay: 50
                toOpacity: 0
                duration: 150
                easingCurve: StockCurve.QuadraticOut
            }
        ]
    }
    Container {
        horizontalAlignment: HorizontalAlignment.Fill
        verticalAlignment: VerticalAlignment.Fill
        topPadding: 4
        leftPadding: 4
        rightPadding: 4
        bottomPadding: 4
        layout: DockLayout {
        }
        ImageView {
            id: selectImage
            horizontalAlignment: HorizontalAlignment.Fill
            verticalAlignment: VerticalAlignment.Fill
            scalingMethod: ScalingMethod.Fill
            opacity: 0
            animations: [
                FadeTransition {
                    id: showContextActivationAnimation
                    toOpacity: 0.4
                    duration: 350
                    delay: 100
                    easingCurve: StockCurve.Linear
                },
                FadeTransition {
                    id: hideContextActivationAnimation
                    delay: 50
                    toOpacity: 0
                    duration: 150
                    easingCurve: StockCurve.QuadraticOut
                },
                FadeTransition {
                    id: showSelectAnimation
                    toOpacity: 1
                    duration: 50
                }
            ]
        }
    }
    
    function onActivationChanged() {
        // TODO: workaround for activeChanged triggered on and off when multi select mode is activated
        if (!itemSelected) {
            if (active) {
                // Fade in the blue background color if the list item have a ContextMenu
                if (thumbListItem.contextActions && thumbListItem.contextActions.length > 0) {
                    hideContextActivationAnimation.stop();
                    showContextActivationAnimation.play();
                }
                // Show the blue border
                showPressedBorderAnimation.play();
            } else {
                showContextActivationAnimation.stop();
                hideContextActivationAnimation.play();
                showPressedBorderAnimation.stop();
                hidePressedBorderAnimation.play();
            }
        }
    }
    
    onItemSelectedChanged: {
        if (itemSelected) {
            showContextActivationAnimation.stop();
            hideContextActivationAnimation.stop();
            showSelectAnimation.play();
            hidePressedBorderAnimation.stop();
            showPressedBorderAnimation.play();
        } else {
            showSelectAnimation.stop();
            showContextActivationAnimation.stop();
            hideContextActivationAnimation.play();
            showPressedBorderAnimation.stop();
            hidePressedBorderAnimation.play();
        }
    }
}
