import bb.cascades 1.0

// The caption
Container {        
	property string topText
	property alias topVisible: topLabel.visible
	property string bottomText
	property alias bottomVisible: bottomLabel.visible

	horizontalAlignment: HorizontalAlignment.Fill
	verticalAlignment: VerticalAlignment.Bottom

	visible: topLabel.visible || bottomLabel.visible
	
	attachedObjects: [
	    // Need to handle onLanguageChanged signal because Retranslate.onLanguageChanged
	    // won't be triggered on an empty string.
	    LocaleHandler {
	        onLanguageChanged: {
	            updateTopLabel();
	            updateBottomLabel();
	        }
	    }
	]

	// property change handling for unknown and default
	onTopTextChanged: {
	    updateTopLabel();
	}

	onBottomTextChanged: {
	    updateBottomLabel();
	}
	
	function updateTopLabel() {
	    topLabel.text = (topText == "" || topText == null) ? qsTr("Unknown") : topText;
	}
	
	function updateBottomLabel() {
	    bottomLabel.text = (bottomText == "" || bottomText == null) ? qsTr("Unknown") : bottomText;
	}

	layout: StackLayout {
		orientation: LayoutOrientation.TopToBottom
	}

	// A horizontal rule
	Container {
		horizontalAlignment: HorizontalAlignment.Fill
		verticalAlignment: VerticalAlignment.Top

		background: Color.White
		opacity: 0.20

		minHeight: 1
		maxHeight: 1
	}

	Container {
		minHeight: 80

		horizontalAlignment: HorizontalAlignment.Fill
		verticalAlignment: VerticalAlignment.Bottom

		layout: DockLayout {}

		// Black background, 60% opaque            
		Container {
			horizontalAlignment: HorizontalAlignment.Fill
			verticalAlignment: VerticalAlignment.Fill

			background: Color.Black
			opacity: 0.60
		}

		// Two rows of label text
		Container {
			horizontalAlignment: HorizontalAlignment.Left
			verticalAlignment: VerticalAlignment.Center

			leftPadding: 14
			rightPadding: 14

			layout: StackLayout {
				orientation: LayoutOrientation.TopToBottom
			}

			Label {
				id: topLabel
				text: qsTr("Unknown") + Retranslate.onLanguageChanged
				visible: true

				horizontalAlignment: HorizontalAlignment.Left
				verticalAlignment: VerticalAlignment.Center
				topMargin: 0
				bottomMargin: 0

				textStyle.base: SystemDefaults.TextStyles.SubtitleText
				textStyle.color: MediaApplication.textColor
				textStyle.fontSize: FontSize.XSmall
				textStyle.fontWeight: FontWeight.W500
                textFormat: TextFormat.Plain
            }

			// label text
			Label {
				id: bottomLabel
				text: qsTr("Unknown") + Retranslate.onLanguageChanged
				visible: false

				horizontalAlignment: HorizontalAlignment.Left
				verticalAlignment: VerticalAlignment.Center
				topMargin: 0
				bottomMargin: 0

				textStyle.base: SystemDefaults.TextStyles.SubtitleText
				textStyle.color: MediaApplication.textColor
				textStyle.fontSize: FontSize.XXSmall
				textStyle.fontWeight: FontWeight.W400
                textFormat: TextFormat.Plain
            }
		}
	}        
}

