import bb.cascades 1.0

Container {
    id: parentContainer
    
    verticalAlignment: VerticalAlignment.Bottom
    horizontalAlignment: HorizontalAlignment.Fill
    
	layoutProperties: DockLayoutProperties{	    
	}
	layout: DockLayout{}
	preferredWidth: 9999
	maxHeight: 0
	background: Color.Transparent

    //'Private' variables
    property string qmlPath
    property variant qmlObject
    property bool isTapToDismiss
    
    //Constants
    property int c_HIDE_DELAY: 2000
    property int c_MAX_HEIGHT: 1000
    
    //Show a toast message using a QML component
    //parameters:
    //string - The QML file to toast
    //boolean - false if you want to disable the default "tap to dismiss" behavior
    //int - timeout duration before the toast hides itself, in milliseconds
    function show(qmlPathString, isTapToDismissFlag, duration){
        if(!(typeof qmlPathString === 'undefined') && qmlPathString != ""){
            clear();
            qmlPath = qmlPathString;                        
            if(typeof isTapToDismissFlag === 'undefined'){
                console.log("bool isTapToDismissFlag undefined");
                isTapToDismiss = true;
            }else{
                isTapToDismiss = isTapToDismissFlag;
            }
            if(typeof duration === 'undefined'){
                console.log("int duration undefined");
                waitAndHide.delay = c_HIDE_DELAY;
            }else{
                waitAndHide.delay = duration;
            }
            qmlObject = Qt.createComponent(qmlPath).createObject(parentContainer);
            parentContainer.controls = [qmlObject];
            parentContainer.maxHeight = c_MAX_HEIGHT;
        }
        waitAndHide.play();
    }
    
    //Hide the toast message
    function hide(){
        stopTimers();
        maxHeight = 0;
        waitAndClear.play();
    }
    
    //'Private' helper method
    function clear(){
        stopTimers();
        maxHeight = 0;
        parentContainer.controls = null;
        if (qmlObject) {
            qmlObject.destroy();
            qmlObject = null;
        }
        qmlPath = "";
        isTapToDismiss = true;
    }
 
    //'Private' helper method
	function stopTimers(){
	    waitAndHide.stop();
	    waitAndClear.stop();
	}
 
    onTouch:{
        if(isTapToDismiss){
            hide();
        }
    }
   
    //animations used as timers
    animations: [
	    ParallelAnimation {
	        id: waitAndHide
            FadeTransition {
			    target: parentContainer
			    toOpacity: 1.0
			    delay: c_HIDE_DELAY
			    repeatCount: 0
			    duration: 1
			    easingCurve: StockCurve.Linear
            }
            onEnded: {
                //console.log("hide time");
                hide();
            }
	    },
	    ParallelAnimation {
	        id: waitAndClear
            FadeTransition {
			    target: parentContainer
			    toOpacity: 1.0
			    delay: 500
			    repeatCount: 0
			    duration: 1
			    easingCurve: StockCurve.Linear
            }
            onEnded: {
                //console.log("clear time");
                clear();
            }
	    }
	]
}
