import bb.cascades 1.0

/**
 * NOTE: There is also a NoContentView which is the exact same but has one button.
 * Any changes to this file should also be made to that file as well if applicable.
 * 
 * A container used to tell the user that no content is available on the device,
 * for a particular page. Shows the message as a title, sub-title, and an optional button.
 *
 * @param   title                  The string for title of the no content message
 * @param   subTitle               The string for subtitle of the no content message
 * @param   image                  The path of the no content image
 * @param   noContentButtonVisible The boolean value if a button should be shown (false by default)
 * @param   noContentButtonLabel   The string for label of the no content button
 * @param   noContentButtonImage   The string for url of the image for no content button 
 * @param   noContentButton2Visible The boolean value if a second button should be shown (false by default)
 * @param   noContentButton2Label  The string for label of the second no content button
 * @param   noContentButton2Image  The string for url of the image for the second no content button 
 * @param   pageDataModel          The data model to which this no content handler will listen to updates for    
 * @signal  addTrackToPlaylist     Emitted when the no content button is pressed                         
 */

Container {
    id: noContentViewContainer
    visible: false
    leftPadding: 30
    rightPadding: 30
    
    property string title
    property string subTitle
    property string image
    property bool noContentButtonVisible
    property string noContentButtonLabel
    property string noContentButtonImage
    property bool noContentButton2Visible
    property string noContentButton2Label
    property string noContentButton2Image
    
    signal noContentButtonClicked()
    signal noContentButton2Clicked()
    
    property variant pageDataModel

    attachedObjects:[
        ComponentDefinition {
            id: mainContent
            
            Container {
                Container {
                    horizontalAlignment: HorizontalAlignment.Center
                    layout: StackLayout {
                        orientation: LayoutOrientation.BottomToTop
                    }
                    layoutProperties: StackLayoutProperties {
                        spaceQuota: 1
                    }
                    Label {
                        id: noContentTitle
                        text: noContentViewContainer.title
                        textStyle{
                            fontWeight: FontWeight.W400
                            fontStyle: FontStyle.Normal
                            fontFamily: "Slate"
                            fontSize: FontSize.Large
                            color: MediaApplication.textColor
                        }
                        horizontalAlignment: HorizontalAlignment.Center
                        topMargin: 20
                    }
                    ImageView {
                        id: noContentImage
                        imageSource: noContentViewContainer.image
                        horizontalAlignment: HorizontalAlignment.Center
                        scalingMethod: ScalingMethod.AspectFit
                    }
                }
                Container {
                    horizontalAlignment: HorizontalAlignment.Center
                    layoutProperties: StackLayoutProperties {
                        spaceQuota: 1
                    }
                    TextArea{
                        id: noContentSubTitle
                        text: noContentViewContainer.subTitle
                        editable: false
                        focusPolicy: FocusPolicy.None
                        scrollMode: TextAreaScrollMode.Stiff
                        horizontalAlignment: HorizontalAlignment.Center
                        textStyle{
                            textAlign: TextAlign.Center
                            fontWeight: FontWeight.W400
                            fontStyle: FontStyle.Normal
                            fontFamily: "Slate"
                            fontSize: FontSize.Small
                            color: Color.create("#b1b1b1")
                        }
                        input {
                            flags: TextInputFlag.SpellCheckOff | TextInputFlag.PredictionOff | TextInputFlag.AutoCorrectionOff 
                        }
                        content
                        {
                            flags: TextContentFlag.ActiveTextOff
                        }
                    }
                    Container {
                        layout: StackLayout {
                            orientation: LayoutOrientation.LeftToRight
                        }
                        Button {
                            id: noContentButton
                            visible: noContentViewContainer.noContentButtonVisible
                            text: noContentViewContainer.noContentButtonLabel
                            imageSource: noContentViewContainer.noContentButtonImage
                            horizontalAlignment: HorizontalAlignment.Center
                            onClicked: {
                                noContentViewContainer.noContentButtonClicked();
                            }
                        }
                        Button {
                            id: noContentButton2
                            visible: noContentViewContainer.noContentButton2Visible
                            text: noContentViewContainer.noContentButton2Label
                            imageSource: noContentViewContainer.noContentButton2Image
                            horizontalAlignment: HorizontalAlignment.Center
                            onClicked: {
                                noContentViewContainer.noContentButton2Clicked();
                            }
                        }
                    }
                }
            }
        },
        ComponentDefinition {
            id: emptyContent
            Container {
            }
        }
    ]
    
    ControlDelegate {
        id: ncvDelegate
        sourceComponent: emptyContent
    }
    onPageDataModelChanged: {
        if ( pageDataModel != null ) {
            var count = pageDataModel.childCount(null);
            console.debug("pageDataModel.childCount="+count);
            pageDataModel.itemsChanged.connect(handlePageDataModelChanged);
            pageDataModel.itemUpdated.connect(handlePageDataModelChanged);
            pageDataModel.itemAdded.connect(handlePageDataModelChanged);
            pageDataModel.itemRemoved.connect(handlePageDataModelChanged);
        }
    }
    
    function handlePageDataModelChanged() {
        var count = pageDataModel.childCount(null);
        noContentViewContainer.visible = (count == null || count==0);
    }
    
    onVisibleChanged: {
        if (noContentViewContainer.visible) {
            ncvDelegate.sourceComponent = mainContent;
        } else {
            ncvDelegate.sourceComponent = emptyContent;
        }
    }
}
