import bb.cascades 1.0
import bb.cascades.multimedia 1.0
import bb.cascades.multimedia.internal 1.0

Container {
    property NetworkStatusManager networkManager
    property SecurityStatus securityStatus 
    layout: StackLayout { orientation: LayoutOrientation.TopToBottom }
    visible: false
    opacity: 0.0
    // if the wifi setting (once determined) is off and there is also no wired network, add the Turn On Wifi button
    property bool wifiNotConnected: (networkManager.wifi != NetworkStatusManager.Connected 
           && networkManager.wifi != NetworkStatusManager.Unknown
           && networkManager.wired != NetworkStatusManager.Connected
           && securityStatus.dlnaPlayOnEnabled)
           
    // If we have both Miracast and DLNA then include "more" in the turn on / connect WiFi warning messages
    property bool multipleTargetsPossible: false

    onWifiNotConnectedChanged: {
        if (wifiNotConnected) {
            opacity = 0.0;
            visible = true;
            fadeIn.play();
        } else {
            // Only fade out if the fade in animation has ended otherwise just hide the container
            if(fadeIn.isEnded()) {
                fadeOut.play();
            } else {
                visible = false;
            }
        }
    }
    property bool showSpinner : false

	property string descriptionTextConnecting : qsTr("Searching for saved networks.") + Retranslate.onLanguageChanged
	
	// Check with the network manager if we should be using Wi-Fi or WLAN as the terminology in our strings and check if we should use "more" or not for
	// the possibility of multiple targets
    property string descriptionTextNoWifi : (multipleTargetsPossible ? 
        (networkManager.wlanStandard ? qsTr("Turn on WLAN to see more devices that you can play on.") :
        	qsTr("Turn on Wi-Fi to see more devices that you can play on.")) :
        (networkManager.wlanStandard ? qsTr("Turn on WLAN to see devices that you can play on.") :
            qsTr("Turn on Wi-Fi to see devices that you can play on."))) + Retranslate.onLanguageChanged
    
    property string descriptionTextNoNetwork : (multipleTargetsPossible ? 
        (networkManager.wlanStandard ? qsTr("Connect to a WLAN network to see more devices you can play on.") : 
            qsTr("Connect to a Wi-Fi network to see more devices you can play on.")) : 
        (networkManager.wlanStandard ? qsTr("Connect to a WLAN network to see devices you can play on.") : 
            qsTr("Connect to a Wi-Fi network to see devices you can play on."))) + Retranslate.onLanguageChanged
    
    property string turnWifiOnText : (networkManager.wlanStandard ? qsTr("Turn WLAN On"): 
    															    qsTr("Turn Wi-Fi On")) + Retranslate.onLanguageChanged
    
    attachedObjects:[
        TextStyleDefinition {
	        id: tsd
	        base: SystemDefaults.TextStyles.BodyText
	        color:MediaApplication.textColor
	    },
	    Timer {
	        id: timer
	        interval: 8000
	        onTimeout: {
	            // hide spinner and show button again
	            showSpinner = false;
	            description.text = descriptionTextNoNetwork;
            }
	    }
	]

    animations: [
        FadeTransition {
            id: fadeOut
            duration: 1000
            easingCurve: StockCurve.CubicOut
            toOpacity: 0.0
            onEnded: {
                visible = false;
            }
        },
        FadeTransition {
            id: fadeIn
            duration: 1000
            easingCurve: StockCurve.CubicOut
            toOpacity: 1.0
        }
    ] // Animations
    
    Container {
        rightPadding:80
        leftPadding:80
        
        ImageView {
            imageSource: "images/play_on_list_no_wifi.png"
            horizontalAlignment: HorizontalAlignment.Center
            visible: (networkManager.wifi == NetworkStatusManager.Disabled)
        }

        Label {
	        id: description
	        multiline:true
	        	        
	        textStyle {
	            base: tsd.style
	            textAlign: TextAlign.Center
	        }
	        
	        horizontalAlignment: HorizontalAlignment.Center
	        
	        text: (networkManager.wifi == NetworkStatusManager.Disabled) ? descriptionTextNoWifi : descriptionTextNoNetwork
	    }
	    
	    ActivityIndicator {
            id: spinner
            visible: showSpinner
            
            running: false
        	 
            preferredWidth: 100
            preferredHeight: 100
         
            horizontalAlignment: HorizontalAlignment.Center
        } 
		
	    Button{
	        id: wifiOnButton
            objectName: "PlayOnListWifi"
	        visible: !showSpinner
	        topMargin: 30
	        
	        //: Button label to turn on Wifi
	    	text: (networkManager.wifi == NetworkStatusManager.Disabled) ?  turnWifiOnText : (qsTr("Connect to network") + Retranslate.onLanguageChanged)
	    	horizontalAlignment: HorizontalAlignment.Center
	        layoutProperties: StackLayoutProperties {
	            
	        }
	        onClicked: {
	            if (networkManager.wifi == NetworkStatusManager.Disabled) {
	                showSpinner = true;
	                spinner.running = true;
	                description.text = descriptionTextConnecting;
	      	        timer.start();
	                networkManager.turnWifiOn();
	            } else {
			invokeHelper.invokeTargetApp("bb.action.OPEN",
				"sys.settings.card",
				"settings://wifi",
				"settings/view",
				[]);
	            }
	        } 
	    }  
	}  	
}
