import bb.cascades 1.0
import bb.system 1.0
import bb.cascades.multimedia 1.0
import bb.cascades.multimedia.internal 1.0
import bb.cascades.music 1.0
import "../core/PlUtils.js" as PlUtils
import "../core/ContentUris.js" as Uris
import "../core/layoutconstants.js" as LayoutConstants

Page {
    id: newPlayerScreen
    objectName: "NewPlayerScreenObjectName" // Required by various methods in SafePushNavPane.qml

    property bool showPlayOnAction: appContext.showPlayOnAction
    property InvokeActionItem p_platformActionItems: null;
    
    // song details
    property string __curSongTitle: ""
    property string __curSongAlbum: ""
    property string __curSongAlbumId: ""
    property string __curSongArtist: ""
    property string __curSongAlbumArtistId: ""
    property string __curSongAlbumArtist: ""
    property string __curSongPlaylistId: ""
    property string __curSongPlaylistName: ""
    property string __curSongFid
    property bool __isPlaylist: (__curSongPlaylistId != "" ? true : false)
    
    property bool useSelectedIndexPath: false
    property bool connectedToLocal: true
    
    property int windowState: appContext.windowState // used to handle "thumbnail" event to disable timeout screen
    property Page topPage: tabbedPane.currentNavPane.top // used to disable timeout screen when the top page has changed

    property M3uPlaylistDataModel createNewPlaylistDataModel // used to save the current dynamic playlist

    paneProperties: NavigationPaneProperties {
        backButton: ActionItem {
            id: newPlayerBackButton
            objectName: "newPlayerBackButton"
            onTriggered: {
                onPagePopped();
                tabbedPane.currentNavPane.pop();
            }
        }
    }
    	
    /*
     * Note: If you wish to add, remove, or change ActionItems, you must also update the updateActions
     * function.
     */	                
    actions: [
        ActionItem {
            id: playOnAction
            objectName: "NewPlayerScreenPlayOn"
            title: qsTr("Play on") + Retranslate.onLanguageChanged
            imageSource: "asset:///images/playing/ic_play_on.png"
            onTriggered: {
                appContext.switchToRemotePlayback();
            }
        },
        ActionItem {
            id: previousAction
            title: qsTr("Previous") + Retranslate.onLanguageChanged //USED BY STABILITY TESTING
            imageSource: "asset:///images/playing/ic_previous.png"
            ActionBar.placement: ActionBarPlacement.OnBar
            objectName: "NewPlayerScreenPreviousActionItem"
            onTriggered: {
                appContext.playbackController.prev();
            }
        },
        ActionItem {
            id: playAction
            objectName: "playAction" //USED BY STABILITY TESTING

            // disable button when not pausable (and still playing)
            // NOTE: Have to re-use "appContext.playbackController.playbackState == PlaybackController.PlaybackPlaying"
            // in multiple locations because trying to refactor it into a property fails when screen in closed and then
            // re-opened (for reasons unknown)
            enabled: (! (! appContext.playbackController.pausable && (appContext.playbackController.playbackState == PlaybackController.PlaybackPlaying ) ) )
            ActionBar.placement: ActionBarPlacement.OnBar
            imageSource: "asset:///images/playing/ic_play.png" // Using play icon as the default icon to start with
            title: qsTr("Play") + Retranslate.onLanguageChanged; // Using play string as the default string to start with
            onTriggered: {
                if (appContext.playbackController.playbackState == PlaybackController.PlaybackPlaying) {
                    analytics.logEvent("PauseEvent", { "position" : appContext.playbackController.position, "volume" : appContext.playbackController.volume });
                    appContext.playbackController.pause()
                } else {
                    analytics.logEvent("PlayEvent", { "position" : appContext.playbackController.position, "volume" : appContext.playbackController.volume });
                    appContext.playbackController.play()
                }
            }
            function setPlayingState(playing) {
                setIconImages();
            }
            function setIconImages() {
                if (appContext.playbackController.playbackState == PlaybackController.PlaybackPlaying) {
                    imageSource = "asset:///images/playing/ic_pause.png"
                    title = qsTr("Pause") + Retranslate.onLanguageChanged; //USED BY STABILITY TESTING
                } else {
                    imageSource = "asset:///images/playing/ic_play.png"
                    title = qsTr("Play") + Retranslate.onLanguageChanged; //USED BY STABILITY TESTING
                }
            }
        },
        ActionItem {
            title: qsTr("Next") + Retranslate.onLanguageChanged //USED BY STABILITY TESTING
            id: nextAction
            imageSource: "asset:///images/playing/ic_next.png"
            ActionBar.placement: ActionBarPlacement.OnBar
            objectName: "NewPlayerScreenNextActionItem"
            onTriggered: {
                appContext.playbackController.next();
            }
        },
        ActionItem {
            id: addToPlaylistAction
            title: qsTr("Add to Playlist") + Retranslate.onLanguageChanged
            imageSource: "asset:///images/playlists/ic_add_to_playlist.png" //USED BY STABILITY TESTING
            onTriggered: {
                //open add to playlist screen
                var page = tabbedPane.currentNavPane.safePushOnePage("core/SelectPlaylist.qml");
                commonAnalytics.instance.logPagePush("SelectPlaylist", "NewPlayerScreen");
                page.contentType = MediaLibraryContentType.AudioType;
                page.idsToAdd = [newPlayerScreen.__curSongFid];
            }
        },
        ActionItem {
            title: qsTr("Edit Order") + Retranslate.onLanguageChanged
            imageSource: "asset:///images/common/ic_edit.png"
            
            // Internal properties for Dynamic playlist editing
            property bool popEditPageOnLoaded: false // pop the edit page when PlaylistList is loaded
            property Page editPage: null
            
            onTriggered: {
                 nowPlayingAlbumList.playlistDataModel.loaded.connect(onPlaylistLoaded);
                 currentNavPane.pushTransitionEnded.connect(onPushTransitionEnded);
                 editPage = currentNavPane.safePushOnePage("core/PlaylistEdit.qml");
                 commonAnalytics.instance.logPagePush("PlaylistEdit", "NewPlayerScreen");
                 editPage.loadDataModel = true;
                 editPage.textField.enabled = false;
                 editPage.setTitle(qsTr("Edit Order") + Retranslate.onLanguageChanged);
                 editPage.playlistSaved.connect(playlistSaved);
            }

            function playlistSaved(plid, filename, four_square_list) {
                popEditPageOnLoaded = true; // pop the edit page after PlaylistList is loaded
                nowPlayingAlbumList.playlistDataModel.load();
                editPage.playlistSaved.disconnect(playlistSaved);
            }
            
            function onPlaylistLoaded() {
                if (popEditPageOnLoaded) {
                    popEditPageOnLoaded = false;
                    editPage.popPlaylistEditPage();
                    editPage = null;
                    nowPlayingAlbumList.playlistDataModel.loaded.disconnect(onPlaylistLoaded);
                }
            }
            
            function onPushTransitionEnded() {
                console.log("NewPlayerScreen - set the model after push transition ended");
                currentNavPane.pushTransitionEnded.disconnect(onPushTransitionEnded);
                editPage.playlistDataModel = musicUtils.createDynamicPlaylistDataModel(Uris.dynamicPlaylistEdit());
            }
        },
        ActionItem {
            id: propertiesAction
            title: qsTr("Properties") + Retranslate.onLanguageChanged
            imageSource: "asset:///images/playing/ic_properties.png"
            onTriggered: {
                appContext.launchProperties(__curSongFid);
            }
        },
        ActionItem {
            id: saveAsPlaylistAction
            title: qsTr("Save as Playlist") + Retranslate.onLanguageChanged
            imageSource: "asset:///images/common/ic_done.png"
            onTriggered: {
                createNewPlaylistDataModel = musicUtils.createEmptyM3uPlaylistDataModel();
                saveDialog.inputField.defaultText = createNewPlaylistDataModel.generateNewFileName();
                saveDialog.show();
            }
        },
        SwitchAudioActionItem {
            id: switchAudioAction
            audioRoutingComponent: AudioRoutingComponent
        }
    ]

    keyListeners: [
        KeyListener {
            onKeyEvent: {
                toggleFullScreen();
            }
        }
    ]
       
    attachedObjects: [
        ComponentDefinition {
            id: imageViewComponent
            ImageView {
                imageSource: "asset:///images/playing/playing_time_slider.png"
            }
        },
        SystemPrompt {
           id: saveDialog
           title: qsTr ("Type a name") + Retranslate.onLanguageChanged
           inputField.emptyText: qsTr("Type Playlist Name") + Retranslate.onLanguageChanged
           confirmButton.label: qsTr ("Create") + Retranslate.onLanguageChanged
           cancelButton.label: qsTr ("Cancel") + Retranslate.onLanguageChanged
        }
   ]
    
    onCreationCompleted: {
        // If we are playing a dynamic playlist add the edit action item
        // and connection to the loaded signal.
        saveDialog.finished.connect(onSaveDialogAccepted);
                
        appContext.addAppWorldActionItem(newPlayerScreen);

        qmlUtils.addActionItemToPage(propertiesAction, newPlayerScreen, ActionBarPlacement.InOverflow);

        appContext.playerScreenClosed.connect(onPagePopped);
        localeHandler.languageChanged.connect(onLocaleHandlerChanged);

        //setup the initial state
        setInitialState();

        //determine if we the app is in remote or local playback mode
        var local = (appContext.playbackController == localPlaybackController);

        //connect appropriate signals
        switchPlaybackMode(local, true);
        playerStub.playOnDeviceChangedFirstSignal.connect(onPlayOnDeviceChanged);

        //if we are playing remotely be sure to show the remote info pane
        if (! local) {
            nowPlayingAlbumList.remoteRendererName = remotePlaybackController.playOnDeviceName;
            nowPlayingHeader.handleRemoteActivityChanged(remotePlaybackController.remoteActivity);
        }
        
        updateActions();

	//Connect Analytics
	playAction.triggered.connect(analytics.logPlayPausePressed);
    }
    onWindowStateChanged: {
        if (windowState == 0) { // do not restart timer if it is in thumbnailed state.
            nowPlayingAlbumList.restartTimer();
        }
    }
    onActionMenuVisualStateChanged: {
        enableTimeoutScreen();
    }
    onTopPageChanged: {
        enableTimeoutScreen();
    }
    function onSaveDialogAccepted() {
		if(saveDialog.result == SystemUiResult.ConfirmButtonSelection){
			createNewPlaylistDataModel.setName(saveDialog.inputFieldTextEntry());
			createNewPlaylistDataModel.append(Uris.saveDynamicPlayList());
            createNewPlaylistDataModel.persistCompleted.connect(onPersistCompleted);
            createNewPlaylistDataModel.persistData();
        } else {
            //clean up since user canceled the dialog
            createNewPlaylistDataModel.destroy();
            createNewPlaylistDataModel = null;
        }
    }

    function onPersistCompleted(result, plid, filename) {
        if (result == PlaylistEditErrorType.PLERROR_OK) {
            systemDialogUtils.showSystemToast(qsTr("A new playlist: \"%1\" has been created").arg(createNewPlaylistDataModel.getName()));
        } else {
            systemDialogUtils.showSystemToast(PlUtils.getErrorText(result));
        }
        //clean up the model
        createNewPlaylistDataModel.destroy();
        createNewPlaylistDataModel = null;
    }

    onShowPlayOnActionChanged: {
        updateActions();
    }
    
    function updateActions() {
        if (appContext.showPlayOnAction) {
            // remove addToPlaylistAction and then re-add afterwards (so that it is
            // below playOnAction in the menu)
            qmlUtils.removeActionItemFromPage(saveAsPlaylistAction, newPlayerScreen);
            qmlUtils.removeActionItemFromPage(switchAudioAction, newPlayerScreen);
            qmlUtils.removeActionItemFromPage(addToPlaylistAction, newPlayerScreen);
            
            // Adding PlayOn, PlatformActionitems(Share, Set As Ringtone), Save as playlist, audio, addToPlaylist
            qmlUtils.addActionItemToPage(playOnAction, newPlayerScreen, ActionBarPlacement.InOverflow);
            updatePlatformActionItems();
            qmlUtils.addActionItemToPage(saveAsPlaylistAction, newPlayerScreen, ActionBarPlacement.InOverflow);
            qmlUtils.addActionItemToPage(switchAudioAction, newPlayerScreen, ActionBarPlacement.InOverflow);
            qmlUtils.addActionItemToPage(addToPlaylistAction, newPlayerScreen, ActionBarPlacement.InOverflow);
        } else {
            qmlUtils.removeActionItemFromPage(playOnAction, newPlayerScreen);
        }
    }

    function enableTimeoutScreen() {
        if (PlaybackController.PlaybackPlaying == appContext.playbackController.playbackState // is currently playing?
               && ActionMenuVisualState.Hidden == actionMenuVisualState // is menu visual hidden.
               && ActionMenuVisualState.Hidden == contextMenuHandler.visualState // is context menu hidden?
               && objectName == topPage.objectName) { // is the this page on the top of the stack?
            nowPlayingAlbumList.fullScreenModeEnabled = LayoutConstants.TIMEOUT_SCREEN_ENABLED
        } else {
            nowPlayingAlbumList.fullScreenModeEnabled = false;
        }
    }
    function onPagePopped() {
        appContext.playerScreenClosed.disconnect(onPagePopped);
        nowPlayingAlbumList.timeSlider0.disconnectSlots();
        volumeControl.disconnectVolumePlaybackController();
        volumeControl.increaseVolume.disconnect(increaseVolume);
        volumeControl.decreaseVolume.disconnect(decreaseVolume);
        saveDialog.finished.disconnect(onSaveDialogAccepted);
        playerStub.playOnDeviceChangedFirstSignal.disconnect(onPlayOnDeviceChanged)

        // exiting while remote playback is active, need to hand off control to the stub
        // and disconnect specific slots
        if (appContext.playbackController == remotePlaybackController) {
            disconnectRemotePlaybackControllerSignals();
        } else {
            disconnectLocalPlaybackControllerSignals();
        }
    }
    function setInitialState() {

        //update the song info and playlist from what is currently playing
        updateCurrentSong();
        updatePlaylist();

        //need to update the time slider manually with a duration
        if (appContext.playbackController.duration) {
            nowPlayingAlbumList.timeSlider0.sliderBarToValue = appContext.playbackController.duration;
        }
        if (appContext.playbackController.position) {
            nowPlayingAlbumList.timeSlider0.updateSliderPosition (appContext.playbackController.position);
        }

        //set default button states
        updateNextButtonState(appContext.playbackController.nextAvailable);
        updatePrevButtonState(appContext.playbackController.prevAvailable);
        updateShuffleButtonState();
        updateRepeatButtonState();

        //ensure we are in the right play state
        onPlaybackStateUpdate();
    }

    /**
     * Connect the signals for the local playback controller.
     */
    function connectLocalPlaybackControllerSignals() {
        localPlaybackController.currentMediaItemChanged.connect(updateCurrentSong)
        localPlaybackController.playlistChanged.connect(updatePlaylist)
        localPlaybackController.nextAvailableChanged.connect(updateNextButtonState)
        localPlaybackController.prevAvailableChanged.connect(updatePrevButtonState)
        localPlaybackController.playbackStateChanged.connect(onPlaybackStateUpdate)
        localPlaybackController.shuffleModeChanged.connect(updateShuffleButtonState)
        localPlaybackController.repeatModeChanged.connect(updateRepeatButtonState)
        nowPlayingAlbumList.timeSlider0.updateOnSeek = true;
        volumeControl.connectVolumePlaybackController(localPlaybackController, false);
        volumeControl.increaseVolume.connect(increaseVolume);
        volumeControl.decreaseVolume.connect(decreaseVolume);
        remotePlaybackController.remoteActivityChanged.disconnect(nowPlayingHeader.handleRemoteActivityChanged);
    }

    /**
     * Connect the signals for the remote playback controller.
     */
    function connectRemotePlaybackControllerSignals() {
        remotePlaybackController.currentMediaItemChanged.connect(updateCurrentSong)
        remotePlaybackController.playlistChanged.connect(updatePlaylist)
        remotePlaybackController.nextAvailableChanged.connect(updateNextButtonState)
        remotePlaybackController.prevAvailableChanged.connect(updatePrevButtonState)
        remotePlaybackController.playbackStateChanged.connect(onPlaybackStateUpdate)
        remotePlaybackController.shuffleModeChanged.connect(updateShuffleButtonState)
        remotePlaybackController.repeatModeChanged.connect(updateRepeatButtonState)
        remotePlaybackController.remoteActivityChanged.connect(nowPlayingHeader.handleRemoteActivityChanged);
        nowPlayingAlbumList.timeSlider0.updateOnSeek = false;
	    volumeControl.connectVolumePlaybackController(remotePlaybackController, true);
    }

    /**
     * Disconnect the signals for the local playback controller.
     */
    function disconnectLocalPlaybackControllerSignals() {
        localPlaybackController.currentMediaItemChanged.disconnect(updateCurrentSong)
        localPlaybackController.playlistChanged.disconnect(updatePlaylist)
        localPlaybackController.nextAvailableChanged.disconnect(updateNextButtonState)
        localPlaybackController.prevAvailableChanged.disconnect(updatePrevButtonState)
        localPlaybackController.playbackStateChanged.disconnect(onPlaybackStateUpdate)
        localPlaybackController.shuffleModeChanged.disconnect(updateShuffleButtonState)
        localPlaybackController.repeatModeChanged.disconnect(updateRepeatButtonState)
        volumeControl.increaseVolume.disconnect(increaseVolume);
        volumeControl.decreaseVolume.disconnect(decreaseVolume);
    }

    /**
     * Disconnect the signals for the remote playback controller.
     */
    function disconnectRemotePlaybackControllerSignals() {
        remotePlaybackController.currentMediaItemChanged.disconnect(updateCurrentSong)
        remotePlaybackController.playlistChanged.disconnect(updatePlaylist)
        remotePlaybackController.nextAvailableChanged.disconnect(updateNextButtonState)
        remotePlaybackController.prevAvailableChanged.disconnect(updatePrevButtonState)
        remotePlaybackController.playbackStateChanged.disconnect(onPlaybackStateUpdate)
        remotePlaybackController.shuffleModeChanged.disconnect(updateShuffleButtonState)
        remotePlaybackController.repeatModeChanged.disconnect(updateRepeatButtonState)
        remotePlaybackController.remoteActivityChanged.disconnect(nowPlayingHeader.handleRemoteActivityChanged);
    }

    /**
     * Connect the playback signals for the remote and local playback controllers.
     * @param bLocal A boolean which indicates if you are connecting the remote or local.
     */
    function switchPlaybackMode(bLocal, force) {
        if (force || connectedToLocal != bLocal) {
            connectedToLocal = bLocal;
            playOnAction.enabled = bLocal;
            nowPlayingAlbumList.remoteInfoVisible = ! bLocal;
            if (bLocal) {
                connectLocalPlaybackControllerSignals();
                disconnectRemotePlaybackControllerSignals();
            } else {
                connectRemotePlaybackControllerSignals();
                disconnectLocalPlaybackControllerSignals();
            }
        }
    }
    function updateShuffleButtonState() {
        nowPlayingAlbumList.updateShuffleState(appContext.playbackController.shuffleMode == PlaybackController.ShuffleOn);
    }
    function updateRepeatButtonState() {
        nowPlayingAlbumList.updateRepeatState(appContext.playbackController.repeatMode);
    }
    function updateNextButtonState(available) {
        nextAction.enabled = available;
    }
    function updatePrevButtonState(available) {
        previousAction.enabled = available;
    }
    function resetSliderPosition() {
        nowPlayingAlbumList.timeSlider0.sliderBarValue = 0
    }
    function hidePlayOn() {
        log("hidePlayOn", "hidePlayOn ->");
        switchPlaybackMode(true, false);
        nowPlayingHeader.handleRemoteActivityChanged(false);
        log("hidePlayOn", "hidePlayOn <-");
    }

    /**
     * Switch from the remote DLNA playback to the local device based playback.
     */
    function hidePlayOnAndSwitchToLocalPlayback() {
        hidePlayOn();
        appContext.switchToLocalPlayback();
    }

    /**
     * Simple log function to normalize output.
     * @param method The name of the method.
     * @param msg The descriptive message you are sending.
     */
    function log(method, msg) {
        console.debug(method + " : " + msg);
    }
    function increaseVolume() {
        appContext.playbackController.increaseVolume();
    }
    function decreaseVolume() {
        appContext.playbackController.decreaseVolume();
    }
    function onPlayOnDeviceChanged(enabled) {
        log("+onPlayOnDeviceChanged()", appContext.playbackController.playOnDevice);
        if (enabled) {
            log("onPlayOnDeviceChanged()", "playOnDevice successful");
            nowPlayingAlbumList.remoteRendererName = remotePlaybackController.playOnDeviceName;

            //We need to connect the signals before we set up playback so that all the right functions will be called
            switchPlaybackMode(false, false);
        } else {
            switchPlaybackMode(true, false);
        }
        log("-onPlayOnDeviceChanged() ", appContext.playbackController.playOnDevice);
    }

    /**
     * Updates the playback state.
     * 
     * Note: For DLNA this method should be being hit for two reasons:
     * 1) When playback state is playing a new song from the library.
     * 2) When playback state is stopped because the remote playback finishes.
     * This should result in the singals being switched back to local and the DLNA info being removed.
     */
    function onPlaybackStateUpdate() {
        switch (appContext.playbackController.playbackState) {
            case PlaybackController.PlaybackPlaying:
                log("onPlaybackStateUpdate", "playbackState:" + appContext.playbackController.playbackState + "=PlaybackController.PlaybackPlaying");
                playAction.setPlayingState(true);
                enableTimeoutScreen();
                nowPlayingAlbumList.restartTimer();
                break;
            case PlaybackController.PlaybackStopped:
                log("onPlaybackStateUpdate", "playbackState:" + appContext.playbackController.playbackState + "=PlaybackController.PlaybackStopped");
                if (playerStub.playbackActive) {
                    // Don't keep this screen alive if we get Stopped but aren't playing.
	                playAction.setPlayingState(false);
	                resetSliderPosition(); //fallthrough
	                nowPlayingAlbumList.fullScreenModeEnabled = false;
	            }
                break;
            case PlaybackController.PlaybackCompleted:
            	log("onPlaybackStateUpdate", "playbackState:" + appContext.playbackController.playbackState + "=PlaybackController.PlaybackCompleted");
            	playAction.setPlayingState(false);
            	// Do nothing for this case.
            	// PlayerStub.qml::__handlePlaybackStateChanged() will
            	// also recieve this signal and will safely pop the player
            	// screen via SafePushNavPane.
                break;
            default:
                log("onPlaybackStateUpdate", "playbackState:" + appContext.playbackController.playbackState);
                playAction.setPlayingState(false);
                nowPlayingAlbumList.fullScreenModeEnabled = false;
        }
        log("onPlaybackStateUpdate", "<-");
    }
    function onLocaleHandlerChanged() {
        if (playAction) {
            // Call function setPlayingState() to reset the 'Play/Pause' button text so that retranslation works
            playAction.setPlayingState(appContext.playbackController.playbackState == PlaybackController.PlaybackPlaying);
        }
    }
    function updatePlaylist() {
        if (appContext.playbackController.playlist) {
            console.debug("updatePlaylist" + " playbackController : " + appContext.playbackController + " playlist : " + appContext.playbackController.playlist);
            nowPlayingAlbumList.playlistDataModel = appContext.playbackController.playlist;
        }
    }
    function updateCurrentSong() {
        if (appContext.playbackController != null && appContext.playbackController.currentSong != null) {
            var currentSong = appContext.playbackController.currentSong
            __curSongTitle = currentSong.title != null ? currentSong.title : (qsTr("Unknown") + Retranslate.onLanguageChanged)
            __curSongArtist = currentSong.artist != null ? currentSong.artist : (qsTr("Unknown") + Retranslate.onLanguageChanged)
            __curSongAlbumArtist = currentSong.album_artist != null ? currentSong.album_artist : ""
            __curSongPlaylistId = currentSong.plid != null ? currentSong.plid : ""
            __curSongPlaylistName = currentSong.playlist_name == null ? (qsTr("Unknown") + Retranslate.onLanguageChanged) : currentSong.playlist_name
            __curSongAlbum = currentSong.playlist_name != null ? currentSong.playlist_name : (currentSong.album != null && currentSong.album != "" ? currentSong.album : qsTr("Unknown"))
            if (__curSongFid != currentSong.fid) {
                nowPlayingAlbumList.timeSlider0.sliderBarValue = 0;
            }
            if (currentSong.fid) {
                __curSongFid = currentSong.fid;
            }
            if (appContext.playbackController.duration) {
                nowPlayingAlbumList.timeSlider0.sliderBarToValue = appContext.playbackController.duration;
            }
            nowPlayingAlbumList.setAlbumArt(currentSong.art_file);
            fullAlbumCover.setAlbumArt(currentSong.art_file);
            nowPlayingAlbumList.setAlbumName(__curSongAlbum);
            nowPlayingAlbumList.restartTimer();
            updateActions();
        }
    }
    
    function updatePlatformActionItems() {
        // Remove old platformActionItems.
        if (p_platformActionItems != null) {
            qmlUtils.removeActionItemFromPage(p_platformActionItems, newPlayerScreen);
        }
        
        var currentSong = appContext.playbackController.currentSong;
        if (currentSong.fullpath != "") {
            // create the invoke action item specific to this list item
            var metadata;
            if (currentSong.art_file) {
                metadata = {"thumb": currentSong.art_file};
            }
            
            // Keep a reference to the old platformActionItem so we can remove it later
            p_platformActionItems = appContext.getPlatformActionItems(currentSong.fullpath, "", metadata);
            qmlUtils.addActionItemToPage(p_platformActionItems, newPlayerScreen, ActionBarPlacement.InOverflow);
        }
    }
    
    function togglePlayback(indexPath) {
        var selectedItem = nowPlayingAlbumList.listView.dataModel.data(indexPath);

        if(selectedItem != null &&
            (selectedItem.fullpath == appContext.playbackController.currentSong.fullpath &&
            appContext.playbackController.currentSong.fullpath != "" &&  appContext.playbackController.currentSong.fullpath != null &&
            selectedItem.np_oid == appContext.playbackController.currentSong.np_oid)){
            //selected an already playing track, play/pause toggle
            if (appContext.playbackController.playbackState == PlaybackController.PlaybackPlaying) {
                appContext.playbackController.pause()
            } else {
                appContext.playbackController.play()
            }
        } else {
            appContext.playbackController.currentIndex = indexPath;
        }
    }
    
    // If time-out screen is on, then change it to off.
    // If time-out screen is off, restart the timer so that when it goes to full screen
    //    after time out.
    function toggleFullScreen() {
        if (fullAlbumCover.visible) {
            nowPlayingAlbumList.setFullCoverOff();
        } else {
            nowPlayingAlbumList.restartTimer();
        }
    }
   
    //top level container
    Container {
        horizontalAlignment: HorizontalAlignment.Fill
        verticalAlignment: VerticalAlignment.Fill
        layout: DockLayout { }
        contextMenuHandler: ContextMenuHandler {
            id: contextMenuHandler
            onVisualStateChanged: {
                enableTimeoutScreen();
            }
        }
        onTouchCapture: {
            toggleFullScreen();
        }
        Container {
            id: mainPlayerScreen
            layout: StackLayout {
                orientation: LayoutOrientation.TopToBottom
            }
            background: ColorUtils.get(0xff121212)
            Container { // a container that allows NowPlayingHeader to be "replaced" by
                // NowPlayingVolumeControl and vice versa. It does this by by
                // setting opacity to 0.0 and 1.0.
                id: mainPlayerScreenTitle
                layout: DockLayout { }

                Container {
                    horizontalAlignment: HorizontalAlignment.Fill
                    preferredHeight: LayoutConstants.SCREEN_TITLE_HEIGHT
                    ImageView {
                        id: titleBar
                        imageSource: "asset:///images/common/core_titlebar_gradient_overlay_d.png"
                        horizontalAlignment: HorizontalAlignment.Fill
                        verticalAlignment: VerticalAlignment.Fill
                    }
                }
                
                Container {
                    attachedObjects: [
                        DockLayout {
                            id: dockLayout
                        },
                        StackLayout {
                            id: stackLayout
                            orientation: LayoutOrientation.TopToBottom
                        }
                    ]

                    layout: LayoutConstants.VOLUME_CONTROL_DEFAULT_VISIBILITY ? stackLayout : dockLayout
                
                    //artist/track/info header
                    NowPlayingHeader {
                        id: nowPlayingHeader
                        bottomMargin: 5
                        songTitle: __curSongTitle
                        artistTitle: __curSongArtist
                    }
                
                    NowPlayingVolumeControl {
                        id: volumeControl
                        objectName: "volumeControl"
                        opacity: LayoutConstants.VOLUME_CONTROL_DEFAULT_OPACITY
                        visible: LayoutConstants.VOLUME_CONTROL_DEFAULT_VISIBILITY
                    }
                }
            }
            Container {
                layout: DockLayout {
                }

                //album information, volume and time slider
                NowPlayingAlbumList {
                    id: nowPlayingAlbumList
                    onShufflePressed: {
                        appContext.playbackController.toggleShuffle();
                    }
                    onRepeatPressed: {
                        appContext.playbackController.cycleRepeatMode();
                        leftImageSpeaker:
                        "asset:///images/playing/volume_icon_off.png"
                        leftImageMute:
                        "asset:///images/playing/volume_icon_mute.png"
                        rightImageSpeaker:
                        "asset:///images/playing/volume_icon_on.png"
                        rightImageBoost:
                        "asset:///images/playing/volume_icon_boost.png"
                    }
                    onEjectPressed: {
                        log("nowPlayingAlbumList.onEjectPressed", "");
			commonAnalytics.instance.logPlayOnEjectPressed();
                        hidePlayOnAndSwitchToLocalPlayback();
                    }
                }
            }
        }

        // Album cover for the full-screen mode.
        // Another instance of NowPlayingAlbumCover is needed because the full-screen album cover
        // has to be on the top of everything -> It is currently not possible to change
        // the z-order of a container; thus a separate album cover is required for full screen mode.
        NowPlayingAlbumCover {
            id: fullAlbumCover
            fullScreen: true
            visible: false // invisible by default (only visible on full screen mode)
            leftPadding: LayoutConstants.ALBUM_COVER_H_PADDING
            rightPadding: LayoutConstants.ALBUM_COVER_H_PADDING
            horizontalAlignment: HorizontalAlignment.Fill
            verticalAlignment: VerticalAlignment.Fill
            translationY: nowPlayingAlbumList.c_ALBUM_COVER_OFFSET
        }

        // NowPlayingHeader for the time-out state screen.
        Container {
            id: fullAlbumCoverHeader
            preferredHeight: LayoutConstants.SCREEN_TITLE_HEIGHT
            layout: DockLayout {
            }
            visible: false
            ImageView {
                imageSource: "asset:///images/common/core_titlebar_gradient_overlay_d.png"
                horizontalAlignment: HorizontalAlignment.Fill
                verticalAlignment: VerticalAlignment.Fill
            }

            //artist/track/info header
            NowPlayingHeader {
                horizontalAlignment: HorizontalAlignment.Fill
                verticalAlignment: VerticalAlignment.Top
                background: Color.Transparent
                songTitle: __curSongTitle
                artistTitle: __curSongArtist
            }
        }
    }
}
