import bb.cascades 1.0
import bb.cascades.multimedia 1.0
import bb.cascades.multimedia.internal 1.0
import bb.cascades.music 1.0
import "../core"
import "../core/layoutconstants.js" as LayoutConstants

/**
 * This page is the Now Playing screen
 */
Container {
    id: nowPlayingAlbumList
        
    property NowPlayingTimeSlider timeSlider0: timeSlider0
    
    property alias listView: nowPlayingListView
    property variant playlistDataModel
    
    property alias remoteRendererName: nowPlayingAlbumCover.remoteRendererName
    property alias remoteInfoVisible: nowPlayingAlbumCover.remoteInfoVisible
    
    property bool fullScreenModeEnabled: false // only enabled on playback only (full screen mode is disabled on stop and pause)
        
    // the animation duration for all animations in this screen    
    property int c_ANIMATION_DURATION: LayoutConstants.ALBUM_ART_ANIMATION_DURATION
    
    // the timer for timeout state
    property int c_TIMEOUT_DURATION: 8000
    
    // album cover position
    property int c_ALBUM_COVER_OFFSET: (mainPlayerScreenTitle.visible ? LayoutConstants.SCREEN_TITLE_HEIGHT : 0)
            + (LayoutConstants.ALBUM_LIST_HEADER_TOP_PADDING << 1) // ALBUM_LIST_HEADER_TOP_PADDING is added twice here
                                                                   // because it is used as margin and padding
            + LayoutConstants.ALBUM_LIST_HEADER_HEIGHT

    property int p_initialY: -1
    property variant p_prevVisibleItem: [0]
    property bool p_volumeControlPullable: false
    
    signal shufflePressed()
    signal repeatPressed()
    signal ejectPressed()
    
    // attached objects for the screen
    attachedObjects: [
        Timer{
            id: timer
            singleShot: true
            
            onTimeout: {
                if (fullScreenModeEnabled && !fullAlbumCover.visible && windowState != 1) {
                    stopAllAnimations();
                    fullCoverOn.play();
                }
            }
        }
    ]
    
    onFullScreenModeEnabledChanged: {
        if (LayoutConstants.TIMEOUT_SCREEN_ENABLED) {
	        if (fullScreenModeEnabled) {
	            restartTimer();
	        } else {
	            stopTimer();
	            setFullCoverOff();
	        }
	    }
    }
    
    function setAlbumArt(uri){
        nowPlayingAlbumCover.setAlbumArt(uri);
    }
    
    function setAlbumName(albumName) {
        albumListHeader.albumName = albumName;
    }
    
    function updateShuffleState(state) {
        albumListHeader.updateShuffleState(state);
    }
    
    function updateRepeatState(state) {
        albumListHeader.updateRepeatState(state);
    }
       
    function restartTimer() {
        if (fullScreenModeEnabled) {
            timer.start(c_TIMEOUT_DURATION);
        }
    }
    
    function stopTimer() {
        timer.stop();
    }
    
    function setFullCoverOff() {
        if (fullAlbumCover.visible) {
	        stopAllAnimations();
	        fullCoverOff.play();
	    }
    }
    
    function stopAllAnimations() {
        openCover.stop();
        closeCover.stop();
        fullCoverOn.stop();
        fullCoverOnNext.stop();
        fullCoverOff.stop();
        fullCoverOffNext.stop();
    }
    
    // resetOpacityAndVisibility() is called when a full-cover animation is stopped, e.g.
    // because of user interaction.
    // It resets the opacity of all components that are changed during full-cover animation. 
    function resetOpacityAndVisibility() {
        timeSliderContainer.opacity = LayoutConstants.TIME_SLIDER_OPACITY;
        nowPlayingAlbumCover.remoteInfo.opacity = LayoutConstants.REMOTE_INFO_OPACITY;        
        mainPlayerScreenTitle.opacity = 1.0;
        fullAlbumCover.visible = false;
        fullAlbumCoverHeader.visible = false;
    }
    
    function scrollToPlayingItem() {
        p_prevVisibleItem = [appContext.playbackController.currentIndex];
        console.debug("scrollToAsyncModelItem: " + appContext.playbackController +  " : " + appContext.playbackController.currentIndex);
        nowPlayingListView.scrollToAsyncModelItem(appContext.playbackController.currentIndex);
    }
    
    layout: StackLayout  {
        orientation: LayoutOrientation.TopToBottom
    }
    
    Container {
        layout: StackLayout {
            orientation: LayoutOrientation.TopToBottom
        }
             
        horizontalAlignment: HorizontalAlignment.Fill
        
        layoutProperties:StackLayoutProperties {
            spaceQuota: 100
        }
            
        Container { // This container is to add padding on the left and right of the NowPlayingAlbumListHeader
            leftPadding: LayoutConstants.TRACKS_LIST_HORIZONTAL_PADDING
            rightPadding: LayoutConstants.TRACKS_LIST_HORIZONTAL_PADDING
            topPadding: LayoutConstants.ALBUM_LIST_HEADER_TOP_PADDING
            
            horizontalAlignment: HorizontalAlignment.Center
            verticalAlignment: VerticalAlignment.Top
            	            
            NowPlayingAlbumListHeader {
                id: albumListHeader
                leftPadding: 20
	            rightPadding: 0

                background: ColorUtils.get(0xffc4c4c4)
                horizontalAlignment: HorizontalAlignment.Fill
 
                onCreationCompleted: {                            
                    albumListHeader.shufflePressed.connect(nowPlayingAlbumList.shufflePressed);
                    albumListHeader.repeatPressed.connect(nowPlayingAlbumList.repeatPressed);
                }
            }
        }
        
        // the list and album art
        Container {
            horizontalAlignment: HorizontalAlignment.Fill
            verticalAlignment: VerticalAlignment.Top
            
            id: listBox
            
            layout: DockLayout{}
            
            Container { // This container is to add padding so that is align with the NowPlayingAlbumListHeader
                leftPadding: LayoutConstants.TRACKS_LIST_HORIZONTAL_PADDING
                rightPadding: LayoutConstants.TRACKS_LIST_HORIZONTAL_PADDING
            
	            //Container that holds the list sans album art
	            Container{
	                //padding added to the bottom of the list so album art does not cover the last track when it's pulled down
	                bottomPadding: LayoutConstants.TRACKS_LIST_BOTTOM_PADDING
	                horizontalAlignment: HorizontalAlignment.Fill 
	                
	                layoutProperties: StackLayoutProperties {
	                    spaceQuota: 100
	                }
	
	                layout: StackLayout {
	                    orientation: LayoutOrientation.TopToBottom                              
	                }
	
	                background: ColorUtils.get(0xffe4e4e4)
	
	                // the labels for the list
	                Container {
	                    id: labels
	                    horizontalAlignment: HorizontalAlignment.Fill
	                    layout: DockLayout {}
	                    background: ColorUtils.get(0xff909090)
	                    ListTextLabel {
	                        //:title for tracks column
	                        text1: qsTr("Song", "column title") + Retranslate.onLanguageChanged
	                        //:title for length of song
	                        text2: qsTr("Length", "column title")  + Retranslate.onLanguageChanged
	                        textColor1: Color.White
	                        textColor2: Color.White
	                    }
	                }
	
	                MusicListView {
	                    id: nowPlayingListView
	                    
	                    scrollRole: ScrollRole.None
	                    
	                    dataModel: CurrentSongDecorator {
	                        id: curSongDecorator
	                        checkKeyId: true 
                            playbackControllerRef: appContext.playbackController
                            dynamicPlaylistManager: appContext.dynamicPlaylistManagerRef
                            model: nowPlayingAlbumList.playlistDataModel
                        }
	                    horizontalAlignment: HorizontalAlignment.Fill
	                    	                    
	                    layoutProperties: StackLayoutProperties {
	                        spaceQuota: 100
	                    }
	                    layout: StackListLayout {}
                        
	                    listItemComponents:[
                            ListItemComponent {
                                type: ""
                                SimpleTwoRowListItemVisual {
                                    id: listItemComponent
                                    title: ((!ListItemData || !ListItemData["title"]) ? "" : ListItemData["title"])
                                    track: ((!ListItemData || !ListItemData["track"]) ? "" : ListItemData["track"])
                                    duration: ((!ListItemData || !ListItemData["duration"]) ? 0 : ListItemData["duration"])
                                    subTitle: ((ListItemData["artist"]) ? ListItemData["artist"] : "")
                                    itemSelected: ListItem.selected
                                    allowMultiSelectAction: false
                                    allowPlayNextAction: false
                                    allowPlayLastAction: false
                                    allowDeleteAction: false
                                    controller: ListItem.view.dataModel
                                    property variant currentIndexPath: ListItem.indexPath
                                }
                            },
	                        ListItemComponent{
	                            type:"fullItem"
	                            ListItemVisual{
	                                id: listFullItem
	                                title: ((!ListItemData || !ListItemData["title"]) ? "" : ListItemData["title"]) 
	                                track: ((!ListItemData || !ListItemData["track"]) ? "" : ListItemData["track"]) 
	                                duration: ((!ListItemData || !ListItemData["duration"]) ? 0 : ListItemData["duration"])
	                                fullpath: ((!ListItemData || !ListItemData["fullpath"]) ? "" : ListItemData["fullpath"])
                                    subTitle:  ((ListItemData["artist"]) ? ListItemData["artist"] : "")
	                                itemSelected: ListItem.selected
	                                allowMultiSelectAction: false
	                                allowPlayNextAction: false
	                                allowPlayLastAction: false
	                                allowDeleteAction: false
                                    controller: ListItem.view.dataModel
                                    currSong: ListItemData["currSong"]
                                    property variant currentIndexPath: ListItem.indexPath
                                }
                            }
                        ]
                        
                        attachedObjects: [
                            ListScrollStateHandler {
                                id: listScrollHandler
                                
                                onFirstVisibleItemChanged: {
                                    if (LayoutConstants.TRACKS_LIST_HIDE_ACTION_BAR && !nowPlayingHeader.visible
                                            && p_initialY >= 0) { // if p_initialY is negative, the user is not touching the list view 
                                        if (firstVisibleItem[0] > p_prevVisibleItem[0]) {
                                            mainPlayerScreenTitle.visible = false;
                                            nowPlayingAlbumCover.translationY = LayoutConstants.ALBUM_COVER_OPEN_POSITION
                                                + LayoutConstants.SCREEN_TITLE_HEIGHT + 100;
                                            newPlayerScreen.actionBarVisibility = ChromeVisibility.Hidden;
                                        } else if (firstVisibleItem[0] < p_prevVisibleItem[0] && !atEnd) {
                                            p_volumeControlPullable = true;
                                            newPlayerScreen.actionBarVisibility = ChromeVisibility.Default;
                                            nowPlayingAlbumCover.translationY = LayoutConstants.ALBUM_COVER_OPEN_POSITION 
                                                + (mainPlayerScreenTitle.visible == true ? 0 : LayoutConstants.SCREEN_TITLE_HEIGHT);
                                        }
                                        p_prevVisibleItem = firstVisibleItem;
                                    }
                                }
                            }
                        ]
	                    
	                    onActivationChanged: {
	                        if (active) {
                        	    nowPlayingListView.selectedIndexPath = indexPath;
	                            nowPlayingListView.activeFid = nowPlayingListView.dataModel.data(indexPath)["fid"];
	                            nowPlayingListView.activeKeyId = nowPlayingListView.dataModel.data(indexPath)["key_id"];
	                            nowPlayingListView.activeFullpath = nowPlayingListView.dataModel.data(indexPath)["fullpath"];
	                        }
	                    }

	                    onTriggered: {
	                        togglePlayback(indexPath);
	                    }

                        onTouch: {
                            if (LayoutConstants.TRACKS_LIST_HIDE_ACTION_BAR) {
                                if (event.touchType == TouchType.Down) {
                                    p_initialY = event.localY; // store the initial location to calculate the delta
                                } else if (event.touchType == TouchType.Move) {
                                    // Pull down the screen title if the first item of the list view is visible
                                    if (event.localY - p_initialY > 75 && listScrollHandler.firstVisibleItem[0] == 0) {
                                        newPlayerScreen.actionBarVisibility = ChromeVisibility.Default;
                                        mainPlayerScreenTitle.visible = true;
                                        nowPlayingAlbumCover.translationY = LayoutConstants.ALBUM_COVER_OPEN_POSITION;
                                    }
                                } else {
                                    p_initialY = -1;
                                }
                            }
                        } // end of onTouch
                    }
               }
            }
            
            // album cover
            NowPlayingAlbumCover {
                id: nowPlayingAlbumCover
                
                stickyPlayOnMode: ((appContext.playbackController == remotePlaybackController)
                                  && (appContext.playbackController.playbackState != PlaybackController.PlaybackPlaying)
                                  && (appContext.playbackController.playbackState != PlaybackController.PlaybackPaused)) 
                                  
                downPos: LayoutConstants.ALBUM_COVER_OPEN_POSITION
                downThreshold: LayoutConstants.ALBUM_COVER_OPEN_THRESHOLD                            
                
                horizontalAlignment: HorizontalAlignment.Fill
                verticalAlignment: VerticalAlignment.Fill
                leftPadding: LayoutConstants.ALBUM_COVER_H_PADDING
                rightPadding: LayoutConstants.ALBUM_COVER_H_PADDING
                
                onCoverOpenStarted: {
                    if (!openCover.isPlaying()) {
	                    stopAllAnimations();
	                    resetOpacityAndVisibility();
	                    openCover.play();
                        analytics.logEvent("NowPlayingCoverOpened");
                    }
                }
                
                onCoverCloseStarted: {
                    if (!closeCover.isPlaying()) {
	                    stopAllAnimations();
	                    resetOpacityAndVisibility();
	                    closeCover.play();
                        analytics.logEvent("NowPlayingCoverClosed");
	                }
                }
                
                onScrollStarted: {
                    if (!nowPlayingAlbumCover.coverOpen) { // if the album cover is closed,
                                                           // then scroll to the currently playing item;
                                                           // so that it is visible when the user starts
                                                           // to drag the album cover
                        scrollToPlayingItem();
                    }
                    
                    // Disable list view to prevent the user from accidentally scroll the list view
                    nowPlayingListView.touchPropagationMode = TouchPropagationMode.None;
                    
                    // Make volume control and now playing header visible because their opacity
                    // will be changed by onScrollProgress slot.
                    volumeControl.visible = true;
                    nowPlayingHeader.visible = true;
                }
                
                onScrollProgress: {
                    // Change the opacity of volume control and now playing header.
                    var progress = delta / LayoutConstants.ALBUM_COVER_OPEN_POSITION;
                    volumeControl.opacity = Math.max(LayoutConstants.VOLUME_CONTROL_DEFAULT_OPACITY, progress);
                    nowPlayingHeader.opacity = Math.max(LayoutConstants.NOW_PLAYING_TITLE_TARGET_OPACITY, 1.0 - progress);
                    
                    // Move the time slider
                    timeSliderContainer.translationY = delta;
                }
                    
                onScrollEnded: {
                    nowPlayingListView.touchPropagationMode = TouchPropagationMode.Full;

                    // Why don't we just call coverOpen.play() or coverClose.play() here?
                    // It is related to MKS 6343818, which happens if the user swipe down
                    // the album art quickly TWICE.
                    if (coverOpen) {
                        nowPlayingAlbumCover.translationY = LayoutConstants.ALBUM_COVER_OPEN_POSITION;
                        timeSliderContainer.translationY = 300;
                        volumeControl.opacity = 1.0;
                        nowPlayingHeader.opacity = LayoutConstants.NOW_PLAYING_TITLE_TARGET_OPACITY;
                        nowPlayingAlbumCover.coverArtCutClosed.opacity = 0.0;
                        nowPlayingAlbumCover.coverArtCutOpen.opacity = 1.0;
                        volumeControl.visible = true;
                        nowPlayingHeader.visible = LayoutConstants.NOW_PLAYING_TITLE_TARGET_VISIBILITY;
                    } else {
                        mainPlayerScreenTitle.visible = true;
                        nowPlayingAlbumCover.translationY = 0;
                        timeSliderContainer.translationY = 0;
                        volumeControl.opacity = LayoutConstants.VOLUME_CONTROL_DEFAULT_OPACITY;
                        nowPlayingHeader.opacity = 1.0;
                        nowPlayingAlbumCover.coverArtCutClosed.opacity = 1.0;
                        nowPlayingAlbumCover.coverArtCutOpen.opacity = 0.0;
                        volumeControl.visible = LayoutConstants.VOLUME_CONTROL_DEFAULT_VISIBILITY;
                        nowPlayingHeader.visible = true;
                        newPlayerScreen.actionBarVisibility = ChromeVisibility.Default;
                    }
                }
            }

            // the slider bar
            Container {
                id: timeSliderContainer
                
                layout: DockLayout {}
                
                horizontalAlignment: HorizontalAlignment.Fill
                verticalAlignment: VerticalAlignment.Bottom
                
                background: LayoutConstants.TIME_SLIDER_COLOR
                topPadding: LayoutConstants.TIME_SLIDER_OUTER_TOP_PADDING
                
                ControlDelegate {
                    // Use ImageView for N-Series so that the knob of the time slider is not transparent.
                    sourceComponent: LayoutConstants.TIME_SLIDER_USE_IMAGE_VIEW ? imageViewComponent : null
                    horizontalAlignment: HorizontalAlignment.Fill
                    verticalAlignment: VerticalAlignment.Fill
                }
                
                NowPlayingTimeSlider {
                    id: timeSlider0 //USED BY STABILITY TESTING
                    objectName: "timeSlider0"
	                
                    horizontalAlignment: HorizontalAlignment.Fill
                    verticalAlignment: VerticalAlignment.Bottom
                }
            }
        }
    } // end of container holding list and album art

    animations: [
        ParallelAnimation {
            id: openCover
            objectName: "openCover"
            TranslateTransition {
                target: nowPlayingAlbumCover
                toY: LayoutConstants.ALBUM_COVER_OPEN_POSITION
                duration: c_ANIMATION_DURATION
                easingCurve: StockCurve.CubicInOut
            }
            TranslateTransition {
                target: timeSliderContainer
                toY: 300
                duration: c_ANIMATION_DURATION
                easingCurve: StockCurve.CubicInOut
            }
            FadeTransition {
                target: volumeControl
                toOpacity: 1.0
                duration: c_ANIMATION_DURATION
                easingCurve: StockCurve.CubicInOut
            }
            FadeTransition {
                target: nowPlayingHeader
                toOpacity: LayoutConstants.NOW_PLAYING_TITLE_TARGET_OPACITY
                duration: c_ANIMATION_DURATION >> 1 // make the "fading effect" shorter
                easingCurve: StockCurve.CubicInOut
            }
            FadeTransition {
                target: nowPlayingAlbumCover.coverArtCutClosed
                toOpacity: 0.0
                duration: c_ANIMATION_DURATION
                easingCurve: StockCurve.CubicInOut
            }
            FadeTransition {
                target: nowPlayingAlbumCover.coverArtCutOpen
                toOpacity: 1.0
                duration: c_ANIMATION_DURATION
                easingCurve: StockCurve.CubicInOut
            }
            onStarted: {
                scrollToPlayingItem();
                volumeControl.visible = true;
            }
            onEnded:{
                nowPlayingHeader.visible = LayoutConstants.NOW_PLAYING_TITLE_TARGET_OPACITY;
            }
            
        },
        ParallelAnimation {
            id: closeCover
            objectName: "closeCover"
            
            TranslateTransition {
                target: nowPlayingAlbumCover
                toY: 0
                duration: c_ANIMATION_DURATION
                easingCurve: StockCurve.CubicInOut
            }
            TranslateTransition {
                target: timeSliderContainer
                toY: 0
                duration: c_ANIMATION_DURATION
                easingCurve: StockCurve.CubicInOut
            }
            FadeTransition {
                target: volumeControl
                toOpacity: LayoutConstants.VOLUME_CONTROL_DEFAULT_OPACITY
                duration: c_ANIMATION_DURATION / 2 // make the "fading effect" shorter
                easingCurve: StockCurve.CubicInOut
            }
            FadeTransition {
                target: nowPlayingHeader
                toOpacity: 1.0
                duration: c_ANIMATION_DURATION
                easingCurve: StockCurve.CubicInOut
            }
            FadeTransition {
                target: nowPlayingAlbumCover.coverArtCutClosed
                toOpacity: 1.0
                duration: c_ANIMATION_DURATION
                easingCurve: StockCurve.CubicInOut
            }
            FadeTransition {
                target: nowPlayingAlbumCover.coverArtCutOpen
                toOpacity: 0.0
                duration: c_ANIMATION_DURATION
                easingCurve: StockCurve.CubicInOut
            }
            onStarted: {
                mainPlayerScreenTitle.visible = true;
                nowPlayingHeader.visible = true;
                newPlayerScreen.actionBarVisibility = ChromeVisibility.Default;
            }
            onEnded: {
                volumeControl.visible = LayoutConstants.VOLUME_CONTROL_DEFAULT_VISIBILITY;
            }
        },
        
        // Animation to go to full screen
        // Step 1: "fullCoverOn"
        //         - Change the opacity of time slider and remote info (a.k.a. "Playing on").
        //         - Change the opacity of now playing header and volume control.
        // Step 2: "fullCoverOnNext"
        //         - Animate the album art to fill-up the screen.
        //
        // Why don't we use SequentialAnimation, instead of doing it in two ParallelAnimation's?
        // It is because we need to do something on onStarted() and onEnded() of the child animations.
        // Currently, these signals are not emitted on the child animations.
        ParallelAnimation {
            id: fullCoverOn
            objectName: "fullCoverOn"
            
            FadeTransition {
                target: timeSliderContainer
                toOpacity: 0.0
                duration: c_ANIMATION_DURATION
                easingCurve: StockCurve.CubicInOut
            }
            FadeTransition {
                target: nowPlayingAlbumCover.remoteInfo
                toOpacity: 0.0
                duration: c_ANIMATION_DURATION
                easingCurve: StockCurve.CubicInOut
            }
            onEnded: {
                fullCoverOnNext.play();
            }
        },
        
        ParallelAnimation {
            id: fullCoverOnNext
            objectName: "fullCoverOnNext"

            FadeTransition {
                target: fullAlbumCoverHeader
                fromOpacity: (nowPlayingAlbumCover.coverOpen ? 0.0 : 0.7)
                toOpacity: 0.7
                duration: c_ANIMATION_DURATION
                easingCurve: StockCurve.CubicInOut
            }
            TranslateTransition {
                target: fullAlbumCover
                fromY: nowPlayingAlbumCover.translationY + c_ALBUM_COVER_OFFSET
                toY: LayoutConstants.FULL_ALBUM_COVER_POSITION
                duration: c_ANIMATION_DURATION
                easingCurve: StockCurve.CubicInOut
            }
            onStarted: {
                fullAlbumCoverHeader.visible = true;
                fullAlbumCover.visible = true;
                newPlayerScreen.actionBarVisibility = ChromeVisibility.Hidden;
            }
        },
        
        // Animation to go to normal screen
        // Step 1: "fullCoverOff"
        //         - Animate the album art to the original position.
        // Step 2: "fullCoverOffNext"
        //         - Change the opacity of time slider and remote info (a.k.a. "Playing on").
        //         - Change the opacity of now playing header and volume control.        
        ParallelAnimation {
            id: fullCoverOff
            objectName: "fullCoverOff"

            FadeTransition {
                target: fullAlbumCoverHeader
                toOpacity: (nowPlayingAlbumCover.coverOpen ? 0.0 : 1.0)
                duration: c_ANIMATION_DURATION
                easingCurve: StockCurve.CubicInOut
            }
            TranslateTransition {
                target: fullAlbumCover
                toY: nowPlayingAlbumCover.translationY + c_ALBUM_COVER_OFFSET
                duration: c_ANIMATION_DURATION
                easingCurve: StockCurve.CubicInOut
            }
            onEnded: {
                mainPlayerScreenTitle.visible = true;
                nowPlayingAlbumCover.translationY = (nowPlayingAlbumCover.coverOpen ? LayoutConstants.ALBUM_COVER_OPEN_POSITION : 0);
                fullAlbumCoverHeader.visible = false;
                fullAlbumCover.visible = false;
                newPlayerScreen.actionBarVisibility = ChromeVisibility.Default;
                fullCoverOffNext.play();
            }
        },
        
        ParallelAnimation {
            id: fullCoverOffNext
            objectName: "fullCoverOffNext"

            FadeTransition {
                target: timeSliderContainer
                toOpacity: LayoutConstants.TIME_SLIDER_OPACITY
                duration: c_ANIMATION_DURATION
                easingCurve: StockCurve.CubicInOut
            }
            FadeTransition {
                target: nowPlayingAlbumCover.remoteInfo
                toOpacity: LayoutConstants.REMOTE_INFO_OPACITY
                duration: c_ANIMATION_DURATION
                easingCurve: StockCurve.CubicInOut
            }
            onEnded: {                
                restartTimer();
            }
        }
    ]
}
