import bb.cascades 1.0
import bb.cascades.multimedia 1.0
import bb.cascades.multimedia.internal 1.0
import "../core/layoutconstants.js" as LayoutConstants
import "../radio/components"
     
     
ControlDelegate {
    id: nowPlayingDelegate
    property bool showNowPlaying: false
    
    property bool musicPlaybackActive: (showNowPlaying && playerStub.playbackActive) || appContext.playbackController == remotePlaybackController
    
	//For sticky play on mode playback will not be active so we need to check if the current controller is the remote playback controller
	//If we're doing play on or in sticky play on mode then the current controller will be the remote one, otherwise it will be the local one.
    delegateActive: musicPlaybackActive || RadioPlaybackContext.radioPlaybackActive
    sourceComponent: RadioPlaybackContext.radioPlaybackActive ? radioPlayingDef : nowPlayingDef
    
    onError: {
        console.log("nowPlayingDelegate, error loading now playing definition, " + errorMessage);
    }
    
    attachedObjects: [
        ComponentDefinition {
            id: radioPlayingDef  
            Container {
                RadioNowPlayingScreenTitle {                    
                }                
            }
        },        
        ComponentDefinition {
          id: nowPlayingDef  
	      Container {
	        	 property variant currentController;
	        	 
	        	 // The animation duration for all album art animations    
				 property int c_ALBUM_ART_ANIMATION_DURATION: 400
				 property int c_ALBUM_ART_ANIMATION_DELAY: 150
	        	
	        	 animations: [       
	            	// Animation to fade out the previous album art and show the play icon momentarily
			    	ParallelAnimation {	 
			    		id: showPlayIcon 		    		
			    		FadeTransition {
			                target: nowPlayingButton
			                toOpacity: 0
			                duration: c_ALBUM_ART_ANIMATION_DURATION
			                easingCurve: StockCurve.CubicInOut
			            }		            
			            FadeTransition {
			                target: nowPlayingBorder
			                toOpacity: 0
			                duration: c_ALBUM_ART_ANIMATION_DURATION
			                easingCurve: StockCurve.CubicInOut
			            } 		                        		   
				        FadeTransition {
				            target: playIcon
				            toOpacity: 1
				            duration: c_ALBUM_ART_ANIMATION_DURATION
				            easingCurve: StockCurve.CubicInOut
				        } 	        
				        onEnded:{
				        	// Update the album art to the new one since we have faded out the previous one
				        	updateCurrentAlbumArt();	        	 
				        	id:showAlbumArt.play()
			            }
				    },
				    // Animation to fade out the play icon and show the next album art
			        ParallelAnimation {	   
			        	id:showAlbumArt     		
			        	delay: c_ALBUM_ART_ANIMATION_DELAY   
			            FadeTransition {
			                target: nowPlayingButton
			                toOpacity: 1
			                duration: c_ALBUM_ART_ANIMATION_DURATION
			                easingCurve: StockCurve.CubicInOut
			            }   		            
			            FadeTransition {
			                target: nowPlayingBorder
			                toOpacity: 1
			                duration: c_ALBUM_ART_ANIMATION_DURATION
			                easingCurve: StockCurve.CubicInOut
			            } 		            
			            FadeTransition {
				            target: playIcon
				            toOpacity: 0
				            duration: c_ALBUM_ART_ANIMATION_DURATION
				            easingCurve: StockCurve.CubicInOut	            
				        }     
                        FadeTransition {
                            target: plusIcon
                            toOpacity: 0
                            duration: c_ALBUM_ART_ANIMATION_DURATION
                            easingCurve: StockCurve.CubicInOut	            
                        }       
			        },
                    // Animation to fade out the previous album art and show the plus icon momentarily
                    ParallelAnimation {	 
                        id: showPlusIcon 		    		
                        FadeTransition {
                            target: nowPlayingButton
                            toOpacity: 0
                            duration: c_ALBUM_ART_ANIMATION_DURATION
                            easingCurve: StockCurve.CubicInOut
                        }		            
                        FadeTransition {
                            target: nowPlayingBorder
                            toOpacity: 0
                            duration: c_ALBUM_ART_ANIMATION_DURATION
                            easingCurve: StockCurve.CubicInOut
                        } 		                        		   
                        FadeTransition {
                            target: plusIcon
                            toOpacity: 1
                            duration: c_ALBUM_ART_ANIMATION_DURATION
                            easingCurve: StockCurve.CubicInOut
                        } 	        
                        onEnded:{
                            // Update the album art to the new one since we have faded out the previous one
                            updateCurrentAlbumArt();	        	 
                            id:showAlbumArt.play()
                        }
                    }
			    ]
			    
			    // Are any of the album animation running?
			    function animationRunning() {
                    return showPlayIcon.isPlaying() || showPlayIcon.isStarted() || showAlbumArt.isPlaying() || showAlbumArt.isStarted() || showPlusIcon.isPlaying() || showPlusIcon.isStarted() ;
			    }
			    
			    // Is show album art animation running?
			    function showAlbumAnimationRunning() {
			    	return showAlbumArt.isPlaying() || showAlbumArt.isStarted();
			    }
			    
                function songChanged () {
                    // TODO: See MKS 6934800
                    // Sometimes we switch the source component from nowPlayingComponent (this)
                    // to the empty component, but these signals still execute, however if this (nowPlayingComponent)
                    // is not the active sourceComponent 'tabbedPane' is not a valid symbol causing an exception.
                    // If it is not caught, we might end up not executing a bunch of JavaScript methods after this.
                    try {
                        if (!animationRunning()){
                            // If we don't have any animation running, start a new one
                            showPlayIcon.play();   
                        } else if (showAlbumAnimationRunning()) {
                            // If we are in currently trying to fade in the new album art, set the correct
                            // album art otherwise we will be showing the wrong one at the end of the animation
                            updateCurrentAlbumArt();
                        }
                    } catch (e) {
                        console.debug("nowplaying/NowPlayingScreenTitle.qml::songChanged() - exception caught, likely due to current source component");
                    }
                }
	 
			    function updateCurrentAlbumArt () {
			         if (currentController != null) {
				         var currentTrack = currentController.currentSong;
				         nowPlayingButton.imageSource = (currentTrack.thumb_file == "" || currentTrack.thumb_file == null) ? 
				             "asset:///images/common/Now_playing_default_album_art.png" : currentTrack.thumb_file;
			         }
			    }     
			          
			    function onPlaybackControllerChanged(playbackController){
					console.log("Now playing screen title controller changed " + playbackController);
					if(currentController != null){
                        currentController.currentMediaItemChanged.disconnect(songChanged);
                    }
					
					if(playbackController != null){
						currentController = playbackController;
						currentController.currentMediaItemChanged.connect(songChanged);					
					}
			    }   

				function onNowPlayingItemQueued () {
                    // TODO: See MKS 6934800
                    // Sometimes we switch the source component from nowPlayingComponent (this)
                    // to the empty component, but these signals still execute, however if this (nowPlayingComponent)
                    // is not the active sourceComponent 'tabbedPane' is not a valid symbol causing an exception.
                    // If it is not caught, we might end up not executing a bunch of JavaScript methods after this.
			        try {
		            	if (!animationRunning()){
		     				// If we don't have any animation running, start a new one
                             showPlusIcon.play();   
		    			} 
                	} catch (e) {
                	    console.debug("ScreenTitle.qml::onNowPlayingItemQueued() - exception caught, likely due to current source component");
                	}
			    }    
			     
			    function playerClosed() {
			        // TODO: See MKS 6934800
			        // Sometimes we switch the source component from nowPlayingComponent (this)
			        // to the empty component, but these signals still execute, however if this (nowPlayingComponent)
			        // is not the active sourceComponent 'tabbedPane' is not a valid symbol causing an exception.
			        // If it is not caught, we might end up not executing a bunch of JavaScript methods after this.
			        try {
				     	// We got out first player closed event. Disconnect
				     	// since we only want to show the album art animation, the 
				     	// very first time we play and exit the Player screen.
				     	appContext.playerScreenClosed.disconnect(playerClosed);
				     	
				     	// Play the album art animation
				     	songChanged();

			      	} catch (e) {
                        console.debug("ScreenTitle.qml::playerClosed() - exception caught, likely due to current source component");
                    }
			    }
			    
	        	onCreationCompleted: {    
	        	    console.log("Now playing screen title created");
	        		// Initialize our controller to the current one       
					onPlaybackControllerChanged(appContext.playbackController);
					
					// Update our album art to the current one
					updateCurrentAlbumArt();
					
					// Register for future playback controller changes
					appContext.newPlaybackController.connect(onPlaybackControllerChanged);
					
                    // Register for a signal when an item is queued to the Now Playing list
                    dynamicPlaylistManager.modifyRequestCompleted.connect(onNowPlayingItemQueued);

					
					// Register for player screen closed events if playback is not active
					// If it is active then we must have opened and closed the Player screen
					// at some point. We only want to show the album art animation, the 
					// very first time we play and exit the Player screen.
					if (!appContext.playerScreenHasBeenClosed) {
						appContext.playerScreenClosed.connect(playerClosed);
					}
	 			}
	 			
	            layout: StackLayout {
	                orientation: LayoutOrientation.LeftToRight
	            }
	            verticalAlignment: VerticalAlignment.Center

	            Container {
	                id: eject
	                layout: DockLayout {
	                }
	                leftPadding: padding
	                rightPadding: padding
	                visible: (appContext.playbackController == remotePlaybackController)
	                MultiStateImageView {
	                    id: ejectButton
	                    upImages: [
	                        "asset:///images/playing/eject-icon.png"
	                    ]
	                    downImages: [
	                        "asset:///images/playing/eject_selected-icon.png"
	                    ]
	                    onTouch: {
	                         if (event.isUp () && enabled) {  
	                           	enabled = false;
                                        commonAnalytics.instance.logPlayOnEjectPressed();
	                         	appContext.switchToLocalPlayback();
	                            enabled = true; 
	                        }
	                    }
	                }
	            }
	            ImageView {
	                visible: (appContext.playbackController == remotePlaybackController) && showNowPlaying && playerStub.playbackActive
	                imageSource: "asset:///images/playing/Tab-divider.png"
	            }
	            Container {
	                id: nowPlaying
	                visible: showNowPlaying && playerStub.playbackActive
	                property bool __fingerDown
	                layout: DockLayout {
	                }
	                leftPadding: padding
	                rightPadding: padding
	                verticalAlignment: VerticalAlignment.Center
	                horizontalAlignment: HorizontalAlignment.Right
	                ImageView {
	                    id: nowPlayingButton
	                    objectName: "nowPlayingButton" //USED BY STABILITY TESTING
	                    preferredHeight: 81
	                    preferredWidth: 81
	                    scalingMethod: ScalingMethod.AspectFill
	                    verticalAlignment: VerticalAlignment.Center
	                    horizontalAlignment: HorizontalAlignment.Center
	                }
	                ImageView {
	                    id: nowPlayingBorder
	                    preferredHeight: 85
	                    preferredWidth: 85
	                    verticalAlignment: VerticalAlignment.Center
	                    horizontalAlignment: HorizontalAlignment.Center
	                    imageSource: nowPlaying.__fingerDown ? "asset:///images/common/core_item_border.amd" : "asset:///images/common/ic_albumart_nowplaying_frame.amd"
	                }
	                ImageView {
			        	id:playIcon
		        		opacity:0
		        		scalingMethod: ScalingMethod.None
			        	verticalAlignment: VerticalAlignment.Center
	 			    	horizontalAlignment: HorizontalAlignment.Center  
			    		imageSource: "asset:///images/playing/control_play_mini.png"			    	
			    	}
                    ImageView {
                        id:plusIcon
                        opacity:0
                        scalingMethod: ScalingMethod.None
                        verticalAlignment: VerticalAlignment.Center
                        horizontalAlignment: HorizontalAlignment.Center  
                        imageSource: "asset:///images/playing/control_plus_mini.png"			    	
                    }
	                onTouch: {
	                    if (event.isDown()) {
	                        __fingerDown = true;
	                    } else if (event.isCancel()) {
	                        __fingerDown = false;
	                    } else if (event.isUp() && __fingerDown) {
	                        __fingerDown = false;
                                analytics.logEvent("NowPlayingPressed");
	                        tabbedPane.currentNavPane.safePushOnePage("nowplaying/NewPlayerScreen.qml");
                            commonAnalytics.instance.logPagePush("NewPlayerScreen", "NowPlayingScreenTitle");
	                    }
	                }
	            }
	        }
	    }
    ]

    
}    
