import bb.cascades 1.0
import bb.cascades.music 1.0
import bb.cascades.multimedia.internal 1.0
import "../components"
import "../../core/layoutconstants.js" as LayoutConstants

Container {
    NoContentView {
        id: noFavouritesView
        image: "asset:///images/radio/fmradio_ic_nofavorites.png"
        title: qsTr("No Favorites") + Retranslate.onLanguageChanged
        subTitle: qsTr("To add your favorite stations, on the Player screen or in the Local Stations list, touch and hold the station that you want to add.")  + Retranslate.onLanguageChanged
        pageDataModel: RadioPlaybackContext.favouritesDataModel
        verticalAlignment: VerticalAlignment.Center
    }
    
    onCreationCompleted: {
        //Because the Favourites Data Model may not signal any changes after it's loaded when it's empty
        //and therefore the NoContentView won't be initialized correctly, we need to manually update it.
        noFavouritesView.handlePageDataModelChanged();
    }
    
    Container {
        visible: ! noFavouritesView.visible
        ListView {
            id: listView
            scrollRole: ScrollRole.None
            leftPadding: LayoutConstants.LIBRARY_LIST_VIEW_LEFT_PADDING
            rightPadding: LayoutConstants.LIBRARY_LIST_VIEW_RIGHT_PADDING

            dataModel: TuneDecorator {
                id: tuneDecorator
                model: RadioPlaybackContext.favouritesDataModel
            }           

            layout: GridListLayout {
                columnCount: LayoutConstants.LIBRARY_LIST_VIEW_COLUMN_COUNT
                cellAspectRatio: 1
                orientation: LayoutOrientation.TopToBottom
                spacingBeforeHeader: LayoutConstants.LIBRARY_LIST_VIEW_SPACING_HEADER;
                spacingAfterHeader: LayoutConstants.LIBRARY_LIST_VIEW_SPACING_HEADER;
                horizontalCellSpacing: 0
                verticalCellSpacing: 4
            }

            listItemComponents: [
                ListItemComponent {
                    type:"item"
                    RadioListItem {
                        favourite:true
                        showStar:false
                    }
                },
                ListItemComponent {
                    type: "tune"
                    TuneInListItem {
                    }
                }
            ]

            onTriggered: {
                var itemType = dataModel.itemType(indexPath);
                
                if (itemType == "tune"){
                    RadioPlaybackContext.launchPlayerScreenForTuneIn();
                } else {
                    RadioPlaybackContext.playIndex(RadioPlaybackContext.favouritesDataModel, tuneDecorator.translateIndex(indexPath));
                }	
            }
        }
    }
}
