import bb.cascades 1.0
import bb.cascades.music 1.0
import bb.cascades.multimedia 1.0
import bb.cascades.multimedia.internal 1.0
import "../../core/layoutconstants.js" as LayoutConstants

Container {
    verticalAlignment: VerticalAlignment.Center
    horizontalAlignment: HorizontalAlignment.Center
    
    property bool scanning: false
    
    onCreationCompleted: {
        RadioPlaybackContext.radioPlayer.scanStarted.connect(onScanStarted);
        RadioPlaybackContext.radioPlayer.scanCompleted.connect(onScanCompleted);
        RadioPlaybackContext.radioPlayer.scanStopped.connect(onScanCompleted);
    }

    function onScanStarted() {
        scanning  = true;
    }
    
    function onScanCompleted() {
        scanning  = false;        
    }
    
    ScanningProgressView {
        id: progressView
        verticalAlignment: VerticalAlignment.Center
        horizontalAlignment: HorizontalAlignment.Center
        visible: scanning && !RadioPlaybackContext.stationsDataModel.hasChildren([])
    }
    
    Container {
        layout: DockLayout {}
        
        visible: !progressView.visible
        verticalAlignment: VerticalAlignment.Center
        TwoButtonNoContentView {
            id: noStationsView
            image: "asset:///images/radio/fmradio_ic_empty_nostations.png"
            title: qsTr("No Stations Found") + Retranslate.onLanguageChanged
            subTitle: qsTr("No FM Radio Stations were found in your local area. Try scanning again or tune to a station manually.") + Retranslate.onLanguageChanged
            noContentButtonVisible: true
            noContentButtonLabel: qsTr("Tune To") + Retranslate.onLanguageChanged
            noContentButton2Visible: true
            noContentButton2Label: qsTr("Refresh") + Retranslate.onLanguageChanged
            pageDataModel: RadioPlaybackContext.stationsDataModel
            
            onNoContentButtonClicked: {
                RadioPlaybackContext.launchPlayerScreenForTuneIn();
            }
    
            onNoContentButton2Clicked: {
                //Why is it necessary to make this connection? It's already been made once
                //RadioPlaybackContext.radioPlayer.scanStarted.connect(onScanStarted);
                RadioPlaybackContext.radioScanner.scan();
            }
        }
        
        onCreationCompleted: {
            //Because the Stations Data Model may not signal any changes after it's loaded when it's empty
            //and therefore the NoContentView won't be initialized correctly, we need to manually update it.
            noStationsView.handlePageDataModelChanged();   
        }     
        
        Container {
            layout: StackLayout {}
            
            //Container for scan in progress indication
            Container {
                id:scanInProgressHeader
                visible: ! noStationsView.visible && RadioPlaybackContext.radioScanner.scanningActive
                horizontalAlignment: HorizontalAlignment.Center
                bottomPadding: LayoutConstants.LIBRARY_LIST_VIEW_SPACING_HEADER + LayoutConstants.CONTROL_PADDING
                preferredHeight: LayoutConstants.RADIO_LIST_SCANNING_INDICATOR_HEIGHT
            
                layout: StackLayout {
                    orientation: LayoutOrientation.LeftToRight;
                }                
                                      
                ActivityIndicator {        
                    id: scanIndicator
                    verticalAlignment: VerticalAlignment.Center
                    running: scanInProgressHeader.visible
                }

                Label {
                    text: qsTr("Scanning") + Retranslate.onLanguageChanged
                    verticalAlignment: VerticalAlignment.Center
               }
            }

            ListView {
                id: listView
                scrollRole: ScrollRole.None
                leftPadding: LayoutConstants.LIBRARY_LIST_VIEW_LEFT_PADDING
                rightPadding: LayoutConstants.LIBRARY_LIST_VIEW_RIGHT_PADDING

                visible: ! noStationsView.visible

                property bool loading: false

                dataModel: TuneDecorator {
                    id: tuneDecorator
                    model: RadioPlaybackContext.stationsDataModel
                }

                layout: GridListLayout {
                    columnCount: LayoutConstants.LIBRARY_LIST_VIEW_COLUMN_COUNT
                    cellAspectRatio: 1
                    orientation: LayoutOrientation.TopToBottom
                    spacingBeforeHeader: LayoutConstants.LIBRARY_LIST_VIEW_SPACING_HEADER
                    spacingAfterHeader: LayoutConstants.LIBRARY_LIST_VIEW_SPACING_HEADER
                    horizontalCellSpacing: 0
                    verticalCellSpacing: 4
                }

                listItemComponents: [
                    ListItemComponent {
                        type: "item"
                        RadioListItem {
                            favourite: ListItemData["isfavourite"]
                        }
                    },
                    ListItemComponent {
                        type: "tune"
                        TuneInListItem {
                        }
                    }
                ]

                onTriggered: {
                    var itemType = dataModel.itemType(indexPath);

                    if (itemType == "tune") {
                        RadioPlaybackContext.launchPlayerScreenForTuneIn();
                    } else {
                        RadioPlaybackContext.playIndex(RadioPlaybackContext.stationsDataModel, tuneDecorator.translateIndex(indexPath));
                    }
                }
            }
        }
    }  
}
