import bb.cascades 1.2
import bb.cascades.multimedia 1.0
import "../../core/layoutconstants.js" as LayoutConstants

Container {
    id:nowPlaying
    
    property bool __fingerDown
    
    layout: DockLayout {}
    
    verticalAlignment: VerticalAlignment.Center
    horizontalAlignment: HorizontalAlignment.Right
    
    // The animation duration for all album art animations    
    property int c_ALBUM_ART_ANIMATION_DURATION: 400
    property int c_ALBUM_ART_ANIMATION_DELAY: 150
        
    onCreationCompleted: {
        label.text = RadioPlaybackContext.currentRadioFrequency
        RadioPlaybackContext.radioPlayer.frequencyChanged.connect(handleCurrentFrequencyChanged);
        
        // Register for player screen closed events if playback is not active
        // If it is active then we must have opened and closed the Player screen
        // at some point. We only want to show the album art animation, the 
        // very first time we play and exit the Player screen.
        if (!RadioPlaybackContext.radioPlayerHasBeenClosed) {
            RadioPlaybackContext.radioPlayerClosed.connect(onRadioplayerClosed);
        }
    }
    
    function handleCurrentFrequencyChanged (frequency){
        // TODO: See MKS 6934800
        // Sometimes we switch the source component from nowPlayingComponent (this)
        // to the empty component, but these signals still execute, however if this (nowPlayingComponent)
        // is not the active sourceComponent 'tabbedPane' is not a valid symbol causing an exception.
        // If it is not caught, we might end up not executing a bunch of JavaScript methods after this.
        try {
            if (!animationRunning()){
                // If we don't have any animation running, start a new one
                showPlayIcon.play();   
            } else if (showAlbumAnimationRunning()) {
                // If we are in currently trying to fade in the new album art, set the correct
                // frequency otherwise we will be showing the wrong one at the end of the animation
                label.text = RadioPlaybackContext.formatFrequency(frequency);
            }
        } catch (e) {
            console.debug("nowplaying/NowPlayingScreenTitle.qml::songChanged() - exception caught, likely due to current source component");
        }
    }
    
    function onRadioplayerClosed() {
        // TODO: See MKS 6934800
        // Sometimes we switch the source component from nowPlayingComponent (this)
        // to the empty component, but these signals still execute, however if this (nowPlayingComponent)
        // is not the active sourceComponent 'tabbedPane' is not a valid symbol causing an exception.
        // If it is not caught, we might end up not executing a bunch of JavaScript methods after this.
        try {
            // We got out first player closed event. Disconnect
            // since we only want to show the album art animation, the 
            // very first time we play and exit the Player screen.
            RadioPlaybackContext.radioPlayerClosed.disconnect(onRadioplayerClosed);
            
            // Play the album art animation
            handleCurrentFrequencyChanged();
        
        } catch (e) {
            console.debug("ScreenTitle.qml::playerClosed() - exception caught, likely due to current source component");
        }
    }    
    
    // Are any of the album animation running?
    function animationRunning() {
        return showPlayIcon.isPlaying() || showPlayIcon.isStarted() || showAlbumArt.isPlaying() || showAlbumArt.isStarted();
    }
    
    // Is show album art animation running?
    function showAlbumAnimationRunning() {
        return showAlbumArt.isPlaying() || showAlbumArt.isStarted();
    }
    
    Container {
        layout: DockLayout {}    
        
	    ImageView {
            id: nowPlayingButton
	        imageSource: "asset:///images/radio/fmradio_default_album_empty.png"
	        preferredHeight: 81
	        preferredWidth: 81
	        
	        scalingMethod: ScalingMethod.AspectFill
	        verticalAlignment: VerticalAlignment.Center
	        horizontalAlignment: HorizontalAlignment.Center
	    }
	
	    Label {
	        id: label
	        overlapTouchPolicy: OverlapTouchPolicy.Allow	        
	        horizontalAlignment: HorizontalAlignment.Center
	        verticalAlignment: VerticalAlignment.Center
	        textStyle.base: SystemDefaults.TextStyles.SubtitleText
	        textStyle.color: MediaApplication.textColor
            textStyle.fontSize: LayoutConstants.RADIO_SCREEN_TITLE_FONT_SIZE
	        textStyle.fontWeight: FontWeight.W500
	        textFormat: TextFormat.Plain
	    }
	}

    ImageView {
        id: nowPlayingBorder
        preferredHeight: 85
        preferredWidth: 85
        verticalAlignment: VerticalAlignment.Center
        horizontalAlignment: HorizontalAlignment.Center
        imageSource: nowPlaying.__fingerDown ? "asset:///images/common/core_item_border.amd" : "asset:///images/common/ic_albumart_nowplaying_frame.amd"
    }
    ImageView {
        id:playIcon
        opacity:0
        scalingMethod: ScalingMethod.None
        verticalAlignment: VerticalAlignment.Center
        horizontalAlignment: HorizontalAlignment.Center  
        imageSource: "asset:///images/playing/control_play_mini.png"			    	
    }
    onTouch: {
        if (event.isDown()) {
            __fingerDown = true;
        } else if (event.isCancel()) {
            __fingerDown = false;
        } else if (event.isUp() && __fingerDown) {
            __fingerDown = false;            
            RadioPlaybackContext.launchRadioPlayerScreen();
        }
    }
    
    animations: [       
        // Animation to fade out the previous album art and show the play icon momentarily
        ParallelAnimation {	 
            id: showPlayIcon 		    		
            FadeTransition {
                target: nowPlayingButton
                toOpacity: 0
                duration: c_ALBUM_ART_ANIMATION_DURATION
                easingCurve: StockCurve.CubicInOut
            }		
            FadeTransition {
                target: label
                toOpacity: 0
                duration: c_ALBUM_ART_ANIMATION_DURATION
                easingCurve: StockCurve.CubicInOut
            }            
            FadeTransition {
                target: nowPlayingBorder
                toOpacity: 0
                duration: c_ALBUM_ART_ANIMATION_DURATION
                easingCurve: StockCurve.CubicInOut
            } 		                        		   
            FadeTransition {
                target: playIcon
                toOpacity: 1
                duration: c_ALBUM_ART_ANIMATION_DURATION
                easingCurve: StockCurve.CubicInOut
            } 	        
            onEnded:{  
                label.text = RadioPlaybackContext.currentRadioFrequency    	 
                id:showAlbumArt.play()
            }
        },
        // Animation to fade out the play icon and show the next album art
        ParallelAnimation {	   
            id:showAlbumArt     		
            delay: c_ALBUM_ART_ANIMATION_DELAY   
            FadeTransition {
                target: nowPlayingButton
                toOpacity: 1
                duration: c_ALBUM_ART_ANIMATION_DURATION
                easingCurve: StockCurve.CubicInOut
            }
            FadeTransition {
                target: label
                toOpacity: 1
                duration: c_ALBUM_ART_ANIMATION_DURATION
                easingCurve: StockCurve.CubicInOut
            }     		            
            FadeTransition {
                target: nowPlayingBorder
                toOpacity: 1
                duration: c_ALBUM_ART_ANIMATION_DURATION
                easingCurve: StockCurve.CubicInOut
            } 		            
            FadeTransition {
                target: playIcon
                toOpacity: 0
                duration: c_ALBUM_ART_ANIMATION_DURATION
                easingCurve: StockCurve.CubicInOut	            
            }     
        }
    ]
}
