import bb.cascades 1.0
import bb.cascades.multimedia 1.0
import bb.cascades.multimedia.internal 1.0
import "../../core/layoutconstants.js" as LayoutConstants

Container {
    id:radioSegmentedControl
    property variant favouritesList
    property variant localStationsList
    
    property variant page
    
    layout: DockLayout {}
    
	horizontalAlignment: HorizontalAlignment.Fill
    verticalAlignment: VerticalAlignment.Fill
    
    attachedObjects: [        
        ScanActionItem {
            id: scanItem
        },
        InvokeHelper {
        	id: invokeHelper
        }
    ]
    
    NoContentView {
        id: noHeadphonesView
        image: "asset:///images/radio/fmradio_ic_empty_headphones.png"
        title: qsTr("No Headphones") + Retranslate.onLanguageChanged
        subTitle: qsTr("To use FM Radio, connect your headphones or hands free headset. They operate as your radio antenna.") + Retranslate.onLanguageChanged
        verticalAlignment: VerticalAlignment.Center
        visible: !RadioPlaybackContext.headphonesConnected
    }
    
    NoContentView {
        id: airplaneNoContent
        visible: RadioPlaybackContext.airplaneModeOn && RadioPlaybackContext.headphonesConnected
        image: "asset:///images/radio/fmradio_ic_empty_airplane.png"
        title: qsTr("Airplane Mode On") + Retranslate.onLanguageChanged
        subTitle: qsTr("FM Radio is disabled when Airplane Mode is on.Turn off Airplane Mode to use FM Radio.") + Retranslate.onLanguageChanged
        noContentButtonVisible: true
        noContentButtonLabel: qsTr("Change Airplane Mode") + Retranslate.onLanguageChanged
        
        onNoContentButtonClicked: {
            //We want to Show WiFi Settings Page as a card
            invokeHelper.invokeTargetApp("bb.action.OPEN",
            "sys.settings.card",
            "settings://networkconnections",
            "settings/view",
            []);
        }
    }

    Container {

        id: radioContentView
        layout: DockLayout {}

        horizontalAlignment: HorizontalAlignment.Fill
        verticalAlignment: VerticalAlignment.Fill

        visible: !noHeadphonesView.visible && !airplaneNoContent.visible
        
        Container {
            id: stationsView
            
            layout: StackLayout {
                orientation: LayoutOrientation.TopToBottom
            }
            horizontalAlignment: HorizontalAlignment.Fill
            verticalAlignment: VerticalAlignment.Fill

            Container {
                leftPadding: 15
                rightPadding: 15
                topPadding: LayoutConstants.CONTROL_PADDING
                bottomPadding: LayoutConstants.CONTROL_PADDING

                layout: DockLayout {
                }

                SegmentedControl {
                    id: segmentedControl
                    objectName: "segmentedControl"
                    horizontalAlignment: HorizontalAlignment.Center

                    Option {
                        id: favouritesOption
                        text: qsTr("Favorites") + Retranslate.onLanguageChanged
                        value: 0
                        selected: true
                    }
                    Option {
                        id: localsOption
                        text: qsTr("Local Stations") + Retranslate.onLanguageChanged
                        value: 1
                    }

                    onSelectedIndexChanged: {
                        radioSegmentedControl.loadSegment(segmentedControl.selectedValue);
                    }
                }
            }

            Container {
                id: switchableContainer                
                layout: DockLayout{}
                
                horizontalAlignment: HorizontalAlignment.Fill
                verticalAlignment: VerticalAlignment.Fill
                
                layoutProperties: StackLayoutProperties {
                    spaceQuota: 1
                }

                onCreationCompleted: {
                    if (RadioPlaybackContext.favouritesDataModel.hasChildren([])) {
                        // Even though the favouritesOption is not selected, the segmented control
                        // must think it is the default option because calling this isn't
                        // triggering a onSelectedIndexChanged().
                        // segmentedControl.setSelectedOption(favouritesOption);
                        // Have to call this to get the favourites list view to show.
                        radioSegmentedControl.loadSegment(favouritesOption.value);
                    } else {
                        segmentedControl.setSelectedOption(localsOption);
                    }
                }
            }
        }
    }

	onPageChanged: {
        // Update the menu items for the new page
        updateMenuItems();
    }
	
	function updateMenuItems(){
        if(page){
            // Add scan menu item for local stations list
            if (segmentedControl.selectedValue == localsOption.value){
                qmlUtils.addActionItemToPage(scanItem,page);
            } else {
                qmlUtils.removeActionItemFromPage(scanItem,page);
            }
        }   
	}
	
    function addPage(page) {
        // Update the menu items for the segment we just loaded
        updateMenuItems();
        
        if (switchableContainer.count() == 0) {
            switchableContainer.add(page);
        } else {
            switchableContainer.replace(0, page);
        }
    }

    function createAndAddPage (switchableContainerQMLPath) {    
    	var component = Qt.createComponent(switchableContainerQMLPath);

		var list;
    	if (!component.errorString()) {
            list = component.createObject(switchableContainer);   
            if (list){
            	addPage(list);
            }            
        } else { 
            console.log("critical error loading radio segments: "  + component.errorString());
        }         
        
        return list;
    }
    
    function loadSegment(stationsList) {        
        
        switch (stationsList) {
            case favouritesOption.value: 	
                if (favouritesList == null) {
                    favouritesList = createAndAddPage("asset:///radio/components/FavouritesListView.qml");
	        	} else {
	        		addPage(favouritesList);
	        	}   
	            break;
	        case localsOption.value:
	            if (localStationsList == null) {
                    localStationsList = createAndAddPage("asset:///radio/components/LocalStationsListView.qml");
                    
                    if (!RadioPlaybackContext.stationsDataModel.hasChildren([])) {
                        RadioPlaybackContext.radioScanner.scan();
                    }
                        
	        	} else {
	        		addPage(localStationsList);
	        	}
	            break;
        }     
    }
}
