import bb.cascades 1.2
import bb.system 1.2

Container {
    //Whether or not to prompt the user that scanning interrupts playback
    property bool warnScanInterruptsPlayback: true
    property bool scanningActive: false
    property bool scanningPossible: !scanningActive && RadioPlaybackContext.headphonesConnected && !RadioPlaybackContext.airplaneModeOn
    property variant confirmScanDialog
    
    attachedObjects: [
        SystemDialog {
            id: confirmScanDialog
            title: qsTr("Scan for FM Stations") + Retranslate.onLanguageChanged
            body: qsTr("When you scan for stations, the current station stops playing.") + Retranslate.onLanguageChanged
            confirmButton.label: qsTr("Scan") + Retranslate.onLanguageChanged
            cancelButton.label: qsTr("Cancel") + Retranslate.onLanguageChanged
            includeRememberMe: true
            rememberMeChecked: false
            rememberMeText: qsTr("Don't show again") + Retranslate.onLanguageChanged
        }
    ]
    
    onCreationCompleted: {
        console.log("+ Scanner.onCreationCompleted");
        
        warnScanInterruptsPlayback = musicUtils.getWarnScanInterruptsFMPlaybackSetting();
        confirmScanDialog.finished.connect(onConfirmScanDialogAccepted);
        
        radioPlayer.scanStarted.connect(onScanStarted);
        radioPlayer.scanCompleted.connect(onScanFinished);
        radioPlayer.scanStopped.connect(onScanFinished);
        console.log("- Scanner.onCreationCompleted");
    }
    
    function onScanStarted() {
        scanningActive = true;
    }
    
    function onScanFinished() {
        scanningActive = false;    
    }
    
    //Start a scan but first check to see if playback is active and if it
    //is confirm that the user wishes to interrupt playback.
    function scan() {
        console.debug(" + RadioPlaybackContext::scanWithPrompt");
        if (scanningPossible) {
            if (! RadioPlaybackContext.radioPlaybackActive || ! warnScanInterruptsPlayback) {
                //If there is no playback, or the user has chosen to ignore warnings about scan
                //interrupting playback then just start scan.  This is happening from an action
                //item so no need to double check headphones and airplane mode.
                radioPlayer.scan();
            } else {
                //We've got to warn about scan
                confirmScanDialog.show();
            }
        } else {
            console.debug("Can't start a scan at this time");
        }
        console.debug(" - RadioPlaybackContext::scanWithPrompt");
    }
    
    function onConfirmScanDialogAccepted(value) {
        console.log(" + onConfirmRefreshDialogAccepted: " + confirmScanDialog.result);
        if(confirmScanDialog.result == SystemUiResult.ConfirmButtonSelection) {
            //We can go ahead and scan as long as headphones are still in and we're not in airplane mode
            //Things could have changed while the dialog is up.
            if(scanningPossible) {
                radioPlayer.scan();    
            } else {
             	console.log("Not scanning on confirm since headphones connected or we're in airplane mode.");   
            }            
        }
        
        //If they checked the do not show again checkbox then make sure we don't show it again
        if(confirmScanDialog.rememberMeSelection()) {
            console.log("ignore scan interrupts playback warning selected");
            warnScanInterruptsPlayback = false;
            musicUtils.setWarnScanInterruptsFMPlaybackSetting(false);
        }
    }
}
