import bb.cascades 1.0
import "../../nowplaying"
import "../components"
import "../../core/layoutconstants.js" as LayoutConstants

Container {
	property string station	
	layout: DockLayout {}
	
	property variant radioStationsModel
	property variant bottomControl
	
	Container {
		layout: DockLayout{}
		// This container has padding  to align with the RadioPlayerListHeader
		leftPadding: LayoutConstants.TRACKS_LIST_HORIZONTAL_PADDING
        rightPadding: LayoutConstants.TRACKS_LIST_HORIZONTAL_PADDING 
		RadioPlayerListView {
	    	id: radioPlayerListView
	        verticalAlignment: VerticalAlignment.Top	        
	    }
    }
    
	NowPlayingAlbumCover {
		id: nowPlayingAlbumCover
		
		leftPadding: LayoutConstants.ALBUM_COVER_H_PADDING
        rightPadding: LayoutConstants.ALBUM_COVER_H_PADDING
        
        albumCoverHeight: LayoutConstants.RADIO_PLAYER_ALBUM_HEIGHT 
		
        defaultAlbumArt: "asset:///images/radio/fmradio_default_album_big.png"
	           
	   	downPos: LayoutConstants.ALBUM_COVER_OPEN_POSITION
	    downThreshold: LayoutConstants.ALBUM_COVER_OPEN_THRESHOLD                            
	           
	    horizontalAlignment: HorizontalAlignment.Fill
	    verticalAlignment: VerticalAlignment.Fill	    
	}
	
	RadioPlayerAlbumOverlay{
		id: overlay
		stationLabel: station
		horizontalAlignment: HorizontalAlignment.Fill
        minHeight: LayoutConstants.RADIO_PLAYER_ALBUM_HEIGHT
        maxHeight: LayoutConstants.RADIO_PLAYER_ALBUM_HEIGHT
	}
    
    // --- Scrolling Logic ---
        
    onCreationCompleted: {
    	//Connect all the scroll handlers to the cover signals
	    nowPlayingAlbumCover.coverOpenStarted.connect(onCoverOpenStarted);
	    nowPlayingAlbumCover.coverCloseStarted.connect(onCoverCloseStarted);
	    nowPlayingAlbumCover.scrollStarted.connect(onScrollStarted);
	    nowPlayingAlbumCover.scrollProgress.connect(onScrollProgress);
	    nowPlayingAlbumCover.scrollEnded.connect(onScrollEnded);    	
    }    
   
    function onCoverOpenStarted () {
        if (!openCover.isPlaying()) {
            stopAllAnimations();
            openCover.play();
        }
    }
		
    function onCoverCloseStarted () {
        if (!closeCover.isPlaying()) {
            stopAllAnimations();
            closeCover.play();
        }
    }
    
    function onScrollStarted () {                    
        if (!nowPlayingAlbumCover.coverOpen) {
            radioPlayerListView.scrollToPlayingItem();
        }
        
        // Disable list view to prevent the user from accidentally scroll the list view
        radioPlayerListView.touchPropagationMode = TouchPropagationMode.None;
        
        // Make volume control and now playing header visible because their opacity
        // will be changed by onScrollProgress slot.
        if (!LayoutConstants.VOLUME_CONTROL_DEFAULT_VISIBILITY) {
            volumeControl.visible = true;
            radioPlayerHeader.visible = true;
        }
    }
    
    function onScrollProgress (delta) {            
        if (!LayoutConstants.VOLUME_CONTROL_DEFAULT_VISIBILITY) {
        	// Change the opacity of volume control and now playing header.
            var progress = delta / LayoutConstants.ALBUM_COVER_OPEN_POSITION;
            volumeControl.opacity = progress;
            radioPlayerHeader.opacity = 1.0 - progress;
        }
                    
        // Move the station label and the tap to button
        overlay.translationY = delta;
        bottomControl.translationY = delta;
    }
		
    function onScrollEnded (coverOpen) {
    	radioPlayerListView.touchPropagationMode = TouchPropagationMode.Full;
    	                        
    	// Snap the album cover into position
        nowPlayingAlbumCover.translationY = coverOpen ? LayoutConstants.ALBUM_COVER_OPEN_POSITION : 0; 
        nowPlayingAlbumCover.coverArtCutClosed.opacity = coverOpen ? 0.0 : 1.0;
        nowPlayingAlbumCover.coverArtCutOpen.opacity = coverOpen ? 1.0 : 0.0 ;
        
        // Snap the station label into position
        overlay.translationY = coverOpen ? LayoutConstants.ALBUM_COVER_OPEN_POSITION : 0
        bottomControl.translationY = coverOpen ? 300 : 0
        
        if (!LayoutConstants.VOLUME_CONTROL_DEFAULT_VISIBILITY) {
            volumeControl.opacity = coverOpen ? 1.0 : 0.0;
            radioPlayerHeader.opacity = coverOpen ? 0.0 : 1.0;
            volumeControl.visible = coverOpen;
            radioPlayerHeader.visible = !coverOpen;
        }
    }	
	
    function stopAllAnimations() {
        openCover.stop();
        closeCover.stop();
    }
	    
	// Animations for opening and closing the album art    
    animations: [
        ParallelAnimation {
            id: openCover
            objectName: "openCover"
            TranslateTransition {
                target: bottomControl
                toY: 300
                duration: LayoutConstants.ALBUM_ART_ANIMATION_DURATION
                easingCurve: StockCurve.CubicInOut
            }  
            TranslateTransition {
                target: nowPlayingAlbumCover
                toY: LayoutConstants.ALBUM_COVER_OPEN_POSITION
                duration: LayoutConstants.ALBUM_ART_ANIMATION_DURATION
                easingCurve: StockCurve.CubicInOut
            }
            TranslateTransition {
                target: overlay
                toY: LayoutConstants.ALBUM_COVER_OPEN_POSITION
                duration: LayoutConstants.ALBUM_ART_ANIMATION_DURATION
                easingCurve: StockCurve.CubicInOut
            }                     
            FadeTransition {
                target: volumeControl
                toOpacity: 1.0
                duration: LayoutConstants.ALBUM_ART_ANIMATION_DURATION
                easingCurve: StockCurve.CubicInOut
            }
            FadeTransition {
                target: radioPlayerHeader
                toOpacity: LayoutConstants.NOW_PLAYING_TITLE_TARGET_OPACITY
                duration: LayoutConstants.ALBUM_ART_ANIMATION_DURATION >> 1 // make the "fading effect" shorter
                easingCurve: StockCurve.CubicInOut
            } 
            FadeTransition {
                target: nowPlayingAlbumCover.coverArtCutClosed
                toOpacity: 0.0
                duration: LayoutConstants.ALBUM_ART_ANIMATION_DURATION
                easingCurve: StockCurve.CubicInOut
            }
            FadeTransition {
                target: nowPlayingAlbumCover.coverArtCutOpen
                toOpacity: 1.0
                duration: LayoutConstants.ALBUM_ART_ANIMATION_DURATION
                easingCurve: StockCurve.CubicInOut
            }
            
            onStarted: {
                radioPlayerListView.scrollToPlayingItem();
                volumeControl.visible = true;
            }
            onEnded:{
                radioPlayerHeader.visible = LayoutConstants.NOW_PLAYING_TITLE_TARGET_OPACITY;
            }
        },
        ParallelAnimation {
            id: closeCover
            objectName: "closeCover"

            TranslateTransition {
                target: nowPlayingAlbumCover
                toY: 0
                duration: LayoutConstants.ALBUM_ART_ANIMATION_DURATION
                easingCurve: StockCurve.CubicInOut
            }
            TranslateTransition {
                target: overlay
                toY: 0
                duration: LayoutConstants.ALBUM_ART_ANIMATION_DURATION
                easingCurve: StockCurve.CubicInOut
            }
            TranslateTransition {
                target: bottomControl
                toY: 0
                duration: LayoutConstants.ALBUM_ART_ANIMATION_DURATION
                easingCurve: StockCurve.CubicInOut
            } 
            FadeTransition {
                target: volumeControl
                toOpacity: LayoutConstants.VOLUME_CONTROL_DEFAULT_OPACITY
                duration: LayoutConstants.ALBUM_ART_ANIMATION_DURATION / 2 // make the "fading effect" shorter
                easingCurve: StockCurve.CubicInOut
            }
            FadeTransition {
                target: radioPlayerHeader
                toOpacity: 1.0
                duration: LayoutConstants.ALBUM_ART_ANIMATION_DURATION
                easingCurve: StockCurve.CubicInOut
            }
            FadeTransition {
                target: nowPlayingAlbumCover.coverArtCutClosed
                toOpacity: 1.0
                duration: LayoutConstants.ALBUM_ART_ANIMATION_DURATION
                easingCurve: StockCurve.CubicInOut
            }
            FadeTransition {
                target: nowPlayingAlbumCover.coverArtCutOpen
                toOpacity: 0.0
                duration: LayoutConstants.ALBUM_ART_ANIMATION_DURATION
                easingCurve: StockCurve.CubicInOut
            }
            onStarted: {
                radioPlayerHeader.visible = true;
            }
            onEnded: {
                volumeControl.visible = LayoutConstants.VOLUME_CONTROL_DEFAULT_VISIBILITY;
            }
        }
    ]
}