import bb.cascades 1.2
import bb.cascades.multimedia 1.0
import bb.cascades.multimedia.internal 1.0
import bb.multimedia 1.0
import "../../core"
import "../../nowplaying"
import "../../core/layoutconstants.js" as LayoutConstants
import "../components"


Page {
	id: radioPlayerScreen
    objectName: "RadioPlayerScreenObjectName" // Required by various methods in SafePushNavPane.qml
    property bool closing: false
    
    paneProperties: NavigationPaneProperties {
        backButton: ActionItem {
            onTriggered: {
                closePlayerScreen();
            }
        }
    }
    
    onCreationCompleted: {
        console.debug("+RadioPlayerScreen: onCreationCompleted..");

        // connect signals
        RadioPlaybackContext.radioPlayerClosed.connect(cleanup);
        
        onMediaStateChanged() 
        RadioPlaybackContext.radioPlayer.mediaStateChanged.connect(onMediaStateChanged);
        
        RadioPlaybackContext.headphoneConnectionStatusChanged.connect(onHeadphoneConnectionStatusChanged);
        RadioPlaybackContext.airplaneModeEnabled.connect(onAirplaneModeOn);
        
        appContext.windowStateChanged.connect(setNowPlayingOverlayStyle);
        RadioPlaybackContext.nowPlaying.setOverlayStyle(OverlayStyle.Plain);
        
        
        console.debug("-RadioPlayerScreen: onCreationCompleted..");
    }
    
    function closePlayerScreen(){
        console.log("+ RadioPlayerScreen closePlayerScreen()");
        if(!closing) {
            closing = true;
            cleanup();
            tabbedPane.currentNavPane.pop();
        }
    }

    function cleanup(){
        console.log("+ RadioPlayerScreen cleanup()");
    
        // disconnect signals
        RadioPlaybackContext.radioPlayerClosed.disconnect(cleanup);
        RadioPlaybackContext.radioPlayer.mediaStateChanged.disconnect(onMediaStateChanged);
        RadioPlaybackContext.headphoneConnectionStatusChanged.disconnect(onHeadphoneConnectionStatusChanged);
        RadioPlaybackContext.airplaneModeEnabled.disconnect(onAirplaneModeOn);
        
        appContext.windowStateChanged.disconnect(setNowPlayingOverlayStyle);
        RadioPlaybackContext.nowPlaying.setOverlayStyle(OverlayStyle.Fancy);
        
        console.log("- RadioPlayerScreen cleanup()");
    }
    
    function setNowPlayingOverlayStyle () {
        console.debug("+ RadioPlaybackContext.setNowPlayingOverlayStyle()");
        // 0 means full screen
        if (appContext.windowState == 0) {
            console.debug("Full screen!");
            console.debug("Player screen is open, be lazy");
            RadioPlaybackContext.nowPlaying.setOverlayStyle(OverlayStyle.Plain);
        } else {
            console.debug("Not full screen, be fancy!");
            RadioPlaybackContext.nowPlaying.setOverlayStyle(OverlayStyle.Fancy);
        }
        console.debug("- RadioPlaybackContext.setNowPlayingOverlayStyle()")
    }
    
    function requestTuneBoxFocus(){
        tuneButton.requestFocus();
    }
    
    function onMediaStateChanged() {
        console.debug("+RadioPlayerScreen::onMediaStateChanged()");
        if (RadioPlaybackContext.radioPlayer.mediaState == MediaState.Started) {
            playPauseButton.imageSource = "asset:///images/playing/ic_pause.png"
            playPauseButton.title = qsTr("Pause") + Retranslate.onLanguageChanged;
            console.debug("..media state is playing..");
        } else {
            playPauseButton.imageSource = "asset:///images/playing/ic_play.png"
            playPauseButton.title = qsTr("Play") + Retranslate.onLanguageChanged;
            console.debug("..media state is stopped..");
        }
        console.debug("- RadioPlayerScreen::onMediaStateChanged()");
    }
    
    function onHeadphoneConnectionStatusChanged(connected){
        console.log("+ RadioPlayerScreen onHeadphoneConnectionStatusChanged: " + connected);
        if(!connected) {
            //We need to pop the player screen and anything on top since headphones are needed for playback            
            closePlayerScreen();
        }
    }
    
    function onAirplaneModeOn() {
        console.log("+ RadioPlayerScreen onAirplaneModeOn");
        closePlayerScreen();
    }
        
    actions: [
        ActionItem {
            ActionBar.placement: ActionBarPlacement.OnBar
            enabled: RadioPlaybackContext.playlistController.prevAvailable && RadioPlaybackContext.radioPlaybackActive
            title: qsTr("Previous") + Retranslate.onLanguageChanged
            imageSource: "asset:///images/playing/ic_previous.png"
            onTriggered: {
                RadioPlaybackContext.playlistController.prev();
            }
        },
        ActionItem {
            id: playPauseButton
            ActionBar.placement: ActionBarPlacement.OnBar
            title: qsTr("Pause") + Retranslate.onLanguageChanged
            imageSource: "asset:///images/playing/ic_pause.png"
            enabled: RadioPlaybackContext.radioPlaybackActive
           
            onTriggered: {
                if (RadioPlaybackContext.radioPlayer.mediaState == MediaState.Started) { 
                    RadioPlaybackContext.radioPlayer.pause();
                } else {
                    if (!RadioPlaybackContext.nowPlaying.acquired) {
                        var error = RadioPlaybackContext.nowPlaying.acquire();
                        if(error == MediaError.None){
                            RadioPlaybackContext.radioPlayer.play();
                        } else {
                            console.debug("Failed to acquire NPC");
                        }
                    } else {
                        RadioPlaybackContext.radioPlayer.play();
                    }
                }
            }            
        },
        ActionItem {
            ActionBar.placement: ActionBarPlacement.OnBar
            enabled: RadioPlaybackContext.playlistController.nextAvailable && RadioPlaybackContext.radioPlaybackActive
            title: qsTr("Next") + Retranslate.onLanguageChanged
            imageSource: "asset:///images/playing/ic_next.png"
            onTriggered: {
               RadioPlaybackContext.playlistController.next();
            }
        },
        ActionItem {
            title: qsTr("Seek down") + Retranslate.onLanguageChanged
            enabled: RadioPlaybackContext.radioPlaybackActive 
            imageSource: "asset:///images/radio/fmradio_seek_down.png"
            onTriggered: {
                RadioPlaybackContext.radioPlayer.seekDown();
            }
        },
        ActionItem {
            title: qsTr("Seek up") + Retranslate.onLanguageChanged
            enabled: RadioPlaybackContext.radioPlaybackActive 
            imageSource: "asset:///images/radio/fmradio_seek_up.png"
            onTriggered: {
                RadioPlaybackContext.radioPlayer.seekUp();
            }
        },
        SwitchAudioActionItem {
            enabled: RadioPlaybackContext.radioPlaybackActive 
            audioRoutingComponent: AudioRoutingComponent
        }
    ]  
        
    
    Container {
     	layout: DockLayout {}	    
	    
	    Container {
		    layout: StackLayout {
		        orientation: LayoutOrientation.TopToBottom
		    }   
		    
            Container {
                id: radioPlayerScreenTitle
                horizontalAlignment: HorizontalAlignment.Fill
                layout: DockLayout {}
                
                Container {
                    horizontalAlignment: HorizontalAlignment.Fill
                    minHeight: LayoutConstants.SCREEN_TITLE_HEIGHT
                    ImageView {
                        id: titleBar
                        imageSource: "asset:///images/common/core_titlebar_gradient_overlay_d.png"
                        horizontalAlignment: HorizontalAlignment.Fill
                        verticalAlignment: VerticalAlignment.Fill
                    }
                }
                
                Container {
                    attachedObjects: [
                        DockLayout { id: dockLayout },
                        StackLayout { 
                            id: stackLayout 
                            orientation: LayoutOrientation.TopToBottom
                        }
                    ]
                    
                    layout: LayoutConstants.VOLUME_CONTROL_DEFAULT_VISIBILITY ? stackLayout : dockLayout
                    Container {
                        // radio player header label
                        leftPadding: 30
                        horizontalAlignment: HorizontalAlignment.Fill
                        verticalAlignment: VerticalAlignment.Center 
                        minHeight: LayoutConstants.SCREEN_TITLE_HEIGHT
                        layout: DockLayout {}
                        
                        Label {
                            id: radioPlayerHeader
                            text: qsTr("Radio") + Retranslate.onLanguageChanged
                            textStyle.base: SystemDefaults.TextStyles.TitleText
                            verticalAlignment: VerticalAlignment.Center              
                        }
                    }
                    
                    RadioVolumeControl {
                        id: volumeControl
                        objectName: "volumeControl"
                        opacity: LayoutConstants.VOLUME_CONTROL_DEFAULT_OPACITY
                        visible: LayoutConstants.VOLUME_CONTROL_DEFAULT_VISIBILITY
                        minHeight: LayoutConstants.VOLUME_CONTROL_HEIGHT
                    }
                }
            }
		
			Container {
			   	leftPadding: LayoutConstants.TRACKS_LIST_HORIZONTAL_PADDING
	            rightPadding: LayoutConstants.TRACKS_LIST_HORIZONTAL_PADDING
	            topPadding: LayoutConstants.ALBUM_LIST_HEADER_TOP_PADDING
		        horizontalAlignment: HorizontalAlignment.Fill
		        
		        layout: DockLayout {}
		    	
		    	RadioPlayerListHeader {
                    station: RadioPlaybackContext.currentRadioFrequency
		    	}
		    	
		    }    
		    
	        RadioPlayerAlbumList {
	         	id: radioPlayerAlbumCover	         	      
	         	bottomControl: tuneButton
                 station: RadioPlaybackContext.currentRadioFrequency
	         	radioStationsModel: RadioPlaybackContext.playlistController.playlist	         	           
	            horizontalAlignment: HorizontalAlignment.Fill
	            verticalAlignment: VerticalAlignment.Top            
	            layoutProperties: StackLayoutProperties {
	  				spaceQuota: 1
				}
	        }
	   }
	   
	   Container {
	        layout: StackLayout {}
	      	overlapTouchPolicy: tuneButton.hasFocus ? OverlapTouchPolicy.Deny : OverlapTouchPolicy.Allow
            horizontalAlignment: HorizontalAlignment.Fill

	        Container {
	            id: invisibleContainer
	            layoutProperties: StackLayoutProperties {
	                spaceQuota: 1
	            }	            
	        }
			
   			TuneToBox {
		    	id: tuneButton
		        verticalAlignment: VerticalAlignment.Bottom
                background:  ColorUtils.get(0xff1c1c1c)
    		}
	    }
	}
}
